! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 原子クラス
module atoms_class
  use error_class
  use basis_class
  implicit none
  !private

  public::atoms_type
  public::init_atoms
  public::new_atoms
  public::read_inp_atoms
  public::read_atoms
  public::replace_atoms
  public::get_atoms
  public::num_atoms
  public::nao_atoms
  public::nele_atoms
  public::load_atoms
  public::load_atoms_f
  public::load_natom_atoms
  public::load_nao_atoms
  public::load_nele_atoms
  public::load_frame_atoms
  public::save_atoms
  public::save_atoms_f
  public::bcast_atoms
  public::free_atoms
  public::merge_atoms
  public::print_atoms
  public::print_brief_atoms
  public::print_xyz
  public::get_atomic_symbol
  public::get_ao_labels

  type::atoms_type
     integer::num = 0 !< the number of coordinates
     integer::nele !< the number of electrons
     integer::natom  !< the number of atoms
     integer::nghost !< the number of ghost atoms (no charge)
     integer::nbare  !< the number of bare atoms (no basis)
     integer::ndummy !< the number of dummy atoms (no basis, no charge)
     integer,allocatable::atnum(:) !< atomic numbers
     real(8),allocatable::chg(:)   !< charges in a.u.
     real(8),allocatable::xyz(:,:) !< cartesian coordinates in a.u.
     integer,allocatable::atom2basis(:) !< basis set assinment
     type(basis_type)::basis !< basis set data
  end type

  character(len=*),private,parameter::DEFAULT_BASIS_SET_NAME  = 'MINI-4'
  character(len=*),private,parameter::DEFAULT_BASIS_FILE_NAME = 'BasisFile'

  integer,private,parameter::MAX_ATOMIC_SYMBOLS = 56
  character(len=*),private,parameter::ATOMIC_SYMBOLS(0:MAX_ATOMIC_SYMBOLS) = (/&
      'X ',&
      'H ',                                     'He',&
      'Li', 'Be', 'B ', 'C ', 'N ', 'O ', 'F ', 'Ne',&
      'Na', 'Mg', 'Al', 'Si', 'P ', 'S ', 'Cl', 'Ar',&
      'K ', 'Ca', 'Sc', 'Ti', 'Va', 'Cr', 'Mn', 'Fe',&
            'Co', 'Ni', 'Cu', 'Zn', 'Ga', 'Ge', 'As', 'Se', 'Br', 'Kr',&
      'Rb', 'Sr', 'Y ', 'Zr', 'Nb', 'Mo', 'Tc', 'Ru',&
            'Rh', 'Pd', 'Ag', 'Cd', 'In', 'Sn', 'Sb', 'Te', 'I ', 'Xe',&
      'Cs', 'Ba'/)

  integer,private,parameter::UNKNOWN  = -1
  integer,private,parameter::IS_ATOM  = 1
  integer,private,parameter::IS_BARE  = 2
  integer,private,parameter::IS_GHOST = 3
  integer,private,parameter::IS_DUMMY = 4
  integer,private,parameter::IS_ANG   = 1
  integer,private,parameter::IS_AU    = 2

  character(len=*),private,parameter::c_lower = 'abcdefghijklmnopqrstuvwxyz'
  character(len=*),private,parameter::c_upper = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
  character(len=*),private,parameter::c_alpha = c_lower // c_upper
  character(len=*),private,parameter::c_digit = '1234567890'
  character(len=*),private,parameter::c_alnum = c_alpha // c_digit
  character(len=*),private,parameter::c_minus = '-'

  character(len=*),private,parameter::url = 'data_atoms' !< BMSファイル名称
  integer,private,parameter::id = 1
  character(len=*),private,parameter::unit_au = 'a.u.'
  character(len=*),private,parameter::no_unit = ' '

contains
  !-----------------------------------------------------------------------------
  function get_atomic_symbol(atnum) result(symbol)
    character(len=2)::symbol
    integer::atnum

    if (0 <= atnum .and. atnum <= MAX_ATOMIC_SYMBOLS) then
       symbol = ATOMIC_SYMBOLS(atnum)
    else
       symbol = '  '
    end if

  end function get_atomic_symbol

  !-----------------------------------------------------------------------------
  subroutine get_ao_labels(self, labels, nao, ldebug, istat)
    type(atoms_type),intent(in)::self
    integer,intent(in)::nao
    character(len=8)::labels(nao)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::iatom, ibasis, ishell, ishell_type, icomp, iao
    character(len=2)::atomic_symbol
    character(len=1)::cshell_type

    labels(1:nao) = ' '
    iao = 1
    do iatom = 1, self%num
       labels(iao)(1:2) = get_atomic_symbol(self%atnum(iatom))
       !write(6,*) 'iatom=',iatom,labels(iao)(1:2)
       ibasis = self%atom2basis(iatom)
       do ishell = 1, self%basis%ntshell(ibasis)
          ishell_type = self%basis%ntshtype(ishell,ibasis)
          cshell_type = get_shell_type(ishell_type)
          !write(6,*) '  ishell=',ishell,cshell_type
          do icomp = 1, ishell_type*(ishell_type+1)/2
             labels(iao)(4:4) = cshell_type
             labels(iao)(5:8) = get_shell_comp(ishell_type,icomp)
             !write(6,*) '    icomp=',get_shell_comp(ishell,icomp),labels(iao)
             if (ldebug > 0) write(6,*) iao,labels(iao)
             iao = iao + 1
          end do
       end do
    end do

  end subroutine get_ao_labels

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_atoms(self, ldebug, istat)
    use pl_bms_class
    type(atoms_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call save_atoms_f(self, url, id, ldebug, istat)

  end subroutine save_atoms

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_atoms_f(self, url, id, ldebug, istat)
    use pl_bms_class
    type(atoms_type),intent(in)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%num, url, 'nxyz', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nele, url, 'nele', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%natom, url, 'natom', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nghost, url, 'nghost', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nbare, url, 'nbare', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%ndummy, url, 'ndummy', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%atnum, self%num, url, 'atnum', id, no_unit, istat)
    if (istat /= 0) return

    if (self%num <= 0) return

    call pl_bms_write(self%chg, self%num, url, 'chg', id, unit_au, istat)
    if (istat /= 0) return
    call pl_bms_write(self%xyz, 3, 3, self%num, url, 'xyz', id, unit_au, istat)
    if (istat /= 0) return
    call pl_bms_write(self%atom2basis, self%num,&
         url, 'atom2basis', id, unit_au, istat)
    if (istat /= 0) return

    call save_basis_f(self%basis, url, id, ldebug, istat)
    if (istat /= 0) return

  end subroutine save_atoms_f

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_atoms(self, ldebug, istat)
    use pl_bms_class
    type(atoms_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call load_atoms_f(self, url, id, ldebug, istat)

  end subroutine load_atoms

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_atoms_f(self, url, id, ldebug, istat)
    use pl_bms_class
    type(atoms_type),intent(inout)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nxyz

    call pl_bms_read(nxyz, url, 'nxyz', id, istat)
    if (istat /= 0) return

    call new_atoms(self, nxyz, istat)
    if (istat /= 0) return

    call pl_bms_read(self%nele, url, 'nele', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%natom, url, 'natom', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%nghost, url, 'nghost', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%nbare, url, 'nbare', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%ndummy, url, 'ndummy', id, istat)
    if (istat /= 0) return

    if (self%num <= 0) return

    call pl_bms_read(self%atnum, self%num, url, 'atnum', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%chg, self%num, url, 'chg', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%xyz, 3, 3, self%num, url, 'xyz', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%atom2basis, self%num, url, 'atom2basis', id, istat)
    if (istat /= 0) return

    call load_basis_f(self%basis, url, id, self%natom, ldebug, istat)
    if (istat /= 0) return

  end subroutine load_atoms_f

  !-----------------------------------------------------------------------------
  !> データBcast
  subroutine bcast_atoms(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(atoms_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nxyz
    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    if (myrank == 0) then
       nxyz = self%num
    end if
    call mpi_bcast(nxyz, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    if (myrank /= 0) then
       call new_atoms(self, nxyz, istat)
       if (istat /= 0) return
    end if

    call mpi_bcast(self%nele, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%natom, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%nghost, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%nbare, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%ndummy, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return

    if (self%num <= 0) return

    call mpi_bcast(self%atnum, nxyz, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%chg, nxyz, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%xyz, 3*nxyz, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%atom2basis, nxyz, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return

    call bcast_basis(self%basis, pl_mpi, ldebug, istat)
    if (istat /= 0) return

  end subroutine bcast_atoms
  
  !-----------------------------------------------------------------------------
  !> 初期化
  subroutine init_atoms(self, ldebug, istat)
    type(atoms_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

!!$    call free_atoms(self, ldebug, istat)
    self%num = 0

    call init_basis(self%basis, ldebug, istat)
    if (istat /= 0) return

  end subroutine init_atoms

  !-----------------------------------------------------------------------------
  !> 配列確保
  subroutine new_atoms(self, num, istat)
    type(atoms_type),intent(inout)::self
    integer,intent(in)::num
    integer,intent(inout)::istat

    istat = 0

    self%num = num
    self%natom = num
    allocate(&
         self%atnum(num), self%chg(num), self%xyz(3,num), self%atom2basis(num),&
         stat=istat)
    if (istat /= 0) return
    self%atnum(num) = 0
    self%chg(num) = 0.0d0
    self%xyz(3,num) = 0.0d0
    self%atom2basis(num) = 0

  end subroutine new_atoms

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_atoms(self, ldebug, istat)
    type(atoms_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    self%num = 0
    deallocate(self%atnum, self%chg, self%xyz, self%atom2basis, stat=istat)
    if (istat /= 0) return

    call free_basis(self%basis, ldebug, istat)
    if (istat /= 0) return

  end subroutine free_atoms

  !-----------------------------------------------------------------------------
  !> \e input_file からの読み込み
  subroutine read_inp_atoms(self, input_file, ldebug, istat)
    use input_class
    use qm_options_class, only: options_type, QM_NO
    type(atoms_type),intent(inout)::self
    character(len=*),intent(in)::input_file
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    type(options_type)::qm_options
    integer::nline, iline
    character(len=MAX_INPUT_LINE_LEN),allocatable::lines(:)
    character(len=MAX_INPUT_LINE_LEN)::line

    open(1, file=input_file, status='old', err=101)
    rewind(1, err=102)
    iline = 0
1   read(1, '(a)', end=2, err=103) line
    iline = iline + 1
    go to 1
2   nline = iline
    allocate(lines(nline), stat=istat)
    if (istat /= 0) return

    rewind(1, err=102)
    iline = 0
3   read(1, '(a)', end=4, err=103) line
    iline = iline + 1
    lines(iline) = line
    go to 3
4   close(1, err=104)

    qm_options%pdbinput = QM_NO
    call read_atoms(self, qm_options, lines, nline, ldebug, istat)
    if (istat /= 0) return


    istat = 0
8   deallocate(lines)
    return

101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(input_file)
    return
  end subroutine read_inp_atoms

  !-----------------------------------------------------------------------------
  subroutine p_read_pdb_n(natom, qm_options, ldebug, istat)
    use qm_options_class, only: options_type, QM_NO
    integer,intent(out)::natom
    type(options_type),intent(in)::qm_options
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    !character(len=MAX_INPUT_LINE_LEN)::line
    character(len=256)::line
    integer::lunit
 
    istat = 0

    lunit = 1

    open(lunit, file=qm_options%qmatomlist, status='old', err=101)
    rewind(lunit, err=102)
    natom = 0
1   read(lunit,'(a)',end=2, err=103 ) line
    natom = natom + 1
    go to 1
2   close(lunit, err=104)
    return

101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(qm_options%pdbfile)
    return
  end subroutine p_read_pdb_n

  !-----------------------------------------------------------------------------
  subroutine p_read_pdb(self, names, symbols, qm_options, ldebug, istat)
    use physical_constants_class
    use qm_options_class, only: options_type, QM_NO
    type(atoms_type),intent(inout)::self
    type(options_type),intent(in)::qm_options
    character(len=*),intent(inout)::names(self%natom)
    character(len=*),intent(inout)::symbols(self%natom)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    character(len=256),allocatable::lines(:)
    !character(len=MAX_INPUT_LINE_LEN)::line
    character(len=256)::line
    integer::lunit, natom, iatom, inum, i
    integer,allocatable::iqmatomlist(:)
    integer,allocatable::ichargelist(:)
    integer::resnum
    real(8)::x,y,z,occ,bfact

    natom = self%natom
    allocate(iqmatomlist(natom),ichargelist(natom))

    istat = 0

    lunit = 1

    open(lunit, file=qm_options%qmatomlist, status='old', err=101)
    rewind(lunit, err=102)
    iatom = 0
1   read(lunit,*,end=2, err=103 ) inum
    iatom = iatom + 1
    iqmatomlist(iatom) = inum
    go to 1
2   close(lunit, err=104)
    !write(6,*) 'trap1',iqmatomlist(:)

    open(lunit, file=qm_options%pdbfile, status='old', err=101)
    rewind(lunit, err=102)
    iatom = 0
3   read(lunit,'(a)',end=4, err=103 ) line
    if ((line(1:4) == 'ATOM' ) .or. (line(1:4) == 'HETA')) then
       iatom = iatom + 1
    end if
    go to 3
4   close(lunit, err=104)
    !write(6,*) 'trap2',iatom

    allocate(lines(iatom))
    open(lunit, file=qm_options%pdbfile, status='old', err=101)
    rewind(lunit, err=102)
    iatom = 0
5   read(lunit,'(a)',end=6, err=103 ) line
    if ((line(1:4) == 'ATOM' ) .or. (line(1:4) == 'HETA')) then
       iatom = iatom + 1
       lines(iatom) = line
    end if
    go to 5
6   close(lunit, err=104)
    !write(6,*) 'trap3',iatom

    open(lunit, file=qm_options%chargelist, status='old', err=101)
    rewind(lunit, err=102)
    iatom = 0
7   read(lunit,*,end=8, err=103 ) inum
    iatom = iatom + 1
    ichargelist(iatom) = inum
    go to 7
8   close(lunit, err=104)

    do i = 1, natom
       read(lines(iqmatomlist(i)),'(22x,i4,4x,3f8.3,2f6.3)')&
            resnum,x,y,z,occ,bfact
       !write(6,'(i4,4x,3f8.3,2f8.3)') resnum,x,y,z,occ,bfact
       self%xyz(1,i) = x
       self%xyz(2,i) = y
       self%xyz(3,i) = z
       self%atnum(i) = ichargelist(i)
       self%chg(i) = dble(ichargelist(i))
       names(i) = ATOMIC_SYMBOLS(ichargelist(i))
       symbols(i) = names(i)
    end do

    self%xyz(:,:) = self%xyz(1:3,1:natom) / bohr
    self%atom2basis(:) = 0

    return

101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(qm_options%pdbfile)
    return
  end subroutine p_read_pdb

  !-----------------------------------------------------------------------------
  !> \e input_file からの読み込み
  subroutine read_atoms(self, qm_options, lines, nline, ldebug, istat)
    use input_class
    use qm_options_class, only: options_type, QM_NO
    type(atoms_type),intent(inout)::self
    type(options_type),intent(in)::qm_options
!!$    character(len=*),intent(in)::input_file
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)

    integer::nbline, iline
    character(len=MAX_INPUT_LINE_LEN),allocatable::blines(:)
    character(len=MAX_INPUT_LINE_LEN)::line

    character(len=MAX_GROUP_NAME_LEN)::group_name !< データグループ名称
    integer::iline_gs  !< 対象データグループの開始行
    integer::iline_ge  !< 対象データグループの終了行
    integer::icol_gs   !< 対象データグループの開始行における開始カラム
    integer::icol_ge   !< 対象データグループの終了行における終了カラム
    integer::iline_err !< エラー行       (=0:正常, /=0:エラー)
    integer::icol_err  !< エラーカラム   (=0:正常, /=0:エラー)
    integer::ngroup    !< 対象データグループの出現回数

    character(len=MAX_INPUT_LINE_LEN),allocatable::names(:)
    character(len=2),allocatable::symbols(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::basissets(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::basisfiles(:)
    integer,allocatable::atspc(:) ! =1:atom, =2:bare, =3:ghost, =4:dummy

    character(len=MAX_INPUT_LINE_LEN),allocatable::set_names(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::set_basissets(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::set_basisfiles(:)
    character(len=2),allocatable::def_symbols(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::def_basissets(:)
    integer,allocatable::def_nshell(:)
    integer,allocatable::def_shtypes(:,:)
    real(8),allocatable::def_scales(:,:)
    integer,allocatable::def_nprim(:,:)
    real(8),allocatable::def_oexp(:,:,:)
    real(8),allocatable::def_coef(:,:,:)

    character(len=MAX_INPUT_LINE_LEN)::basisset_default
    character(len=MAX_INPUT_LINE_LEN)::basisfile_default

    character(len=MAX_INPUT_LINE_LEN),allocatable::res_symbols(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::res_basissets(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::res_basisfiles(:)
    integer,allocatable::res_ncount(:)
    integer,allocatable::atom2basis(:)

    integer::nxyz, natom, nbare, nghost, ndummy, iunits
    integer::nset, ndef, def_mxshell, def_mxprim, nbasis, mxshell, mxprim
    integer::nele, i, nao, ibasis, ishell, shtype, nshell

    istat = 0

    if (qm_options%pdbinput == QM_NO) then
       group_name = 'ATOMS'
       call get_data_group_position(&
            lines, nline, group_name,&
            iline_gs, icol_gs, iline_ge, icol_ge, ngroup,&
            ldebug, iline_err, icol_err, istat)
       if (istat /= 0) return
       if (ngroup == 0) then
          istat = PL_ERR_GROUP_NOT_SPECIFIED
          return
       end if

       call p_read_atoms_n(&
            lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
            nxyz, iunits, ldebug, iline_err, icol_err, istat)
       if (istat /= 0) return

       call new_atoms(self, nxyz, istat)
       if (istat /= 0) return

       allocate(names(nxyz), symbols(nxyz),&
            basissets(nxyz), basisfiles(nxyz), atspc(nxyz), stat=istat)
       if (istat /= 0) return

       call p_read_atoms(&
            self, lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
            names, symbols, basissets, basisfiles, atspc,&
            nxyz, natom, nbare, nghost, ndummy, iunits,&
            ldebug, istat)
       if (istat /= 0) return
    else
       call p_read_pdb_n(nxyz, qm_options, ldebug, istat)
       if (istat /= 0) return
       write(6,*) 'nxyz=',nxyz

       call new_atoms(self, nxyz, istat)
       if (istat /= 0) return
       write(6,*) 'nxyz=',nxyz,self%natom
       self%natom = nxyz

       allocate(names(nxyz), symbols(nxyz),&
            basissets(nxyz), basisfiles(nxyz), atspc(nxyz), stat=istat)
       if (istat /= 0) return

       call p_read_pdb(self, names, symbols, qm_options, ldebug, istat)
       if (istat /= 0) return

       basissets(:) = ' '
       basisfiles(:) = ' '
       atspc(:) = IS_ATOM
       iunits = IS_ANG
       natom  = nxyz
       nbare  = 0
       nghost = 0
       ndummy = 0
    end if

!!$    call print_atoms(self)
    !write(6,*) names
    !write(6,*) symbols
    !stop

    group_name = 'BASIS'
    call get_data_group_position(&
         lines, nline, group_name,&
         iline_gs, icol_gs, iline_ge, icol_ge, ngroup,&
         ldebug, iline_err, icol_err, istat)
    if (istat /= 0) return

    call p_read_basis_n(&
         lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
         nset, ndef, def_mxshell, def_mxprim,&
         basisset_default, basisfile_default,&
         ldebug, iline_err, icol_err, istat)
    if (istat /= 0) return

    allocate(set_names(nset), set_basissets(nset), set_basisfiles(nset),&
         def_symbols(ndef), def_basissets(ndef),&
         def_nshell(ndef), def_shtypes(def_mxshell,ndef),&
         def_scales(def_mxshell,ndef), def_nprim(def_mxshell,ndef),&
         def_oexp(def_mxprim, def_mxshell,ndef),&
         def_coef(def_mxprim, def_mxshell,ndef), stat=istat)
    if (istat /= 0) return

    call p_read_basis(&
         lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
         set_names, set_basissets, set_basisfiles, def_symbols,&
         def_basissets, def_nshell, def_shtypes,&
         def_scales, def_nprim, def_oexp, def_coef,&
         nset, ndef, def_mxshell, def_mxprim,&
         ldebug, istat)
    if (istat /= 0) return

!!$    deallocate(lines)

    allocate(res_symbols(nxyz), res_basissets(nxyz), res_basisfiles(nxyz),&
         res_ncount(nxyz), atom2basis(nxyz), stat=istat)
    if (istat /= 0) return

    call p_assign_basis(&
         names, symbols, basissets, basisfiles, atspc,&
         nxyz, nbare, nghost, ndummy,&
         set_names, set_basissets, set_basisfiles,&
         def_symbols, def_basissets, def_nshell, def_shtypes,&
         def_scales, def_nprim, def_oexp, def_coef,&
         nset, ndef, def_mxshell, def_mxprim,&
         basisset_default, basisfile_default,&
         res_symbols, res_basissets, res_basisfiles, res_ncount, atom2basis,&
         nbasis,&
         ldebug, istat)
    if (istat /= 0) return

    open(1, file=basisfile_default, status='old', err=101)
    rewind(1, err=102)
    iline = 0
11  read(1, '(a)', end=12, err=103) line
    iline = iline + 1
    go to 11
12  nbline = iline
    allocate(blines(nbline), stat=istat)
    if (istat /= 0) return

    rewind(1, err=102)
    iline = 0
13  read(1, '(a)', end=14, err=103) line
    iline = iline + 1
    blines(iline) = line
    go to 13
14  close(1, err=104)

    call p_get_basis_n(&
         blines, nbline,&
         def_symbols, def_basissets, def_nshell, def_nprim,&
         res_symbols, res_basissets, res_basisfiles,&
         ndef, def_mxshell,&
         nbasis, mxshell, mxprim,&
         basisfile_default,&
         ldebug, istat)
    if (istat /= 0) return

    call new_basis(self%basis, nxyz, nbasis, mxshell, mxprim, istat)
    if (istat /= 0) return

    self%basis%ntao(:) = 0!
    self%basis%ntshell(:) = 0!
    call p_get_basis(&
         self%basis, blines, nbline,&
         def_symbols, def_basissets, def_nshell, def_shtypes,&
         def_scales, def_nprim, def_oexp, def_coef,&
         res_symbols, res_basissets, res_basisfiles,&
         ndef, def_mxshell, def_mxprim,&
         nbasis,&
         basisfile_default,&
         ldebug, istat)
    if (istat /= 0) return

!!$    self%basis%atom2basis(1:nxyz) = atom2basis(1:nxyz)
    self%atom2basis(1:nxyz) = atom2basis(1:nxyz)
    !self%basis%ntao(nbasis)

    deallocate(blines)
    deallocate(names, symbols, basissets, basisfiles, atspc)
    deallocate(set_names, set_basissets, set_basisfiles, def_symbols,&
         def_basissets, def_nshell, def_shtypes,&
         def_scales, def_nprim, def_oexp, def_coef)
    deallocate(res_symbols, res_basissets, res_basisfiles,&
         res_ncount, atom2basis)

    nele = 0
    do i = 1, self%num
       nele = nele + self%atnum(i)
    end do
    self%nele = nele

    do ibasis = 1, self%basis%num
       nao = 0
       do ishell = 1, self%basis%ntshell(ibasis)
          shtype = self%basis%ntshtype(ishell,ibasis)
          nao = nao + shtype*(shtype+1)/2
       end do
       self%basis%ntao(ibasis) = nao
    end do

    nao = 0
    nshell = 0
    do i = 1, self%num
!!$       ibasis = self%basis%atom2basis(i)
       ibasis = self%atom2basis(i)
       if (ibasis == 0) cycle
       nao = nao + self%basis%ntao(ibasis)
       nshell = nshell + self%basis%ntshell(ibasis)
    end do
    self%basis%nao = nao
    self%basis%nshell = nshell

    if (ldebug > 0) then
       call print_atoms(self)
       call print_basis(self%basis)
    end if

    call normalize_basis(self%basis, istat)
    if (istat /= 0) return

    if (ldebug > 0) then
       call print_atoms(self)
       call print_basis(self%basis)
    end if

    !stop!!
    istat = 0
    return

101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(basisfile_default)
    return
  end subroutine read_atoms

  !-----------------------------------------------------------------------------
  !> xyz, chg 配列の置き換え
  subroutine replace_atoms(self, num, xyz, chg, istat)
    type(atoms_type),intent(inout)::self
    integer,intent(in)::num
    real(8),intent(in)::xyz(3,num)
    real(8),intent(in)::chg(num)
    integer,intent(inout)::istat

    istat = 0
    if (self%num /= num) then
      istat = 1
      return
    end if

    self%xyz(:,:) = xyz(1:3,1:num)
    self%chg(:)   = chg(1:num)

  end subroutine replace_atoms

  !-----------------------------------------------------------------------------
  !> データ取得
  subroutine get_atoms(self, num, xyz, chg, istat)
    type(atoms_type),intent(in)::self
    integer,intent(in)::num
    real(8),intent(inout)::xyz(3,num)
    real(8),intent(inout)::chg(num)
    integer,intent(inout)::istat

    istat = 0

    if (self%num /= num) then
      istat = 1
      return
    end if

    xyz(:,:) = self%xyz(1:3,1:num)
    chg(:)   = self%chg(1:num)

  end subroutine get_atoms

  !-----------------------------------------------------------------------------
  !> 原子数の取得
  integer function num_atoms(self)
    type(atoms_type),intent(in)::self
    num_atoms = self%num
  end function num_atoms

  !-----------------------------------------------------------------------------
  !> AO数の取得
  integer function nao_atoms(self)
    type(atoms_type),intent(in)::self
    nao_atoms = self%basis%nao
  end function nao_atoms

  !-----------------------------------------------------------------------------
  !> 電子数の取得
  integer function nele_atoms(self)
    type(atoms_type),intent(in)::self
    nele_atoms = self%nele
  end function nele_atoms

  !-----------------------------------------------------------------------------
  !> 原子数の取得
  subroutine load_natom_atoms(natom, istat)
    use pl_bms_class
    integer,intent(out)::natom
    integer,intent(inout)::istat

    call pl_bms_read(natom, url, 'natom', id, istat)
    if (istat /= 0) return

  end subroutine load_natom_atoms

  !-----------------------------------------------------------------------------
  !> AO数の取得
  subroutine load_nao_atoms(nao, istat)
    use pl_bms_class
    integer,intent(out)::nao
    integer,intent(inout)::istat

    call pl_bms_read(nao, url, 'nao', id, istat)
    if (istat /= 0) return

  end subroutine load_nao_atoms

  !-----------------------------------------------------------------------------
  !> 電子数の取得
  subroutine load_nele_atoms(nele, istat)
    use pl_bms_class
    integer,intent(out)::nele
    integer,intent(inout)::istat

    call pl_bms_read(nele, url, 'nele', id, istat)
    if (istat /= 0) return

  end subroutine load_nele_atoms

  !-----------------------------------------------------------------------------
  !> フレームの取得
  subroutine load_frame_atoms(xyz_min, xyz_max, istat)
    use pl_bms_class
    real(8),intent(inout)::xyz_min(3)
    real(8),intent(inout)::xyz_max(3)
    integer,intent(inout)::istat

    real(8),allocatable::xyz(:,:)
    real(8)::x_min, y_min, z_min, x_max, y_max, z_max
    integer::nxyz, ixyz

    xyz_min(1:3) = 0.0d0
    xyz_max(1:3) = 0.0d0

    call pl_bms_read(nxyz, url, 'nxyz', id, istat)
    if (istat /= 0) return

    if (nxyz <= 0) return

    allocate(xyz(3,nxyz), stat=istat)
    if (istat /= 0) return

    call pl_bms_read(xyz, 3, 3, nxyz, url, 'xyz', id, istat)
    if (istat /= 0) return

    x_min = 1.0d+10
    y_min = 1.0d+10
    z_min = 1.0d+10
    x_max =-1.0d+10
    y_max =-1.0d+10
    z_max =-1.0d+10
    do ixyz = 1, nxyz
       x_min = min(x_min, xyz(1,ixyz))
       y_min = min(y_min, xyz(2,ixyz))
       z_min = min(z_min, xyz(3,ixyz))
       x_max = max(x_max, xyz(1,ixyz))
       y_max = max(y_max, xyz(2,ixyz))
       z_max = max(z_max, xyz(3,ixyz))
    end do

    deallocate(xyz, stat=istat)
    if (istat /= 0) return

    xyz_min(1) = x_min
    xyz_min(2) = y_min
    xyz_min(3) = z_min
    xyz_max(1) = x_max
    xyz_max(2) = y_max
    xyz_max(3) = z_max

  end subroutine load_frame_atoms

  !-----------------------------------------------------------------------------
  !> データの簡易表示
  subroutine print_brief_atoms(self)
    type(atoms_type),intent(in)::self
    integer::i,j

    write(6,'(/''Atoms:'')')
    !write(6,'(1x,''Number of atoms ='',i6)') self%num
    write(6,'(1x,''num ='',i6)') self%num

  end subroutine print_brief_atoms

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_atoms(self)
    type(atoms_type),intent(in)::self
    integer::i,j

    call print_brief_atoms(self)

    if (self%num > 0) then
      write(6,'(/6x,''No. Atoms         X             Y             Z'',&
          &''       Charges  Basis'')')
      do i = 1, self%num
        write(6,'(i8,4x,a2,3f14.8,f11.5,1x,i6)')&
            i, ATOMIC_SYMBOLS(self%atnum(i)),&
!!$            (self%xyz(j,i),j=1,3), self%chg(i), self%basis%atom2basis(i)
            (self%xyz(j,i),j=1,3), self%chg(i), self%atom2basis(i)
      end do
    end if

    call print_brief_basis(self%basis)
  end subroutine print_atoms

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_xyz(self)
    use physical_constants_class
    type(atoms_type),intent(in)::self
    integer::i,j

    if (self%num > 0) then
      do i = 1, self%num
         write(6,'(a2,2x,3f16.10)')&
              get_atomic_symbol(self%atnum(i)),(self%xyz(j,i)*bohr,j=1,3)
      end do
    end if

  end subroutine print_xyz

  !=============================================================================
  ! private
  !-----------------------------------------------------------------------------
  subroutine p_read_atoms_n(&
       lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
       nxyz, iunits, ldebug, iline_err, icol_err, istat)
    use input_class
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::iline_gs  !< 対象データグループの開始行
    integer,intent(in)::iline_ge  !< 対象データグループの終了行
    integer,intent(in)::icol_gs   !< 対象データグループの開始行における開始カラム
    integer,intent(in)::icol_ge   !< 対象データグループの終了行における終了カラム
    integer,intent(out)::iline_err !< エラー行       (=0:正常, /=0:エラー)
    integer,intent(out)::icol_err  !< エラーカラム   (=0:正常, /=0:エラー)
    integer,intent(out)::nxyz
    integer,intent(out)::iunits
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    integer,parameter::nopt = 1                          !< オプション数
    character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt)& !< オプションの名称
         =(/'UNITS'/)
    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::opt_count(nopt)

    integer,parameter::QM_UNITS_NUM = 2
    character(len=*),parameter::QM_UNITS(QM_UNITS_NUM) = (/ 'ANG', 'AU '/)

    integer::iopt, ipos, ipos_default

    integer,parameter::mxval = 7 ! atom x y z charge basisset basisfile
    character(len=MAX_INPUT_LINE_LEN)::vals(mxval)
    integer::nval, ival, iline, icol_s, icol_e
    integer::ixyz
    logical::is_option

    character(len=*),parameter::c_assignment_op = '='

    opt_count(:) = 0
    opt_values(:) = ' '
    nxyz = 0
    do iline = iline_gs, iline_ge
       !write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'
       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if
       !write(6,*) 'str=',lines(iline)(icol_s:icol_e),'$'

       is_option = .false.
       call split_str_eq(lines(iline)(icol_s:icol_e), vals, mxval, nval, istat)
       if (istat /= 0) then
          istat = PL_ERR_INVALID_FORMAT
          iline_err = iline; icol_err = icol_s
          go to 9
       end if
       do ival = 1, nval
          if (trim(vals(ival)) == c_assignment_op) then
             is_option = .true.
             exit
          end if
       end do
       if (is_option) then
          call parse_options(&
               lines(iline), 1, 1, icol_s, 1, icol_e,&
               opt_names, opt_values, opt_count, nopt,&
               ldebug, iline_err, icol_err, istat)
          if (istat /= 0) return
       else
          select case (nval)
          case (0)
             cycle
          case (4:7)
             nxyz = nxyz + 1
          case default
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end select
       endif
    end do
    if (ldebug > 0) write(6,*) 'nxyz=',nxyz

    do iopt = 1, nopt
       if (opt_count(iopt) > 1) then
          istat = PL_ERR_DUPLICATE_OPTION
          call pl_error_print_message(istat)
          call print_error_invalid_option(opt_names(iopt))
          return
       end if
    end do
    
    opt_name = 'units'
    ipos_default = pl_str_search_name_ignorecase('ang', QM_UNITS, QM_UNITS_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_UNITS, QM_UNITS_NUM, ipos_default, ipos, istat)
    if (istat /= 0) return
    iunits = ipos
    if (ldebug > 0) write(6,*) 'units:',QM_UNITS(iunits)

    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine p_read_atoms_n

  !-----------------------------------------------------------------------------
  subroutine p_read_atoms(&
       self, lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
       names, symbols, basissets, basisfiles, atspc,&
       nxyz, natom, nbare, nghost, ndummy, iunits,&
       ldebug, istat)
    use physical_constants_class
    use input_class
    type(atoms_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::iline_gs  !< 対象データグループの開始行
    integer,intent(in)::iline_ge  !< 対象データグループの終了行
    integer,intent(in)::icol_gs   !< 対象データグループの開始行における開始カラム
    integer,intent(in)::icol_ge   !< 対象データグループの終了行における終了カラム
    integer,intent(in)::nxyz
    integer,intent(out)::natom
    integer,intent(out)::nbare
    integer,intent(out)::nghost
    integer,intent(out)::ndummy
    integer,intent(in)::iunits
    character(len=*),intent(inout)::names(nxyz)
    character(len=*),intent(inout)::symbols(nxyz)
    character(len=*),intent(inout)::basissets(nxyz)
    character(len=*),intent(inout)::basisfiles(nxyz)
    integer,intent(inout)::atspc(nxyz)
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    integer,parameter::mxval = 7
    character(len=MAX_INPUT_LINE_LEN)::vals(mxval)
    integer::nval, ival, iline, icol_s, icol_e, ipos
    integer::ixyz
    real(8)::val
    logical::is_option

    character(len=*),parameter::c_assignment_op = '='

    character(len=MAX_INPUT_LINE_LEN)::name
    integer::ln
    integer::iline_err, icol_err

    names(:) = ' '
    symbols(:) = ' '
    basissets(:) = ' '
    basisfiles(:) = ' '
    atspc(:) = 0
    self%chg(:) = 0.0d0
    self%atnum(:) = 1

    ixyz = 0
    natom = 0
    nbare = 0
    nghost = 0
    ndummy = 0
    do iline = iline_gs, iline_ge
       !write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'
       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if
       !write(6,*) 'str=',lines(iline)(icol_s:icol_e),'$'

       is_option = .false.
       call split_str_eq(lines(iline)(icol_s:icol_e), vals, mxval, nval, istat)
       if (istat /= 0) return
       !write(6,*) 'nval=',nval
       do ival = 1, nval
          if (trim(vals(ival)) == c_assignment_op) then
             is_option = .true.
             exit
          end if
       end do
       if (is_option) then
          cycle
       else
          select case (nval)
          case (0)
             cycle
          case (4:7)
             ixyz = ixyz + 1
             ln = len_trim(vals(1))
             names(ixyz) = vals(1)
             call split_atomic_name(&
                  names(ixyz),&
                  symbols(ixyz), self%atnum(ixyz), atspc(ixyz), istat)
             !write(6,*) 'name  =',trim(names(ixyz))
             !write(6,*) '  symbol=',trim(symbols(ixyz))
             !write(6,*) '  atnum =',self%atnum(ixyz)
             !write(6,*) '  atspc =',atspc(ixyz)
             if (istat /= 0) then
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             select case (atspc(ixyz))
             case (IS_ATOM)
                natom = natom + 1
             case (IS_BARE)
                nbare = nbare + 1
             case (IS_GHOST)
                nghost = nghost + 1
             case (IS_DUMMY)
                ndummy = ndummy + 1
             end select
             if (istat /= 0) then
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             call pl_str_to_dbl(vals(2), self%xyz(1,ixyz), istat)
             if (istat /= 0) then
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             call pl_str_to_dbl(vals(3), self%xyz(2,ixyz), istat)
             if (istat /= 0) then
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             call pl_str_to_dbl(vals(4), self%xyz(3,ixyz), istat)
             if (istat /= 0) then
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             if (atspc(ixyz) == IS_ATOM .or. atspc(ixyz) == IS_BARE) then
                self%chg(ixyz) = dble(self%atnum(ixyz))
             else
                self%chg(ixyz) = 0.0d0
             end if
             if (nval == 4) cycle

             if (atspc(ixyz) == IS_DUMMY) then
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if

             call pl_str_to_dbl(vals(5), val, istat)
             if (istat == 0) then
                if (atspc(ixyz) == IS_GHOST) then
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if

                self%chg(ixyz) = val
                if (nval == 5) cycle

                if (atspc(ixyz) == IS_BARE) then
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if

                select case (nval)
                case (6)
                   basissets(ixyz)  = vals(6)
                case (7)
                   basissets(ixyz)  = vals(6)
                   basisfiles(ixyz) = vals(7)
                case default
                end select
             else
                if (atspc(ixyz) == IS_BARE) then
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if

                select case (nval)
                case (5)
                   basissets(ixyz)  = vals(5)
                case (6)
                   basissets(ixyz)  = vals(5)
                   basisfiles(ixyz) = vals(6)
                case default
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end select
             end if
          case default
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end select
       end if
    end do
    if (ldebug > 0) then
       write(6,*) 'natom =',natom
       write(6,*) 'nbare =',nbare
       write(6,*) 'nghost=',nghost
       write(6,*) 'ndummy=',ndummy
    end if

    self%natom  = natom
    self%nbare  = nbare
    self%nghost = nghost
    self%ndummy = ndummy

    if (iunits == IS_ANG) self%xyz(:,:) = self%xyz(1:3,1:nxyz) / bohr

    if (ldebug > 0) then
       do ixyz = 1, nxyz
          write(6,'(i3,'':'',i1,'':'',i3,'':'',&
               &4f12.6,1x,'':'',a,'':'',a,'':'',a,'':'',a)')&
               ixyz,atspc(ixyz),&
               self%atnum(ixyz),self%xyz(1:3,ixyz),self%chg(ixyz),&
               trim(names(ixyz)),trim(symbols(ixyz)),&
               trim(basissets(ixyz)),trim(basisfiles(ixyz))
       end do
    end if

    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine p_read_atoms

  !-----------------------------------------------------------------------------
  subroutine p_read_basis_n(&
       lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
       nset, ndef, def_mxshell, def_mxprim,&
       basisset_default, basisfile_default,&
       ldebug, iline_err, icol_err, istat)
    use input_class
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::iline_gs  !< 対象データグループの開始行
    integer,intent(in)::iline_ge  !< 対象データグループの終了行
    integer,intent(in)::icol_gs   !< 対象データグループの開始行における開始カラム
    integer,intent(in)::icol_ge   !< 対象データグループの終了行における終了カラム
    integer,intent(out)::iline_err !< エラー行       (=0:正常, /=0:エラー)
    integer,intent(out)::icol_err  !< エラーカラム   (=0:正常, /=0:エラー)
    integer,intent(out)::nset
    integer,intent(out)::ndef
    integer,intent(out)::def_mxshell
    integer,intent(out)::def_mxprim
    character(len=*),intent(inout)::basisset_default
    character(len=*),intent(inout)::basisfile_default
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    integer,parameter::nopt = 2                          !< オプション数
    character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt)& !< オプションの名称
         =(/'BASISFILE', 'BASISSET '/)
    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::opt_count(nopt)
    integer::iopt, ipos, ipos_default

    integer,parameter::mxval = 6
    character(len=MAX_INPUT_LINE_LEN)::vals(mxval)
    real(8)::val
    integer::nval, ival, iline, icol_s, icol_e
    logical::is_option
    
    character(len=*),parameter::c_assignment_op = '='

    integer::iset

    integer::count_down
    integer::idef, nshell, ishell, nprim, iprim

    character(len=MAX_INPUT_LINE_LEN)::shtype
    character(len=MAX_INPUT_LINE_LEN)::str, str_default
    character(len=MAX_INPUT_LINE_LEN)::cval

    opt_count(:) = 0
    opt_values(:) = ' '
    nset = 0
    ndef = 0
    def_mxshell = 0
    def_mxprim = 0
    count_down = 0
    do iline = iline_gs, iline_ge
       !write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'
       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if
       !write(6,*) 'str=',lines(iline)(icol_s:icol_e),'$'

       is_option = .false.
       call split_str_eq(lines(iline)(icol_s:icol_e), vals, mxval, nval, istat)
       if (istat /= 0) then
          istat = PL_ERR_INVALID_FORMAT
          iline_err = iline; icol_err = icol_s
          go to 9
       end if
       !write(6,*) 'nval=',nval
       if (nval == 0) cycle
       call pl_str_tolower(vals(1), cval)
       !write(6,*) 'cval=',trim(cval)
       if (trim(cval) /= 'set' .and. trim(cval) /= 'def') then
          do ival = 1, nval
             if (trim(vals(ival)) == c_assignment_op) then
                is_option = .true.
                exit
             end if
          end do
       end if
       if (is_option) then
          call parse_options(&
               lines(iline), 1, 1, icol_s, 1, icol_e,&
               opt_names, opt_values, opt_count, nopt,&
               ldebug, iline_err, icol_err, istat)
          if (istat /= 0) then
             iline_err = iline
             go to 9
          end if
       else
          select case (trim(cval))
          case ('set')
             nset = nset + 1
             !write(6,*) 'set-line:',trim(vals(2)),':',trim(vals(4))
             if (count_down /= 0) then
                !write(6,*) 'error in set'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
          case ('def')
             if (ndef /= 0) then
                if (ldebug > 0) write(6,*) 'nshell=',nshell
                def_mxshell = max(def_mxshell, nshell)
             end if
             ndef = ndef + 1
             !write(6,*) 'start-def:',trim(vals(2)),':',trim(vals(4))
             if (count_down /= 0) then
                !write(6,*) 'error in def'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             nshell = 0
          case ('s','p','d','sp')
             shtype = cval
             !write(6,*) ' shell:',trim(vals(1)),':',trim(vals(2))
             if (count_down /= 0) then
                !write(6,*) 'error in shell'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             nshell = nshell + 1
             select case (nval)
             case (2:3)
                call pl_str_to_int(trim(vals(2)), nprim, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in nprim'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
             case default
                if (istat /= 0) then
                   !write(6,*) 'error in shell'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
             end select
             !write(6,*) 'nprim=',nprim
             def_mxprim = max(def_mxprim, nprim)
             if (trim(shtype) == 'sp') nshell = nshell + 1
             count_down = nprim
          case default
             !write(6,*) '   primitive:',trim(vals(1)),':',trim(vals(2))
             if (count_down <= 0) then
                !write(6,*) 'error in prim'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             count_down = count_down - 1
          end select
       endif
    end do
    if (ndef /= 0) then
       !write(6,*) 'nshell=',nshell
       def_mxshell = max(def_mxshell, nshell)
    end if
    if (ldebug > 0) then
       write(6,*) 'nset      =',nset
       write(6,*) 'ndef      =',ndef
       write(6,*) 'def_mxshell=',def_mxshell
       write(6,*) 'def_mxprim =',def_mxprim
    end if

    do iopt = 1, nopt
       if (opt_count(iopt) > 1) then
          istat = PL_ERR_DUPLICATE_OPTION
          call pl_error_print_message(istat)
          call print_error_invalid_option(opt_names(iopt))
          return
       end if
    end do
    
    opt_name = 'basisset'
    str_default = DEFAULT_BASIS_SET_NAME
    call get_option_values_str(&
         opt_name, opt_names, opt_values, nopt, str_default, str, istat)
    if (istat /= 0) return
    basisset_default = str
    if (ldebug > 0) write(6,*) 'basisset:',trim(basisset_default)

    opt_name = 'basisfile'
    str_default = DEFAULT_BASIS_FILE_NAME
    call get_option_values_str(&
         opt_name, opt_names, opt_values, nopt, str_default, str, istat)
    if (istat /= 0) return
    basisfile_default = str
    if (ldebug > 0) write(6,*) 'basisfile:',trim(basisfile_default)

    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine p_read_basis_n

  !-----------------------------------------------------------------------------
  subroutine p_read_basis(&
       lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
       set_names, set_basissets, set_basisfiles, def_symbols,&
       def_basissets, def_nshell, def_shtypes,&
       def_scales, def_nprim, def_oexp, def_coef,&
       nset, ndef, def_mxshell, def_mxprim,&
       ldebug, istat)
    use input_class
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::iline_gs !< 対象データグループの開始行
    integer,intent(in)::iline_ge !< 対象データグループの終了行
    integer,intent(in)::icol_gs  !< 対象データグループの開始行における開始カラム
    integer,intent(in)::icol_ge  !< 対象データグループの終了行における終了カラム
    integer,intent(in)::nset
    integer,intent(in)::ndef
    integer,intent(in)::def_mxshell
    integer,intent(in)::def_mxprim
    character(len=*),intent(inout)::set_names(nset)
    character(len=*),intent(inout)::set_basissets(nset)
    character(len=*),intent(inout)::set_basisfiles(nset)
    character(len=*),intent(inout)::def_symbols(ndef)
    character(len=*),intent(inout)::def_basissets(ndef)
    integer,intent(inout)::def_nshell(ndef)
    integer,intent(inout)::def_shtypes(def_mxshell,ndef)
    real(8),intent(inout)::def_scales(def_mxshell,ndef)
    integer,intent(inout)::def_nprim(def_mxshell,ndef)
    real(8),intent(inout)::def_oexp(def_mxprim,def_mxshell,ndef)
    real(8),intent(inout)::def_coef(def_mxprim,def_mxshell,ndef)
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    character(len=*),parameter::group_name = 'BASIS'
    integer,parameter::nopt = 2                          !< オプション数
    character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt)& !< オプションの名称
         =(/'BASISFILE', 'BASISSET '/)
    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::opt_count(nopt)
    integer::iopt, ipos, ipos_default

    integer,parameter::mxval = 6
    character(len=MAX_INPUT_LINE_LEN)::vals(mxval)
    real(8)::val
    integer::nval, ival, iline, icol_s, icol_e
    logical::is_option
    
    character(len=*),parameter::c_assignment_op = '='

    integer::count_down
    integer::iset, jset, idef, jdef, nshell, ishell, nprim, iprim
    character(len=MAX_INPUT_LINE_LEN)::shtype

    character(len=MAX_INPUT_LINE_LEN)::str, str_default
    character(len=MAX_INPUT_LINE_LEN)::cval
    integer::iline_err, icol_err
    integer::nerror, ierror
    character(len=2)::symbol
    integer::atnum, atspc

    set_basisfiles(:) = ' '
    iset = 0
    idef = 0
    count_down = 0
    do iline = iline_gs, iline_ge
       !write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'
       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if
       !write(6,*) 'str=',lines(iline)(icol_s:icol_e),'$'

       is_option = .false.
       call split_str_eq(lines(iline)(icol_s:icol_e), vals, mxval, nval, istat)
       if (istat /= 0) then
          istat = PL_ERR_INVALID_FORMAT
          iline_err = iline; icol_err = icol_s
          go to 9
       end if
       !write(6,*) 'nval=',nval
       if (nval == 0) cycle
       call pl_str_tolower(vals(1), cval)
       !write(6,*) 'cval=',trim(cval)
       if (trim(cval) /= 'set' .and. trim(cval) /= 'def') then
          do ival = 1, nval
             if (trim(vals(ival)) == c_assignment_op) then
                is_option = .true.
                exit
             end if
          end do
       end if
       if (is_option) then
          cycle
       else
          select case (trim(cval))
          case ('set')
             iset = iset + 1
             !write(6,*) 'set-line:',trim(vals(2)),':',trim(vals(4))
             if (count_down /= 0) then
                !write(6,*) 'error in set 1'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             select case (nval)
             case (4:5)
                if (trim(vals(3)) /= c_assignment_op) then
                   !write(6,*) 'error in set 2'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                call split_atomic_name(vals(2), symbol, atnum, atspc, istat)
                if (istat /= 0 .or. atspc /= IS_ATOM) then
                   !write(6,*) 'error in set 2b'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                set_names(iset) = vals(2)
                set_basissets(iset) = vals(4)
                if (nval == 5) set_basisfiles(iset) = vals(5)
             case default
                !write(6,*) 'error in set 3'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end select
          case ('def')
             if (idef /= 0) then
                !write(6,*) 'ishell=',ishell
                def_nshell(idef) = ishell
             end if
             idef = idef + 1
             !write(6,*) 'start-def:',trim(vals(2)),':',trim(vals(4))
             if (count_down /= 0) then
                !write(6,*) 'error in def 1'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             select case (nval)
             case (4)
                if (trim(vals(3)) /= c_assignment_op) then
                   !write(6,*) 'error in def 2'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                call split_atomic_name(vals(2), symbol, atnum, atspc, istat)
                if (istat /= 0 .or. atspc /= IS_ATOM .or.&
                     trim(vals(2)) /= trim(symbol)) then
                   !write(6,*) 'error in def 2b'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                def_symbols(idef) = vals(2)
                def_basissets(idef) = vals(4)
             case default
                !write(6,*) 'error in def 3'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end select
             ishell = 0
          case ('s','p','d','sp')
             shtype = cval
             !write(6,*) ' shell:',trim(vals(1)),':',trim(vals(2))
             if (count_down /= 0) then
                !write(6,*) 'error in shell 1'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             select case (nval)
             case (2:3)
                call pl_str_to_int(trim(vals(2)), nprim, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in nprim'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                ishell = ishell + 1
                if (nval == 3) then
                   !write(6,*) ' scale:',trim(vals(3))
                   call pl_str_to_dbl(trim(vals(3)), val, istat)
                   if (istat /= 0) then
                      !write(6,*) 'error in scale'
                      istat = PL_ERR_INVALID_FORMAT
                      iline_err = iline; icol_err = icol_s
                      go to 9
                   end if
                   def_scales(ishell,idef) = val
                else
                   def_scales(ishell,idef) = 1.0d0
                end if
             case default
                !write(6,*) 'error in shell 2'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end select
             !write(6,*) 'nprim=',nprim
             def_nprim(ishell,idef) = nprim
             select case(trim(shtype))
             case ('s','sp')
                def_shtypes(ishell, idef) = 1
             case ('p')
                def_shtypes(ishell, idef) = 2
             case ('d')
                def_shtypes(ishell, idef) = 3
             case default
                !write(6,*) 'error in shell type'
             end select
             if (trim(shtype) == 'sp') then
                ishell = ishell + 1
                def_scales(ishell,idef) = def_scales(ishell-1,idef)
                def_nprim(ishell,idef) = def_nprim(ishell-1,idef)
                def_shtypes(ishell, idef) = 2
             end if
             count_down = nprim
             iprim = 0
          case default
             !write(6,*) '   primitive:',trim(vals(1)),':',trim(vals(2))
             if (count_down <= 0) then
                !write(6,*) 'error in prim 1'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             iprim = iprim + 1
             select case (nval)
             case (2)
                if (trim(shtype) /= 's' .and. trim(shtype) /= 'p'&
                     .and. trim(shtype) /= 'd') then
                   !write(6,*) 'error in prim 2'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                call pl_str_to_dbl(trim(vals(1)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 3'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                def_oexp(iprim, ishell, idef) = val
                call pl_str_to_dbl(trim(vals(2)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 4'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                def_coef(iprim, ishell, idef) = val
             case (3)
                if (trim(shtype) /= 'sp') then
                   !write(6,*) 'error in prim 5'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                call pl_str_to_dbl(trim(vals(1)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 6'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                def_oexp(iprim, ishell-1, idef) = val
                def_oexp(iprim, ishell  , idef) = val
                call pl_str_to_dbl(trim(vals(2)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 7'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                def_coef(iprim, ishell-1, idef) = val
                call pl_str_to_dbl(trim(vals(3)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 8'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                def_coef(iprim, ishell, idef) = val
             case default
                !write(6,*) 'error in prim 9'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end select
             count_down = count_down - 1
          end select
       endif
    end do
    if (idef /= 0) then
       !write(6,*) 'ishell=',ishell
       def_nshell(idef) = ishell
    end if

    if (ldebug > 0) then
       write(6,*) 'nset=',nset
       do iset = 1, nset
          write(6,'(i3,'':'',a,'':'',a,'':'',a)')&
               iset,trim(set_names(iset)),&
               trim(set_basissets(iset)),trim(set_basisfiles(iset))
       end do
    end if

    nerror = 0
    do iset = 1, nset
       do jset = iset+1, nset
          if (pl_str_cmp_ignorecase(set_names(iset), set_names(jset))) then
             istat = PL_ERR_DUPLICATE
             call pl_error_print_message(istat)
             write(6,'(2x,''set :'',a)') trim(set_names(iset))
             nerror = nerror + 1
          end if
       end do
    end do
    if (nerror /= 0) return

    if (ldebug > 0) then
       write(6,*) 'ndef=',ndef
       do idef = 1, ndef
          write(6,'(i3,'':'',a,'':'',a,'':'',i3)')&
               idef,&
               trim(def_symbols(idef)),trim(def_basissets(idef)),&
               def_nshell(idef)
          do ishell = 1, def_nshell(idef)
             write(6,'(5x,i2,f8.4,i5)')&
                  def_shtypes(ishell,idef),def_scales(ishell,idef),&
                  def_nprim(ishell,idef)
             do iprim = 1, def_nprim(ishell,idef)
                write(6,'(8x,2f12.7)')&
                     def_oexp(iprim,ishell,idef),def_coef(iprim,ishell,idef)
             end do
          end do
       end do
    end if

    nerror = 0
    do idef = 1, ndef
       do jdef = idef+1, ndef
          if (pl_str_cmp_ignorecase(def_symbols(idef), def_symbols(jdef)) .and.&
               pl_str_cmp_ignorecase(def_basissets(idef), def_basissets(jdef))&
               ) then
             istat = PL_ERR_DUPLICATE
             call pl_error_print_message(istat)
             write(6,'(2x,''def :'',a,'', '',a)')&
                  trim(def_symbols(idef)), trim(def_basissets(idef))
             nerror = nerror + 1
          end if
       end do
    end do
    if (nerror /= 0) return

    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine p_read_basis

  !-----------------------------------------------------------------------------
  subroutine p_assign_basis(&
       names, symbols, basissets, basisfiles, atspc,&
       nxyz, nbare, nghost, ndummy,&
       set_names, set_basissets, set_basisfiles,&
       def_symbols, def_basissets, def_nshell, def_shtypes,&
       def_scales, def_nprim, def_oexp, def_coef,&
       nset, ndef, def_mxshell, def_mxprim,&
       basisset_default, basisfile_default,&
       res_symbols, res_basissets, res_basisfiles, res_ncount, atom2basis,&
       nbasis,&
       ldebug, istat)
    use input_class
    integer,intent(in)::nxyz
    integer,intent(in)::nbare
    integer,intent(in)::nghost
    integer,intent(in)::ndummy
    character(len=*),intent(in)::names(nxyz)
    character(len=*),intent(in)::symbols(nxyz)
    character(len=*),intent(in)::basissets(nxyz)
    character(len=*),intent(inout)::basisfiles(nxyz) !update
    integer,intent(in)::atspc(:)
    integer,intent(in)::nset
    integer,intent(in)::ndef
    integer,intent(in)::def_mxshell
    integer,intent(in)::def_mxprim
    character(len=*),intent(in)::set_names(nset)
    character(len=*),intent(in)::set_basissets(nset)
    character(len=*),intent(inout)::set_basisfiles(nset) !update
    character(len=*),intent(in)::def_symbols(ndef)
    character(len=*),intent(in)::def_basissets(ndef)
    integer,intent(in)::def_nshell(ndef)
    integer,intent(in)::def_shtypes(def_mxshell,ndef)
    real(8),intent(in)::def_scales(def_mxshell,ndef)
    integer,intent(in)::def_nprim(def_mxshell,ndef)
    real(8),intent(in)::def_oexp(def_mxprim,def_mxshell,ndef)
    real(8),intent(in)::def_coef(def_mxprim,def_mxshell,ndef)
    character(len=*),intent(in)::basisset_default
    character(len=*),intent(in)::basisfile_default
    character(len=*),intent(inout)::res_symbols(nxyz)
    character(len=*),intent(inout)::res_basissets(nxyz)
    character(len=*),intent(inout)::res_basisfiles(nxyz)
    integer,intent(inout)::res_ncount(nxyz)
    integer,intent(inout)::atom2basis(nxyz)
    integer,intent(out)::nbasis
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    integer::nset1, iset1, nset2, iset2, iset
    character(len=MAX_INPUT_LINE_LEN),allocatable::set_names1(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::set_basissets1(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::set_basisfiles1(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::set_names2(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::set_basissets2(:)
    character(len=MAX_INPUT_LINE_LEN),allocatable::set_basisfiles2(:)
    character(len=5)::set_symbol
    integer::idef, kdef

    integer::ixyz, jxyz, ibasis, dbasis, kbasis, atnum, atspc_dum
    integer::ln

    istat = 0
    nset1 = 0
    nset2 = 0
    do iset = 1, nset
       call split_atomic_name(&
            set_names(iset), set_symbol, atnum, atspc_dum, istat)
       if (istat /= 0) return
       if (pl_str_cmp_ignorecase(set_names(iset), set_symbol)) then
          nset1 = nset1 + 1
       else
          nset2 = nset2 + 1
       end if
       if (len_trim(set_basisfiles(iset)) == 0) then
          ! &Basisのdefで割り当てかBasisFileで割り当てか判定
          kdef = 0
          do idef = 1, ndef
             !write(6,*) 'idef',idef
             !write(6,*) ' ',trim(def_symbols(idef)),' ',trim(set_symbol)
             !write(6,*)&
             !     ' ',trim(def_basissets(idef)),' ',trim(set_basissets(iset))
             if (pl_str_cmp_ignorecase(def_symbols(idef), set_symbol) .and.&
                  pl_str_cmp_ignorecase(&
                  def_basissets(idef), set_basissets(iset))) then
!!$                  trim(def_basissets(idef)) == trim(set_basissets(iset))) then
                kdef = idef
                exit
             end if
          end do
          if (kdef == 0) then ! defになかったらデフォルトのBasisFile
             set_basisfiles(iset) = basisfile_default
          end if
       end if
    end do
    !write(6,*) 'nset1,nset2=',nset1,nset2

    allocate(set_names1(nset1), set_basissets1(nset1), set_basisfiles1(nset1))
    allocate(set_names2(nset2), set_basissets2(nset2), set_basisfiles2(nset2))

    iset1 = 0
    iset2 = 0
    do iset = 1, nset
       call split_atomic_name(&
            set_names(iset), set_symbol, atnum, atspc_dum, istat)
       if (istat /= 0) return
       if (pl_str_cmp_ignorecase(set_names(iset), set_symbol)) then
          iset1 = iset1 + 1
          set_names1(iset1)      = trim(set_names(iset))
          set_basissets1(iset1)  = trim(set_basissets(iset))
          set_basisfiles1(iset1) = trim(set_basisfiles(iset))
       else
          iset2 = iset2 + 1
          set_names2(iset2)      = trim(set_names(iset))
          set_basissets2(iset2)  = trim(set_basissets(iset))
          set_basisfiles2(iset2) = trim(set_basisfiles(iset))
       end if
    end do
    if (ldebug > 0) then
       do iset1 = 1, nset1
          write(6,*) 'iset1:',iset1,' ',trim(set_names1(iset1)),&
               ' ',trim(set_basissets1(iset1)), ' ',trim(set_basisfiles1(iset1))
       end do
       do iset2 = 1, nset2
          write(6,*) 'iset2:',iset2,' ',trim(set_names2(iset2)),&
               ' ',trim(set_basissets2(iset2)), ' ',trim(set_basisfiles2(iset2))
       end do
    end if

    ! assignment
    res_symbols(:) = ' '
    res_basissets(:) = ' '
    res_basisfiles(:) = ' '
    res_ncount(:) = 0
    atom2basis(:) = 0
    nbasis = 0
    ! &Atomsで定義されている基底関数系を集計
    do ixyz = 1, nxyz
       if (len_trim(basissets(ixyz)) == 0) cycle
       if (len_trim(basisfiles(ixyz)) == 0) then
          ! &Basisのdefで割り当てかBasisFileで割り当てか判定
          kdef = 0
          do idef = 1, ndef
             !write(6,*) 'idef',idef
             !write(6,*) ' ',trim(def_symbols(idef)),' ',trim(symbol)
             !write(6,*)&
             !     ' ',trim(def_basissets(idef)),' ',trim(basissets(iset))
             if (pl_str_cmp_ignorecase(def_symbols(idef), symbols(ixyz)) .and.&
                  pl_str_cmp_ignorecase(&
                  def_basissets(idef), basissets(ixyz))) then
!!$                  trim(def_basissets(idef)) == trim(basissets(ixyz))) then
                kdef = idef
                exit
             end if
          end do
          if (kdef == 0) then ! defになかったらデフォルトのBasisFile
             basisfiles(ixyz) = basisfile_default
          end if
       end if
       !write(6,*) 'ixyz:',ixyz,' ',trim(names(ixyz)),&
       !     ' ',trim(basissets(ixyz)),' ',trim(basisfiles(ixyz))
       dbasis = 0
       do ibasis = 1, nbasis
          if (pl_str_cmp_ignorecase(res_symbols(ibasis), symbols(ixyz)).and.&
               pl_str_cmp_ignorecase(&
               res_basissets(ibasis), basissets(ixyz)) .and.&
!!$               trim(res_basissets(ibasis)) == trim(basissets(ixyz)) .and.&
               trim(res_basisfiles(ibasis)) == trim(basisfiles(ixyz))) then
             dbasis = ibasis
             exit
          end if
       end do
       if (dbasis == 0) then
          nbasis = nbasis + 1
          atom2basis(ixyz)       = nbasis
          res_ncount(nbasis)     = 1
          res_symbols(nbasis)    = symbols(ixyz)
          res_basissets(nbasis)  = trim(basissets(ixyz))
          res_basisfiles(nbasis) = trim(basisfiles(ixyz))
       else
          atom2basis(ixyz) = dbasis
          res_ncount(dbasis) = res_ncount(dbasis) + 1
       end if
    end do

    ! &Basisのsetで割り当て指定(個別指定)された基底関数系を集計
    do iset = 1, nset2
       do ixyz = 1, nxyz
          if (atom2basis(ixyz) /= 0) cycle
          ln = len_trim(names(ixyz))
          if (atspc(ixyz) == IS_GHOST) ln = ln - 3 ! '-bq'
          if (pl_str_cmp_ignorecase(set_names2(iset), names(ixyz)(:ln))) then
             !write(6,*) 'iset2->ixyz:',iset,' ',trim(set_names2(iset)),&
             !     '->',ixyz,' ',trim(names(ixyz)),' ',names(ixyz)(:ln)
             dbasis = 0
             do ibasis = 1, nbasis
                !write(6,*) 'ibasis=',ibasis
                !write(6,*) ' ',trim(res_symbols(ibasis)),&
                !     ' ',trim(symbols(ixyz))
                !write(6,*) ' ',trim(res_basissets(ibasis)),&
                !     ' ',trim(set_basissets2(iset))
                !write(6,*) ' ',trim(res_basisfiles(ibasis)),&
                !     ' ',trim(set_basisfiles2(iset))
                if (pl_str_cmp_ignorecase(res_symbols(ibasis), symbols(ixyz))&
                     .and.&
                     pl_str_cmp_ignorecase(&
                     res_basissets(ibasis), set_basissets2(iset))&
!!$                     trim(res_basissets(ibasis)) == trim(set_basissets2(iset))&
                     .and. trim(res_basisfiles(ibasis))&
                     == trim(set_basisfiles2(iset))) then
                   dbasis = ibasis
                   !write(6,*) 'dbasis=',dbasis
                   exit
                end if
             end do
             if (dbasis == 0) then
                kbasis = nbasis + 1
                res_ncount(kbasis) = 1
             else
                kbasis = dbasis
                res_ncount(kbasis) = res_ncount(kbasis) + 1
             end if
             atom2basis(ixyz) = kbasis
          end if
       end do
       if (kbasis == nbasis + 1) then
          nbasis = nbasis + 1
          call split_atomic_name(&
               set_names2(iset), res_symbols(nbasis), atnum, atspc_dum, istat)
!!$       call get_atomic_symbol(set_names2(iset), res_symbols(nbasis), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             call pl_error_print_message(istat)
             write(6,*) '  : ',trim(set_names2(iset))
          end if
          res_basissets(nbasis)  = trim(set_basissets2(iset))
          res_basisfiles(nbasis) = trim(set_basisfiles2(iset))
       end if
    end do

    ! &Basisのsetで割り当て指定(総称名指定)された基底関数系を集計
    do iset = 1, nset1
       do ixyz = 1, nxyz
          if (atspc(ixyz) == IS_BARE .or. atspc(ixyz) == IS_DUMMY) cycle
          if (atom2basis(ixyz) /= 0) cycle
          if (pl_str_cmp_ignorecase(set_names1(iset), symbols(ixyz))) then
             !write(6,*) 'iset1->ixyz:',iset,' ',trim(set_names1(iset)),&
             !     '->',ixyz,' ',trim(names(ixyz))
             dbasis = 0
             do ibasis = 1, nbasis
                !write(6,*) 'ibasis=',ibasis
                !write(6,*) ' ',trim(res_symbols(ibasis)),&
                !     ' ',trim(symbols(ixyz))
                !write(6,*) ' ',trim(res_basissets(ibasis)),&
                !     ' ',trim(set_basissets2(iset))
                !write(6,*) ' ',trim(res_basisfiles(ibasis)),&
                !     ' ',trim(set_basisfiles2(iset))
                if (pl_str_cmp_ignorecase(res_symbols(ibasis), symbols(ixyz))&
                     .and.&
                     pl_str_cmp_ignorecase(&
                     res_basissets(ibasis), set_basissets1(iset))&
!!$                     trim(res_basissets(ibasis)) == trim(set_basissets1(iset))&
                     .and. trim(res_basisfiles(ibasis))&
                     == trim(set_basisfiles1(iset))) then
                   dbasis = ibasis
                   !write(6,*) 'dbasis=',dbasis
                   exit
                end if
             end do
             if (dbasis == 0) then
                kbasis = nbasis + 1
                res_ncount(kbasis) = 1
             else
                kbasis = dbasis
                res_ncount(kbasis) = res_ncount(kbasis) + 1
             end if
             atom2basis(ixyz) = kbasis
          end if
       end do
       if (kbasis == nbasis + 1) then
          nbasis = nbasis + 1
          res_symbols(nbasis)    = trim(set_names1(iset))
          res_basissets(nbasis)  = trim(set_basissets1(iset))
          res_basisfiles(nbasis) = trim(set_basisfiles1(iset))
       end if
    end do

    ! &Basisのデフォルト割り当て指定された基底関数系を集計
    do ixyz = 1, nxyz
       if (atspc(ixyz) == IS_BARE .or. atspc(ixyz) == IS_DUMMY) cycle
       if (atom2basis(ixyz) /= 0) cycle
       !write(6,*) 'last',ixyz,' ',trim(names(ixyz))
       dbasis = 0
       do ibasis = 1, nbasis
          !write(6,*) 'ibasis=',ibasis
          !write(6,*) ' ',trim(res_symbols(ibasis)),' ',trim(symbols(ixyz))
          !write(6,*) ' ',trim(res_basissets(ibasis)),' ',trim(basisset_default)
          !write(6,*) ' ',trim(res_basisfiles(ibasis)),&
          !     ' ',trim(basisfile_default)
          if (pl_str_cmp_ignorecase(res_symbols(ibasis), symbols(ixyz)) .and.&
               pl_str_cmp_ignorecase(res_basissets(ibasis), basisset_default)&
               .and.&
!!$               trim(res_basissets(ibasis)) == trim(basisset_default) .and.&
               trim(res_basisfiles(ibasis)) == trim(basisfile_default)) then
             dbasis = ibasis
             !write(6,*) 'dbasis=',dbasis
             exit
          end if
       end do
       if (dbasis == 0) then
          kbasis = nbasis + 1
          res_ncount(kbasis) = 1
       else
          kbasis = dbasis
          res_ncount(kbasis) = res_ncount(kbasis) + 1
       end if
       atom2basis(ixyz) = kbasis
       do jxyz = ixyz + 1, nxyz
          if (atom2basis(jxyz) /= 0) cycle
          if (pl_str_cmp_ignorecase(symbols(ixyz), symbols(jxyz))) then
             atom2basis(jxyz) = kbasis
             res_ncount(kbasis) = res_ncount(kbasis) + 1
          end if
       end do
       if (kbasis == nbasis + 1) then
          nbasis = nbasis + 1
          res_symbols(nbasis)    = trim(symbols(ixyz))
          res_basissets(nbasis)  = basisset_default
          res_basisfiles(nbasis) = basisfile_default
       end if
    end do

    if (ldebug > 0) then
       do ibasis = 1, nbasis
          write(6,*) 'basis:',ibasis,trim(res_symbols(ibasis)),&
               ' ',trim(res_basissets(ibasis)),&
               ' ',trim(res_basisfiles(ibasis)),res_ncount(ibasis)
       end do

       do ixyz = 1, nxyz
          write(6,*) 'xyz:',ixyz,trim(names(ixyz)),' ',trim(symbols(ixyz)),&
               ' ',atom2basis(ixyz),atspc(ixyz)
       end do
    end if

  end subroutine p_assign_basis

  !-----------------------------------------------------------------------------
  subroutine p_get_basis_n(&
       lines, nline,&
       def_symbols, def_basissets, def_nshell, def_nprim,&
       res_symbols, res_basissets, res_basisfiles,&
       ndef, def_mxshell,&
       nbasis, mxshell, mxprim,&
       basisfile_default,&
       ldebug, istat)
    use input_class
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::ndef
    integer,intent(in)::def_mxshell
    integer,intent(in)::nbasis
    character(len=*),intent(in)::def_symbols(ndef)
    character(len=*),intent(in)::def_basissets(ndef)
    integer,intent(in)::def_nshell(ndef)
    integer,intent(in)::def_nprim(def_mxshell,ndef)
    character(len=*),intent(in)::res_symbols(nbasis)
    character(len=*),intent(in)::res_basissets(nbasis)
    character(len=*),intent(in)::res_basisfiles(nbasis)
    integer,intent(out)::mxshell
    integer,intent(out)::mxprim
    character(len=*),intent(in)::basisfile_default
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    integer::idef, kdef, ibasis, nshell, ishell, nprim

    mxshell = 0
    mxprim = 0
    do ibasis = 1, nbasis
       !write(6,*) 'ibasis=',ibasis
       if (len_trim(res_basisfiles(ibasis)) == 0) then
          !write(6,*) 'symbol   =',trim(res_symbols(ibasis))
          !write(6,*) 'basisset =',trim(res_basissets(ibasis))
          !write(6,*) 'basisfile =','_____'
          kdef = 0
          do idef = 1, ndef
             !write(6,*) '  ',trim(def_symbols(idef)),&
             !     ' ',trim(def_basissets(idef))
             if (pl_str_cmp_ignorecase(res_symbols(ibasis), def_symbols(idef))&
                  .and.&
                  pl_str_cmp_ignorecase(&
                  res_basissets(ibasis), def_basissets(idef))) then
!!$                  trim(res_basissets(ibasis)) == trim(def_basissets(idef))) then
                !write(6,*) '  nshell=',def_nshell(idef)
                mxshell = max(mxshell, def_nshell(idef))
                do ishell = 1, def_nshell(idef)
                   !write(6,*) '  nprim=',def_nprim(ishell,idef)
                   mxprim  = max(mxprim, def_nprim(ishell,idef))
                end do
                kdef = 1
                exit
             end if
          end do
          if (kdef == 0) then
             istat = PL_ERR_INVALID_FORMAT
             call pl_error_print_message(istat)
             write(6,*) '  : kdef error'
             return
          end if
       else
          if (trim(res_basisfiles(ibasis)) == trim(basisfile_default)) then
             call p_basisfile_read_n(&
                  lines, nline,&
                  res_symbols(ibasis), res_basissets(ibasis),&
                  nshell, nprim, ldebug, istat)
             if (istat /= 0) return
          else
             call p_basisfile_read2_n(&
                  res_symbols(ibasis), res_basissets(ibasis),&
                  res_basisfiles(ibasis), nshell, nprim, ldebug, istat)
             if (istat /= 0) return
          end if
          mxshell = max(mxshell, nshell)
          mxprim  = max(mxprim, nprim)
       end if
       !write(6,*) 'mxshell=',mxshell
       !write(6,*) 'mxprim =',mxprim
    end do
    if (ldebug > 0) then
       write(6,*) 'mxshell=',mxshell
       write(6,*) 'mxprim =',mxprim
    end if

  end subroutine p_get_basis_n

  !-----------------------------------------------------------------------------
  subroutine p_get_basis(&
       self, lines, nline,&
       def_symbols, def_basissets, def_nshell, def_shtypes,&
       def_scales, def_nprim, def_oexp, def_coef,&
       res_symbols, res_basissets, res_basisfiles,&
       ndef, def_mxshell, def_mxprim,&
       nbasis,&
       basisfile_default,&
       ldebug, istat)
    use input_class
    type(basis_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::ndef
    integer,intent(in)::def_mxshell
    integer,intent(in)::def_mxprim
    integer,intent(in)::nbasis
    character(len=*),intent(in)::def_symbols(ndef)
    character(len=*),intent(in)::def_basissets(ndef)
    integer,intent(in)::def_nshell(ndef)
    integer,intent(in)::def_shtypes(def_mxshell,ndef)
    real(8),intent(in)::def_scales(def_mxshell,ndef)
    integer,intent(in)::def_nprim(def_mxshell,ndef)
    real(8),intent(in)::def_oexp(def_mxprim,def_mxshell,ndef)
    real(8),intent(in)::def_coef(def_mxprim,def_mxshell,ndef)
    character(len=*),intent(in)::res_symbols(nbasis)
    character(len=*),intent(in)::res_basissets(nbasis)
    character(len=*),intent(in)::res_basisfiles(nbasis)
    character(len=*),intent(in)::basisfile_default
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    integer::idef, kdef, ibasis, nshell, ishell, nprim, iprim, scale

    do ibasis = 1, nbasis
       !write(6,*) 'ibasis=',ibasis
       if (len_trim(res_basisfiles(ibasis)) == 0) then
          !write(6,*) 'symbol   =',trim(res_symbols(ibasis))
          !write(6,*) 'basisset =',trim(res_basissets(ibasis))
          !write(6,*) 'basisfile =','_____'
          kdef = 0
          do idef = 1, ndef
             !write(6,*) '  ',trim(def_symbols(idef)),&
             !     ' ',trim(def_basissets(idef))
             if (pl_str_cmp_ignorecase(res_symbols(ibasis), def_symbols(idef))&
                  .and.&
                  pl_str_cmp_ignorecase(&
                  res_basissets(ibasis), def_basissets(idef))) then
!!$               trim(res_basissets(ibasis)) == trim(def_basissets(idef))) then
                !write(6,*) '  nshell=',def_nshell(idef)
                self%ntshell(ibasis) = def_nshell(idef)
                do ishell = 1, def_nshell(idef)
                   !write(6,*) '  nprim=',def_nprim(ishell,idef)
                   self%ntshtype(ishell,ibasis) = def_shtypes(ishell,idef)
                   self%ntprim(ishell,ibasis) = def_nprim(ishell,idef)
                   scale = def_scales(ishell,idef) * def_scales(ishell,idef)
                   do iprim = 1, def_nprim(ishell,idef)
                      self%z(iprim,ishell,ibasis) = def_oexp(iprim,ishell,idef)&
                           *scale
                      self%c(iprim,ishell,ibasis) = def_coef(iprim,ishell,idef)
                   end do
                end do
                kdef = 1
                exit
             end if
          end do
          if (kdef == 0) then
             istat = PL_ERR_INVALID_FORMAT
             call pl_error_print_message(istat)
             write(6,*) '  : kdef error'
             return
          end if
       else
          if (trim(res_basisfiles(ibasis)) == trim(basisfile_default)) then
             call p_basisfile_read(&
                  self, ibasis, lines, nline,&
                  res_symbols(ibasis), res_basissets(ibasis),&
                  ldebug, istat)
             if (istat /= 0) return
          else
             call p_basisfile_read2(&
                  self, ibasis,&
                  res_symbols(ibasis), res_basissets(ibasis),&
                  res_basisfiles(ibasis), ldebug, istat)
             if (istat /= 0) return
          end if
       end if
    end do

  end subroutine p_get_basis

  !-----------------------------------------------------------------------------
  subroutine p_basisfile_read_n(&
       lines, nline, symbol, basisset,&
       mxshell, mxprim,&
       ldebug, istat)
    use input_class
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    character(len=*),intent(in)::symbol
    character(len=*),intent(in)::basisset
    integer,intent(out)::mxshell
    integer,intent(out)::mxprim
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    character(len=MAX_INPUT_LINE_LEN),allocatable::vals(:)
    character(len=MAX_INPUT_LINE_LEN)::cval
    character(len=MAX_INPUT_LINE_LEN)::shtype
    integer::mxval, nval, ival
    integer::iline
    integer::ncount, ndef, nshell, nprim, count_down
    logical::in
    integer::iline_err, icol_err, icol_s

    !write(6,*) 'symbol    =',trim(symbol)
    !write(6,*) 'basisset  =',trim(basisset)
    !write(6,*) 'basisfile =','__default__'

    mxval = 0
    do iline = 1, nline
       call split_str_eq_n(lines(iline), nval, istat)
       if (istat /= 0) return
       mxval = max(mxval,nval)
    end do
    !write(6,*) 'mxval=',mxval
    allocate(vals(mxval))

    ncount = 0
    ndef = 0
    count_down = 0
    mxshell = 0
    mxprim = 0
    icol_s = 1
    do iline = 1, nline
       call split_str_eq(lines(iline), vals, mxval, nval, istat)
       if (istat /= 0) return
       if (nval == 0) cycle
       call pl_str_tolower(vals(1), cval)
       select case (trim(cval))
       case ('def')
          if (ndef /= 0) then
             !write(6,*) 'nshell=',nshell
             if (in .and. ncount == 1) then
                mxshell = max(mxshell, nshell)
             end if
          end if
          if (pl_str_cmp_ignorecase(vals(2), symbol)&
               .and. pl_str_cmp_ignorecase(vals(4), basisset)) then
!!$               .and. trim(vals(4)) == basisset) then
             in = .true.
             ncount = ncount + 1
          else
             in = .false.
          end if
          ndef = ndef + 1
          !write(6,*) 'start-def:',trim(vals(2)),':',trim(vals(4))
          if (count_down /= 0) then
             !write(6,*) 'error in def',iline
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          nshell = 0
       case ('s','p','d','f','g','h','sp')
          shtype = cval
          !write(6,*) ' shell:',trim(vals(1)),':',trim(vals(2))
          if (count_down /= 0) then
             !write(6,*) 'error in shell',iline
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          nshell = nshell + 1
          select case (nval)
          case (2:3)
             call pl_str_to_int(trim(vals(2)), nprim, istat)
             if (istat /= 0) then
                !write(6,*) 'error in nprim',iline
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
          case default
             if (istat /= 0) then
                !write(6,*) 'error in shell',iline
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
          end select
          !write(6,*) 'nprim=',nprim
          if (in .and. ncount == 1) then
             mxprim = max(mxprim, nprim)
             if (trim(shtype) == 'sp') nshell = nshell + 1
          end if
          count_down = nprim
       case default
          !write(6,*) '   primitive:',trim(vals(1)),':',trim(vals(2))
          if (count_down <= 0) then
             !write(6,*) 'error in prim',iline
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          count_down = count_down - 1
       end select
    end do
    if (ncount == 0) then
       istat = PL_ERR_NO_BASIS_SET
       call pl_error_print_message(istat)
       write(6,'(2x,''Basis set :'',a,'', '',a)')&
            trim(symbol), trim(basisset)
       go to 8
    end if
    if (in .and. ncount == 1) then
       !write(6,*) 'nshell=',nshell
       mxshell = max(mxshell, nshell)
    end if
    
    !write(6,*) '  nshell=',mxshell
    !write(6,*) '  nprim=',mxprim

    istat = 0

8   deallocate(vals)
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine p_basisfile_read_n

  !-----------------------------------------------------------------------------
  subroutine p_basisfile_read(&
       self, ibasis, lines, nline, symbol, basisset,&
       ldebug, istat)
    use input_class
    type(basis_type),intent(inout)::self
    integer,intent(in)::ibasis
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    character(len=*),intent(in)::symbol
    character(len=*),intent(in)::basisset
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    character(len=MAX_INPUT_LINE_LEN),allocatable::vals(:)
    character(len=MAX_INPUT_LINE_LEN)::cval
    character(len=MAX_INPUT_LINE_LEN)::shtype
    integer::mxval, nval, ival
    real(8)::val
    integer::iline
    integer::ndef, idef
    integer::ncount, nshell, ishell, nprim, iprim, count_down
    logical::in
    real(8)::scale

    character(len=*),parameter::c_assignment_op = '='
    integer::iline_err, icol_err, icol_s

!!$    write(6,*) 'symbol    =',trim(symbol)
!!$    write(6,*) 'basisset  =',trim(basisset)
!!$    write(6,*) 'basisfile =','__default__'

    mxval = 0
    do iline = 1, nline
       call split_str_eq_n(lines(iline), nval, istat)
       if (istat /= 0) return
       mxval = max(mxval,nval)
    end do
    !write(6,*) 'mxval=',mxval
    allocate(vals(mxval))

    ncount = 0
    idef = 0
    count_down = 0
    icol_s = 1
    do iline = 1, nline
       call split_str_eq(lines(iline), vals, mxval, nval, istat)
       if (istat /= 0) return
       if (nval == 0) cycle
       call pl_str_tolower(vals(1), cval)
       select case (trim(cval))
       case ('def')
          if (idef /= 0) then
             !write(6,*) 'ishell=',ishell
             if (in .and. ncount == 1) then
                self%ntshell(ibasis) = nshell
             end if
          end if
          idef = idef + 1
!!$          write(6,*) 'start-def:',trim(vals(2)),':',trim(vals(4))
          if (count_down /= 0) then
             !write(6,*) 'error in def 1'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          select case (nval)
          case (4)
             if (trim(vals(3)) /= c_assignment_op) then
                !write(6,*) 'error in def 2'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             if (pl_str_cmp_ignorecase(vals(2), symbol)&
                  .and. pl_str_cmp_ignorecase(vals(4), basisset)) then
!!$                  .and. trim(vals(4)) == basisset) then
                in = .true.
                ncount = ncount + 1
             else
                in = .false.
             end if
          case default
             !write(6,*) 'error in def 3'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end select
          nshell = 0
       case ('s','p','d','f','g','h','sp')
          shtype = cval
!!$          write(6,*) ' shell:',trim(vals(1)),':',trim(vals(2))
          if (count_down /= 0) then
             !write(6,*) 'error in shell 1'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          select case (nval)
          case (2:3)
             call pl_str_to_int(trim(vals(2)), nprim, istat)
             if (istat /= 0) then
                !write(6,*) 'error in nprim'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             nshell = nshell + 1
             if (nval == 3) then
                call pl_str_to_dbl(trim(vals(3)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in scale'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                scale = val
             else
                scale = 1.0d0
             end if
          case default
             !write(6,*) 'error in shell 2'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end select
!!$          write(6,*) 'nprim=',nprim
          if (in .and. ncount == 1) then
             self%ntprim(nshell,ibasis) = nprim
             select case(trim(shtype))
             case ('s','sp')
                self%ntshtype(nshell, ibasis) = 1
             case ('p')
                self%ntshtype(nshell, ibasis) = 2
             case ('d')
                self%ntshtype(nshell, ibasis) = 3
             case default
                !write(6,*) 'error in shell type'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end select
             if (trim(shtype) == 'sp') then
                nshell = nshell + 1
                self%ntprim(nshell,ibasis) = self%ntprim(nshell-1,ibasis)
                self%ntshtype(nshell,ibasis) = 2
             end if
          end if
          count_down = nprim
          iprim = 0
       case default
!!$          write(6,*) '   primitive:',trim(vals(1)),':',trim(vals(2))
          if (count_down <= 0) then
             !write(6,*) 'error in prim 1'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          iprim = iprim + 1
          if (in .and. ncount == 1) then
             select case (nval)
             case (2)
                if (trim(shtype) /= 's' .and. trim(shtype) /= 'p'&
                     .and. trim(shtype) /= 'd') then
                   !write(6,*) 'error in prim 2'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                call pl_str_to_dbl(trim(vals(1)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 3'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%z(iprim, nshell, ibasis) = val*scale*scale
                call pl_str_to_dbl(trim(vals(2)), val, istat)
!!$                write(6,*) 'c=',trim(vals(2)), val
                if (istat /= 0) then
                   !write(6,*) 'error in prim 4'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%c(iprim, nshell, ibasis) = val
             case (3)
                if (trim(shtype) /= 'sp') then
                   !write(6,*) 'error in prim 5'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                call pl_str_to_dbl(trim(vals(1)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 6'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%z(iprim, nshell-1, ibasis) = val
                self%z(iprim, nshell,   ibasis) = val
                call pl_str_to_dbl(trim(vals(2)), val, istat)
!!$                write(6,*) 'c=',trim(vals(2)), val
                if (istat /= 0) then
                   !write(6,*) 'error in prim 7'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%c(iprim, nshell-1, ibasis) = val
                call pl_str_to_dbl(trim(vals(3)), val, istat)
!!$                write(6,*) 'c=',trim(vals(3)), val
                if (istat /= 0) then
                   !write(6,*) 'error in prim 8'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%c(iprim, nshell, ibasis) = val
             case default
                !write(6,*) 'error in prim 9'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end select
          end if
          count_down = count_down - 1
       end select
    end do
    if (in .and. ncount == 1) then
       write(6,*) 'nshell=',nshell
       self%ntshell(ibasis) = ishell
    end if
    !write(6,*) 'self%c=',self%c

    deallocate(vals)
    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine p_basisfile_read

  !-----------------------------------------------------------------------------
  subroutine p_basisfile_read2_n(&
       symbol, basisset, basisfile,&
       mxshell, mxprim,&
       ldebug, istat)
    use input_class
    character(len=*),intent(in)::symbol
    character(len=*),intent(in)::basisset
    character(len=*),intent(in)::basisfile
    integer,intent(out)::mxshell
    integer,intent(out)::mxprim
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    character(len=MAX_INPUT_LINE_LEN)::line
    character(len=MAX_INPUT_LINE_LEN),allocatable::vals(:)
    character(len=MAX_INPUT_LINE_LEN)::cval
    character(len=MAX_INPUT_LINE_LEN)::shtype
    integer::mxval, nval, ival
    integer::iline
    integer::ncount, ndef, nshell, nprim, count_down
    logical::in
    integer::iline_err, icol_err, icol_s

    !write(6,*) 'symbol   =',trim(symbol)
    !write(6,*) 'basisset =',trim(basisset)
    !write(6,*) 'basisfile=',trim(basisfile)

    mxval = 0
    iline = 0
    open(1, file=basisfile, status='old', err=101)
    rewind(1, err=102)
1   read(1, '(a)', end=2, err=103) line
    iline = iline + 1
    call split_str_eq_n(line, nval, istat)
    if (istat /= 0) return
    mxval = max(mxval,nval)
    go to 1
2   rewind(1, err=102)
    !write(6,*) 'mxval=',mxval
    allocate(vals(mxval))
    

    iline = 0
    ncount = 0
    ndef = 0
    count_down = 0
    mxshell = 0
    mxprim = 0
    icol_s = 1
3   read(1, '(a)', end=4, err=103) line
    iline = iline + 1
       call split_str_eq(line, vals, mxval, nval, istat)
       if (istat /= 0) return
       if (nval == 0) go to 3
       call pl_str_tolower(vals(1), cval)
       select case (trim(cval))
       case ('def')
          if (ndef /= 0) then
             !write(6,*) 'nshell=',nshell
             if (in .and. ncount == 1) then
                mxshell = max(mxshell, nshell)
             end if
          end if
          if (pl_str_cmp_ignorecase(vals(2), symbol)&
               .and. pl_str_cmp_ignorecase(vals(4), basisset)) then
!!$               .and. trim(vals(4)) == basisset) then
             in = .true.
             ncount = ncount + 1
          else
             in = .false.
          end if
          ndef = ndef + 1
          !write(6,*) 'start-def:',trim(vals(2)),':',trim(vals(4))
          if (count_down /= 0) then
             !write(6,*) 'error in def',iline
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          nshell = 0
       case ('s','p','d','f','g','h','sp')
          shtype = cval
          !write(6,*) ' shell:',trim(vals(1)),':',trim(vals(2))
          if (count_down /= 0) then
             !write(6,*) 'error in shell',iline
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          nshell = nshell + 1
          select case (nval)
          case (2:3)
             call pl_str_to_int(trim(vals(2)), nprim, istat)
             if (istat /= 0) then
                !write(6,*) 'error in nprim',iline
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
          case default
             if (istat /= 0) then
                !write(6,*) 'error in shell',iline
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
          end select
          !write(6,*) 'nprim=',nprim
          if (in .and. ncount == 1) then
             mxprim = max(mxprim, nprim)
             if (trim(shtype) == 'sp') nshell = nshell + 1
          end if
          count_down = nprim
       case default
          !write(6,*) '   primitive:',trim(vals(1)),':',trim(vals(2))
          if (count_down <= 0) then
             !write(6,*) 'error in prim',iline
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          count_down = count_down - 1
       end select
    go to 3
4   close(1, err=104)
    if (ncount == 0) then
       istat = PL_ERR_NO_BASIS_SET
       call pl_error_print_message(istat)
       write(6,'(2x,''Basis set :'',a,'', '',a,'', '',a)')&
            trim(symbol), trim(basisset), trim(basisfile)
       go to 8
    end if
    if (in .and. ncount == 1) then
       !write(6,*) 'nshell=',nshell
       mxshell = max(mxshell, nshell)
    end if

    !write(6,*) '  nshell=',mxshell
    !write(6,*) '  nprim=',mxprim

    istat = 0

8   deallocate(vals)
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(line)
    return
101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(basisfile)
    return
  end subroutine p_basisfile_read2_n

  !-----------------------------------------------------------------------------
  subroutine p_basisfile_read2(&
       self, ibasis,&
       symbol, basisset, basisfile,&
       ldebug, istat)
    use input_class
    type(basis_type),intent(inout)::self
    integer,intent(in)::ibasis
    character(len=*),intent(in)::symbol
    character(len=*),intent(in)::basisset
    character(len=*),intent(in)::basisfile
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    character(len=MAX_INPUT_LINE_LEN)::line
    character(len=MAX_INPUT_LINE_LEN),allocatable::vals(:)
    character(len=MAX_INPUT_LINE_LEN)::cval
    character(len=MAX_INPUT_LINE_LEN)::shtype
    integer::mxval, nval, ival
    real(8)::val
    integer::iline
    integer::ndef, idef
    integer::ncount, nshell, ishell, nprim, iprim, count_down
    logical::in
    real(8)::scale

    character(len=*),parameter::c_assignment_op = '='
    integer::iline_err, icol_err, icol_s

    !write(6,*) 'symbol    =',trim(symbol)
    !write(6,*) 'basisset  =',trim(basisset)
    !write(6,*) 'basisfile =','__default__'

    mxval = 0
    iline = 0
    open(1, file=basisfile, status='old', err=101)
    rewind(1, err=102)
1   read(1, '(a)', end=2, err=103) line
    iline = iline + 1
    call split_str_eq_n(line, nval, istat)
    if (istat /= 0) return
    mxval = max(mxval,nval)
    go to 1
2   rewind(1, err=102)
    !write(6,*) 'mxval=',mxval
    allocate(vals(mxval))

    iline = 0
    ncount = 0
    ndef = 0
    count_down = 0
    icol_s = 1
3   read(1, '(a)', end=4, err=103) line
    iline = iline + 1
       call split_str_eq(line, vals, mxval, nval, istat)
       if (istat /= 0) return
       if (nval == 0) go to 3
       call pl_str_tolower(vals(1), cval)
       select case (trim(cval))
       case ('def')
          if (idef /= 0) then
             !write(6,*) 'ishell=',ishell
             if (in .and. ncount == 1) then
                self%ntshell(ibasis) = nshell
             end if
          end if
          idef = idef + 1
          !write(6,*) 'start-def:',trim(vals(2)),':',trim(vals(4))
          if (count_down /= 0) then
             !write(6,*) 'error in def 1'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          select case (nval)
          case (4)
             if (trim(vals(3)) /= c_assignment_op) then
                !write(6,*) 'error in def 2'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             if (pl_str_cmp_ignorecase(vals(2), symbol)&
                  .and. pl_str_cmp_ignorecase(vals(4), basisset)) then
!!$                  .and. trim(vals(4)) == basisset) then
                in = .true.
                ncount = ncount + 1
             else
                in = .false.
             end if
          case default
             !write(6,*) 'error in def 3'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end select
          nshell = 0
       case ('s','p','d','f','g','h','sp')
          shtype = cval
          !write(6,*) ' shell:',trim(vals(1)),':',trim(vals(2))
          if (count_down /= 0) then
             !write(6,*) 'error in shell 1'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          select case (nval)
          case (2:3)
             call pl_str_to_int(trim(vals(2)), nprim, istat)
             if (istat /= 0) then
                !write(6,*) 'error in nprim'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end if
             nshell = nshell + 1
             if (nval == 3) then
                call pl_str_to_dbl(trim(vals(3)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in scale'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                scale = val
             else
                scale = 1.0d0
             end if
          case default
             !write(6,*) 'error in shell 2'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end select
          !write(6,*) 'nprim=',nprim
          if (in .and. ncount == 1) then
             self%ntprim(nshell,ibasis) = nprim
             select case(trim(shtype))
             case ('s','sp')
                self%ntshtype(nshell, ibasis) = 1
             case ('p')
                self%ntshtype(nshell, ibasis) = 2
             case ('d')
                self%ntshtype(nshell, ibasis) = 3
             case default
                !write(6,*) 'error in shell type'
                istat = PL_ERR_INVALID_FORMAT
                iline_err = iline; icol_err = icol_s
                go to 9
             end select
             if (trim(shtype) == 'sp') then
                nshell = nshell + 1
                self%ntprim(nshell,ibasis) = self%ntprim(nshell-1,ibasis)
                self%ntshtype(nshell,ibasis) = 2
             end if
          end if
          count_down = nprim
          iprim = 0
       case default
          !write(6,*) '   primitive:',trim(vals(1)),':',trim(vals(2))
          if (count_down <= 0) then
             !write(6,*) 'error in prim 1'
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          iprim = iprim + 1
          if (in .and. ncount == 1) then
             select case (nval)
             case (2)
                if (trim(shtype) /= 's' .and. trim(shtype) /= 'p'&
                     .and. trim(shtype) /= 'd') then
                   !write(6,*) 'error in prim 2'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                call pl_str_to_dbl(trim(vals(1)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 3'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%z(iprim, nshell, ibasis) = val*scale*scale
                call pl_str_to_dbl(trim(vals(2)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 4'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%c(iprim, nshell, ibasis) = val
             case (3)
                if (trim(shtype) /= 'sp') then
                   !write(6,*) 'error in prim 5'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                call pl_str_to_dbl(trim(vals(1)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 6'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%z(iprim, nshell-1, ibasis) = val
                self%z(iprim, nshell,   ibasis) = val
                call pl_str_to_dbl(trim(vals(2)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 7'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%c(iprim, nshell-1, ibasis) = val
                call pl_str_to_dbl(trim(vals(3)), val, istat)
                if (istat /= 0) then
                   !write(6,*) 'error in prim 8'
                   istat = PL_ERR_INVALID_FORMAT
                   iline_err = iline; icol_err = icol_s
                   go to 9
                end if
                self%c(iprim, nshell, ibasis) = val
             case default
             end select
          end if
          count_down = count_down - 1
       end select
    go to 3
4   close(1, err=104)
    if (in .and. ncount == 1) then
       !write(6,*) 'nshell=',nshell
       self%ntshell(ibasis) = ishell
    end if

    deallocate(vals)
    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(line)
    return
101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(basisfile)
    return
  end subroutine p_basisfile_read2

!!$  !-----------------------------------------------------------------------------
!!$  !> @brief 原子名が正当か調べる
!!$  !>
!!$  !> 元素記号(1〜2文字) [ + 識別番号:0-9(0〜文字) ] [ + { -Ba | -Bq } ]
!!$  function is_valid_atomic_name(name) result(res)
!!$    use pl_str_utils_class
!!$    character(len=*),intent(in)::name
!!$    logical::res
!!$
!!$    character(len=2)::symbol
!!$    integer::ln, i, ipos, atnum
!!$
!!$    res = .false.
!!$    ln = len_trim(name) 
!!$    if (ln == 0) return ! 長さがゼロ (invalid)
!!$    !write(6,*) 1,name(1:1)
!!$    if (index(c_alpha, name(1:1)) == 0) return ! 1文字目がA-Za-zでない (invalid)
!!$    ! 1文字目がA-Za-zである
!!$    !write(6,*) 2,name(2:2)
!!$    if (index(c_alpha, name(2:2)) == 0) then
!!$       ! 2文字目はA-Za-zでない
!!$       ipos = 1
!!$    else
!!$       ! 2文字目もA-Za-zである
!!$       ipos = 2
!!$    end if
!!$    symbol = name(1:ipos)
!!$    !write(6,*) 3,symbol
!!$    do i = ipos+1, ln
!!$       !write(6,*) 4,name(i:i)
!!$       if (index(c_digit, name(i:i)) /= 0) cycle ! 0-9ならばOK
!!$       if (c_minus /= name(i:i)) return ! '-' でないとき (invalid)
!!$       ! '-' のとき
!!$       ipos = i + 1
!!$       go to 1
!!$    end do
!!$    ! ipos+1以降最後までc_digitである
!!$    go to 2
!!$
!!$1   if (ipos > ln) return ! '-' が末端 (invalid)
!!$    !write(6,*) 6,name(ipos:ipos),ipos,ln
!!$    if (index('Bb',name(ipos:ipos)) == 0) return ! 'B'でない (invalid)
!!$    ipos = ipos + 1
!!$    if (ipos > ln) return ! 'B' が末端 (invalid)
!!$    if (ipos /= ln) return ! まだつづく (invalid)
!!$    !write(6,*) 7,name(ipos:ipos),ipos,ln
!!$    if (index('AaQq',name(ipos:ipos)) == 0) return ! 'A'か'Q' でない (invalid)
!!$
!!$2   atnum = pl_str_search_name_ignorecase(&
!!$         symbol, ATOMIC_SYMBOLS, MAX_ATOMIC_SYMBOLS + 1)
!!$    !write(6,*) 8,atnum,symbol
!!$    if (atnum == 0) return ! 元素記号でない (invalid)
!!$    res = .true.
!!$    return ! OK
!!$
!!$  end function is_valid_atomic_name

  !-----------------------------------------------------------------------------
  !> @brief 原子名を調べる
  !>
  !> 元素記号(1〜2文字) [ + 識別番号:0-9(0〜文字) ] [ + { -Ba | -Bq } ]
  subroutine split_atomic_name(name, symbol, atnum, atspc, istat)
    use pl_str_utils_class
    character(len=*),intent(in)::name
    character(len=*),intent(inout)::symbol
    integer,intent(out)::atspc
    integer,intent(out)::istat
    
    integer::ln, i, ipos, atnum

    istat = PL_ERR_INVALID_NAME
    atspc = UNKNOWN
    atnum = UNKNOWN
    symbol = ' '

    ln = len_trim(name) 
    if (ln == 0) return ! 長さがゼロ (invalid)
    !write(6,*) 1,name(1:1)
    if (index(c_alpha, name(1:1)) == 0) return ! 1文字目がA-Za-zでない (invalid)
    ! 1文字目がA-Za-zである
    !write(6,*) 2,name(2:2)
    if (index(c_alpha, name(2:2)) == 0) then
       ! 2文字目はA-Za-zでない
       ipos = 1
    else
       ! 2文字目もA-Za-zである
       ipos = 2
    end if
    symbol = name(1:ipos)
    !write(6,*) 3,symbol
    do i = ipos+1, ln
       !write(6,*) 4,name(i:i)
       if (index(c_digit, name(i:i)) /= 0) cycle ! 0-9ならばOK
       if (c_minus /= name(i:i)) return ! '-' でないとき (invalid)
       ! '-' のとき
       ipos = i + 1
       go to 1
    end do
    ! ipos+1以降最後までc_digitである
    go to 2

1   if (ipos > ln) return ! '-' が末端 (invalid)
    !write(6,*) 6,name(ipos:ipos),ipos,ln
    if (index('Bb',name(ipos:ipos)) == 0) return ! 'B'でない (invalid)
    ipos = ipos + 1
    if (ipos > ln) return ! 'B' が末端 (invalid)
    if (ipos /= ln) return ! まだつづく (invalid)
    !write(6,*) 7,name(ipos:ipos),ipos,ln
    select case (name(ipos:ipos))
    case ('A','a')
       atspc = IS_BARE
    case('Q','q')
       atspc = IS_GHOST
    case default
       return ! 'A'でも'Q' でない (invalid)
    end select

2   atnum = pl_str_search_name_ignorecase(&
         symbol, ATOMIC_SYMBOLS, MAX_ATOMIC_SYMBOLS + 1) - 1
    !write(6,*) 8,atnum,symbol
    if (atnum < 0) return ! 元素記号でない (invalid)
    if (atnum == 0) then
       if (atspc /= UNKNOWN) then
          return ! dummyに'-ba','-bq'が付いている
       else
          atspc = IS_DUMMY ! OK
       end if
    else
       if (atspc == UNKNOWN) then
          atspc = IS_ATOM ! OK
       end if
    end if

8   istat = 0 ! OK
    !write(6,*) 'name  =',trim(name)
    !write(6,*) '  symbol=',trim(symbol)
    !write(6,*) '  atnum =',atnum
    !write(6,*) '  atspc =',atspc
    return
  end subroutine split_atomic_name

  !-----------------------------------------------------------------------------
  subroutine merge_atoms(self, atoms1, atoms2, ldebug, istat)
    type(atoms_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms1
    type(atoms_type),intent(in)::atoms2
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nxyz, ib

    nxyz = atoms1%num + atoms2%num

    call new_atoms(self, nxyz, istat)
    if (istat /= 0) return

    ib = atoms1%basis%num
    self%atnum(1:atoms1%num)           = atoms1%atnum(1:atoms1%num)
    self%atnum(atoms1%num+1:nxyz)      = atoms2%atnum(1:atoms2%num)
    self%chg(1:atoms1%num)             = atoms1%chg(1:atoms1%num)
    self%chg(atoms1%num+1:nxyz)        = atoms2%chg(1:atoms2%num)
    self%xyz(1:3,1:atoms1%num)         = atoms1%xyz(1:3,1:atoms1%num)
    self%xyz(1:3,atoms1%num+1:nxyz)    = atoms2%xyz(1:3,1:atoms2%num)
    self%atom2basis(1:atoms1%num)      = atoms1%atom2basis(1:atoms1%num)
    self%atom2basis(atoms1%num+1:nxyz) = atoms2%atom2basis(1:atoms2%num) + ib

    call merge_basis(self%basis, atoms1%basis, atoms2%basis, ldebug, istat)
    if (istat /= 0) return

  end subroutine merge_atoms

  !-----------------------------------------------------------------------------
end module atoms_class
