! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> RHFエネルギークラス
!!
module energy_rhf_class
  use energy_rhf_options_class
  implicit none
  private

  public::read_data
  public::print_data
  public::save_data
  public::load_data
  public::bcast_data
  public::free_data
  public::new_data
  public::guess
  public::orthogonalize
  public::solve
!!$  public::solve_dft
  public::data_type

  public::set_dmax

  type::data_type
     integer::nao
     integer::nmo
!!$    integer::nion
     integer::nclosed
!!$    integer::nfzcore
!!$    integer::nfzvir
     real(8),allocatable::c(:,:)   !< Molecular orbital coefficients
     real(8),allocatable::oe(:)    !< Orbital energies
     real(8),allocatable::d(:,:)   !< Density matrix
     real(8),allocatable::eij(:,:) !< Lagrangian matrix
  end type

  character(len=*),private,parameter::url = 'data_rhf'
  integer,private,parameter::id = 1
  character(len=*),private,parameter::unit_au = 'a.u.'
  character(len=*),private,parameter::no_unit = ' '

contains
  !-----------------------------------------------------------------------------
  !> 前処理
  subroutine new_data(self, nao, ldebug, istat)
    type(data_type),intent(inout)::self
    integer,intent(in)::nao
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nmo

    istat = 0

    nmo = nao
    allocate(&
         self%c(nao,nmo), self%oe(nmo), self%d(nao,nao), self%eij(nao,nao),&
         stat=istat)
    if (istat /= 0) return
    self%nao = nao
    self%nmo = nmo

  end subroutine new_data

  !-----------------------------------------------------------------------------
  !> 初期値生成
  subroutine guess(self, integral, ldebug, istat)
    use integral_class,  only: integral_type
    use initmo_class
    type(data_type),intent(inout)::self
    type(integral_type),intent(in)::integral
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

!!$    real(8),allocatable::t(:,:)
    integer::nao, nmo

    nao = self%nao
    nmo = nao

    if (ldebug > 0) then
       call ummtw('s',integral%s,nao,nao,nao,10,6,istat)
       call ummtw('h',integral%h,nao,nao,nao,10,6,istat)
    end if

!!$    allocate(t(nao,nao))
!!$    call rhf_t_matrix(t, integral%s, nao, ldebug, istat)
!!$    if (istat /= 0) return
!!$    if (ldebug > 0) then
!!$      call ummtw('T', t, nao, nao, nao, 10, 6, istat)
!!$    end if
!!$
!!$    call initmo_gbe(&
!!$        self%c, self%oe, integral%s, integral%h, t, nao, nmo, ldebug, istat)
!!$    call initmo_hcore(c, oe, w, s, h, t, nao, nmo, ldebug, istat)
    call initmo_gbe(&
         self%c, self%oe, nao, nao, nmo,&
         integral%s, integral%h, nao, ldebug, istat)
    if (istat /= 0) return

    if (ldebug > 0) then
       call ummtw('guess mo', self%c, nao, nao, nmo, 10, 6, istat)
       if (istat /= 0) return
    end if

!!$    deallocate(t)

  end subroutine guess

  !-----------------------------------------------------------------------------
  !> 直交化
  subroutine orthogonalize(self, integral, ldebug, istat)
    use integral_class,  only: integral_type
    type(data_type),intent(inout)::self
    type(integral_type),intent(in)::integral
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nao, nclosed

    nao     = self%nao
    nclosed = self%nclosed

    call orthogonalize_mo(self%c, integral%s, nao, nclosed, ldebug, istat)
    if (istat /= 0) return

  end subroutine orthogonalize

  !-----------------------------------------------------------------------------
  !> エネルギー計算
  subroutine solve(&
       self, options, e_total, e_rep,&
       atoms, ptcharges, gscharges, fswolf, integral,&
       pl_mpi, lprint, ldebug, istat)
    use atoms_class,     only: atoms_type
    use ptcharges_class, only: ptcharges_type
    use gscharges_class, only: gscharges_type
    use fswolf_class, only: fswolf_type
    use integral_class, only: integral_type!, integral_calc_hcore
!!$    use integral_class, only: integral_get_fock_2j_k
!!$    use integral_class, only: integral_get_fock_2j_k_direct
    use pl_mpi_class
    use pl_prof_class
    type(data_type),intent(inout)::self
    type(options_type),intent(in)::options
    real(8),intent(inout)::e_total
    real(8),intent(in)::e_rep
    type(atoms_type),intent(in)::atoms
    type(ptcharges_type),intent(in)::ptcharges
    type(gscharges_type),intent(in)::gscharges
    type(fswolf_type),intent(in)::fswolf
    type(integral_type),intent(inout)::integral
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::lprint
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    real(8),allocatable::t(:,:)
    real(8),allocatable::d_new(:,:)
    real(8),allocatable::f(:,:)
    real(8),allocatable::fx(:,:),fxw(:,:),fjx(:,:),fkx(:,:)!!checking
    real(8),allocatable::f_new(:,:)
    real(8),allocatable::fw(:,:)
    real(8),allocatable::diis_vec(:,:)
    real(8),allocatable::diis_err(:,:)
    real(8),allocatable::diis_b(:,:)
    real(8),allocatable::wm1(:,:)
    real(8),allocatable::pop(:)
    integer::nao, nmo, nclosed
    integer::mxdiis, mxiter, niter
    real(8)::conv_thresh_d, conv_thresh_e

    logical::is_conv
    integer::idiis_curpos, idiis_cursize
    real(8)::total_energy, total_energy_prev, e_one, e_two, e_ele
    real(8)::e_zd1, e_zd2, e_zd3, e_zd4
    real(8)::conv_d_max, conv_d_rms, conv_e, shift_val
    real(8)::ddot

    real(8),allocatable::oen(:)
    integer::icomm, myrank, nproc
    integer::natom
    integer::istat2
    integer::iao,jao
    integer::method_zd

    real(8),allocatable::dmax(:)

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank
    nproc  = pl_mpi%nprocs

    nao = self%nao
    nmo = nao
    nclosed = options%nclosed
    self%nclosed = nclosed
    natom  = atoms%num

    if (myrank == 0) then
       if (lprint >= 0) then
          write(6,'(1x)')
          write(6,'(''RHF start'')')
          call print_data(self)
          write(6,'(1x)')
          write(6,'(2x,''Iter       '',&
               &''Total Energy         Energy Conv   Density Conv'')')
       end if
    end if

    mxiter        = options%mxiter
    mxdiis        = min(options%mxdiis, mxiter)
    conv_thresh_d = options%conv_thresh_d
    conv_thresh_e = options%conv_thresh_e
    shift_val     = options%shift_val

    if (myrank == 0) then
       allocate(&
            t(nao,nao),&
            f(nao,nao), f_new(nao,nao),&
            diis_vec(nao*(nao+1)/2,mxdiis),&
            diis_err(nao*(nao+1)/2,mxdiis),&
            diis_b(mxdiis,mxdiis),&
            stat=istat)
    else
       !allocate(f(1,1), stat=istat)
       allocate(f(nao,nao), stat=istat)!! checking
    endif
    call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
    if (istat /= 0) return

    allocate(&
         d_new(nao,nao),&
         fw(nao,nao),&
         stat=istat)
    if (istat /= 0) return

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('s:Chlsk')
    if (myrank == 0) then
       call rhf_t_matrix(t, integral%s, nao, ldebug, istat)
    end if
    call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
    if (istat /= 0) return
    if (myrank == 0) then
       if (ldebug > 0) then
          call ummtw('T', t, nao, nao, nao, 10, 6, istat)
       end if
    end if
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('s:Chlsk')

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('s:Den')
    call density_matrix(self%d, self%c, nao, nclosed, ldebug, istat)
    if (istat /= 0) return
!!!
!!$    open(file='test_data_d.bin',unit=1,form='unformatted',status='unknown')
!!$    write(1) nao
!!$    write(1) ((self%d(iao,jao),iao=1,nao),jao=1,nao)
!!$    close(1)
!!$    open(file='test_data_d.txt',unit=1,form='formatted',status='unknown')
!!$    write(1,'(i8)') nao
!!$    do jao = 1, nao
!!$       do iao = 1, nao
!!$          write(1,'(2i8,f20.10)') iao,jao,self%d(iao,jao)
!!$       end do
!!$    end do
!!$    close(1)
!!$    !call ummtw('d', self%d, nao, nao, nao, 10, 6, istat)
!!!
    call dscal(nao*nao, 2.0d0, self%d, 1)
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('s:Den')

!!$       ! dmax -------------------------------------------------------------
!!$    allocate(dmax(integral%tables%nshelp))
!!$       ! dmax -------------------------------------------------------------

    niter = 0
    is_conv = .false.
    idiis_curpos  = 0
    idiis_cursize = 0
    total_energy = 0.0d0
    total_energy_prev = 0.0d0

    do while (.not. is_conv)
       if (niter + 1 > mxiter) exit
       niter = niter + 1
       if (ldebug > 0) write(6,*) 'niter=',niter

       if (niter > 1) self%d(:,:) = d_new(1:nao,1:nao)

       !---checking--->
       allocate(fjx(nao,nao), fkx(nao,nao), stat=istat)
       call pl_prof_region_begin('Fock')
       call os_fock_ov5(fjx,fkx,self%d,nao,&
            integral%rsrs,integral%tables,&
            atoms,integral%fmt,integral%options%thresh_eri,myrank,nproc)
       call pl_prof_region_end('Fock')
       !---
       fw(:,:) = fjx(:,:)*2.0d0 - fkx(:,:)*0.5d0
       deallocate(fjx, fkx)
       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('m:Fock_reduce')
       call mpi_reduce(&
            fw, f, nao*nao, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
       if (istat /= 0) return
       if (myrank == 0) then
          if (ldebug > 0) call ummtw('2J-K', f, nao, nao, nao, 10, 6, istat)
       end if
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('m:Fock_reduce')
       !---
       method_zd = integral%options%zerodipole
       if (myrank == 0) then
          total_energy_prev = total_energy
          e_one = ddot(nao*nao, self%d, 1, integral%h, 1)
          e_two = ddot(nao*nao, self%d, 1, f, 1) * 0.5d0
          if (ptcharges%num > 0 .and.&
               (method_zd == 1 .or. method_zd == 2 .or. method_zd == 3)) then
             e_zd1 = ddot(nao*nao, self%d, 1, integral%h_zd1, 1)
             e_zd2 = ddot(nao*nao, self%d, 1, integral%h_zd2, 1)
             e_zd3 = ddot(nao*nao, self%d, 1, integral%h_zd3, 1)
             e_zd4 = ddot(nao*nao, self%d, 1, integral%h_zd4, 1)
          else
             e_zd1 = 0.0d0
             e_zd2 = 0.0d0
             e_zd3 = 0.0d0
             e_zd4 = 0.0d0
          end if
          e_ele = e_one + e_two
          total_energy = e_ele + e_rep
          if (ldebug > 0) then
             write(6,'(1x,a,f20.10)') 'e_one  =', e_one
             write(6,'(1x,a,f20.10)') 'e_two  =', e_two
             write(6,'(1x,a,f20.10)') 'e_ele  =', e_ele
             write(6,'(1x,a,f20.10)') 'e_rep  =', e_rep
             write(6,'(1x,a,f20.10)') 'e_zd1  =', e_zd1
             write(6,'(1x,a,f20.10)') 'e_zd2  =', e_zd2
             write(6,'(1x,a,f20.10)') 'e_zd3  =', e_zd3
             write(6,'(1x,a,f20.10)') 'e_zd4  =', e_zd4
             write(6,'(1x,a,f20.10)') 'e_total=', total_energy
          end if
       endif

       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('s:DIIS')
       if (myrank == 0) then
          f(:,:) = f(1:nao,1:nao) + integral%h(1:nao,1:nao)
          if (ldebug > 0) then
             call ummtw('F', f, nao, nao, nao, 10, 6, istat)
          endif

          if (shift_val > 0.0d0) then
             !write(6,*) '...Level shifting'
             allocate(wm1(nao,nao))
             call density_matrix(wm1, self%c(1,nclosed+1),&
                  nao, nmo-nclosed, ldebug, istat)
             if (istat /= 0) return
             call dscal(nao*nao, shift_val, wm1, 1)
             f(:,:) = f(1:nao,1:nao) + wm1(1:nao,1:nao)
             deallocate(wm1)
          end if

          idiis_curpos = idiis_curpos + 1
          if ( idiis_curpos  > mxdiis ) idiis_curpos = 1
          if ( idiis_cursize < mxdiis ) idiis_cursize = idiis_cursize + 1
          call diis(&
               f_new, f, diis_vec, diis_err, diis_b, self%d, integral%s, t,&
               nao, mxdiis, idiis_curpos, idiis_cursize, ldebug, istat)
          if (istat /= 0) return
       end if
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('s:DIIS')

       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('s:newMO')

#ifdef PL_SCALAPACK
       allocate(wm1(nao,nao))
#else
       if (myrank == 0) then
          allocate(wm1(nao,nao))
       endif
#endif
       if (myrank == 0) then
          ! W = T^transpose * F * T
          call similar_trans(wm1, f_new, t, nao, ldebug, istat)
       end if
       call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
       if (istat /= 0) return

#ifdef PL_SCALAPACK
       ! wm1:local
       allocate(oen(nao), stat=istat)
       if (istat /= 0) return
       call peigen(wm1, oen, nao,&
            icomm, myrank, nproc, ldebug, istat)
       if (istat /= 0) return
       deallocate(oen, stat=istat)
       if (istat /= 0) return
#else
       if (myrank == 0) then
          call eigen(wm1, self%oe, nao, istat)
       end if
       call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
       if (istat /= 0) return
#endif
       if (myrank == 0) then
          if (ldebug > 0) then
             call ummtw2('U', wm1, self%oe, nao, nao, nao, 10, 6, istat)
          end if

          ! C = T * U
          call dgemm('n', 'n',&
               nao, nao, nao, 1.0d0, t, nao, wm1, nao, 0.0d0, self%c, nao )
          if (ldebug > 0) then
             call ummtw2('C', self%c, self%oe, nao, nao, nmo, 10, 6, istat)
          end if
       end if

#ifdef PL_SCALAPACK
       deallocate(wm1)
#else
       if (myrank == 0) then
          deallocate(wm1)
       end if
#endif
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('s:newMO')

       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('m:newMO_bcast')
       call mpi_bcast(self%c, nao*nclosed, MPI_DOUBLE_PRECISION,&
            0, icomm, istat)
       if (istat /= 0) return
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('m:newMO_bcast')

       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('s:Den')
       call density_matrix(d_new, self%c, nao, nclosed, ldebug, istat)
       if (istat /= 0) return
       call dscal(nao*nao, 2.0d0, d_new, 1)
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('s:Den')

       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('s:Conv')
       if (myrank == 0) then
          call convergence(conv_d_max, conv_d_rms, self%d, d_new, nao)
          conv_e = total_energy - total_energy_prev

          if (lprint >= 0) then
             write(6,'(1x,i4,f20.10,f20.10,d14.3)')&
                  niter, total_energy, conv_e, conv_d_rms
          end if

          if (dabs(conv_e) < conv_thresh_e .and.&
               conv_d_rms < conv_thresh_d) is_conv = .true.
       end if

       call mpi_bcast(is_conv, 1, MPI_LOGICAL, 0, icomm, istat)
       if (istat /= 0) return
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('s:Conv')

    end do

    if (myrank == 0) then
       if (lprint >= 0) then
          if (is_conv) then
             write(6,'(50x,''Converged'')')
          else
             write(6,'(46x,''Not converged'')')
          endif
          write(6,'(1x,''Electronic energy ='',f20.10)') e_ele
          write(6,'(1x,''Total energy      ='',f20.10)') total_energy

!!$          write(6,'(1x,a,f20.10)') 'e_one  =', e_one
!!$          write(6,'(1x,a,f20.10)') 'e_two  =', e_two
!!$          write(6,'(1x,a,f20.10)') 'e_ele  =', e_ele
!!$          write(6,'(1x,a,f20.10)') 'e_rep  =', e_rep
!!$          write(6,'(1x,a,f20.10)') 'e_zd1  =', e_zd1
!!$          write(6,'(1x,a,f20.10)') 'e_zd2  =', e_zd2
!!$          write(6,'(1x,a,f20.10)') 'e_zd3  =', e_zd3
!!$          write(6,'(1x,a,f20.10)') 'e_zd4  =', e_zd4
!!$          write(6,'(1x,a,f20.10)') 'e_total=', total_energy
       end if
    end if

    call mpi_bcast(total_energy, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    e_total = total_energy

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('s:Lag')
    if (myrank == 0) then
       call lagrangian_matrix(self%eij, f, self%d, nao, ldebug, istat)
       if (istat /= 0) return

       call dscal(nao*nao, -0.5d0, self%eij, 1)
       !                     0.5
    endif
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('s:Lag')

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('m:Lag_bcast')
    call mpi_bcast(self%eij, nao*nao, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('m:Lag_bcast')

    if (myrank == 0) then
       deallocate(t, f, f_new, diis_vec, diis_err, diis_b, stat=istat)
    else
       deallocate(f, stat=istat)
    end if
    deallocate(d_new, fw, stat=istat)
    if (ptcharges%num > 0 .and.&
         (method_zd == 1 .or. method_zd == 2 .or. method_zd == 3)) then
       deallocate(integral%h_zd1,integral%h_zd2,&
            integral%h_zd3,integral%h_zd4)
    end if

    if (myrank == 0) then
       if (lprint >= 10) then
          call ummtw2('MO Coefficients', self%c, self%oe, nao,nao,nmo, 8,6,istat)
       end if
    end if

!!$    if (myrank == 0) then
!!$      if (lprint >= 3) then
!!$        allocate(pop(natom), stat=istat)
!!$        if (istat /= 0) return
!!$        call population_mulliken(&
!!$             pop, integral%s, self%d, atoms, natom, nao, lprint, ldebug, istat)
!!$        if (istat /= 0) return
!!$        deallocate(pop, stat=istat)
!!$        if (istat /= 0) return
!!$      end if
!!$    end if

  end subroutine solve

!!$  !-----------------------------------------------------------------------------
!!$  !> エネルギー計算
!!$  subroutine solve_dft(&
!!$       self, options, dft_data, dft_options,&
!!$       e_total, e_rep, atoms, ptcharges, gscharges, fswolf, integral,&
!!$       pl_mpi, lprint, ldebug, istat)
!!$    use dft_options_class, only: dft_options_type => options_type
!!$    use dft_class, only: dft_data_type, dft_get_vxc_rdft
!!$    use atoms_class,     only: atoms_type
!!$    use ptcharges_class, only: ptcharges_type
!!$    use gscharges_class, only: gscharges_type
!!$    use fswolf_class, only: fswolf_type
!!$    use integral_class, only: integral_type
!!$    use integral_class, only: integral_get_fock_2j_k_rdft
!!$    use integral_class, only: integral_get_fock_2j_rdft
!!$    use integral_class, only: integral_get_fock_2j_k_rdft_direct
!!$    use integral_class, only: integral_get_fock_2j_rdft_direct
!!$    use pl_mpi_class
!!$    use pl_prof_class
!!$    type(data_type),intent(inout)::self
!!$    type(options_type),intent(in)::options
!!$    type(dft_data_type),intent(in)::dft_data
!!$    type(dft_options_type),intent(in)::dft_options
!!$    real(8),intent(inout)::e_total
!!$    real(8),intent(in)::e_rep
!!$    type(atoms_type),intent(in)::atoms
!!$    type(ptcharges_type),intent(in)::ptcharges
!!$    type(gscharges_type),intent(in)::gscharges
!!$    type(fswolf_type),intent(in)::fswolf
!!$    type(integral_type),intent(in)::integral
!!$    type(pl_mpi_type),intent(in)::pl_mpi
!!$    integer,intent(in)::lprint
!!$    integer,intent(in)::ldebug
!!$    integer,intent(inout)::istat
!!$
!!$    real(8),allocatable::t(:,:)
!!$    real(8),allocatable::d_new(:,:)
!!$    real(8),allocatable::f(:,:)
!!$    real(8),allocatable::fx(:,:),fxw(:,:),fjx(:,:),fkx(:,:)
!!$    real(8),allocatable::f_new(:,:)
!!$    real(8),allocatable::fw(:,:)
!!$    real(8),allocatable::diis_vec(:,:)
!!$    real(8),allocatable::diis_err(:,:)
!!$    real(8),allocatable::diis_b(:,:)
!!$    real(8),allocatable::wm1(:,:)
!!$    real(8),allocatable::pop(:)
!!$    integer::nao, nmo, nclosed
!!$    integer::mxdiis, mxiter, niter
!!$    real(8)::conv_thresh_d, conv_thresh_e
!!$
!!$    logical::is_conv
!!$    integer::idiis_curpos, idiis_cursize
!!$    real(8)::total_energy, total_energy_prev, e_one, e_two, e_twow, e_ele
!!$    real(8)::conv_d_max, conv_d_rms, conv_e, shift_val
!!$    real(8)::ddot
!!$
!!$    integer::icomm, myrank, nproc
!!$
!!$    real(8),allocatable::oen(:)
!!$    real(8),allocatable::vxc(:)
!!$    real(8),allocatable::vxc_(:)
!!$    real(8)::scaleh, exc, excw, totpn, totpnw
!!$    integer::natom, i, j, ij, iao, jao
!!$
!!$    istat = 0
!!$
!!$    icomm  = pl_mpi%comm
!!$    myrank = pl_mpi%myrank
!!$    nproc  = pl_mpi%nprocs
!!$
!!$    nao = self%nao
!!$    nmo = nao
!!$    nclosed = options%nclosed
!!$    self%nclosed = nclosed
!!$    natom  = atoms%num
!!$
!!$    if (myrank == 0) then
!!$       if (lprint >= 0) then
!!$          write(6,'(1x)')
!!$          write(6,'(''RDFT start'')')
!!$          call print_data(self)
!!$          write(6,'(1x)')
!!$          write(6,'(2x,''Iter       '',&
!!$               &''Total Energy         Energy Conv   Density Conv'',&
!!$               &''         Exc'')')
!!$       end if
!!$    end if
!!$
!!$    mxiter        = options%mxiter
!!$    mxdiis        = min(options%mxdiis, mxiter)
!!$    conv_thresh_d = options%conv_thresh_d
!!$    conv_thresh_e = options%conv_thresh_e
!!$    shift_val     = options%shift_val
!!$
!!$    scaleh = dft_options%scaleh
!!$
!!$    if (myrank == 0) then
!!$       allocate(&
!!$            t(nao,nao),&
!!$            f(nao,nao), f_new(nao,nao),&
!!$            diis_vec(nao*(nao+1)/2,mxdiis),&
!!$            diis_err(nao*(nao+1)/2,mxdiis),&
!!$            diis_b(mxdiis,mxdiis),&
!!$            stat=istat)
!!$       if (istat /= 0) return
!!$    else
!!$       !allocate(f(1,1), stat=istat)
!!$       allocate(f(nao,nao), stat=istat)
!!$       if (istat /= 0) return
!!$    endif
!!$
!!$    allocate(&
!!$         d_new(nao,nao),&
!!$         fw(nao,nao),&
!!$         stat=istat)
!!$    if (istat /= 0) return
!!$
!!$    if (myrank == 0) then
!!$       !call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('s:Chlsk')
!!$       call rhf_t_matrix(t, integral%s, nao, ldebug, istat)
!!$       if (istat /= 0) return
!!$       if (ldebug > 0) then
!!$          call ummtw('T', t, nao, nao, nao, 10, 6, istat)
!!$       end if
!!$       !call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('s:Chlsk')
!!$    end if
!!$
!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_begin('s:Den')
!!$    call density_matrix(self%d, self%c, nao, nclosed, ldebug, istat)
!!$    if (istat /= 0) return
!!$    call dscal(nao*nao, 2.0d0, self%d, 1)
!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_end('s:Den')
!!$
!!$    !allocate(vxc(nao*(nao+1)/2), stat=istat)
!!$    allocate(vxc(nao*(nao+1)/2+1), stat=istat)
!!$
!!$    niter = 0
!!$    is_conv = .false.
!!$    idiis_curpos  = 0
!!$    idiis_cursize = 0
!!$    total_energy = 0.0d0
!!$    total_energy_prev = 0.0d0
!!$
!!$    !is_not_conv = .true.
!!$    !do while (is_not_conv)
!!$    do while (.not. is_conv)
!!$       if (niter + 1 > mxiter) exit
!!$       niter = niter + 1
!!$       if (ldebug > 0) write(6,*) 'niter=',niter
!!$
!!$       if (niter > 1) self%d(:,:) = d_new(1:nao,1:nao)
!!$
!!$       !---checking--->
!!$       allocate(fjx(nao,nao), fkx(nao,nao), stat=istat)
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('Fock')
!!$       call os_fock_ov5(fjx,fkx,self%d,nao,&
!!$            integral%rsrs,integral%tables,&
!!$            atoms,integral%fmt,integral%options%thresh_eri,myrank,nproc)
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('Fock')
!!$       fw(:,:) = fjx(:,:)*2.0d0 - fkx(:,:)*0.5d0*scaleh
!!$       deallocate(fjx,fkx)
!!$
!!$       !call mpi_barrier(icomm, istat)
!!$       !call pl_prof_region_begin('s:HF_2e')
!!$       e_twow = ddot(nao*nao, self%d, 1, fw, 1) * 0.5d0
!!$       !call mpi_barrier(icomm, istat)
!!$       !call pl_prof_region_end('s:HF_2e')
!!$       call mpi_reduce(&
!!$            e_twow, e_two, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$
!!$       call mpi_barrier(icomm, istat)
!!$!$omp parallel
!!$       call pl_prof_region_begin('p:Vxc')
!!$!$omp end parallel
!!$       call dft_get_vxc_rdft(&
!!$            dft_data, dft_options, vxc, excw, totpnw,&
!!$            atoms%xyz, self%c, natom, nao, nclosed, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$!$omp parallel
!!$       call pl_prof_region_end('p:Vxc')
!!$!$omp end parallel
!!$       if (ldebug > 0) then
!!$           allocate(vxc_(nao*(nao+1)/2+1), stat=istat)
!!$           call mpi_reduce(&
!!$              vxc,vxc_,nao*(nao+1)/2+1,MPI_DOUBLE_PRECISION,MPI_SUM,0,icomm,istat)
!!$           if (myrank == 0) then
!!$              call umtrw('Vxc', vxc_, nao*(nao+1)/2, nao, 10, 6, istat)
!!$           end if
!!$           deallocate(vxc_)
!!$       end if
!!$
!!$       call mpi_reduce(&
!!$            excw, exc, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$
!!$       if (ldebug > 0) then
!!$          call mpi_reduce(&
!!$               totpnw, totpn, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
!!$          if ( myrank == 0 ) then
!!$             write(6,'(''exc    ='',f20.10)') exc
!!$             write(6,'(''totpn  ='',f20.10)') totpn
!!$          end if
!!$       end if
!!$
!!$       !call mpi_barrier(icomm, istat)
!!$       !call pl_prof_region_begin('s:Fock+Vxc')
!!$       ij = 0
!!$       do i = 1, nao
!!$          do j = 1, i
!!$             ij = ij + 1
!!$             fw(i,j) = fw(i,j) + vxc(ij)
!!$             fw(j,i) = fw(i,j)
!!$          end do
!!$       end do
!!$       !call mpi_barrier(icomm, istat)
!!$       !call pl_prof_region_end('s:Fock+Vxc')
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('m:Fock_reduce')
!!$       call mpi_reduce(&
!!$            fw, f, nao*nao, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('m:Fock_reduce')
!!$       if (myrank == 0) then
!!$          if (ldebug > 0) call ummtw('2J-K', f, nao, nao, nao, 10, 6, istat)
!!$       end if
!!$       !call ummtw('2J-K', f, nao, nao, nao, 10, 6, istat)
!!$
!!$       if (myrank == 0) then
!!$          total_energy_prev = total_energy
!!$          e_one = ddot(nao*nao, self%d, 1, integral%h, 1)
!!$          !e_two = ddot(nao*nao, self%d, 1, f, 1) * 0.5d0
!!$          e_ele = e_one + e_two + exc
!!$          total_energy = e_ele + e_rep
!!$          if (ldebug > 0) then
!!$             write(6,*) 'e_one  =', e_one
!!$             write(6,*) 'e_two  =', e_two
!!$             write(6,*) 'e_ele  =', e_ele
!!$             write(6,*) 'e_rep  =', e_rep
!!$             write(6,*) 'e_exc  =', exc
!!$             write(6,*) 'e_total=', total_energy
!!$          end if
!!$       end if
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('s:DIIS')
!!$       if (myrank == 0) then
!!$          f(:,:) = f(1:nao,1:nao) + integral%h(1:nao,1:nao)
!!$          if (ldebug > 0) then
!!$             call ummtw('F', f, nao, nao, nao, 10, 6, istat)
!!$          endif
!!$
!!$          if (shift_val > 0.0d0) then
!!$             !write(6,*) '...Level shifting'
!!$             allocate(wm1(nao,nao))
!!$             call density_matrix(wm1, self%c(1,nclosed+1),&
!!$                  nao, nmo-nclosed, ldebug, istat)
!!$             if (istat /= 0) return
!!$             call dscal(nao*nao, shift_val, wm1, 1)
!!$             f(:,:) = f(1:nao,1:nao) + wm1(1:nao,1:nao)
!!$             deallocate(wm1)
!!$          end if
!!$
!!$          idiis_curpos = idiis_curpos + 1
!!$          if ( idiis_curpos  > mxdiis ) idiis_curpos = 1
!!$          if ( idiis_cursize < mxdiis ) idiis_cursize = idiis_cursize + 1
!!$          call diis(&
!!$               f_new, f, diis_vec, diis_err, diis_b, self%d, integral%s, t,&
!!$               nao, mxdiis, idiis_curpos, idiis_cursize, ldebug, istat)
!!$          if (istat /= 0) return
!!$       end if
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('s:DIIS')
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('s:newMO')
!!$
!!$#ifdef PL_SCALAPACK
!!$       allocate(wm1(nao,nao))
!!$#else
!!$       if (myrank == 0) then
!!$          allocate(wm1(nao,nao))
!!$       endif
!!$#endif
!!$       if (myrank == 0) then
!!$          ! W = T^transpose * F * T
!!$          call similar_trans(wm1, f_new, t, nao, ldebug, istat)
!!$          if (istat /= 0) return
!!$       endif
!!$
!!$#ifdef PL_SCALAPCK
!!$       ! wm1:local
!!$       allocate(oen(nao), stat=istat)
!!$       if (istat /= 0) return
!!$       call peigen(wm1, oen, nao,&
!!$            icomm, nproc, myrank, ldebug, istat)
!!$       if (istat /= 0) return
!!$       deallocate(oen, stat=istat)
!!$       if (istat /= 0) return
!!$#else
!!$       if (myrank == 0) then
!!$          call eigen(wm1, self%oe, nao, istat)
!!$          if (istat /= 0) return
!!$       end if
!!$#endif
!!$       if (myrank == 0) then
!!$          if (ldebug > 0) then
!!$             call ummtw2('U', wm1, self%oe, nao, nao, nao, 10, 6, istat)
!!$          end if
!!$
!!$          ! C = T * U
!!$          call dgemm('n', 'n',&
!!$               nao, nao, nao, 1.0d0, t, nao, wm1, nao, 0.0d0, self%c, nao )
!!$          if (ldebug > 0) then
!!$             call ummtw2('C', self%c, self%oe, nao, nao, nmo, 10, 6, istat)
!!$          end if
!!$       end if
!!$
!!$#ifdef PL_SCALAPACK
!!$       deallocate(wm1)
!!$#else
!!$       if (myrank == 0) then
!!$          deallocate(wm1)
!!$       end if
!!$#endif
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('s:newMO')
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('m:newMO_bcast')
!!$       call mpi_bcast(self%c, nao*nclosed, MPI_DOUBLE_PRECISION,&
!!$            0, icomm, istat)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('m:newMO_bcast')
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('s:Den')
!!$       call density_matrix(d_new, self%c, nao, nclosed, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call dscal(nao*nao, 2.0d0, d_new, 1)
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('s:Den')
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('s:Conv')
!!$       if (myrank == 0) then
!!$          call convergence(conv_d_max, conv_d_rms, self%d, d_new, nao)
!!$          conv_e = total_energy - total_energy_prev
!!$
!!$          if (lprint >= 0) then
!!$             write(6,'(1x,i4,f20.10,f20.10,d14.3,f17.10)')&
!!$                  niter, total_energy, conv_e, conv_d_rms, exc
!!$          end if
!!$
!!$          if (dabs(conv_e) < conv_thresh_e .and.&
!!$               conv_d_rms < conv_thresh_d) is_conv = .true.
!!$       end if
!!$
!!$       call mpi_bcast(is_conv, 1, MPI_LOGICAL, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('s:Conv')
!!$
!!$    end do
!!$
!!$    if (myrank == 0) then
!!$       if (lprint >= 0) then
!!$          if (is_conv) then
!!$             write(6,'(50x,''Converged'')')
!!$          else
!!$             write(6,'(46x,''Not converged'')')
!!$          endif
!!$          write(6,'(1x,''Electronic energy ='',f20.10)') e_ele
!!$          write(6,'(1x,''Total energy      ='',f20.10)') total_energy
!!$       end if
!!$    end if
!!$
!!$    call mpi_bcast(total_energy, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$    if (istat /= 0) return
!!$    e_total = total_energy
!!$
!!$    ! for Pa of old rdftgrd
!!$    !call density_matrix(self%d, self%c, nao, nclosed, ldebug, istat)
!!$    !call dscal(nao*nao, 2.0d0, self%d, 1)
!!$
!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_begin('s:Lag')
!!$    if (myrank == 0) then
!!$       call lagrangian_matrix(self%eij, f, self%d, nao, ldebug, istat)
!!$       if (istat /= 0) return
!!$
!!$       call dscal(nao*nao, -0.5d0, self%eij, 1)
!!$       !                     0.5
!!$    endif
!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_end('s:Lag')
!!$
!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_begin('m:Lag_bcast')
!!$    call mpi_bcast(self%eij, nao*nao, MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$    if (istat /= 0) return
!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_end('m:Lag_bcast')
!!$
!!$    !call ummtw('Density Matrix', self%d, nao,nao,nao, 8,6,istat)
!!$    !call ummtw('Lagrangian Matrix', self%eij, nao,nao,nao, 8,6,istat)
!!$
!!$    if (myrank == 0) then
!!$       deallocate(t, f, f_new, diis_vec, diis_err, diis_b, stat=istat)
!!$    else
!!$       deallocate(f, stat=istat)
!!$    end if
!!$    deallocate(vxc, stat=istat)
!!$    deallocate(d_new, fw, stat=istat)
!!$
!!$    if (myrank == 0) then
!!$       if (lprint >= 10) then
!!$          call ummtw2('MO Coefficients', self%c, self%oe, nao,nao,nmo, 8,6,istat)
!!$       end if
!!$    end if
!!$
!!$  end subroutine solve_dft

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_data(self)
    type(data_type),intent(in)::self

    write(6,'(1x)')
    write(6,'('' Number of AOs        = '',i5)') self%nao
    write(6,'('' Number of MOs        = '',i5)') self%nmo
    write(6,'('' Number of closed MOs = '',i5)') self%nclosed

  end subroutine print_data

  !-----------------------------------------------------------------------------
  !> データの読み込み
  subroutine read_data(self, istat)
    type(data_type),intent(inout)::self
    integer,intent(inout)::istat

    istat = 0

  end subroutine read_data

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_data(self, istat)
    use pl_bms_class
    type(data_type),intent(in)::self
    integer,intent(inout)::istat

    call pl_bms_write(self%nao, url, 'nao', id, no_unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%nmo, url, 'nmo', id, no_unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%nclosed, url, 'nclosed', id, no_unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%oe, self%nmo,&
         url, 'oe', id, no_unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%c, self%nao, self%nao, self%nmo,&
         url,'c', id, no_unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%d, self%nao, self%nao, self%nao,&
         url,'d', id, no_unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%eij, self%nao, self%nao, self%nao,&
         url,'lag', id, no_unit, istat)
    if (istat /= 0) return

  end subroutine save_data

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_data(self, ldebug, istat)
    use pl_bms_class
    type(data_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nao, nmo, nclosed

    call pl_bms_read(nao, url, 'nao', id, istat)
    if (istat /= 0) return

    call pl_bms_read(nmo, url, 'nmo', id, istat)
    if (istat /= 0) return

    call pl_bms_read(nclosed, url, 'nclosed', id, istat)
    if (istat /= 0) return

    call new_data(self, nao, ldebug, istat)
    if (istat /= 0) return
    self%nclosed = nclosed

    call pl_bms_read(self%oe, nmo, url, 'oe', id, istat)
    if (istat /= 0) return

    call pl_bms_read(self%c, nao, nao, nmo, url,'c', id, istat)
    if (istat /= 0) return

    call pl_bms_read(self%d, nao, nao, nao, url,'d', id, istat)
    if (istat /= 0) return

    call pl_bms_read(self%eij, nao, nao, nao, url,'lag', id, istat)
    if (istat /= 0) return

  end subroutine load_data

  !-----------------------------------------------------------------------------
  !> データBcast
  subroutine bcast_data(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(data_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nao, nmo, nclosed
    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    if (myrank == 0) then
       nao = self%nao
       nmo = self%nmo
       nclosed = self%nclosed
    end if
    call mpi_bcast(nao, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(nmo, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(nclosed, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return

    if (myrank /= 0) then
       call new_data(self, nao, ldebug, istat)
       if (istat /= 0) return
       self%nclosed = nclosed
    end if

    call mpi_bcast(self%c, nao*nmo, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return

    call mpi_bcast(self%d, nao*nao, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return

    call mpi_bcast(self%eij, nao*nao, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return

  end subroutine bcast_data
  
  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_data(self, ldebug, istat)
    type(data_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    deallocate(self%c, self%oe, self%d, self%eij, stat=istat)
    if (istat /= 0) return

  end subroutine free_data

  !-----------------------------------------------------------------------------
  !> set Dmax
  subroutine set_dmax(dmax, d, tables, nshellpair, nao)
    use integral_tables_class
    type(tables_type),intent(in)::tables
    integer,intent(in)::nshellpair, nao
    real(8),intent(inout)::dmax(nshellpair)
    real(8),intent(in)::d(nao,nao)

    real(8)::dmax_
    integer::ij, i,j, ir, is, ix, iy, ir_, is_

    dmax(:) = 0.0d0

    ! ss
    do ij = 1, tables%ishp(1)
       i = tables%npsh(1,ij)
       j = tables%npsh(2,ij)
       ir = tables%nsh2ao(i)
       is = tables%nsh2ao(j)
       dmax(ij) = dabs(d(ir,is))
    end do

    ! ps
    do ij = tables%ishp(1)+1, tables%ishp(2)
       i = tables%npsh(1,ij)
       j = tables%npsh(2,ij)
       ir = tables%nsh2ao(i)
       is = tables%nsh2ao(j)
       dmax_ = 0.0d0
       do ix = 1, 3
          ir_ = ir + ix - 1
          dmax_ = dmax1(dmax_, dabs(d(ir_,is)))
       end do
       dmax(ij) = dmax_
    end do

    ! pp
    do ij = tables%ishp(2)+1, tables%ishp(3)
       i = tables%npsh(1,ij)
       j = tables%npsh(2,ij)
       ir = tables%nsh2ao(i)
       is = tables%nsh2ao(j)
       dmax_ = 0.0d0
       do ix = 1, 3
          ir_ = ir + ix - 1
          do iy = 1, 3
             is_ = is + iy - 1
             dmax_ = dmax1(dmax_, dabs(d(ir_,is_)))
          end do
       end do
       dmax(ij) = dmax_
    end do

    ! ds
    do ij = tables%ishp(3)+1, tables%ishp(4)
       i = tables%npsh(1,ij)
       j = tables%npsh(2,ij)
       ir = tables%nsh2ao(i)
       is = tables%nsh2ao(j)
       dmax_ = 0.0d0
       do ix = 1, 6
          ir_ = ir + ix - 1
          dmax_ = dmax1(dmax_, dabs(d(ir_,is)))
       end do
       dmax(ij) = dmax_
    end do

    ! dp
    do ij = tables%ishp(4)+1, tables%ishp(5)
       i = tables%npsh(1,ij)
       j = tables%npsh(2,ij)
       ir = tables%nsh2ao(i)
       is = tables%nsh2ao(j)
       dmax_ = 0.0d0
       do ix = 1, 6
          ir_ = ir + ix - 1
          do iy = 1, 3
             is_ = is + iy - 1
             dmax_ = dmax1(dmax_, dabs(d(ir_,is_)))
          end do
       end do
       dmax(ij) = dmax_
    end do

    ! dd
    do ij = tables%ishp(5)+1, tables%ishp(6)
       i = tables%npsh(1,ij)
       j = tables%npsh(2,ij)
       ir = tables%nsh2ao(i)
       is = tables%nsh2ao(j)
       dmax_ = 0.0d0
       do ix = 1, 6
          ir_ = ir + ix - 1
          do iy = 1, 6
             is_ = is + iy - 1
             dmax_ = dmax1(dmax_, dabs(d(ir_,is_)))
          end do
       end do
       dmax(ij) = dmax_
    end do

  end subroutine set_dmax

  !-----------------------------------------------------------------------------
end module energy_rhf_class
