! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> RHFオプションクラス
!! 
module energy_rhf_options_class
  implicit none
  private

  public::options_type
  public::read_options
  public::save_options
  public::load_options
  public::load_nclosed
  public::load_nfzcore
  public::bcast_options
  public::free_options
  public::print_options

  type::options_type
     integer::mxiter
     integer::mxdiis
     real(8)::conv_thresh_d
     real(8)::conv_thresh_e
     real(8)::shift_val
     !real(8)::shift_thresh
     integer::nmo     !< number of MOs
     integer::nele    !< number of electrons
     integer::nclosed !< number of closed MOs
     integer::nfzcore !< number of frozen core MOs
     integer::nfzvir  !< number of frozen virtual MOs
     integer::nion
  end type

  character(len=*),parameter::group_name = 'RHF' !< データグループ名称

  integer,parameter::MAX_OPTION_NAME_LEN = 16
  integer,parameter::nopt = 7                               !< オプション数
  character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt) = (/& !< オプションの名称
       'ION       ',&
       'FROZENCORE',&
       'ENERGYCONV',&
       'SCFCONV   ',&
       'DIISSIZE  ',&
       'MAXCYC    ',&
       'SHIFTVALUE'/)

  character(len=*),parameter::url = 'options_rhf' !< BMSファイル名称
  integer,parameter::id = 1
  character(len=*),parameter::no_unit = ' '

contains
  !-----------------------------------------------------------------------------
  !> オプションの表示
  subroutine print_options(self)
    type(options_type),intent(in)::self

    write(6,'(/a,'' Options'')') trim(group_name)
    write(6,'(2x,a,'' : '',i6)')       'ION       ',self%nion
    !write(6,'(2x,a,'' : '',i6)')       'FROZENCORE',self%nfzcore
    write(6,'(2x,a,'' : '',i6)')       'MAXCYC    ',self%mxiter
    write(6,'(2x,a,'' : '',1p,d10.3)') 'SCFCONV   ',self%conv_thresh_d
    write(6,'(2x,a,'' : '',1p,d10.3)') 'ENERGYCONV',self%conv_thresh_e
    write(6,'(2x,a,'' : '',i6)')       'DIISSIZE  ',self%mxdiis
    write(6,'(2x,a,'' : '',1p,d10.3)') 'SHIFTVALUE',self%shift_val
    !write(6,'('' Maximum number of SCF iterations         = '',i5)')&
    !     self%mxiter
    !write(6,'('' Convergence threshold for density matrix ='',1p,d10.3)')&
    !     self%conv_thresh_d
    !write(6,'('' Convergence threshold for energy         ='',1p,d10.3)')&
    !     self%conv_thresh_e
    !write(6,'('' DIIS size                                = '',i5)')&
    !     self%mxdiis
    !write(6,'('' Level shifting value                     ='',1p,d10.3)')&
    !     self%shift_val
    !write(6,'('' Level shifting threshold                 ='',1p,d10.3)')&
    !     self%shift_thresh
    !write(6,'('' DMREAD                                   = '',i5)')&
    !     self%dmread

    write(6,'(1x)')
    write(6,'(2x,''Molecular charge     ='',i5)') self%nion
    write(6,'(2x,''Number of electrons  ='',i5)') self%nele
    write(6,'(2x,''Number of MOs        ='',i5)') self%nmo
    write(6,'(2x,''Number of closed MOs ='',i5)') self%nclosed

  end subroutine print_options

  !-----------------------------------------------------------------------------
  !> オプションの入力ファイルからの読み込み
  subroutine read_options(self, lines, nline, nmo, nele, ldebug, istat)
    use input_class
    type(options_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::nmo
    integer,intent(in)::nele
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    integer::opt_count(nopt)                             !< オプションの出現回数
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::iopt, ival, ipos, ival_default, ipos_default
    real(8)::dval, dval_default
    character(len=MAX_OPTION_VALUE_LEN)::str, str_default

    integer::nerror, ierror

    call get_options(&
       lines, nline,&
       group_name, opt_names, opt_values, opt_count, nopt, ldebug, istat)
    if (istat /= 0) return

    nerror = 0
    ierror = 0

    opt_name = 'energyconv'
    dval_default = 1.d-10
    call get_option_values_dbl(&
         opt_name, opt_names, opt_values, nopt, dval_default, dval, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (dval < 0.0d0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%conv_thresh_e = dval

    opt_name = 'scfconv'
    dval_default = 1.d-04
    call get_option_values_dbl(&
         opt_name, opt_names, opt_values, nopt, dval_default, dval, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (dval < 0.0d0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%conv_thresh_d = dval

    opt_name = 'diissize'
    ival_default = 8
    call get_option_values_int(&
         opt_name, opt_names, opt_values, nopt, ival_default, ival, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (ival < 0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%mxdiis = ival

    opt_name = 'maxcyc'
    ival_default = 100
    call get_option_values_int(&
         opt_name, opt_names, opt_values, nopt, ival_default, ival, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (ival < 0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%mxiter = ival

    opt_name = 'ion'
    ival_default = 0
    call get_option_values_int(&
         opt_name, opt_names, opt_values, nopt, ival_default, ival, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%nion = ival

    opt_name = 'frozencore'
    ival_default = 0
    call get_option_values_int(&
         opt_name, opt_names, opt_values, nopt, ival_default, ival, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (ival < 0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%nfzcore = ival

    self%nfzvir  = 0

    opt_name = 'shiftvalue'
    dval_default = 0.0d0
    call get_option_values_dbl(&
         opt_name, opt_names, opt_values, nopt, dval_default, dval, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (dval < 0.0d0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%shift_val = dval

    if (nerror /= 0) then
       istat = ierror; return
    end if

    self%nmo = nmo
    self%nele = nele - self%nion
    self%nclosed = (nele - self%nion)/2
    if (nele - self%nion > nmo*2 .or.&
         self%nclosed*2 /= nele - self%nion .or.&
         self%nclosed < 0 .or.&
         nmo - self%nclosed < 0) then
       istat = PL_ERR_INCONSISTENT_VALUE
       call pl_error_print_message(istat)
       iopt = pl_str_search_name_ignorecase('ion', opt_names, nopt)
       call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
       write(6,'(2x,''Number of electrons  ='',i5)') nele - self%nion
       write(6,'(2x,''Number of closed MOs ='',i5)') self%nclosed
       write(6,'(2x,''Number of MOs        ='',i5)') nmo
       return
    end if

    if (self%nfzcore /= 0) then
       write(6,*) 'Error: Not implemented'
       write(6,*) 'frozencore'
       istat = PL_ERR_INCONSISTENT_VALUE
       call pl_error_print_message(istat)
       iopt = pl_str_search_name_ignorecase('frozencore', opt_names, nopt)
       call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
       return
    end if

    if (ldebug > 0) then
       write(6,'(2x,''Number of electrons  ='',i5)') nele - self%nion
       write(6,'(2x,''Number of closed MOs ='',i5)') self%nclosed
       write(6,'(2x,''Number of MOs        ='',i5)') nmo
    end if

  end subroutine read_options

  !-----------------------------------------------------------------------------
  !> オプションのBMS形式ファイルへの保存
  subroutine save_options(self, ldebug, istat)
    use pl_bms_class
    type(options_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%mxiter,  url, 'maxcyc',     id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%mxdiis,  url, 'diissize',   id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nclosed, url, 'closed',     id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nfzcore, url, 'frozencore', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nion,    url, 'ion',        id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%conv_thresh_d, url, 'scfconv',    id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%conv_thresh_e, url, 'energyconv', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%shift_val, url, 'shiftvalue', id, no_unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%nmo, url, 'nmo', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nele, url, 'nele', id, no_unit, istat)
    if (istat /= 0) return

  end subroutine save_options

  !-----------------------------------------------------------------------------
  !> オプションのBMS形式ファイルからの読み込み
  subroutine load_options(self, ldebug, istat)
    use pl_bms_class
    type(options_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_read(self%mxiter,  url, 'maxcyc',     id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%mxdiis,  url, 'diissize',   id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%nclosed, url, 'closed',     id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%nfzcore, url, 'frozencore', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%nion,    url, 'ion',        id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%conv_thresh_d, url, 'scfconv',    id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%conv_thresh_e, url, 'energyconv', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%shift_val, url, 'shiftvalue', id, istat)
    if (istat /= 0) return

    call pl_bms_read(self%nmo, url, 'nmo', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%nele, url, 'nele', id, istat)
    if (istat /= 0) return

  end subroutine load_options

  !-----------------------------------------------------------------------------
  !> 閉殻軌道数の取得
  subroutine load_nclosed(nclosed, istat)
    use pl_bms_class
    integer,intent(out)::nclosed
    integer,intent(inout)::istat

    call pl_bms_read(nclosed, url, 'closed', id, istat)
    if (istat /= 0) return

  end subroutine load_nclosed

  !-----------------------------------------------------------------------------
  !> フローズン軌道数の取得
  subroutine load_nfzcore(nfzcore, istat)
    use pl_bms_class
    integer,intent(out)::nfzcore
    integer,intent(inout)::istat

    call pl_bms_read(nfzcore, url, 'frozencore', id, istat)
    if (istat /= 0) return

  end subroutine load_nfzcore

  !-----------------------------------------------------------------------------
  !> オプションのBcast
  subroutine bcast_options(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(options_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat
    integer::icomm

    icomm = pl_mpi%comm

    call mpi_bcast(self%mxiter,  1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%mxdiis,  1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%nclosed, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%nfzcore, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%nion,    1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%conv_thresh_d, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%conv_thresh_e, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%shift_val, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return

    call mpi_bcast(self%nmo, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%nele, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return

  end subroutine bcast_options

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_options(self, ldebug, istat)
    type(options_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0
  end subroutine free_options

  !-----------------------------------------------------------------------------
end module energy_rhf_options_class
