! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 力の計算モジュール
!!
!! 
!   Force on QM atom
!     = deriv_N <Phi| Te + Vee + VNe + VNN
!                              + Vpe + VpN + Vge + VgN |Phi>
!   Force on point charge
!     = deriv_p <Phi|          + Vpe + VpN
!                                    + Vpp       + Vpg |Phi>
!   Force on gauss charge
!     = deriv_g <Phi|                      + Vge + VgN
!                                          + Vgg + Vpg |Phi>
module force_class
  implicit none
  private

  public::force_init
  public::force_new
  public::force_set_wftype
  public::force_bcast_options
  public::force_new_data
  public::force_solve
  public::force_print_data
  public::force_print_data_brief
  public::force_save_data
  public::force_free_data
  public::force_type

  type::force_type
     integer::wftype
     integer::natom
     integer::npc
     integer::ngc
     integer::nfsw
     real(8),allocatable::qm_atom(:,:)   ! dE_QM/da,   a in QM_atom
     real(8),allocatable::qmmm_atom(:,:) ! dE_QMMM/da, a in QM_atom
     real(8),allocatable::qmmm_pc(:,:)   ! dE_QMMM/dp, p in MM_pc
     real(8),allocatable::qmmm_gc(:,:)   ! dE_QMMM/dg, g in MM_gc
     real(8),allocatable::qmmm_fsw(:,:)  ! dE_QMMM/df, g in MM_fsw
     real(8),allocatable::mm_pc(:,:)     ! dE_MM/dp,   p in MM_pc
     real(8),allocatable::mm_gc(:,:)     ! dE_MM/dg,   g in MM_gc
     real(8),allocatable::mm_fsw(:,:)    ! dE_MM/df,   g in MM_fsw
     real(8),allocatable::force_atom(:,:)
     real(8),allocatable::force_pc(:,:)
     real(8),allocatable::force_gc(:,:)
     real(8),allocatable::force_fsw(:,:)
  end type

contains
  !-----------------------------------------------------------------------------
  !> 初期化
  subroutine force_init(self, ldebug, istat)
    type(force_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0
  end subroutine force_init

  !-----------------------------------------------------------------------------
  !> 波動関数タイプのセット
  subroutine force_set_wftype(self, wftype, istat)
    type(force_type),intent(inout)::self
    integer,intent(in)::wftype
    integer,intent(inout)::istat

    istat = 0

    self%wftype = wftype

  end subroutine force_set_wftype

  !-----------------------------------------------------------------------------
  !> OptionデータBcast
  subroutine force_bcast_options(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(force_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    call mpi_bcast(self%wftype, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%natom, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%npc, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%ngc, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%nfsw, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return

  end subroutine force_bcast_options

  !-----------------------------------------------------------------------------
  !> 前処理
  subroutine force_new(self, ldebug, istat)
    type(force_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

  end subroutine force_new

  !-----------------------------------------------------------------------------
  !> 配列確保
  subroutine force_new_data(self, natom, npc, ngc, nfsw, istat)
    type(force_type),intent(inout)::self
    integer,intent(in)::natom
    integer,intent(in)::npc
    integer,intent(in)::ngc
    integer,intent(in)::nfsw
    integer,intent(inout)::istat

    istat = 0

    allocate(&
         self%force_atom(3,natom),&
         self%force_pc  (3,npc),&
         self%force_gc  (3,ngc),&
         self%force_fsw (3,nfsw),&
         stat=istat)
    if (istat /= 0) return

    self%natom = natom
    self%npc   = npc
    self%ngc   = ngc
    self%nfsw  = nfsw

    if (natom > 0) self%force_atom(:,:) = 0.0d0
    if (npc   > 0) self%force_pc  (:,:) = 0.0d0
    if (ngc   > 0) self%force_gc  (:,:) = 0.0d0
    if (nfsw  > 0) self%force_fsw (:,:) = 0.0d0

  end subroutine force_new_data

  !-----------------------------------------------------------------------------
  !> 力の計算
  subroutine force_solve(&
       self, energy,&
!!$       dft_data, dft_options,&
       atoms, ptcharges, gscharges, fswolf, integral,&
       pl_mpi, lprint, ldebug, istat)
    use qm_options_class
    use force_rhf_class, only: rhf_solve => solve
!!$    use force_rhf_class, only: rhf_solve => solve, rhf_solve_dft => solve_dft
!!$    use force_uhf_class, only: uhf_solve => solve, uhf_solve_dft => solve_dft
!!$    use force_casscf_class, only: casscf_solve => solve
!!$    use force_cis_class, only: cis_solve => solve
    use energy_class
!!$    use dft_options_class, only: dft_options_type => options_type
!!$    use dft_class, only: dft_data_type
    use atoms_class, only: atoms_type
    use ptcharges_class, only: ptcharges_type
    use gscharges_class, only: gscharges_type
    use fswolf_class, only: fswolf_type
    use integral_class, only: integral_type,&
         integral_get_force_pc, integral_get_force_gc
!!$         integral_get_force_pc, integral_get_force_gc, integral_get_force_fsw
    use mo_data_class, only: mo_data_type,&
         mo_data_new => new_data,&
         mo_data_bcast => bcast_data,&
         mo_data_free => free_data
    use pl_mpi_class
    use pl_prof_class
    use physical_constants_class
    type(force_type),intent(inout)::self
    type(energy_type),intent(inout)::energy
!!$    type(dft_options_type),intent(in)::dft_options
!!$    type(dft_data_type),intent(in)::dft_data
    type(atoms_type),intent(in)::atoms
    type(ptcharges_type),intent(in)::ptcharges
    type(gscharges_type),intent(in)::gscharges
    type(fswolf_type),intent(in)::fswolf
    type(integral_type),intent(in)::integral
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::lprint
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    type(mo_data_type)::mo_data
    integer::wftype
    integer::icomm, myrank, nproc
    real(8)::time0 ,time1
    integer::i
    real(8)::rcut
    integer::method_zd
    real(8),allocatable::gw(:,:)

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank
    nproc  = pl_mpi%nprocs

    rcut      = integral%options%rcut / Bohr
    method_zd = integral%options%zerodipole
    if (ldebug > 0) then
       write(6,*) 'rcut@force_solve=',rcut
       write(6,*) 'method_zd@force_solve=',method_zd
    end if

    wftype = self%wftype
    !write(6,*) 'wftype=',wftype

    self%force_atom(:,:) = 0.0d0
    self%force_pc(:,:)   = 0.0d0
    self%force_gc(:,:)   = 0.0d0
    self%force_fsw(:,:)  = 0.0d0
    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('::dAtom')
    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_solve(&
            self%force_atom, self%natom,&
            energy%rhf_data,&
            atoms, ptcharges, gscharges, fswolf, integral,&
            pl_mpi, ldebug, istat)
       if (istat /= 0) return
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_solve(&
!!$            self%force_atom, self%natom,&
!!$            energy%uhf_data,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_solve(&
!!$            self%force_atom, self%natom,&
!!$            energy%casscf_data, energy%casscf_options,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_solve_dft(&
!!$            self%force_atom, self%natom,&
!!$            energy%rhf_data,&
!!$            dft_data, dft_options,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_solve_dft(&
!!$            self%force_atom, self%natom,&
!!$            energy%uhf_data,&
!!$            dft_data, dft_options,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_CIS)
!!$       if (myrank == 0) then
!!$          call mo_data_new(&
!!$               mo_data, energy%rhf_data%nao, energy%rhf_data%nao,&
!!$               energy%rhf_data%nmo, ldebug, istat)
!!$          if (istat /= 0) return
!!$          mo_data%c(:,:) = energy%rhf_data%c(:,:)
!!$          mo_data%oe(:)  = energy%rhf_data%oe(:)
!!$       end if
!!$       call mo_data_bcast(mo_data, pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call cis_solve(&
!!$            self%force_atom, self%natom,&
!!$            energy%cis_data, energy%cis_options, mo_data,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call mo_data_free(mo_data, ldebug, istat)
!!$       if (istat /= 0) return
    case default
       istat = 1
    end select
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('::dAtom')

!!$    if (myrank == 0) then
!!$       allocate(gw(3,atoms%natom))
!!$       call gconrg(gw,atoms%chg,atoms%xyz,atoms%natom,lprint,ldebug)
!!$       call gsum_gnr(self%force_atom,gw,atoms%natom)
!!$
!!$       call gconrg_zd(&
!!$            self%force_atom, atoms%xyz,atoms%chg, atoms%natom, rcut,&
!!$            ptcharges%xyz, ptcharges%chg, ptcharges%num, ldebug)
!!$       if (ldebug > 0) then
!!$          call ummtw('+gw_nuc-nuc', gw, 3, 3, atoms%natom, 10, 6, istat)
!!$       end if
!!$       deallocate(gw)
!!$    end if


    if (ldebug > 0) then
       write(6,*) 'force_atom 1:'
       do i = 1, self%natom
          write(6,*)&
               self%force_atom(1,i),self%force_atom(2,i),self%force_atom(3,i)
       end do
    end if

!!$    if (wftype == QM_WFTYPE_RHF .or. wftype == QM_WFTYPE_RDFT) then
    if (wftype == QM_WFTYPE_RHF) then
       call integral_get_force_pc(&
            integral,&
            self%force_pc, self%npc,&
            energy%rhf_data%d, energy%rhf_data%eij, energy%rhf_data%nao,&
            atoms, ptcharges,&
            integral%tables, integral%fmt, integral%options,&
            pl_mpi, ldebug, istat)
       if (istat /= 0) return
       call integral_get_force_gc(&
            integral,&
            self%force_gc, self%ngc,&
            energy%rhf_data%d, energy%rhf_data%eij, energy%rhf_data%nao,&
            atoms, gscharges,&
            integral%tables, integral%fmt,&
            pl_mpi, ldebug, istat)
       if (istat /= 0) return
!!$       call integral_get_force_fsw(&
!!$            integral,&
!!$            self%force_fsw, self%nfsw,&
!!$            energy%rhf_data%d, energy%rhf_data%eij, energy%rhf_data%nao,&
!!$            atoms, fswolf,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$    else if (wftype == QM_WFTYPE_UHF .or. wftype == QM_WFTYPE_UDFT) then
!!$       call integral_get_force_pc(&
!!$            integral,&
!!$            self%force_pc, self%npc,&
!!$            energy%uhf_data%d, energy%uhf_data%eij, energy%uhf_data%nao,&
!!$            atoms, ptcharges,&
!!$            integral%tables, integral%fmt, integral%options,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call integral_get_force_gc(&
!!$            integral,&
!!$            self%force_gc, self%ngc,&
!!$            energy%uhf_data%d, energy%uhf_data%eij, energy%uhf_data%nao,&
!!$            atoms, gscharges,&
!!$            integral%tables, integral%fmt,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call integral_get_force_fsw(&
!!$            integral,&
!!$            self%force_fsw, self%nfsw,&
!!$            energy%uhf_data%d, energy%uhf_data%eij, energy%uhf_data%nao,&
!!$            atoms, fswolf,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$    else if (wftype == QM_WFTYPE_CASSCF) then
!!$       call integral_get_force_pc(&
!!$            integral,&
!!$            self%force_pc, self%npc,&
!!$            energy%casscf_data%d, energy%casscf_data%eij,&
!!$            energy%casscf_data%nao,&
!!$            atoms, ptcharges,&
!!$            integral%tables, integral%fmt, integral%options,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call integral_get_force_gc(&
!!$            integral,&
!!$            self%force_gc, self%ngc,&
!!$            energy%casscf_data%d, energy%casscf_data%eij,&
!!$            energy%casscf_data%nao,&
!!$            atoms, gscharges,&
!!$            integral%tables, integral%fmt,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call integral_get_force_fsw(&
!!$            integral,&
!!$            self%force_fsw, self%nfsw,&
!!$            energy%casscf_data%d, energy%casscf_data%eij,&
!!$            energy%casscf_data%nao,&
!!$            atoms, fswolf,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$    else if (wftype == QM_WFTYPE_CIS) then
!!$       call integral_get_force_pc(&
!!$            integral,&
!!$            self%force_pc, self%npc,&
!!$            energy%cis_data%d, energy%cis_data%eij,&
!!$            energy%cis_data%nao,&
!!$            atoms, ptcharges,&
!!$            integral%tables, integral%fmt, integral%options,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call integral_get_force_gc(&
!!$            integral,&
!!$            self%force_gc, self%ngc,&
!!$            energy%cis_data%d, energy%cis_data%eij,&
!!$            energy%cis_data%nao,&
!!$            atoms, gscharges,&
!!$            integral%tables, integral%fmt,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call integral_get_force_fsw(&
!!$            integral,&
!!$            self%force_fsw, self%nfsw,&
!!$            energy%cis_data%d, energy%cis_data%eij,&
!!$            energy%cis_data%nao,&
!!$            atoms, fswolf,&
!!$            pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       deallocate(energy%cis_data%d, energy%cis_data%eij, stat=istat)
!!$       if (istat /= 0) return
    end if

!!$    if (myrank == 0) then
!!$       allocate(gw(3,ptcharges%num))
!!$       call gconrg_pc_zd(&
!!$            gw,ptcharges%chg,ptcharges%xyz,ptcharges%num,&
!!$            atoms%xyz,atoms%natom,rcut,lprint,ldebug)
!!$       call gsum_gnr(self%force_pc,gw,ptcharges%num)
!!$
!!$       call gconrg_on_zd(&
!!$            gw,ptcharges%xyz,ptcharges%chg,ptcharges%num,rcut,&
!!$            atoms%xyz,atoms%chg,atoms%natom,ldebug)
!!$       if (ldebug > 0) then
!!$          call ummtw('+gw_chg_on_zd', gw, 3, 3, ptcharges%num, 10, 6, istat)
!!$       end if
!!$       deallocate(gw)
!!$    end if

    call interaction_forces(&
         self%force_atom,&
         self%force_gc,&
         self%force_pc,&
         self%force_fsw,&
         atoms%xyz, atoms%chg, atoms%num,&
         gscharges%xyz, gscharges%chg, gscharges%rc2v, gscharges%num,&
         ptcharges%xyz, ptcharges%chg, ptcharges%num,&
         fswolf%xyz, fswolf%chg, fswolf%rc2v, fswolf%num,&
         rcut, method_zd,&
         icomm, nproc, myrank,&
         ldebug,&
         istat)
    if (istat /= 0) return

    if (ldebug > 0) then
       write(6,*) 'force_atom 2:'
       do i = 1, self%natom
          write(6,*)&
               self%force_atom(1,i),self%force_atom(2,i),self%force_atom(3,i)
       end do
    end if

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('::Force_bcast')
    call mpi_bcast(&
         self%force_atom, 3*self%natom, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call dscal(3*self%natom, -1.0d0, self%force_atom, 1)

    call mpi_bcast(&
         self%force_gc, 3*self%ngc, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call dscal(3*self%ngc, -1.0d0, self%force_gc, 1)

    call mpi_bcast(&
         self%force_pc, 3*self%npc, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call dscal(3*self%npc, -1.0d0, self%force_pc, 1)
    call mpi_barrier(icomm, istat)

    call mpi_bcast(&
         self%force_fsw, 3*self%nfsw, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call dscal(3*self%nfsw, -1.0d0, self%force_fsw, 1)
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('::Force_bcast')

  end subroutine force_solve

  !-----------------------------------------------------------------------------
  !> データの簡易表示
  subroutine force_print_data_brief(self)
    type(force_type),intent(in)::self

    real(8)::force_x_tot, force_y_tot, force_z_tot
    real(8)::force_x, force_y, force_z
    integer::i, j

    write(6,'(1x)')
    write(6,'(''Forces:'')')

    force_x_tot = 0.0d0
    force_y_tot = 0.0d0
    force_z_tot = 0.0d0

    if (self%natom > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'('' Atoms:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, self%natom
          !write(6,'(i8,3f18.10)') i, (self%force_atom(j,i), j = 1, 3)
          force_x = force_x + self%force_atom(1,i)
          force_y = force_y + self%force_atom(2,i)
          force_z = force_z + self%force_atom(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    if (self%ngc > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'(/'' Gaussian charges:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, self%ngc
          !write(6,'(i8,3f18.10)') i, (self%force_gc(j,i), j = 1, 3)
          force_x = force_x + self%force_gc(1,i)
          force_y = force_y + self%force_gc(2,i)
          force_z = force_z + self%force_gc(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    if (self%npc > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'(/'' Point charges:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, self%npc
          !write(6,'(i8,3f18.10)') i, (self%force_pc(j,i), j = 1, 3)
          force_x = force_x + self%force_pc(1,i)
          force_y = force_y + self%force_pc(2,i)
          force_z = force_z + self%force_pc(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    if (self%nfsw > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'(/'' Force-Switching Wolf:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, self%nfsw
          !write(6,'(i8,3f18.10)') i, (self%force_fsw(j,i), j = 1, 3)
          force_x = force_x + self%force_fsw(1,i)
          force_y = force_y + self%force_fsw(2,i)
          force_z = force_z + self%force_fsw(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    write(6,'(/'' Total:'')')
    write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
    write(6,'(8x,3(6x,''------------''))')
    write(6,'(4x,''sum:'',3f18.10)') force_x_tot, force_y_tot, force_z_tot

  end subroutine force_print_data_brief

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine force_print_data(self)
    type(force_type),intent(in)::self

    real(8)::force_x_tot, force_y_tot, force_z_tot
    real(8)::force_x, force_y, force_z
    integer::i, j

    write(6,'(1x)')
    write(6,'(''Forces:'')')

    force_x_tot = 0.0d0
    force_y_tot = 0.0d0
    force_z_tot = 0.0d0

    if (self%natom > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'('' Atoms:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, self%natom
          write(6,'(i8,3f18.10)') i, (self%force_atom(j,i), j = 1, 3)
          force_x = force_x + self%force_atom(1,i)
          force_y = force_y + self%force_atom(2,i)
          force_z = force_z + self%force_atom(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    if (self%ngc > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'(/'' Gaussian charges:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, self%ngc
          write(6,'(i8,3f18.10)') i, (self%force_gc(j,i), j = 1, 3)
          force_x = force_x + self%force_gc(1,i)
          force_y = force_y + self%force_gc(2,i)
          force_z = force_z + self%force_gc(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    if (self%npc > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'(/'' Point charges:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, self%npc
          write(6,'(i8,3f18.10)') i, (self%force_pc(j,i), j = 1, 3)
          force_x = force_x + self%force_pc(1,i)
          force_y = force_y + self%force_pc(2,i)
          force_z = force_z + self%force_pc(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    if (self%nfsw > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'(/'' Force-Switching Wolf:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, self%nfsw
          write(6,'(i8,3f18.10)') i, (self%force_fsw(j,i), j = 1, 3)
          force_x = force_x + self%force_fsw(1,i)
          force_y = force_y + self%force_fsw(2,i)
          force_z = force_z + self%force_fsw(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    write(6,'(/'' Total:'')')
    write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
    write(6,'(8x,3(6x,''------------''))')
    write(6,'(4x,''sum:'',3f18.10)') force_x_tot, force_y_tot, force_z_tot

  end subroutine force_print_data

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine force_save_data(self, istat)
    type(force_type),intent(in)::self
    integer,intent(inout)::istat

  end subroutine force_save_data

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine force_free_data(self, ldebug, istat)
    type(force_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    !if (self%natom > 0) then
    deallocate(self%force_atom, stat=istat)
    if (istat /= 0) return
    !end if

    !if (self%ngc > 0) then
    deallocate(self%force_gc, stat=istat)
    if (istat /= 0) return
    !end if

    !if (self%npc > 0) then
    deallocate(self%force_pc, stat=istat)
    if (istat /= 0) return
    !end if

    !if (self%nfsw > 0) then
    deallocate(self%force_fsw, stat=istat)
    if (istat /= 0) return
    !end if

  end subroutine force_free_data

end module force_class
