! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> ガウス電荷クラス
!!
module gscharges_class
  use error_class
  implicit none
!!$  private

  public::init_gscharges
  public::new_gscharges
  public::free_gscharges
  public::read_inp_gscharges
  public::read_gscharges
  public::set_gscharges
  public::get_gscharges
  public::num_gscharges
  public::print_gscharges
  public::print_brief_gscharges

  public::save_gscharges
  public::save_gscharges_f
  public::load_gscharges
  public::load_gscharges_f
  public::bcast_gscharges

  public::gscharges_type

  type::gscharges_type
    integer::num                 !< the number of gaussian charges
    real(8),allocatable::chg(:)  !< charges in a.u.
    real(8),allocatable::xyz(:,:)!< cartesian coordinates in a.u.
    real(8),allocatable::rc2v(:)
  end type

  character(len=*),private,parameter::url = 'data_gaussiancharges'
  integer,private,parameter::id = 1
  character(len=*),private,parameter::unit_au = 'a.u.'
  character(len=*),private,parameter::unit = ' '

contains
  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_gscharges(self, ldebug, istat)
    use pl_bms_class
    type(gscharges_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call save_gscharges_f(self, url, id, ldebug, istat)

  end subroutine save_gscharges

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_gscharges_f(self, url, id, ldebug, istat)
    use pl_bms_class
    type(gscharges_type),intent(in)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%num, url, 'nxyz', id, unit, istat)
    if (istat /= 0) return

    if (self%num <= 0) return

    call pl_bms_write(self%chg, self%num, url, 'chg', id, unit_au, istat)
    if (istat /= 0) return
    call pl_bms_write(self%xyz, 3, 3, self%num, url, 'xyz', id, unit_au, istat)
    if (istat /= 0) return
    call pl_bms_write(self%rc2v, self%num, url, 'rc2v', id, unit_au, istat)
    if (istat /= 0) return

  end subroutine save_gscharges_f

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_gscharges(self, ldebug, istat)
    use pl_bms_class
    type(gscharges_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call load_gscharges_f(self, url, id, ldebug, istat)

  end subroutine load_gscharges

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_gscharges_f(self, url, id, ldebug, istat)
    use pl_bms_class
    type(gscharges_type),intent(inout)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nxyz

    call pl_bms_read(nxyz, url, 'nxyz', id, istat)
    if (istat /= 0) return

    call new_gscharges(self, nxyz, istat)
    if (istat /= 0) return

    if (self%num <= 0) return

    call pl_bms_read(self%chg, self%num, url, 'chg', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%xyz, 3, 3, self%num, url, 'xyz', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%rc2v, self%num, url, 'rc2v', id, istat)
    if (istat /= 0) return

  end subroutine load_gscharges_f

  !-----------------------------------------------------------------------------
  !> データBcast
  subroutine bcast_gscharges(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(gscharges_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nxyz
    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    if (myrank == 0) then
      nxyz = self%num
    end if
    call mpi_bcast(nxyz, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    if (myrank /= 0) then
      call new_gscharges(self, nxyz, istat)
      if (istat /= 0) return
    end if

    if (self%num <= 0) return

    call mpi_bcast(self%chg, nxyz, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%xyz, 3*nxyz, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%rc2v, nxyz, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return

  end subroutine bcast_gscharges
  
  !-----------------------------------------------------------------------------
  !> 初期化
  subroutine init_gscharges(self, ldebug, istat)
    type(gscharges_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

!!$    call free_gscharges(self, ldebug, istat)
    self%num = 0

  end subroutine init_gscharges

  !-----------------------------------------------------------------------------
  !> 配列確保
  subroutine new_gscharges(self, num, istat)
    type(gscharges_type),intent(inout)::self
    integer,intent(in)::num
    integer,intent(inout)::istat

    istat = 0

    self%num = num
    allocate(self%chg(num), self%xyz(3,num), self%rc2v(num), stat=istat)
    if (istat .ne. 0) return

  end subroutine new_gscharges

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_gscharges(self, ldebug, istat)
    type(gscharges_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    if (allocated(self%chg)) deallocate(self%chg, stat=istat)
    if (istat .ne. 0) return
    if (allocated(self%xyz)) deallocate(self%xyz, stat=istat)
    if (istat .ne. 0) return
    if (allocated(self%rc2v)) deallocate(self%rc2v, stat=istat)
    if (istat .ne. 0) return

    self%num = 0
  end subroutine free_gscharges

  !-----------------------------------------------------------------------------
  !> \e input_file からの読み込み
  subroutine read_inp_gscharges(self, input_file, ldebug, istat)
    use input_class
    type(gscharges_type),intent(inout)::self
    character(len=*),intent(in)::input_file
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nline, iline
    character(len=MAX_INPUT_LINE_LEN),allocatable::lines(:)
    character(len=MAX_INPUT_LINE_LEN)::line

    istat = 0

    open(1, file=input_file, status='old', err=101)
    rewind(1, err=102)
    iline = 0
1   read(1, '(a)', end=2, err=103) line
    iline = iline + 1
    go to 1
2   nline = iline
    allocate(lines(nline))

    rewind(1, err=102)
    iline = 0
3   read(1, '(a)', end=4, err=103) line
    iline = iline + 1
    lines(iline) = line
    go to 3
4   close(1, err=104)

!!$    call p_read_gscharges(self,input_file,ldebug,istat)
    call read_gscharges(self, lines, nline, ldebug, istat)
    if (istat .ne. 0) return

!   self%num = 0

    if (ldebug > 100)&
         call print_gscharges(self)

    deallocate(lines)
    return
101 write(6,*) 'open error'
    return
102 write(6,*) 'rewind error'
    return
103 write(6,*) 'read error'
    return
104 write(6,*) 'close error'
    return
  end subroutine read_inp_gscharges

  !-----------------------------------------------------------------------------
  !> xyz, chg 配列の置き換え
  subroutine set_gscharges(self, num, xyz, chg, rc2v, istat)
    type(gscharges_type),intent(inout)::self
    integer,intent(in)::num
    real(8),intent(in)::xyz(3,num)
    real(8),intent(in)::chg(num)
    real(8),intent(in)::rc2v(num)
    integer,intent(inout)::istat

    istat = 0

    call new_gscharges(self, num, istat)
    if (istat .ne. 0) return

    if (num > 0) then
      self%xyz(:,:) = xyz(1:3,1:num)
      self%chg(:)   = chg(1:num)
      self%rc2v(:)  = rc2v(1:num)
    end if

  end subroutine set_gscharges

  !-----------------------------------------------------------------------------
  !> データ取得
  subroutine get_gscharges(self, num, xyz, chg, rc2v, istat)
    type(gscharges_type),intent(in)::self
    integer,intent(in)::num
    real(8),intent(inout)::xyz(3,num)
    real(8),intent(inout)::chg(num)
    real(8),intent(inout)::rc2v(num)
    integer,intent(inout)::istat

    istat = 0

    if (self%num .ne. num) then
      istat = 1
      return
    end if

    if (num > 0) then
      xyz(:,:) = self%xyz(1:3,1:num)
      chg(:)   = self%chg(1:num)
      rc2v(:)  = self%rc2v(1:num)
    end if

  end subroutine get_gscharges

  !-----------------------------------------------------------------------------
  !> ガウス電荷数の取得
  integer function num_gscharges(self)
    type(gscharges_type),intent(in)::self
    num_gscharges = self%num
  end function num_gscharges

  !-----------------------------------------------------------------------------
  !> データの簡易表示
  subroutine print_brief_gscharges(self)
    type(gscharges_type),intent(in)::self
    integer::i,j

    write(6,'(/''Gaussian charges:'')')
    !write(6,'(1x,''Number of gaussian charges ='',i6)') self%num
    write(6,'(1x,''num ='',i6)') self%num

  end subroutine print_brief_gscharges

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_gscharges(self)
    type(gscharges_type),intent(in)::self
    integer::i,j

    call print_brief_gscharges(self)

    if (self%num > 0) then
      write(6,'(/6x,''No.         X             Y             Z'',&
!!!$'          &''       Charges       Alpha'')')
           &''       Charges       Rc**2'')')
      do i = 1, self%num
        write(6,'(i8,3f14.8,f11.5,f14.8)')&
            i, (self%xyz(j,i),j=1,3), self%chg(i), self%rc2v(i)
      end do
    end if

  end subroutine print_gscharges

  !-----------------------------------------------------------------------------
  subroutine read_gscharges(self, lines, nline, ldebug, istat)
    use physical_constants_class
    use input_class
    type(gscharges_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    character(len=*),parameter::group_name = 'GAUSSIANCHARGES'
    integer,parameter::nopt = 1                          !< オプション数
    character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt)& !< オプションの名称
         =(/'UNITS'/)
    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::opt_count(nopt)
    integer::iopt, ipos, ipos_default

    integer,parameter::QM_UNITS_NUM = 2
    character(len=*),parameter::QM_UNITS(QM_UNITS_NUM) = (/ 'ANG', 'AU '/)
    integer::iunits

    integer::iline_gs  !< 対象データグループの開始行
    integer::iline_ge  !< 対象データグループの終了行
    integer::icol_gs   !< 対象データグループの開始行における開始カラム
    integer::icol_ge   !< 対象データグループの終了行における終了カラム
    integer::iline_err !< エラー行       (=0:正常, /=0:エラー)
    integer::icol_err  !< エラーカラム   (=0:正常, /=0:エラー)
    integer::ngroup

    integer,parameter::mxval = 5
    character(len=MAX_INPUT_LINE_LEN)::vals(mxval)
    integer::nval, ival, iline, icol_s, icol_e, nxyz, ixyz
    logical::is_option

    character(len=*),parameter::c_assignment_op = '='

    call get_data_group_position(&
         lines, nline, group_name,&
         iline_gs, icol_gs, iline_ge, icol_ge, ngroup,&
         ldebug, iline_err, icol_err, istat)
    if (istat /= 0) return
    if (ngroup == 0) return

    nxyz = 0
    opt_count(:) = 0
    opt_values(:) = ' '
    do iline = iline_gs, iline_ge
       !write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'
       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if
       !write(6,*) 'str=',lines(iline)(icol_s:icol_e),'$'

       is_option = .false.
       call split_str_eq(lines(iline)(icol_s:icol_e), vals, mxval, nval, istat)
       if (istat /= 0) then
          istat = PL_ERR_INVALID_FORMAT
          iline_err = iline; icol_err = icol_s
          go to 9
       end if
       !write(6,*) 'nval=',nval
       if (nval == 0) cycle
       do ival = 1, nval
          if (trim(vals(ival)) == c_assignment_op) then
             is_option = .true.
             exit
          end if
       end do
       if (is_option) then
          call parse_options(&
               lines(iline), 1, 1, icol_s, 1, icol_e,&
               opt_names, opt_values, opt_count, nopt,&
               ldebug, iline_err, icol_err, istat)
          if (istat /= 0) return
       else
          if (nval == 5) then
             nxyz = nxyz + 1
          else
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
       endif
    end do
    !write(6,*) 'nxyz=',nxyz

    do iopt = 1, nopt
       if (opt_count(iopt) > 1) then
          istat = PL_ERR_DUPLICATE_OPTION
          call pl_error_print_message(istat)
          call print_error_invalid_option(opt_names(iopt))
          return
       end if
    end do
    
    call new_gscharges(self, nxyz, istat)
    if (istat /= 0) return

    ixyz = 0
    do iline = iline_gs, iline_ge
       !write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'
       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if
       !write(6,*) 'str=',lines(iline)(icol_s:icol_e),'$'

       call split_str_eq(lines(iline)(icol_s:icol_e), vals, mxval, nval, istat)
       if (nval == 0) cycle
       if (nval == 5) then
          ixyz = ixyz + 1
          call pl_str_to_dbl(vals(1), self%xyz(1,ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          call pl_str_to_dbl(vals(2), self%xyz(2,ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          call pl_str_to_dbl(vals(3), self%xyz(3,ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          call pl_str_to_dbl(vals(4), self%chg(ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          call pl_str_to_dbl(vals(5), self%rc2v(ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
       end if
    end do

    opt_name = 'units'
    ipos_default = pl_str_search_name_ignorecase('ang', QM_UNITS, QM_UNITS_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_UNITS, QM_UNITS_NUM, ipos_default, ipos, istat)
    if (istat /= 0) return
    iunits = ipos
    !write(6,*) 'units:',QM_UNITS(iunits)

    if (self%num /= 0 .and. iunits == 1)&
         self%xyz(:,:) = self%xyz(1:3,1:nxyz) / bohr

    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine read_gscharges

end module gscharges_class
