! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 積分クラス
!!
module integral_class
  use integral_options_class
  use integral_fmt_class
  use integral_tables_class
  implicit none
!!$  private

  public::integral_prep
  public::integral_free
  public::integral_set_params
  public::integral_free_params
  public::integral_calc_overlap
  public::integral_calc_hcore
  public::integral_calc_swz
  public::integral_free_overlap
  public::integral_free_hcore
  public::integral_get_force_pc
  public::integral_get_force_gc
!!$  public::integral_get_force_fsw
  public::integral_get_force_rhf

  type,public::eri_type
     integer(8)::ntei1cut
     integer(8)::ntei2cut
     integer(2),allocatable::idx(:,:)
     real(8),allocatable::val(:)
     real(8),allocatable::swzij(:,:)
     integer::npswz
     integer,allocatable::ishp(:,:)
  end type eri_type

  type,public::control_type
     logical::l_incore
     integer::incore_p
     integer::incore_q
  end type control_type

  type,public::integral_type
     type(options_type)::options
     type(control_type)::control
     type(fmt_type)::fmt
     type(tables_type)::tables
     real(8),allocatable::rsrs(:)

     real(8),allocatable::zetpq2(:,:,:,:,:,:)
     real(8),allocatable::zxi(:,:,:,:,:,:)
     real(8),allocatable::zp32(:,:,:,:,:,:)
     integer,allocatable::nang2n(:,:,:,:)
     real(8),allocatable::zetpq(:,:,:,:,:,:)
     real(8),allocatable::zz2(:,:,:)
     real(8),allocatable::e_rrrr(:)
     integer::mik


     real(8),allocatable::s(:,:)
     real(8),allocatable::h(:,:)
     real(8),allocatable::h_zd1(:,:)
     real(8),allocatable::h_zd2(:,:)
     real(8),allocatable::h_zd3(:,:)
     real(8),allocatable::h_zd4(:,:)

     real(8),allocatable :: rzet(:)

     type(eri_type)::eri

  end type integral_type

contains
  !-----------------------------------------------------------------------------
  !> 前処理
  subroutine integral_prep(self, atoms, ldebug, istat)
    use atoms_class
    type(integral_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    call fmt_new(self%fmt, ldebug, istat)
    if (istat /= 0) return

    call fmt_prep(self%fmt, ldebug, istat)
    if (istat /= 0) return

    call tables_new(self%tables, atoms, ldebug, istat)
    if (istat /= 0) return

    call tables_prep(self%tables, atoms, ldebug, istat)
    if (istat /= 0) return

  end subroutine integral_prep

  !-----------------------------------------------------------------------------
  !> パラメータテーブルの計算
  subroutine integral_set_params(self, atoms, ldebug, istat)
    use atoms_class
    type(integral_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nasp, nshell, nao, mxsa, mxprm
    integer::mik

    nasp   = self%tables%nasp
    nshell = self%tables%nshell
    nao    = self%tables%nao
    mxsa   = self%tables%mxsa
    mxprm  = self%tables%mxprm

    allocate(&
         self%zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp),&
         self%zz2(mxprm,mxsa,nasp), self%nang2n(5,5,5,5),&
         self%e_rrrr(nshell),&
         stat=istat)
    if (istat /= 0) return

    call iprepara(&
         self%zetpq, self%zz2, self%nang2n, self%e_rrrr,&
         mik,&
         atoms%basis%z, atoms%basis%c,&
         self%tables%nsh2sht, self%tables%nsh2ssh,&
         self%tables%nsh2asp, self%tables%ntprm, self%tables%ntsht,&
         nshell, mxsa, nasp, mxprm,&
         istat)
    if (istat /= 0) return
    self%mik = mik

    allocate(&
         self%zetpq2(mxprm,mxprm,mxsa,nasp,mxsa,nasp),&
         self%zxi   (mxprm,mxprm,mxsa,nasp,mxsa,nasp),&
         self%zp32  (mxprm,mxprm,mxsa,nasp,mxsa,nasp),&
         stat=istat)
    if (istat /= 0) return

    call iopara_zet(&
         self%zetpq2, self%zxi, self%zp32,&
         atoms%basis%z,&
         nasp, mxprm,  mxsa)

  end subroutine integral_set_params

  !-----------------------------------------------------------------------------
  !> 重なり積分の計算
  subroutine integral_calc_overlap(self, atoms, pl_mpi, ldebug, istat)
    use atoms_class
    use pl_mpi_class
    use pl_prof_class
    type(integral_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nao
    integer::nr_s0, nr_e0, ns_s0, ns_e0
    integer::icomm
    integer::lprint

    integer::i,j
    real(8)::diffmax,diff
    real(8),allocatable::s(:,:)

    nao = self%tables%nao
    allocate(self%s(nao,nao), stat=istat)
    if (istat /= 0) return

    self%s(:,:) = 0.0d0
    call pl_prof_region_begin('s:Ovi')
    call os_ovi_s1(self%s,nao,self%tables,atoms)
    call pl_prof_region_end('s:Ovi')
    do j = 1, nao
       do i = j+1, nao
          self%s(j,i) = self%s(i,j)
       end do
    end do

  end subroutine integral_calc_overlap

  !-----------------------------------------------------------------------------
  !> Hcore積分の計算
  subroutine integral_calc_hcore(&
       self, atoms, ptcharges, gscharges, fswolf, pl_mpi, ldebug, istat)
    use atoms_class
    use ptcharges_class
    use gscharges_class
    use fswolf_class
    use pl_mpi_class
    use pl_prof_class
    use physical_constants_class
    type(integral_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms
    type(ptcharges_type),intent(in)::ptcharges
    type(gscharges_type),intent(in)::gscharges
    type(fswolf_type),intent(in)::fswolf
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    real(8),allocatable :: w(:,:)
    real(8),allocatable :: hw(:,:)
    real(8),allocatable :: hww(:,:)
    integer::mpi_istat(MPI_STATUS_SIZE)
    integer::myrank, nproc, icomm, irank
    integer::nr_s1, nr_e1, ns_s1, ns_e1
    integer::nrs_s1, nrs_e1, nss_s1, nss_e1
    integer::nr_sx, nr_ex, ns_sx, ns_ex
    integer::ncount
    integer::i, j
    integer::lprint

    integer::nao
    integer::natom, nasp, nshell, mxsa, mxprm
    real(8)::rcut,alpha_zd
    integer::method_zd

    real(8)::diffmax,diff
    real(8),allocatable::t(:,:),tw(:,:),t_(:,:),t1(:,:),t2(:,:)

    integer,save::isave = 0

    lprint = 0

    myrank = pl_mpi%myrank
    nproc  = pl_mpi%nprocs
    icomm  = pl_mpi%comm

    rcut      = self%options%rcut / Bohr
    alpha_zd  = self%options%alpha
    method_zd = self%options%zerodipole

    nao = self%tables%nao
    allocate(t_(nao,nao))

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('::Hcore')

    allocate(self%h(nao,nao), stat=istat)
    if (istat /= 0) return
    self%h(:,:) = 0.0d0

    ! -- kinetic energy integrals --
    allocate(t(nao,nao), stat=istat)

    call pl_prof_region_begin('s:Kei')
    call os_kei_s1(self%h,nao,self%tables,atoms)
    call pl_prof_region_end('s:Kei')
    if (ldebug > 0) then
       call ummtw('kei',self%h,nao,nao,nao,10,6,istat)
    end if

    ! -- nuclear attraction integrals --
    allocate(tw(nao,nao), stat=istat)
    if (istat /= 0) return
    tw(:,:) = 0.0d0
    t_(:,:) = 0.0d0
    call pl_prof_region_begin('p:Nai_v1')
    call os_nai_v1(tw,nao,self%tables,atoms,self%fmt,myrank,nproc)
    call pl_prof_region_end('p:Nai_v1')
    if (ldebug > 0) then
       call ummtw('nai',tw,nao,nao,nao,10,6,istat)
    end if

    ! nuclear - point charge
    if (ptcharges%num > 0) then

       t(:,:) = 0.0d0
!!$       !rcut = 16.0d0/0.5291772083d0
!!$       if (ldebug > 0) then
!!$          write(6,*) 'rcut@integral_calc_hcore=',rcut
!!$          write(6,*) 'alpha@integral_calc_hcore=',alpha_zd
!!$       end if
!!$       if (method_zd == 1 .or. method_zd == 2 .or. method_zd == 3) then
!!$          allocate(self%h_zd1(nao,nao),self%h_zd2(nao,nao),&
!!$               self%h_zd3(nao,nao),self%h_zd4(nao,nao))
!!$          call pl_prof_region_begin('s:ZeroDipole')
!!$          call zero_dipole(&
!!$               self%h_zd1,self%h_zd2,self%h_zd3,self%h_zd4,&
!!$               nao,self%tables,atoms,ptcharges,rcut,alpha_zd,method_zd,&
!!$               self%fmt,myrank,nproc)
!!$          call pl_prof_region_end('s:ZeroDipole')
!!$          do j = 1, nao
!!$             do i = j+1, nao
!!$                self%h_zd1(j,i) = self%h_zd1(i,j)
!!$                self%h_zd2(j,i) = self%h_zd2(i,j)
!!$                self%h_zd3(j,i) = self%h_zd3(i,j)
!!$                self%h_zd4(j,i) = self%h_zd4(i,j)
!!$             end do
!!$          end do
!!$          t(:,:) = self%h_zd1(:,:) + self%h_zd2(:,:)&
!!$                 + self%h_zd3(:,:) + self%h_zd4(:,:)
!!$          if (ldebug > 0) then
!!$             call ummtw('zd',t,nao,nao,nao,10,6,istat)
!!$             call ummtw('self%h:before',self%h,nao,nao,nao,10,6,istat)
!!$          end if
!!$          self%h(:,:) = self%h(:,:) + t(:,:)
!!$          if (ldebug > 0) then
!!$             call ummtw('self%h:after',self%h,nao,nao,nao,10,6,istat)
!!$          end if
!!$       else if (method_zd == 0) then

       t_(:,:) = tw(:,:)
       tw(:,:) = 0.0d0
       call pl_prof_region_begin('p:Nai_pc_v1')
       call os_nai_pc_v1(&
            tw,nao,self%tables,atoms,ptcharges,self%fmt,myrank,nproc)
       call pl_prof_region_end('p:Nai_pc_v1')
       if (ldebug > 0) then
          call ummtw('nai_pc',tw,nao,nao,nao,10,6,istat)
       end if
       tw(:,:) = tw(:,:) + t_(:,:)
       if (ldebug > 0) then
          call ummtw('nai+nai_pc',tw,nao,nao,nao,10,6,istat)
       end if
!!$       end if
    end if


    ! nuclear - gauss charge
    if (gscharges%num > 0) then
       t_(:,:) = tw(:,:)
       tw(:,:) = 0.0d0
       call pl_prof_region_begin('p:Nai_gc')
       call os_nai_gc_v1(&
            tw,nao,self%tables,atoms,gscharges,self%fmt,myrank,nproc)
       call pl_prof_region_end('p:Nai_gc')
       if (ldebug > 0) then
          call ummtw('nai_gc',tw,nao,nao,nao,10,6,istat)
       end if
       tw(:,:) = tw(:,:) + t_(:,:)
       if (ldebug > 0) then
          call ummtw('nai+nai_gc',tw,nao,nao,nao,10,6,istat)
       end if
    end if

!!$    ! nuclear - force swithing wolf
!!$    if (fswolf%num > 0) then
!!$       if (check) then
!!$          hw = 0.0d0
!!$          call pl_prof_region_begin('p:Nai_fsw:old')
!!$          call inuc_pc(&
!!$               hw,&
!!$               self%tables%nsh2atm, self%tables%nsh2sht, self%tables%nsh2ssh,&
!!$               self%tables%natm2asp, self%tables%nao2dir, self%tables%nao2sh ,&
!!$               self%fmt%fdat, self%fmt%edat,&
!!$               atoms%xyz, atoms%basis%z, atoms%basis%c, atoms%chg,&
!!$               fswolf%xyz, fswolf%chg, fswolf%num,&
!!$               self%zetpq2, self%zxi,&
!!$               self%tables%ntprm,&
!!$               self%fmt%nfsz, self%fmt%nesz,&
!!$               nr_s1, nr_e1, ns_s1, ns_e1,&
!!$               nao, nasp, natom, nshell,&
!!$               mxprm, mxsa,&
!!$               lprint, ldebug,&
!!$               istat)
!!$          if (istat /= 0) return
!!$          call inuc_gc(&
!!$               hw,&
!!$               self%tables%nsh2atm, self%tables%nsh2sht, self%tables%nsh2ssh,&
!!$               self%tables%natm2asp, self%tables%nao2dir, self%tables%nao2sh ,&
!!$               self%fmt%fdat, self%fmt%edat,&
!!$               atoms%xyz, atoms%basis%z, atoms%basis%c, atoms%chg,&
!!$               fswolf%xyz, fswolf%chg, fswolf%num,&
!!$               self%zetpq2, self%zxi,&
!!$               self%tables%ntprm,&
!!$               self%fmt%nfsz, self%fmt%nesz,&
!!$               nr_s1, nr_e1, ns_s1, ns_e1,&
!!$               nao, nasp, natom, nshell,&
!!$               mxprm, mxsa,&
!!$               fswolf%rc2v,&
!!$               lprint, ldebug,&
!!$               istat)
!!$          if (istat /= 0) return
!!$          call pl_prof_region_end('p:Nai_fsw:old')
!!$          diffmax = 0.0d0
!!$          do j = 1, nao
!!$             do i = j, nao
!!$                diff = dabs((hw(i,j))-t(i,j))
!!$                diffmax = dmax1(diffmax,diff)
!!$                if (diff <= 1.d-10) cycle
!!$                write(6,'(2i5,2f20.15)')i,j,hw(i,j),t(i,j)
!!$                stop
!!$             end do
!!$          end do
!!$          write(6,'(3x,''nai_fsw1:diffmax='',1pd12.5)') diffmax
!!$       end if
!!$    end if

    if (ldebug > 0) then
       call ummtw('tw',tw,nao,nao,nao,10,6,istat)
    end if
    call pl_prof_region_begin('m:Hcore_allreduce')
    call mpi_reduce(tw,t,nao*nao,MPI_DOUBLE_PRECISION,MPI_SUM,0,icomm,istat)
    if (istat /= 0) return
    if (ldebug > 0) then
       call ummtw('t',t,nao,nao,nao,10,6,istat)
    end if
    call mpi_bcast(t,nao*nao,MPI_DOUBLE_PRECISION,0,icomm,istat)
    if (istat /= 0) return
    call pl_prof_region_end('m:Hcore_allreduce')
    if (ldebug > 0) then
       call ummtw('self%h',self%h,nao,nao,nao,10,6,istat)
       call ummtw('t',t,nao,nao,nao,10,6,istat)
    end if
    self%h(:,:) = self%h(:,:) + t(:,:)

    do j = 1, nao
       do i = j+1, nao
          self%h(j,i) = self%h(i,j)
       end do
    end do
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('::Hcore')
    if (ldebug > 0) then
       call ummtw('T+V',self%h,nao,nao,nao,10,6,istat)
    end if

  end subroutine integral_calc_hcore

  !-----------------------------------------------------------------------------
  !> 電子間反発積分の計算
  subroutine integral_calc_swz(self, atoms, pl_mpi, ldebug, istat)
    use atoms_class
    use pl_mpi_class
    use pl_prof_class
    type(integral_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    call os_csi_os3(self%rsrs,self%tables,atoms,self%fmt)

  end subroutine integral_calc_swz

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine integral_free_overlap(self, ldebug, istat)
    type(integral_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    deallocate(self%s, stat=istat)
    if (istat /= 0) return

  end subroutine integral_free_overlap

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine integral_free_hcore(self, ldebug, istat)
    type(integral_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    deallocate(self%h, stat=istat)
    if (istat /= 0) return

  end subroutine integral_free_hcore

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine integral_free_params(self, ldebug, istat)
    type(integral_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    deallocate(&
         self%zetpq,&
         self%zz2, self%nang2n,&
         self%e_rrrr,&
         self%zetpq2,&
         self%zxi,&
         self%zp32,&
         stat=istat)
    if (istat /= 0) return

!!$    deallocate(self%s, stat=istat)
!!$    if (istat /= 0) return
!!$
!!$    deallocate(self%h, stat=istat)
!!$    if (istat /= 0) return

  end subroutine integral_free_params

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine integral_free(self, ldebug, istat)
    type(integral_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call fmt_free(self%fmt, ldebug, istat)
    if (istat /= 0) return

    call tables_free(self%tables, ldebug, istat)
    if (istat /= 0) return

  end subroutine integral_free

  !-----------------------------------------------------------------------------
  !> 力の計算
  subroutine integral_get_force_pc(&
       self,&
       force_pc, npc,&
       d, eij, nao, atoms, ptcharges,&
       tables, fmt, options,&
       pl_mpi, ldebug, istat)
    use integral_tables_class
    use integral_fmt_class
    use integral_options_class
    use atoms_class, only: atoms_type
    use ptcharges_class, only: ptcharges_type
    use pl_mpi_class
    use pl_prof_class
    use physical_constants_class
    type(integral_type),intent(in)::self
    integer,intent(in)::npc
    real(8),intent(inout)::force_pc(3,npc)
    integer,intent(in)::nao
    real(8),intent(in)::d(nao,nao)
    real(8),intent(in)::eij(nao,nao)
    type(atoms_type),intent(in)::atoms
    type(ptcharges_type),intent(in)::ptcharges
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat
    type(tables_type),intent(in)::tables
    type(fmt_type),intent(in)::fmt
    type(options_type),intent(in)::options

    real(8),allocatable::gw(:,:)

    integer::i, j
    integer::icomm, myrank, nproc
    integer::nr_s1, nr_e1, ns_s1, ns_e1
    integer::nr_s, nr_e, ns_s, ns_e, nt_s, nt_e, nu_s, nu_e
    integer::nasp, nshell, mxsa, mxprm, natom
    real(8)::erithr

    integer::lprint
    real(8)::rcut, alpha_zd
    integer::method_zd

    istat = 0

    lprint = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank
    nproc  = pl_mpi%nprocs

    rcut      = self%options%rcut / Bohr
    alpha_zd  = self%options%alpha
    method_zd = self%options%zerodipole

    nasp   = self%tables%nasp
    nshell = self%tables%nshell
    !nao    = self%tables%nao
    mxsa   = self%tables%mxsa
    mxprm  = self%tables%mxprm
    erithr = self%options%thresh_eri
    natom  = atoms%num

!!$    call get_range1(nr_s1, nr_e1, ns_s1, ns_e1, nao, myrank, nproc)
    nr_s = 1; nr_e = nao
    ns_s = 1; ns_e = nao

    if (npc > 0) then
       allocate(gw(3,npc), stat=istat)
       if (istat /= 0) go to 9900
       gw(:,:) = 0.0d0

!!$       !rcut = 0.0d0
!!$       !rcut = 16.0d0/0.5291772083d0
!!$       if (ldebug > 0) then
!!$          write(6,*) 'rcut@integral_get_force_pc=',rcut
!!$          write(6,*) 'alpha@integral_get_force_pc=',alpha_zd
!!$          write(6,*) 'method_zd@integral_get_force_pc=',method_zd
!!$       end if
!!$
!!$       if (method_zd == 1 .or. method_zd == 2 .or. method_zd == 3) then
!!$          call pl_prof_region_begin('p:hfi_on_zd')
!!$          call os_hfi_on_zd_v2(&
!!$               gw,d,nao,tables,atoms,ptcharges,rcut,alpha_zd,method_zd,fmt,myrank,nproc)
!!$          call pl_prof_region_end('p:hfi_on_zd')
!!$          if (ldebug > 0) then
!!$             call ummtw( 'gw_hfi_on_zd', gw, 3, 3, npc, 10, 6, istat)
!!$          end if
!!$          call pl_prof_region_begin('p:demq_on_zd')
!!$          call os_demq_on_zd_t1(gw,d,nao,tables,atoms,ptcharges,rcut,alpha_zd,method_zd)
!!$          call pl_prof_region_end('p:demq_on_zd')
!!$          if (ldebug > 0) then
!!$             call ummtw( '+gw_demq_on_zd', gw, 3, 3, npc, 10, 6, istat)
!!$          end if
!!$       else if (method_zd == 0) then
          call pl_prof_region_begin('p:hfi_on_pc')
          call os_hfi_on_pc_v2(&
               gw,d,nao,tables,atoms,ptcharges,fmt,myrank,nproc)
          call pl_prof_region_end('p:hfi_on_pc')
          if (ldebug > 0) then
             call ummtw( 'gw_hfi_on_pc', gw, 3, 3, npc, 10, 6, istat)
          end if
!!$       end if

       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('m:hfi_pc_reduce')
       call mpi_reduce(&
            gw, force_pc, 3*npc, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
       if (istat /= 0) return
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('m:hfi_pc_reduce')
       if (myrank == 0) then
          if (ldebug > 0) then
             call ummtw('@@force_pc', force_pc, 3, 3, npc, 10, 6, istat)
          end if
       end if

       deallocate(gw)
    end if

    go to 9999
9900 write(6,'(''Error : Array not allocated'')')
    istat = 1
    go to 9999
9999 return

  end subroutine integral_get_force_pc

  !-----------------------------------------------------------------------------
  !> 力の計算
  subroutine integral_get_force_gc(&
       self,&
       force_gc, ngc,&
       d, eij, nao, atoms, gscharges,&
       tables, fmt,&
       pl_mpi, ldebug, istat)
    use integral_tables_class
    use integral_fmt_class
    use atoms_class, only: atoms_type
    use gscharges_class, only: gscharges_type
    use pl_mpi_class
    use pl_prof_class
    type(integral_type),intent(in)::self
    integer,intent(in)::ngc
    real(8),intent(inout)::force_gc(3,ngc)
    integer,intent(in)::nao
    real(8),intent(in)::d(nao,nao)
    real(8),intent(in)::eij(nao,nao)
    type(atoms_type),intent(in)::atoms
    type(gscharges_type),intent(in)::gscharges
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat
    type(tables_type),intent(in)::tables
    type(fmt_type),intent(in)::fmt

    real(8),allocatable::gw(:,:)

    integer::i, j
    integer::icomm, myrank, nproc
    integer::nr_s1, nr_e1, ns_s1, ns_e1
    integer::nr_s, nr_e, ns_s, ns_e, nt_s, nt_e, nu_s, nu_e
    integer::nasp, nshell, mxsa, mxprm, natom
    real(8)::erithr

    integer::lprint

    istat = 0

    lprint = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank
    nproc  = pl_mpi%nprocs

    nasp   = self%tables%nasp
    nshell = self%tables%nshell
    mxsa   = self%tables%mxsa
    mxprm  = self%tables%mxprm
    erithr = self%options%thresh_eri
    natom  = atoms%num

!!$    call get_range1(nr_s1, nr_e1, ns_s1, ns_e1, nao, myrank, nproc)
    nr_s = 1; nr_e = nao
    ns_s = 1; ns_e = nao

    if (ngc > 0) then
       allocate(gw(3,ngc), stat=istat)
       if (istat /= 0) go to 9900

       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('p:hfi_on_gc')
       gw(:,:) = 0.0d0
       call os_hfi_on_gc_v2(gw,d,nao,tables,atoms,gscharges,fmt,myrank,nproc)
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('p:hfi_on_gc')
       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('m:hfi_gc_reduce')
       call mpi_reduce(&
            gw, force_gc, 3*ngc, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
       if (istat /= 0) return
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('m:hfi_gc_reduce')
       if (myrank == 0) then
          if (ldebug > 0) then
             call ummtw('@@force_gc', force_gc, 3, 3, ngc, 10, 6, istat)
          end if
       end if

       deallocate(gw)
    end if

    go to 9999
9900 write(6,'(''Error : Array not allocated'')')
    istat = 1
    go to 9999
9999 return

  end subroutine integral_get_force_gc

!!$  !-----------------------------------------------------------------------------
!!$  !> 力の計算
!!$  subroutine integral_get_force_fsw(&
!!$       self,&
!!$       force_fsw, nfsw,&
!!$       d, eij, nao, atoms, fswolf,&
!!$       pl_mpi, ldebug, istat)
!!$    use atoms_class, only: atoms_type
!!$    use fswolf_class, only: fswolf_type
!!$    use pl_mpi_class
!!$    use pl_prof_class
!!$    type(integral_type),intent(in)::self
!!$    integer,intent(in)::nfsw
!!$    real(8),intent(inout)::force_fsw(3,nfsw)
!!$    integer,intent(in)::nao
!!$    real(8),intent(in)::d(nao,nao)
!!$    real(8),intent(in)::eij(nao,nao)
!!$    type(atoms_type),intent(in)::atoms
!!$    type(fswolf_type),intent(in)::fswolf
!!$    type(pl_mpi_type),intent(in)::pl_mpi
!!$    integer,intent(in)::ldebug
!!$    integer,intent(inout)::istat
!!$
!!$    real(8),allocatable::gw(:,:)
!!$
!!$    integer::i, j
!!$    integer::icomm, myrank, nproc
!!$    integer::nr_s1, nr_e1, ns_s1, ns_e1
!!$    integer::nr_s, nr_e, ns_s, ns_e, nt_s, nt_e, nu_s, nu_e
!!$    integer::nasp, nshell, mxsa, mxprm, natom
!!$    real(8)::erithr
!!$
!!$    integer::lprint
!!$
!!$    istat = 0
!!$
!!$    lprint = 0
!!$
!!$    icomm  = pl_mpi%comm
!!$    myrank = pl_mpi%myrank
!!$    nproc  = pl_mpi%nprocs
!!$
!!$    nasp   = self%tables%nasp
!!$    nshell = self%tables%nshell
!!$    mxsa   = self%tables%mxsa
!!$    mxprm  = self%tables%mxprm
!!$    erithr = self%options%thresh_eri
!!$    natom  = atoms%num
!!$
!!$    call get_range1(nr_s1, nr_e1, ns_s1, ns_e1, nao, myrank, nproc)
!!$
!!$    nr_s = 1; nr_e = nao
!!$    ns_s = 1; ns_e = nao
!!$
!!$    if (nfsw > 0) then
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('p:dhfm_fsw')
!!$
!!$       allocate(gw(3,nfsw), stat=istat)
!!$       if (istat /= 0) go to 9900
!!$
!!$       gw(:,:) = 0.d0
!!$
!!$       ! Force Switching Wolf <- electron
!!$       call ihfm_gc(&
!!$            gw,&
!!$            d,&
!!$            self%tables%nsh2atm, self%tables%nsh2sht,&
!!$            self%tables%nsh2ssh,&
!!$            self%tables%natm2asp, self%tables%nao2dir,&
!!$            self%tables%nao2sh,&
!!$            self%fmt%fdat, self%fmt%edat,&
!!$            atoms%xyz, atoms%basis%z, atoms%basis%c, atoms%chg,&
!!$            fswolf%xyz, fswolf%chg, fswolf%num,&
!!$            self%zetpq2, self%zxi,&
!!$            self%tables%ntprm,&
!!$            self%fmt%nfsz, self%fmt%nesz,&
!!$            nr_s, nr_e, ns_s, ns_e,&
!!$            nao, nasp, natom, nshell,&
!!$            mxprm, mxsa,&
!!$            erithr,&
!!$            fswolf%rc2v,&
!!$            icomm, nproc, myrank,&
!!$            lprint, ldebug, istat)
!!$       if (istat /= 0) return
!!$       gw(:,:) = -gw(:,:)
!!$
!!$       call ihfm_pc(&
!!$            gw,&
!!$            d,&
!!$            self%tables%nsh2atm, self%tables%nsh2sht,&
!!$            self%tables%nsh2ssh,&
!!$            self%tables%natm2asp, self%tables%nao2dir,&
!!$            self%tables%nao2sh,&
!!$            self%fmt%fdat, self%fmt%edat,&
!!$            atoms%xyz, atoms%basis%z, atoms%basis%c, atoms%chg,&
!!$            fswolf%xyz, fswolf%chg, fswolf%num,&
!!$            self%zetpq2, self%zxi,&
!!$            self%tables%ntprm,&
!!$            self%fmt%nfsz, self%fmt%nesz,&
!!$            nr_s, nr_e, ns_s, ns_e,&
!!$            nao, nasp, natom, nshell,&
!!$            mxprm, mxsa,&
!!$            erithr,&
!!$            icomm, nproc, myrank,&
!!$            lprint, ldebug, istat)
!!$       if (istat /= 0) return
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('p:dhfm_fsw')
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('m:dhfm_fsw_reduce')
!!$
!!$       call mpi_reduce(&
!!$            gw, force_fsw, 3*nfsw, MPI_DOUBLE_PRECISION, MPI_SUM,&
!!$            0, icomm, istat)
!!$       if (istat /= 0) return
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('m:dhfm_fsw_reduce')
!!$
!!$       deallocate(gw)
!!$    end if
!!$
!!$    go to 9999
!!$9900 write(6,'(''Error : Array not allocated'')')
!!$    istat = 1
!!$    go to 9999
!!$9999 return
!!$
!!$  end subroutine integral_get_force_fsw

  !-----------------------------------------------------------------------------
  !> 力の計算
  subroutine integral_get_force_rhf(&
       self,&
       force_atom, natom,&
!!$       force_pc, npc,&
!!$       force_gc, ngc,&
       d, eij, nao, atoms, ptcharges, gscharges, fswolf,&
       pl_mpi, ldebug, istat)
    use atoms_class, only: atoms_type
    use ptcharges_class, only: ptcharges_type
    use gscharges_class, only: gscharges_type
    use fswolf_class, only: fswolf_type
    use pl_mpi_class
    use pl_prof_class
    use physical_constants_class
    type(integral_type),intent(in)::self
    integer,intent(in)::natom
    real(8),intent(inout)::force_atom(3,natom)
!!$    integer,intent(in)::npc
!!$    real(8),intent(inout)::force_pc(3,npc)
!!$    integer,intent(in)::ngc
!!$    real(8),intent(inout)::force_gc(3,ngc)
    integer,intent(in)::nao
    real(8),intent(in)::d(nao,nao)
    real(8),intent(in)::eij(nao,nao)
    type(atoms_type),intent(in)::atoms
    type(ptcharges_type),intent(in)::ptcharges
    type(gscharges_type),intent(in)::gscharges
    type(fswolf_type),intent(in)::fswolf
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

!!$    real(8),allocatable::gw(:,:)

    integer::i, j
    integer::icomm, myrank, nproc
    integer::nr_s1, nr_e1, ns_s1, ns_e1
    integer::nr_s, nr_e, ns_s, ns_e, nt_s, nt_e, nu_s, nu_e
    integer::nasp, nshell, mxsa, mxprm
    real(8)::erithr

    integer::lprint
    real(8)::rcut,alpha_zd
    integer::method_zd

    istat = 0

    lprint = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank
    nproc  = pl_mpi%nprocs

    rcut      = self%options%rcut / Bohr
    alpha_zd  = self%options%alpha
    method_zd = self%options%zerodipole

    nasp   = self%tables%nasp
    nshell = self%tables%nshell
    !nao    = self%tables%nao
    mxsa   = self%tables%mxsa
    mxprm  = self%tables%mxprm
    erithr = self%options%thresh_eri

!!$    call get_range1(nr_s1, nr_e1, ns_s1, ns_e1, nao, myrank, nproc)
    nr_s = 1; nr_e = nao
    ns_s = 1; ns_e = nao
    nt_s = 1; nt_e = nao
    nu_s = 1; nu_e = nao

    ! QM atoms
!!$    allocate(gw(3,natom), stat=istat)
!!$    if (istat /= 0) go to 9900

!!$    gw(:,:) = 0.d0
!!$    force_atom(:,:) = 0.d0
    call grmain(&
!!$         gw,&
         force_atom,&
         self%tables%nsh2atm, self%tables%nsh2sht,&
         self%tables%nsh2ssh, self%tables%nsh2asp,&
         self%tables%natm2asp, self%tables%nao2dir,&
         self%tables%nao2sh,&
         self%fmt%fdat, self%fmt%edat,&
         atoms%xyz, atoms%basis%z, self%zz2, atoms%basis%c, atoms%chg,&
         gscharges%xyz, gscharges%chg, gscharges%rc2v, gscharges%num,&
         ptcharges%xyz, ptcharges%chg, ptcharges%num,&
         fswolf%xyz, fswolf%chg, fswolf%rc2v, fswolf%num,&
         self%zetpq2, self%zxi, self%zp32, self%zetpq,&
         self%tables%ntprm, self%tables%ntsht,&
         eij, d,&
         self%nang2n,&
         self%fmt%nfsz, self%fmt%nesz,&
         nr_s, nr_e, ns_s, ns_e,&
         nr_s, nr_e, ns_s, ns_e,&
         nt_s, nt_e, nu_s, nu_e,&
         nao, nasp, natom, nshell,&
         mxprm, mxsa,&
         erithr,&
         icomm, nproc, myrank,&
         lprint, ldebug, istat,&
         self%tables, atoms, ptcharges, gscharges, self%fmt,&
         rcut, alpha_zd, method_zd)
    if (istat /= 0) return

  end subroutine integral_get_force_rhf

end module integral_class
