! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> Hellmann-Feynman integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_hfi_v2(g_,d,nao,tables,atoms,fmt,myrank,nproc)
  use integral_tables_class
  use atoms_class
  use integral_fmt_class
  use pl_prof_class
  integer,intent(in)::nao
  real(8),intent(in)::d(nao,nao)
  type(tables_type),intent(in)::tables
  type(atoms_type),intent(in)::atoms
  real(8),intent(inout)::g_(3,atoms%natom)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),allocatable::g(:,:)

  allocate(g(3,atoms%natom))
  g(:,:) = 0.0d0

!$omp parallel&
!$omp& default(private)&
!$omp& shared(d,nao,tables,atoms,fmt,myrank,nproc)&
!$omp& reduction(+:g)
  call pl_prof_region_begin('os_hfi_ss_v2')
  call os_hfi_ss_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_ss_v2')
  if (tables%mang == 1) go to 9

  call pl_prof_region_begin('os_hfi_ps_v2')
  call os_hfi_ps_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_ps_v2')

  call pl_prof_region_begin('os_hfi_pp_v2')
  call os_hfi_pp_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_pp_v2')
  if (tables%mang == 2) go to 9

  call pl_prof_region_begin('os_hfi_ds_v2')
  call os_hfi_ds_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_ds_v2')

  call pl_prof_region_begin('os_hfi_dp_v2')
  call os_hfi_dp_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_dp_v2')

  call pl_prof_region_begin('os_hfi_dd_v2')
  call os_hfi_dd_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_dd_v2')
9 continue
!$omp end parallel

  g_(:,:) = g_(:,:) + g(:,:)

end subroutine os_hfi_v2

!------------------------------------------------------------------------------
!> ss Hellmann-Feynman integrals
!
subroutine os_hfi_ss_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,nc)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(1,3,nc)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:1,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = myrank+1, ishp(1), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

v(:,:,:) = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        zeta2 = zeta_*2.0d0
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(2)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(8,it)*dt6 + fmt%fdat(7,it))*dt5&
                  + fmt%fdat(6,it))*dt4 + fmt%fdat(5,it))*dt3&
                  + fmt%fdat(4,it))*dt2 + fmt%fdat(3,it))*dt + fmt%fdat(2,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(2)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss1 = ft * ossx
              !ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
           end if

a_ss1 = ssss1
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
ax_ss0 = zeta2*PCx*a_ss1
ay_ss0 = zeta2*PCy*a_ss1
az_ss0 = zeta2*PCz*a_ss1

v(1,1,k) = v(1,1,k) + ax_ss0
v(1,2,k) = v(1,2,k) + ay_ss0
v(1,3,k) = v(1,3,k) + az_ss0
        end do

     end do

!!$     do k = 1, nc
!!$        write(6,'(''v(ss)='',i4,3f12.7)') k,(v(1,iq,k),iq=1,3)
!!$     end do

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     ci = 2.0d0
     if (iao == jao) ci = 1.0d0
     if (iao >= jao) then
        do ixyz = 1, 3
           do k = 1, nc
              g(ixyz,k) = g(ixyz,k) + dm(iao,jao)*v(1,ixyz,k)*ci
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_hfi_ss_v2

!------------------------------------------------------------------------------
!> ps Hellmann-Feynman integrals
!
subroutine os_hfi_ps_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,nc)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(3,3,nc)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:2,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(1)+myrank+1, ishp(2), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

v(:,:,:) = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        zeta2 = zeta_*2.0d0
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(3)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(9,it)*dt6 + fmt%fdat(8,it))*dt5&
                  + fmt%fdat(7,it))*dt4 + fmt%fdat(6,it))*dt3&
                  + fmt%fdat(5,it))*dt2 + fmt%fdat(4,it))*dt + fmt%fdat(3,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(3)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss2 = ft * ossx
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              !ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
           end if

a_ss1 = ssss1
a_ss2 = ssss2
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
ax_ss0 = zeta2*PCx*a_ss1
ay_ss0 = zeta2*PCy*a_ss1
az_ss0 = zeta2*PCz*a_ss1
ax_ss1 = zeta2*PCx*a_ss2
ay_ss1 = zeta2*PCy*a_ss2
az_ss1 = zeta2*PCz*a_ss2
ax_pxs0 = PAx*ax_ss0 - PCx*ax_ss1 + a_ss1
ay_pxs0 = PAx*ay_ss0 - PCx*ay_ss1
az_pxs0 = PAx*az_ss0 - PCx*az_ss1
ax_pys0 = PAy*ax_ss0 - PCy*ax_ss1
ay_pys0 = PAy*ay_ss0 - PCy*ay_ss1 + a_ss1
az_pys0 = PAy*az_ss0 - PCy*az_ss1
ax_pzs0 = PAz*ax_ss0 - PCz*ax_ss1
ay_pzs0 = PAz*ay_ss0 - PCz*ay_ss1
az_pzs0 = PAz*az_ss0 - PCz*az_ss1 + a_ss1

v(1,1,k) = v(1,1,k) + ax_pxs0
v(2,1,k) = v(2,1,k) + ax_pys0
v(3,1,k) = v(3,1,k) + ax_pzs0
v(1,2,k) = v(1,2,k) + ay_pxs0
v(2,2,k) = v(2,2,k) + ay_pys0
v(3,2,k) = v(3,2,k) + ay_pzs0
v(1,3,k) = v(1,3,k) + az_pxs0
v(2,3,k) = v(2,3,k) + az_pys0
v(3,3,k) = v(3,3,k) + az_pzs0
        end do

     end do

!!$     do k = 1, nc
!!$        write(6,'(''v(ps)='',i4,9f12.7)') k,((v(ip,iq,k),ip=1,3),iq=1,3)
!!$     end do

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     ci = 2.0d0
     if (i < j) then
        do ix = iao, iao + 3-1
           key = ix-iao + 1
           do ixyz = 1, 3
              do k = 1, nc
                 g(ixyz,k) = g(ixyz,k) + dm(jao,ix)*v(key,ixyz,k)*ci
              end do
           end do
        end do
     else
        do ix = iao, iao + 3-1
           key = ix-iao + 1
           do ixyz = 1, 3
              do k = 1, nc
                 g(ixyz,k) = g(ixyz,k) + dm(ix,jao)*v(key,ixyz,k)*ci
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_hfi_ps_v2

!------------------------------------------------------------------------------
!> pp Hellmann-Feynman integrals
!
subroutine os_hfi_pp_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,nc)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(9,3,nc)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:3,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(2)+myrank+1, ishp(3), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

v(:,:,:) = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        zeta2 = zeta_*2.0d0
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(4)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(10,it)*dt6 + fmt%fdat(9,it))*dt5&
                  + fmt%fdat(8,it))*dt4 + fmt%fdat(7,it))*dt3&
                  + fmt%fdat(6,it))*dt2 + fmt%fdat(5,it))*dt + fmt%fdat(4,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(4)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss3 = ft * ossx
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              !ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
           end if

a_ss1 = ssss1
a_ss2 = ssss2
a_ss3 = ssss3
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
ax_ss0 = zeta2*PCx*a_ss1
ay_ss0 = zeta2*PCy*a_ss1
az_ss0 = zeta2*PCz*a_ss1
ax_ss1 = zeta2*PCx*a_ss2
ay_ss1 = zeta2*PCy*a_ss2
az_ss1 = zeta2*PCz*a_ss2
ax_ss2 = zeta2*PCx*a_ss3
ay_ss2 = zeta2*PCy*a_ss3
az_ss2 = zeta2*PCz*a_ss3
ax_pxs0 = PAx*ax_ss0 - PCx*ax_ss1 + a_ss1
ay_pxs0 = PAx*ay_ss0 - PCx*ay_ss1
az_pxs0 = PAx*az_ss0 - PCx*az_ss1
ax_pys0 = PAy*ax_ss0 - PCy*ax_ss1
ay_pys0 = PAy*ay_ss0 - PCy*ay_ss1 + a_ss1
az_pys0 = PAy*az_ss0 - PCy*az_ss1
ax_pzs0 = PAz*ax_ss0 - PCz*ax_ss1
ay_pzs0 = PAz*ay_ss0 - PCz*ay_ss1
az_pzs0 = PAz*az_ss0 - PCz*az_ss1 + a_ss1
ax_pxs1 = PAx*ax_ss1 - PCx*ax_ss2 + a_ss2
ay_pxs1 = PAx*ay_ss1 - PCx*ay_ss2
az_pxs1 = PAx*az_ss1 - PCx*az_ss2
ax_pys1 = PAy*ax_ss1 - PCy*ax_ss2
ay_pys1 = PAy*ay_ss1 - PCy*ay_ss2 + a_ss2
az_pys1 = PAy*az_ss1 - PCy*az_ss2
ax_pzs1 = PAz*ax_ss1 - PCz*ax_ss2
ay_pzs1 = PAz*ay_ss1 - PCz*ay_ss2
az_pzs1 = PAz*az_ss1 - PCz*az_ss2 + a_ss2
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_ax_ss0 = oo2zeta*ax_ss0
ai_ax_ss1 = oo2zeta*ax_ss1
ax_pxpx0 = PBx*ax_pxs0 - PCx*ax_pxs1 + ai_ax_ss0 - ai_ax_ss1 + a_pxs1
ai_ay_ss0 = oo2zeta*ay_ss0
ai_ay_ss1 = oo2zeta*ay_ss1
ay_pxpx0 = PBx*ay_pxs0 - PCx*ay_pxs1 + ai_ay_ss0 - ai_ay_ss1
ai_az_ss0 = oo2zeta*az_ss0
ai_az_ss1 = oo2zeta*az_ss1
az_pxpx0 = PBx*az_pxs0 - PCx*az_pxs1 + ai_az_ss0 - ai_az_ss1
ax_pxpy0 = PBy*ax_pxs0 - PCy*ax_pxs1
ay_pxpy0 = PBy*ay_pxs0 - PCy*ay_pxs1 + a_pxs1
az_pxpy0 = PBy*az_pxs0 - PCy*az_pxs1
ax_pxpz0 = PBz*ax_pxs0 - PCz*ax_pxs1
ay_pxpz0 = PBz*ay_pxs0 - PCz*ay_pxs1
az_pxpz0 = PBz*az_pxs0 - PCz*az_pxs1 + a_pxs1
ax_pypx0 = PBx*ax_pys0 - PCx*ax_pys1 + a_pys1
ay_pypx0 = PBx*ay_pys0 - PCx*ay_pys1
az_pypx0 = PBx*az_pys0 - PCx*az_pys1
ax_pypy0 = PBy*ax_pys0 - PCy*ax_pys1 + ai_ax_ss0 - ai_ax_ss1
ay_pypy0 = PBy*ay_pys0 - PCy*ay_pys1 + ai_ay_ss0 - ai_ay_ss1 + a_pys1
az_pypy0 = PBy*az_pys0 - PCy*az_pys1 + ai_az_ss0 - ai_az_ss1
ax_pypz0 = PBz*ax_pys0 - PCz*ax_pys1
ay_pypz0 = PBz*ay_pys0 - PCz*ay_pys1
az_pypz0 = PBz*az_pys0 - PCz*az_pys1 + a_pys1
ax_pzpx0 = PBx*ax_pzs0 - PCx*ax_pzs1 + a_pzs1
ay_pzpx0 = PBx*ay_pzs0 - PCx*ay_pzs1
az_pzpx0 = PBx*az_pzs0 - PCx*az_pzs1
ax_pzpy0 = PBy*ax_pzs0 - PCy*ax_pzs1
ay_pzpy0 = PBy*ay_pzs0 - PCy*ay_pzs1 + a_pzs1
az_pzpy0 = PBy*az_pzs0 - PCy*az_pzs1
ax_pzpz0 = PBz*ax_pzs0 - PCz*ax_pzs1 + ai_ax_ss0 - ai_ax_ss1
ay_pzpz0 = PBz*ay_pzs0 - PCz*ay_pzs1 + ai_ay_ss0 - ai_ay_ss1
az_pzpz0 = PBz*az_pzs0 - PCz*az_pzs1 + ai_az_ss0 - ai_az_ss1 + a_pzs1

v(1,1,k) = v(1,1,k) + ax_pxpx0
v(2,1,k) = v(2,1,k) + ax_pxpy0
v(3,1,k) = v(3,1,k) + ax_pxpz0
v(4,1,k) = v(4,1,k) + ax_pypx0
v(5,1,k) = v(5,1,k) + ax_pypy0
v(6,1,k) = v(6,1,k) + ax_pypz0
v(7,1,k) = v(7,1,k) + ax_pzpx0
v(8,1,k) = v(8,1,k) + ax_pzpy0
v(9,1,k) = v(9,1,k) + ax_pzpz0
v(1,2,k) = v(1,2,k) + ay_pxpx0
v(2,2,k) = v(2,2,k) + ay_pxpy0
v(3,2,k) = v(3,2,k) + ay_pxpz0
v(4,2,k) = v(4,2,k) + ay_pypx0
v(5,2,k) = v(5,2,k) + ay_pypy0
v(6,2,k) = v(6,2,k) + ay_pypz0
v(7,2,k) = v(7,2,k) + ay_pzpx0
v(8,2,k) = v(8,2,k) + ay_pzpy0
v(9,2,k) = v(9,2,k) + ay_pzpz0
v(1,3,k) = v(1,3,k) + az_pxpx0
v(2,3,k) = v(2,3,k) + az_pxpy0
v(3,3,k) = v(3,3,k) + az_pxpz0
v(4,3,k) = v(4,3,k) + az_pypx0
v(5,3,k) = v(5,3,k) + az_pypy0
v(6,3,k) = v(6,3,k) + az_pypz0
v(7,3,k) = v(7,3,k) + az_pzpx0
v(8,3,k) = v(8,3,k) + az_pzpy0
v(9,3,k) = v(9,3,k) + az_pzpz0
        end do

     end do

!!$     do k = 1, nc
!!$        write(6,'(''v(pp)='',i4,27f12.7)') k,((v(ip,iq,k),ip=1,9),iq=1,3)
!!$     end do

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do jx = jao, jao + 3-1
           do ix = jx, iao + 3-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = (ix-iao)*3 + jx-jao + 1
              do ixyz = 1, 3
                 do k = 1, nc
                    g(ixyz,k) = g(ixyz,k) + dm(ix,jx)*v(key,ixyz,k)*ci
                 end do
              end do
           end do
        end do
     else
        do jx = jao, jao + 3-1
           do ix = iao, iao + 3-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = (ix-iao)*3 + jx-jao + 1
              do ixyz = 1, 3
                 do k = 1, nc
                    g(ixyz,k) = g(ixyz,k) + dm(ix,jx)*v(key,ixyz,k)*ci
                 end do
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_hfi_pp_v2

!------------------------------------------------------------------------------
!> ds Hellmann-Feynman integrals
!
subroutine os_hfi_ds_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,nc)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(6,3,nc)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:3,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(3)+myrank+1, ishp(4), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

v(:,:,:) = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        zeta2 = zeta_*2.0d0
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(4)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(10,it)*dt6 + fmt%fdat(9,it))*dt5&
                  + fmt%fdat(8,it))*dt4 + fmt%fdat(7,it))*dt3&
                  + fmt%fdat(6,it))*dt2 + fmt%fdat(5,it))*dt + fmt%fdat(4,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(4)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss3 = ft * ossx
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              !ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
           end if

a_ss1 = ssss1
a_ss2 = ssss2
a_ss3 = ssss3
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
ax_ss0 = zeta2*PCx*a_ss1
ay_ss0 = zeta2*PCy*a_ss1
az_ss0 = zeta2*PCz*a_ss1
ax_ss1 = zeta2*PCx*a_ss2
ay_ss1 = zeta2*PCy*a_ss2
az_ss1 = zeta2*PCz*a_ss2
ax_ss2 = zeta2*PCx*a_ss3
ay_ss2 = zeta2*PCy*a_ss3
az_ss2 = zeta2*PCz*a_ss3
ax_pxs0 = PAx*ax_ss0 - PCx*ax_ss1 + a_ss1
ay_pxs0 = PAx*ay_ss0 - PCx*ay_ss1
az_pxs0 = PAx*az_ss0 - PCx*az_ss1
ax_pys0 = PAy*ax_ss0 - PCy*ax_ss1
ay_pys0 = PAy*ay_ss0 - PCy*ay_ss1 + a_ss1
az_pys0 = PAy*az_ss0 - PCy*az_ss1
ax_pzs0 = PAz*ax_ss0 - PCz*ax_ss1
ay_pzs0 = PAz*ay_ss0 - PCz*ay_ss1
az_pzs0 = PAz*az_ss0 - PCz*az_ss1 + a_ss1
ax_pxs1 = PAx*ax_ss1 - PCx*ax_ss2 + a_ss2
ay_pxs1 = PAx*ay_ss1 - PCx*ay_ss2
az_pxs1 = PAx*az_ss1 - PCx*az_ss2
ax_pys1 = PAy*ax_ss1 - PCy*ax_ss2
ay_pys1 = PAy*ay_ss1 - PCy*ay_ss2 + a_ss2
az_pys1 = PAy*az_ss1 - PCy*az_ss2
ax_pzs1 = PAz*ax_ss1 - PCz*ax_ss2
ay_pzs1 = PAz*ay_ss1 - PCz*ay_ss2
az_pzs1 = PAz*az_ss1 - PCz*az_ss2 + a_ss2
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_ax_ss0 = oo2zeta*ax_ss0
ai_ax_ss1 = oo2zeta*ax_ss1
ax_dxxs0 = PAx*ax_pxs0 - PCx*ax_pxs1 + ai_ax_ss0 - ai_ax_ss1 + a_pxs1
ai_ay_ss0 = oo2zeta*ay_ss0
ai_ay_ss1 = oo2zeta*ay_ss1
ay_dxxs0 = PAx*ay_pxs0 - PCx*ay_pxs1 + ai_ay_ss0 - ai_ay_ss1
ai_az_ss0 = oo2zeta*az_ss0
ai_az_ss1 = oo2zeta*az_ss1
az_dxxs0 = PAx*az_pxs0 - PCx*az_pxs1 + ai_az_ss0 - ai_az_ss1
ax_dyys0 = PAy*ax_pys0 - PCy*ax_pys1 + ai_ax_ss0 - ai_ax_ss1
ay_dyys0 = PAy*ay_pys0 - PCy*ay_pys1 + ai_ay_ss0 - ai_ay_ss1 + a_pys1
az_dyys0 = PAy*az_pys0 - PCy*az_pys1 + ai_az_ss0 - ai_az_ss1
ax_dzzs0 = PAz*ax_pzs0 - PCz*ax_pzs1 + ai_ax_ss0 - ai_ax_ss1
ay_dzzs0 = PAz*ay_pzs0 - PCz*ay_pzs1 + ai_ay_ss0 - ai_ay_ss1
az_dzzs0 = PAz*az_pzs0 - PCz*az_pzs1 + ai_az_ss0 - ai_az_ss1 + a_pzs1
ax_dxys0 = PAy*ax_pxs0 - PCy*ax_pxs1
ay_dxys0 = PAy*ay_pxs0 - PCy*ay_pxs1 + a_pxs1
az_dxys0 = PAy*az_pxs0 - PCy*az_pxs1
ax_dxzs0 = PAz*ax_pxs0 - PCz*ax_pxs1
ay_dxzs0 = PAz*ay_pxs0 - PCz*ay_pxs1
az_dxzs0 = PAz*az_pxs0 - PCz*az_pxs1 + a_pxs1
ax_dyzs0 = PAz*ax_pys0 - PCz*ax_pys1
ay_dyzs0 = PAz*ay_pys0 - PCz*ay_pys1
az_dyzs0 = PAz*az_pys0 - PCz*az_pys1 + a_pys1

v(1,1,k) = v(1,1,k) + ax_dxxs0
v(2,1,k) = v(2,1,k) + ax_dyys0
v(3,1,k) = v(3,1,k) + ax_dzzs0
v(4,1,k) = v(4,1,k) + ax_dxys0*root3
v(5,1,k) = v(5,1,k) + ax_dxzs0*root3
v(6,1,k) = v(6,1,k) + ax_dyzs0*root3
v(1,2,k) = v(1,2,k) + ay_dxxs0
v(2,2,k) = v(2,2,k) + ay_dyys0
v(3,2,k) = v(3,2,k) + ay_dzzs0
v(4,2,k) = v(4,2,k) + ay_dxys0*root3
v(5,2,k) = v(5,2,k) + ay_dxzs0*root3
v(6,2,k) = v(6,2,k) + ay_dyzs0*root3
v(1,3,k) = v(1,3,k) + az_dxxs0
v(2,3,k) = v(2,3,k) + az_dyys0
v(3,3,k) = v(3,3,k) + az_dzzs0
v(4,3,k) = v(4,3,k) + az_dxys0*root3
v(5,3,k) = v(5,3,k) + az_dxzs0*root3
v(6,3,k) = v(6,3,k) + az_dyzs0*root3
        end do

     end do

!!$     do k = 1, nc
!!$        write(6,'(''v(ds)='',i4,18f12.7)') k,((v(ip,iq,k),ip=1,6),iq=1,3)
!!$     end do

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     ci = 2.0d0
     if (i < j) then
        do ix = iao, iao + 6-1
           key = ix-iao + 1
           do ixyz = 1, 3
              do k = 1, nc
                 g(ixyz,k) = g(ixyz,k) + dm(jao,ix)*v(key,ixyz,k)*ci
              end do
           end do
        end do
     else
        do ix = iao, iao + 6-1
           key = ix-iao + 1
           do ixyz = 1, 3
              do k = 1, nc
                 g(ixyz,k) = g(ixyz,k) + dm(ix,jao)*v(key,ixyz,k)*ci
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_hfi_ds_v2

!------------------------------------------------------------------------------
!> dp Hellmann-Feynman integrals
!
subroutine os_hfi_dp_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,nc)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(18,3,nc)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:4,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(4)+myrank+1, ishp(5), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

v(:,:,:) = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        zeta2 = zeta_*2.0d0
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(5)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(11,it)*dt6 + fmt%fdat(10,it))*dt5&
                  + fmt%fdat(9,it))*dt4 + fmt%fdat(8,it))*dt3&
                  + fmt%fdat(7,it))*dt2 + fmt%fdat(6,it))*dt + fmt%fdat(5,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(5)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss4 = ft * ossx
              ssss3 = (ssss4 * t2 + etcn) * rminv(4)
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              !ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
              ssss4 = ssss3 * tinv * 3.5d0
           end if

a_ss1 = ssss1
a_ss2 = ssss2
a_ss3 = ssss3
a_ss4 = ssss4
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
ax_ss0 = zeta2*PCx*a_ss1
ay_ss0 = zeta2*PCy*a_ss1
az_ss0 = zeta2*PCz*a_ss1
ax_ss1 = zeta2*PCx*a_ss2
ay_ss1 = zeta2*PCy*a_ss2
az_ss1 = zeta2*PCz*a_ss2
ax_ss2 = zeta2*PCx*a_ss3
ay_ss2 = zeta2*PCy*a_ss3
az_ss2 = zeta2*PCz*a_ss3
ax_ss3 = zeta2*PCx*a_ss4
ay_ss3 = zeta2*PCy*a_ss4
az_ss3 = zeta2*PCz*a_ss4
ax_pxs0 = PAx*ax_ss0 - PCx*ax_ss1 + a_ss1
ay_pxs0 = PAx*ay_ss0 - PCx*ay_ss1
az_pxs0 = PAx*az_ss0 - PCx*az_ss1
ax_pys0 = PAy*ax_ss0 - PCy*ax_ss1
ay_pys0 = PAy*ay_ss0 - PCy*ay_ss1 + a_ss1
az_pys0 = PAy*az_ss0 - PCy*az_ss1
ax_pzs0 = PAz*ax_ss0 - PCz*ax_ss1
ay_pzs0 = PAz*ay_ss0 - PCz*ay_ss1
az_pzs0 = PAz*az_ss0 - PCz*az_ss1 + a_ss1
ax_pxs1 = PAx*ax_ss1 - PCx*ax_ss2 + a_ss2
ay_pxs1 = PAx*ay_ss1 - PCx*ay_ss2
az_pxs1 = PAx*az_ss1 - PCx*az_ss2
ax_pys1 = PAy*ax_ss1 - PCy*ax_ss2
ay_pys1 = PAy*ay_ss1 - PCy*ay_ss2 + a_ss2
az_pys1 = PAy*az_ss1 - PCy*az_ss2
ax_pzs1 = PAz*ax_ss1 - PCz*ax_ss2
ay_pzs1 = PAz*ay_ss1 - PCz*ay_ss2
az_pzs1 = PAz*az_ss1 - PCz*az_ss2 + a_ss2
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_ax_ss0 = oo2zeta*ax_ss0
ai_ax_ss1 = oo2zeta*ax_ss1
ax_dxxs0 = PAx*ax_pxs0 - PCx*ax_pxs1 + ai_ax_ss0 - ai_ax_ss1 + a_pxs1
ai_ay_ss0 = oo2zeta*ay_ss0
ai_ay_ss1 = oo2zeta*ay_ss1
ay_dxxs0 = PAx*ay_pxs0 - PCx*ay_pxs1 + ai_ay_ss0 - ai_ay_ss1
ai_az_ss0 = oo2zeta*az_ss0
ai_az_ss1 = oo2zeta*az_ss1
az_dxxs0 = PAx*az_pxs0 - PCx*az_pxs1 + ai_az_ss0 - ai_az_ss1
ax_dyys0 = PAy*ax_pys0 - PCy*ax_pys1 + ai_ax_ss0 - ai_ax_ss1
ay_dyys0 = PAy*ay_pys0 - PCy*ay_pys1 + ai_ay_ss0 - ai_ay_ss1 + a_pys1
az_dyys0 = PAy*az_pys0 - PCy*az_pys1 + ai_az_ss0 - ai_az_ss1
ax_dzzs0 = PAz*ax_pzs0 - PCz*ax_pzs1 + ai_ax_ss0 - ai_ax_ss1
ay_dzzs0 = PAz*ay_pzs0 - PCz*ay_pzs1 + ai_ay_ss0 - ai_ay_ss1
az_dzzs0 = PAz*az_pzs0 - PCz*az_pzs1 + ai_az_ss0 - ai_az_ss1 + a_pzs1
ax_dxys0 = PAy*ax_pxs0 - PCy*ax_pxs1
ay_dxys0 = PAy*ay_pxs0 - PCy*ay_pxs1 + a_pxs1
az_dxys0 = PAy*az_pxs0 - PCy*az_pxs1
ax_dxzs0 = PAz*ax_pxs0 - PCz*ax_pxs1
ay_dxzs0 = PAz*ay_pxs0 - PCz*ay_pxs1
az_dxzs0 = PAz*az_pxs0 - PCz*az_pxs1 + a_pxs1
ax_dyzs0 = PAz*ax_pys0 - PCz*ax_pys1
ay_dyzs0 = PAz*ay_pys0 - PCz*ay_pys1
az_dyzs0 = PAz*az_pys0 - PCz*az_pys1 + a_pys1
ax_pxs2 = PAx*ax_ss2 - PCx*ax_ss3 + a_ss3
ay_pxs2 = PAx*ay_ss2 - PCx*ay_ss3
az_pxs2 = PAx*az_ss2 - PCx*az_ss3
ax_pys2 = PAy*ax_ss2 - PCy*ax_ss3
ay_pys2 = PAy*ay_ss2 - PCy*ay_ss3 + a_ss3
az_pys2 = PAy*az_ss2 - PCy*az_ss3
ax_pzs2 = PAz*ax_ss2 - PCz*ax_ss3
ay_pzs2 = PAz*ay_ss2 - PCz*ay_ss3
az_pzs2 = PAz*az_ss2 - PCz*az_ss3 + a_ss3
a_pxs2 = PAx*a_ss2 - PCx*a_ss3
a_pys2 = PAy*a_ss2 - PCy*a_ss3
a_pzs2 = PAz*a_ss2 - PCz*a_ss3
ai_ax_ss2 = oo2zeta*ax_ss2
ax_dxxs1 = PAx*ax_pxs1 - PCx*ax_pxs2 + ai_ax_ss1 - ai_ax_ss2 + a_pxs2
ai_ay_ss2 = oo2zeta*ay_ss2
ay_dxxs1 = PAx*ay_pxs1 - PCx*ay_pxs2 + ai_ay_ss1 - ai_ay_ss2
ai_az_ss2 = oo2zeta*az_ss2
az_dxxs1 = PAx*az_pxs1 - PCx*az_pxs2 + ai_az_ss1 - ai_az_ss2
ax_dyys1 = PAy*ax_pys1 - PCy*ax_pys2 + ai_ax_ss1 - ai_ax_ss2
ay_dyys1 = PAy*ay_pys1 - PCy*ay_pys2 + ai_ay_ss1 - ai_ay_ss2 + a_pys2
az_dyys1 = PAy*az_pys1 - PCy*az_pys2 + ai_az_ss1 - ai_az_ss2
ax_dzzs1 = PAz*ax_pzs1 - PCz*ax_pzs2 + ai_ax_ss1 - ai_ax_ss2
ay_dzzs1 = PAz*ay_pzs1 - PCz*ay_pzs2 + ai_ay_ss1 - ai_ay_ss2
az_dzzs1 = PAz*az_pzs1 - PCz*az_pzs2 + ai_az_ss1 - ai_az_ss2 + a_pzs2
ax_dxys1 = PAy*ax_pxs1 - PCy*ax_pxs2
ay_dxys1 = PAy*ay_pxs1 - PCy*ay_pxs2 + a_pxs2
az_dxys1 = PAy*az_pxs1 - PCy*az_pxs2
ax_dxzs1 = PAz*ax_pxs1 - PCz*ax_pxs2
ay_dxzs1 = PAz*ay_pxs1 - PCz*ay_pxs2
az_dxzs1 = PAz*az_pxs1 - PCz*az_pxs2 + a_pxs2
ax_dyzs1 = PAz*ax_pys1 - PCz*ax_pys2
ay_dyzs1 = PAz*ay_pys1 - PCz*ay_pys2
az_dyzs1 = PAz*az_pys1 - PCz*az_pys2 + a_pys2
ai_a_ss1 = oo2zeta*a_ss1
ai_a_ss2 = oo2zeta*a_ss2
a_dxxs1 = PAx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_dyys1 = PAy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_dzzs1 = PAz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
a_dxys1 = PAy*a_pxs1 - PCy*a_pxs2
a_dxzs1 = PAz*a_pxs1 - PCz*a_pxs2
a_dyzs1 = PAz*a_pys1 - PCz*a_pys2
ai_ax_pxs0 = oo2zeta*ax_pxs0
ai_ax_pxs1 = oo2zeta*ax_pxs1
ax_dxxpx0 = PBx*ax_dxxs0 - PCx*ax_dxxs1 + 2.0d0*ai_ax_pxs0 - 2.0d0*ai_ax_pxs1 + a_dxxs1
ai_ay_pxs0 = oo2zeta*ay_pxs0
ai_ay_pxs1 = oo2zeta*ay_pxs1
ay_dxxpx0 = PBx*ay_dxxs0 - PCx*ay_dxxs1 + 2.0d0*ai_ay_pxs0 - 2.0d0*ai_ay_pxs1
ai_az_pxs0 = oo2zeta*az_pxs0
ai_az_pxs1 = oo2zeta*az_pxs1
az_dxxpx0 = PBx*az_dxxs0 - PCx*az_dxxs1 + 2.0d0*ai_az_pxs0 - 2.0d0*ai_az_pxs1
ax_dxxpy0 = PBy*ax_dxxs0 - PCy*ax_dxxs1
ay_dxxpy0 = PBy*ay_dxxs0 - PCy*ay_dxxs1 + a_dxxs1
az_dxxpy0 = PBy*az_dxxs0 - PCy*az_dxxs1
ax_dxxpz0 = PBz*ax_dxxs0 - PCz*ax_dxxs1
ay_dxxpz0 = PBz*ay_dxxs0 - PCz*ay_dxxs1
az_dxxpz0 = PBz*az_dxxs0 - PCz*az_dxxs1 + a_dxxs1
ax_dyypx0 = PBx*ax_dyys0 - PCx*ax_dyys1 + a_dyys1
ay_dyypx0 = PBx*ay_dyys0 - PCx*ay_dyys1
az_dyypx0 = PBx*az_dyys0 - PCx*az_dyys1
ai_ax_pys0 = oo2zeta*ax_pys0
ai_ax_pys1 = oo2zeta*ax_pys1
ax_dyypy0 = PBy*ax_dyys0 - PCy*ax_dyys1 + 2.0d0*ai_ax_pys0 - 2.0d0*ai_ax_pys1
ai_ay_pys0 = oo2zeta*ay_pys0
ai_ay_pys1 = oo2zeta*ay_pys1
ay_dyypy0 = PBy*ay_dyys0 - PCy*ay_dyys1 + 2.0d0*ai_ay_pys0 - 2.0d0*ai_ay_pys1 + a_dyys1
ai_az_pys0 = oo2zeta*az_pys0
ai_az_pys1 = oo2zeta*az_pys1
az_dyypy0 = PBy*az_dyys0 - PCy*az_dyys1 + 2.0d0*ai_az_pys0 - 2.0d0*ai_az_pys1
ax_dyypz0 = PBz*ax_dyys0 - PCz*ax_dyys1
ay_dyypz0 = PBz*ay_dyys0 - PCz*ay_dyys1
az_dyypz0 = PBz*az_dyys0 - PCz*az_dyys1 + a_dyys1
ax_dzzpx0 = PBx*ax_dzzs0 - PCx*ax_dzzs1 + a_dzzs1
ay_dzzpx0 = PBx*ay_dzzs0 - PCx*ay_dzzs1
az_dzzpx0 = PBx*az_dzzs0 - PCx*az_dzzs1
ax_dzzpy0 = PBy*ax_dzzs0 - PCy*ax_dzzs1
ay_dzzpy0 = PBy*ay_dzzs0 - PCy*ay_dzzs1 + a_dzzs1
az_dzzpy0 = PBy*az_dzzs0 - PCy*az_dzzs1
ai_ax_pzs0 = oo2zeta*ax_pzs0
ai_ax_pzs1 = oo2zeta*ax_pzs1
ax_dzzpz0 = PBz*ax_dzzs0 - PCz*ax_dzzs1 + 2.0d0*ai_ax_pzs0 - 2.0d0*ai_ax_pzs1
ai_ay_pzs0 = oo2zeta*ay_pzs0
ai_ay_pzs1 = oo2zeta*ay_pzs1
ay_dzzpz0 = PBz*ay_dzzs0 - PCz*ay_dzzs1 + 2.0d0*ai_ay_pzs0 - 2.0d0*ai_ay_pzs1
ai_az_pzs0 = oo2zeta*az_pzs0
ai_az_pzs1 = oo2zeta*az_pzs1
az_dzzpz0 = PBz*az_dzzs0 - PCz*az_dzzs1 + 2.0d0*ai_az_pzs0 - 2.0d0*ai_az_pzs1 + a_dzzs1
ax_dxypx0 = PBx*ax_dxys0 - PCx*ax_dxys1 + ai_ax_pys0 - ai_ax_pys1 + a_dxys1
ay_dxypx0 = PBx*ay_dxys0 - PCx*ay_dxys1 + ai_ay_pys0 - ai_ay_pys1
az_dxypx0 = PBx*az_dxys0 - PCx*az_dxys1 + ai_az_pys0 - ai_az_pys1
ax_dxypy0 = PBy*ax_dxys0 - PCy*ax_dxys1 + ai_ax_pxs0 - ai_ax_pxs1
ay_dxypy0 = PBy*ay_dxys0 - PCy*ay_dxys1 + ai_ay_pxs0 - ai_ay_pxs1 + a_dxys1
az_dxypy0 = PBy*az_dxys0 - PCy*az_dxys1 + ai_az_pxs0 - ai_az_pxs1
ax_dxypz0 = PBz*ax_dxys0 - PCz*ax_dxys1
ay_dxypz0 = PBz*ay_dxys0 - PCz*ay_dxys1
az_dxypz0 = PBz*az_dxys0 - PCz*az_dxys1 + a_dxys1
ax_dxzpx0 = PBx*ax_dxzs0 - PCx*ax_dxzs1 + ai_ax_pzs0 - ai_ax_pzs1 + a_dxzs1
ay_dxzpx0 = PBx*ay_dxzs0 - PCx*ay_dxzs1 + ai_ay_pzs0 - ai_ay_pzs1
az_dxzpx0 = PBx*az_dxzs0 - PCx*az_dxzs1 + ai_az_pzs0 - ai_az_pzs1
ax_dxzpy0 = PBy*ax_dxzs0 - PCy*ax_dxzs1
ay_dxzpy0 = PBy*ay_dxzs0 - PCy*ay_dxzs1 + a_dxzs1
az_dxzpy0 = PBy*az_dxzs0 - PCy*az_dxzs1
ax_dxzpz0 = PBz*ax_dxzs0 - PCz*ax_dxzs1 + ai_ax_pxs0 - ai_ax_pxs1
ay_dxzpz0 = PBz*ay_dxzs0 - PCz*ay_dxzs1 + ai_ay_pxs0 - ai_ay_pxs1
az_dxzpz0 = PBz*az_dxzs0 - PCz*az_dxzs1 + ai_az_pxs0 - ai_az_pxs1 + a_dxzs1
ax_dyzpx0 = PBx*ax_dyzs0 - PCx*ax_dyzs1 + a_dyzs1
ay_dyzpx0 = PBx*ay_dyzs0 - PCx*ay_dyzs1
az_dyzpx0 = PBx*az_dyzs0 - PCx*az_dyzs1
ax_dyzpy0 = PBy*ax_dyzs0 - PCy*ax_dyzs1 + ai_ax_pzs0 - ai_ax_pzs1
ay_dyzpy0 = PBy*ay_dyzs0 - PCy*ay_dyzs1 + ai_ay_pzs0 - ai_ay_pzs1 + a_dyzs1
az_dyzpy0 = PBy*az_dyzs0 - PCy*az_dyzs1 + ai_az_pzs0 - ai_az_pzs1
ax_dyzpz0 = PBz*ax_dyzs0 - PCz*ax_dyzs1 + ai_ax_pys0 - ai_ax_pys1
ay_dyzpz0 = PBz*ay_dyzs0 - PCz*ay_dyzs1 + ai_ay_pys0 - ai_ay_pys1
az_dyzpz0 = PBz*az_dyzs0 - PCz*az_dyzs1 + ai_az_pys0 - ai_az_pys1 + a_dyzs1

v(1,1,k) = v(1,1,k) + ax_dxxpx0
v(2,1,k) = v(2,1,k) + ax_dxxpy0
v(3,1,k) = v(3,1,k) + ax_dxxpz0
v(4,1,k) = v(4,1,k) + ax_dyypx0
v(5,1,k) = v(5,1,k) + ax_dyypy0
v(6,1,k) = v(6,1,k) + ax_dyypz0
v(7,1,k) = v(7,1,k) + ax_dzzpx0
v(8,1,k) = v(8,1,k) + ax_dzzpy0
v(9,1,k) = v(9,1,k) + ax_dzzpz0
v(10,1,k) = v(10,1,k) + ax_dxypx0*root3
v(11,1,k) = v(11,1,k) + ax_dxypy0*root3
v(12,1,k) = v(12,1,k) + ax_dxypz0*root3
v(13,1,k) = v(13,1,k) + ax_dxzpx0*root3
v(14,1,k) = v(14,1,k) + ax_dxzpy0*root3
v(15,1,k) = v(15,1,k) + ax_dxzpz0*root3
v(16,1,k) = v(16,1,k) + ax_dyzpx0*root3
v(17,1,k) = v(17,1,k) + ax_dyzpy0*root3
v(18,1,k) = v(18,1,k) + ax_dyzpz0*root3
v(1,2,k) = v(1,2,k) + ay_dxxpx0
v(2,2,k) = v(2,2,k) + ay_dxxpy0
v(3,2,k) = v(3,2,k) + ay_dxxpz0
v(4,2,k) = v(4,2,k) + ay_dyypx0
v(5,2,k) = v(5,2,k) + ay_dyypy0
v(6,2,k) = v(6,2,k) + ay_dyypz0
v(7,2,k) = v(7,2,k) + ay_dzzpx0
v(8,2,k) = v(8,2,k) + ay_dzzpy0
v(9,2,k) = v(9,2,k) + ay_dzzpz0
v(10,2,k) = v(10,2,k) + ay_dxypx0*root3
v(11,2,k) = v(11,2,k) + ay_dxypy0*root3
v(12,2,k) = v(12,2,k) + ay_dxypz0*root3
v(13,2,k) = v(13,2,k) + ay_dxzpx0*root3
v(14,2,k) = v(14,2,k) + ay_dxzpy0*root3
v(15,2,k) = v(15,2,k) + ay_dxzpz0*root3
v(16,2,k) = v(16,2,k) + ay_dyzpx0*root3
v(17,2,k) = v(17,2,k) + ay_dyzpy0*root3
v(18,2,k) = v(18,2,k) + ay_dyzpz0*root3
v(1,3,k) = v(1,3,k) + az_dxxpx0
v(2,3,k) = v(2,3,k) + az_dxxpy0
v(3,3,k) = v(3,3,k) + az_dxxpz0
v(4,3,k) = v(4,3,k) + az_dyypx0
v(5,3,k) = v(5,3,k) + az_dyypy0
v(6,3,k) = v(6,3,k) + az_dyypz0
v(7,3,k) = v(7,3,k) + az_dzzpx0
v(8,3,k) = v(8,3,k) + az_dzzpy0
v(9,3,k) = v(9,3,k) + az_dzzpz0
v(10,3,k) = v(10,3,k) + az_dxypx0*root3
v(11,3,k) = v(11,3,k) + az_dxypy0*root3
v(12,3,k) = v(12,3,k) + az_dxypz0*root3
v(13,3,k) = v(13,3,k) + az_dxzpx0*root3
v(14,3,k) = v(14,3,k) + az_dxzpy0*root3
v(15,3,k) = v(15,3,k) + az_dxzpz0*root3
v(16,3,k) = v(16,3,k) + az_dyzpx0*root3
v(17,3,k) = v(17,3,k) + az_dyzpy0*root3
v(18,3,k) = v(18,3,k) + az_dyzpz0*root3
        end do

     end do

!!$     do k = 1, nc
!!$        write(6,'(''v(dp)='',i4,18f12.7)') k,((v(ip,iq,k),ip=1,18),iq=1,3)
!!$     end do

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     ci = 2.0d0
     if (i < j) then
        do jx = jao, jao + 3-1
           do ix = iao, iao + 6-1
              key = ( ix-iao ) * 3 + jx-jao +1
              do ixyz = 1, 3
                 do k = 1, nc
                    g(ixyz,k) = g(ixyz,k) + dm(jx,ix)*v(key,ixyz,k)*ci
                 end do
              end do
           end do
        end do
     else
        do jx = jao, jao + 3-1
           do ix = iao, iao + 6-1
              key = ( ix-iao ) * 3 + jx-jao +1
              do ixyz = 1, 3
                 do k = 1, nc
                    g(ixyz,k) = g(ixyz,k) + dm(ix,jx)*v(key,ixyz,k)*ci
                 end do
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_hfi_dp_v2

!------------------------------------------------------------------------------
!> dd Hellmann-Feynman integrals
!
subroutine os_hfi_dd_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,nc)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(36,3,nc)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:5,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(5)+myrank+1, ishp(6), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

v(:,:,:) = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        zeta2 = zeta_*2.0d0
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(6)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(12,it)*dt6 + fmt%fdat(11,it))*dt5&
                   + fmt%fdat(10,it))*dt4 + fmt%fdat(9,it))*dt3&
                   + fmt%fdat(8,it))*dt2 + fmt%fdat(7,it))*dt + fmt%fdat(6,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                   + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(6)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss5 = ft * ossx
              ssss4 = (ssss5 * t2 + etcn) * rminv(5)
              ssss3 = (ssss4 * t2 + etcn) * rminv(4)
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              !ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
              ssss4 = ssss3 * tinv * 3.5d0
              ssss5 = ssss4 * tinv * 4.5d0
           end if

a_ss1 = ssss1
a_ss2 = ssss2
a_ss3 = ssss3
a_ss4 = ssss4
a_ss5 = ssss5
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
ax_ss0 = zeta2*PCx*a_ss1
ay_ss0 = zeta2*PCy*a_ss1
az_ss0 = zeta2*PCz*a_ss1
ax_ss1 = zeta2*PCx*a_ss2
ay_ss1 = zeta2*PCy*a_ss2
az_ss1 = zeta2*PCz*a_ss2
ax_ss2 = zeta2*PCx*a_ss3
ay_ss2 = zeta2*PCy*a_ss3
az_ss2 = zeta2*PCz*a_ss3
ax_ss3 = zeta2*PCx*a_ss4
ay_ss3 = zeta2*PCy*a_ss4
az_ss3 = zeta2*PCz*a_ss4
ax_ss4 = zeta2*PCx*a_ss5
ay_ss4 = zeta2*PCy*a_ss5
az_ss4 = zeta2*PCz*a_ss5
ax_pxs0 = PAx*ax_ss0 - PCx*ax_ss1 + a_ss1
ay_pxs0 = PAx*ay_ss0 - PCx*ay_ss1
az_pxs0 = PAx*az_ss0 - PCx*az_ss1
ax_pys0 = PAy*ax_ss0 - PCy*ax_ss1
ay_pys0 = PAy*ay_ss0 - PCy*ay_ss1 + a_ss1
az_pys0 = PAy*az_ss0 - PCy*az_ss1
ax_pzs0 = PAz*ax_ss0 - PCz*ax_ss1
ay_pzs0 = PAz*ay_ss0 - PCz*ay_ss1
az_pzs0 = PAz*az_ss0 - PCz*az_ss1 + a_ss1
ax_pxs1 = PAx*ax_ss1 - PCx*ax_ss2 + a_ss2
ay_pxs1 = PAx*ay_ss1 - PCx*ay_ss2
az_pxs1 = PAx*az_ss1 - PCx*az_ss2
ax_pys1 = PAy*ax_ss1 - PCy*ax_ss2
ay_pys1 = PAy*ay_ss1 - PCy*ay_ss2 + a_ss2
az_pys1 = PAy*az_ss1 - PCy*az_ss2
ax_pzs1 = PAz*ax_ss1 - PCz*ax_ss2
ay_pzs1 = PAz*ay_ss1 - PCz*ay_ss2
az_pzs1 = PAz*az_ss1 - PCz*az_ss2 + a_ss2
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_ax_ss0 = oo2zeta*ax_ss0
ai_ax_ss1 = oo2zeta*ax_ss1
ax_dxxs0 = PAx*ax_pxs0 - PCx*ax_pxs1 + ai_ax_ss0 - ai_ax_ss1 + a_pxs1
ai_ay_ss0 = oo2zeta*ay_ss0
ai_ay_ss1 = oo2zeta*ay_ss1
ay_dxxs0 = PAx*ay_pxs0 - PCx*ay_pxs1 + ai_ay_ss0 - ai_ay_ss1
ai_az_ss0 = oo2zeta*az_ss0
ai_az_ss1 = oo2zeta*az_ss1
az_dxxs0 = PAx*az_pxs0 - PCx*az_pxs1 + ai_az_ss0 - ai_az_ss1
ax_dyys0 = PAy*ax_pys0 - PCy*ax_pys1 + ai_ax_ss0 - ai_ax_ss1
ay_dyys0 = PAy*ay_pys0 - PCy*ay_pys1 + ai_ay_ss0 - ai_ay_ss1 + a_pys1
az_dyys0 = PAy*az_pys0 - PCy*az_pys1 + ai_az_ss0 - ai_az_ss1
ax_dzzs0 = PAz*ax_pzs0 - PCz*ax_pzs1 + ai_ax_ss0 - ai_ax_ss1
ay_dzzs0 = PAz*ay_pzs0 - PCz*ay_pzs1 + ai_ay_ss0 - ai_ay_ss1
az_dzzs0 = PAz*az_pzs0 - PCz*az_pzs1 + ai_az_ss0 - ai_az_ss1 + a_pzs1
ax_dxys0 = PAy*ax_pxs0 - PCy*ax_pxs1
ay_dxys0 = PAy*ay_pxs0 - PCy*ay_pxs1 + a_pxs1
az_dxys0 = PAy*az_pxs0 - PCy*az_pxs1
ax_dxzs0 = PAz*ax_pxs0 - PCz*ax_pxs1
ay_dxzs0 = PAz*ay_pxs0 - PCz*ay_pxs1
az_dxzs0 = PAz*az_pxs0 - PCz*az_pxs1 + a_pxs1
ax_dyzs0 = PAz*ax_pys0 - PCz*ax_pys1
ay_dyzs0 = PAz*ay_pys0 - PCz*ay_pys1
az_dyzs0 = PAz*az_pys0 - PCz*az_pys1 + a_pys1
ax_pxs2 = PAx*ax_ss2 - PCx*ax_ss3 + a_ss3
ay_pxs2 = PAx*ay_ss2 - PCx*ay_ss3
az_pxs2 = PAx*az_ss2 - PCx*az_ss3
ax_pys2 = PAy*ax_ss2 - PCy*ax_ss3
ay_pys2 = PAy*ay_ss2 - PCy*ay_ss3 + a_ss3
az_pys2 = PAy*az_ss2 - PCy*az_ss3
ax_pzs2 = PAz*ax_ss2 - PCz*ax_ss3
ay_pzs2 = PAz*ay_ss2 - PCz*ay_ss3
az_pzs2 = PAz*az_ss2 - PCz*az_ss3 + a_ss3
a_pxs2 = PAx*a_ss2 - PCx*a_ss3
a_pys2 = PAy*a_ss2 - PCy*a_ss3
a_pzs2 = PAz*a_ss2 - PCz*a_ss3
ai_ax_ss2 = oo2zeta*ax_ss2
ax_dxxs1 = PAx*ax_pxs1 - PCx*ax_pxs2 + ai_ax_ss1 - ai_ax_ss2 + a_pxs2
ai_ay_ss2 = oo2zeta*ay_ss2
ay_dxxs1 = PAx*ay_pxs1 - PCx*ay_pxs2 + ai_ay_ss1 - ai_ay_ss2
ai_az_ss2 = oo2zeta*az_ss2
az_dxxs1 = PAx*az_pxs1 - PCx*az_pxs2 + ai_az_ss1 - ai_az_ss2
ax_dyys1 = PAy*ax_pys1 - PCy*ax_pys2 + ai_ax_ss1 - ai_ax_ss2
ay_dyys1 = PAy*ay_pys1 - PCy*ay_pys2 + ai_ay_ss1 - ai_ay_ss2 + a_pys2
az_dyys1 = PAy*az_pys1 - PCy*az_pys2 + ai_az_ss1 - ai_az_ss2
ax_dzzs1 = PAz*ax_pzs1 - PCz*ax_pzs2 + ai_ax_ss1 - ai_ax_ss2
ay_dzzs1 = PAz*ay_pzs1 - PCz*ay_pzs2 + ai_ay_ss1 - ai_ay_ss2
az_dzzs1 = PAz*az_pzs1 - PCz*az_pzs2 + ai_az_ss1 - ai_az_ss2 + a_pzs2
ax_dxys1 = PAy*ax_pxs1 - PCy*ax_pxs2
ay_dxys1 = PAy*ay_pxs1 - PCy*ay_pxs2 + a_pxs2
az_dxys1 = PAy*az_pxs1 - PCy*az_pxs2
ax_dxzs1 = PAz*ax_pxs1 - PCz*ax_pxs2
ay_dxzs1 = PAz*ay_pxs1 - PCz*ay_pxs2
az_dxzs1 = PAz*az_pxs1 - PCz*az_pxs2 + a_pxs2
ax_dyzs1 = PAz*ax_pys1 - PCz*ax_pys2
ay_dyzs1 = PAz*ay_pys1 - PCz*ay_pys2
az_dyzs1 = PAz*az_pys1 - PCz*az_pys2 + a_pys2
ai_a_ss1 = oo2zeta*a_ss1
ai_a_ss2 = oo2zeta*a_ss2
a_dxxs1 = PAx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_dyys1 = PAy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_dzzs1 = PAz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
a_dxys1 = PAy*a_pxs1 - PCy*a_pxs2
a_dxzs1 = PAz*a_pxs1 - PCz*a_pxs2
a_dyzs1 = PAz*a_pys1 - PCz*a_pys2
ai_ax_pxs0 = oo2zeta*ax_pxs0
ai_ax_pxs1 = oo2zeta*ax_pxs1
ax_dxxpx0 = PBx*ax_dxxs0 - PCx*ax_dxxs1 + 2.0d0*ai_ax_pxs0 - 2.0d0*ai_ax_pxs1 + a_dxxs1
ai_ay_pxs0 = oo2zeta*ay_pxs0
ai_ay_pxs1 = oo2zeta*ay_pxs1
ay_dxxpx0 = PBx*ay_dxxs0 - PCx*ay_dxxs1 + 2.0d0*ai_ay_pxs0 - 2.0d0*ai_ay_pxs1
ai_az_pxs0 = oo2zeta*az_pxs0
ai_az_pxs1 = oo2zeta*az_pxs1
az_dxxpx0 = PBx*az_dxxs0 - PCx*az_dxxs1 + 2.0d0*ai_az_pxs0 - 2.0d0*ai_az_pxs1
ax_dxxpy0 = PBy*ax_dxxs0 - PCy*ax_dxxs1
ay_dxxpy0 = PBy*ay_dxxs0 - PCy*ay_dxxs1 + a_dxxs1
az_dxxpy0 = PBy*az_dxxs0 - PCy*az_dxxs1
ax_dxxpz0 = PBz*ax_dxxs0 - PCz*ax_dxxs1
ay_dxxpz0 = PBz*ay_dxxs0 - PCz*ay_dxxs1
az_dxxpz0 = PBz*az_dxxs0 - PCz*az_dxxs1 + a_dxxs1
ax_dyypx0 = PBx*ax_dyys0 - PCx*ax_dyys1 + a_dyys1
ay_dyypx0 = PBx*ay_dyys0 - PCx*ay_dyys1
az_dyypx0 = PBx*az_dyys0 - PCx*az_dyys1
ai_ax_pys0 = oo2zeta*ax_pys0
ai_ax_pys1 = oo2zeta*ax_pys1
ax_dyypy0 = PBy*ax_dyys0 - PCy*ax_dyys1 + 2.0d0*ai_ax_pys0 - 2.0d0*ai_ax_pys1
ai_ay_pys0 = oo2zeta*ay_pys0
ai_ay_pys1 = oo2zeta*ay_pys1
ay_dyypy0 = PBy*ay_dyys0 - PCy*ay_dyys1 + 2.0d0*ai_ay_pys0 - 2.0d0*ai_ay_pys1 + a_dyys1
ai_az_pys0 = oo2zeta*az_pys0
ai_az_pys1 = oo2zeta*az_pys1
az_dyypy0 = PBy*az_dyys0 - PCy*az_dyys1 + 2.0d0*ai_az_pys0 - 2.0d0*ai_az_pys1
ax_dyypz0 = PBz*ax_dyys0 - PCz*ax_dyys1
ay_dyypz0 = PBz*ay_dyys0 - PCz*ay_dyys1
az_dyypz0 = PBz*az_dyys0 - PCz*az_dyys1 + a_dyys1
ax_dzzpx0 = PBx*ax_dzzs0 - PCx*ax_dzzs1 + a_dzzs1
ay_dzzpx0 = PBx*ay_dzzs0 - PCx*ay_dzzs1
az_dzzpx0 = PBx*az_dzzs0 - PCx*az_dzzs1
ax_dzzpy0 = PBy*ax_dzzs0 - PCy*ax_dzzs1
ay_dzzpy0 = PBy*ay_dzzs0 - PCy*ay_dzzs1 + a_dzzs1
az_dzzpy0 = PBy*az_dzzs0 - PCy*az_dzzs1
ai_ax_pzs0 = oo2zeta*ax_pzs0
ai_ax_pzs1 = oo2zeta*ax_pzs1
ax_dzzpz0 = PBz*ax_dzzs0 - PCz*ax_dzzs1 + 2.0d0*ai_ax_pzs0 - 2.0d0*ai_ax_pzs1
ai_ay_pzs0 = oo2zeta*ay_pzs0
ai_ay_pzs1 = oo2zeta*ay_pzs1
ay_dzzpz0 = PBz*ay_dzzs0 - PCz*ay_dzzs1 + 2.0d0*ai_ay_pzs0 - 2.0d0*ai_ay_pzs1
ai_az_pzs0 = oo2zeta*az_pzs0
ai_az_pzs1 = oo2zeta*az_pzs1
az_dzzpz0 = PBz*az_dzzs0 - PCz*az_dzzs1 + 2.0d0*ai_az_pzs0 - 2.0d0*ai_az_pzs1 + a_dzzs1
ax_dxypx0 = PBx*ax_dxys0 - PCx*ax_dxys1 + ai_ax_pys0 - ai_ax_pys1 + a_dxys1
ay_dxypx0 = PBx*ay_dxys0 - PCx*ay_dxys1 + ai_ay_pys0 - ai_ay_pys1
az_dxypx0 = PBx*az_dxys0 - PCx*az_dxys1 + ai_az_pys0 - ai_az_pys1
ax_dxypy0 = PBy*ax_dxys0 - PCy*ax_dxys1 + ai_ax_pxs0 - ai_ax_pxs1
ay_dxypy0 = PBy*ay_dxys0 - PCy*ay_dxys1 + ai_ay_pxs0 - ai_ay_pxs1 + a_dxys1
az_dxypy0 = PBy*az_dxys0 - PCy*az_dxys1 + ai_az_pxs0 - ai_az_pxs1
ax_dxypz0 = PBz*ax_dxys0 - PCz*ax_dxys1
ay_dxypz0 = PBz*ay_dxys0 - PCz*ay_dxys1
az_dxypz0 = PBz*az_dxys0 - PCz*az_dxys1 + a_dxys1
ax_dxzpx0 = PBx*ax_dxzs0 - PCx*ax_dxzs1 + ai_ax_pzs0 - ai_ax_pzs1 + a_dxzs1
ay_dxzpx0 = PBx*ay_dxzs0 - PCx*ay_dxzs1 + ai_ay_pzs0 - ai_ay_pzs1
az_dxzpx0 = PBx*az_dxzs0 - PCx*az_dxzs1 + ai_az_pzs0 - ai_az_pzs1
ax_dxzpy0 = PBy*ax_dxzs0 - PCy*ax_dxzs1
ay_dxzpy0 = PBy*ay_dxzs0 - PCy*ay_dxzs1 + a_dxzs1
az_dxzpy0 = PBy*az_dxzs0 - PCy*az_dxzs1
ax_dxzpz0 = PBz*ax_dxzs0 - PCz*ax_dxzs1 + ai_ax_pxs0 - ai_ax_pxs1
ay_dxzpz0 = PBz*ay_dxzs0 - PCz*ay_dxzs1 + ai_ay_pxs0 - ai_ay_pxs1
az_dxzpz0 = PBz*az_dxzs0 - PCz*az_dxzs1 + ai_az_pxs0 - ai_az_pxs1 + a_dxzs1
ax_dyzpx0 = PBx*ax_dyzs0 - PCx*ax_dyzs1 + a_dyzs1
ay_dyzpx0 = PBx*ay_dyzs0 - PCx*ay_dyzs1
az_dyzpx0 = PBx*az_dyzs0 - PCx*az_dyzs1
ax_dyzpy0 = PBy*ax_dyzs0 - PCy*ax_dyzs1 + ai_ax_pzs0 - ai_ax_pzs1
ay_dyzpy0 = PBy*ay_dyzs0 - PCy*ay_dyzs1 + ai_ay_pzs0 - ai_ay_pzs1 + a_dyzs1
az_dyzpy0 = PBy*az_dyzs0 - PCy*az_dyzs1 + ai_az_pzs0 - ai_az_pzs1
ax_dyzpz0 = PBz*ax_dyzs0 - PCz*ax_dyzs1 + ai_ax_pys0 - ai_ax_pys1
ay_dyzpz0 = PBz*ay_dyzs0 - PCz*ay_dyzs1 + ai_ay_pys0 - ai_ay_pys1
az_dyzpz0 = PBz*az_dyzs0 - PCz*az_dyzs1 + ai_az_pys0 - ai_az_pys1 + a_dyzs1
ax_pxs3 = PAx*ax_ss3 - PCx*ax_ss4 + a_ss4
ay_pxs3 = PAx*ay_ss3 - PCx*ay_ss4
az_pxs3 = PAx*az_ss3 - PCx*az_ss4
ax_pys3 = PAy*ax_ss3 - PCy*ax_ss4
ay_pys3 = PAy*ay_ss3 - PCy*ay_ss4 + a_ss4
az_pys3 = PAy*az_ss3 - PCy*az_ss4
ax_pzs3 = PAz*ax_ss3 - PCz*ax_ss4
ay_pzs3 = PAz*ay_ss3 - PCz*ay_ss4
az_pzs3 = PAz*az_ss3 - PCz*az_ss4 + a_ss4
a_pxs3 = PAx*a_ss3 - PCx*a_ss4
a_pys3 = PAy*a_ss3 - PCy*a_ss4
a_pzs3 = PAz*a_ss3 - PCz*a_ss4
ai_ax_ss3 = oo2zeta*ax_ss3
ax_dxxs2 = PAx*ax_pxs2 - PCx*ax_pxs3 + ai_ax_ss2 - ai_ax_ss3 + a_pxs3
ai_ay_ss3 = oo2zeta*ay_ss3
ay_dxxs2 = PAx*ay_pxs2 - PCx*ay_pxs3 + ai_ay_ss2 - ai_ay_ss3
ai_az_ss3 = oo2zeta*az_ss3
az_dxxs2 = PAx*az_pxs2 - PCx*az_pxs3 + ai_az_ss2 - ai_az_ss3
ax_dyys2 = PAy*ax_pys2 - PCy*ax_pys3 + ai_ax_ss2 - ai_ax_ss3
ay_dyys2 = PAy*ay_pys2 - PCy*ay_pys3 + ai_ay_ss2 - ai_ay_ss3 + a_pys3
az_dyys2 = PAy*az_pys2 - PCy*az_pys3 + ai_az_ss2 - ai_az_ss3
ax_dzzs2 = PAz*ax_pzs2 - PCz*ax_pzs3 + ai_ax_ss2 - ai_ax_ss3
ay_dzzs2 = PAz*ay_pzs2 - PCz*ay_pzs3 + ai_ay_ss2 - ai_ay_ss3
az_dzzs2 = PAz*az_pzs2 - PCz*az_pzs3 + ai_az_ss2 - ai_az_ss3 + a_pzs3
ax_dxys2 = PAy*ax_pxs2 - PCy*ax_pxs3
ay_dxys2 = PAy*ay_pxs2 - PCy*ay_pxs3 + a_pxs3
az_dxys2 = PAy*az_pxs2 - PCy*az_pxs3
ax_dxzs2 = PAz*ax_pxs2 - PCz*ax_pxs3
ay_dxzs2 = PAz*ay_pxs2 - PCz*ay_pxs3
az_dxzs2 = PAz*az_pxs2 - PCz*az_pxs3 + a_pxs3
ax_dyzs2 = PAz*ax_pys2 - PCz*ax_pys3
ay_dyzs2 = PAz*ay_pys2 - PCz*ay_pys3
az_dyzs2 = PAz*az_pys2 - PCz*az_pys3 + a_pys3
ai_a_ss3 = oo2zeta*a_ss3
a_dxxs2 = PAx*a_pxs2 - PCx*a_pxs3 + ai_a_ss2 - ai_a_ss3
a_dyys2 = PAy*a_pys2 - PCy*a_pys3 + ai_a_ss2 - ai_a_ss3
a_dzzs2 = PAz*a_pzs2 - PCz*a_pzs3 + ai_a_ss2 - ai_a_ss3
a_dxys2 = PAy*a_pxs2 - PCy*a_pxs3
a_dxzs2 = PAz*a_pxs2 - PCz*a_pxs3
a_dyzs2 = PAz*a_pys2 - PCz*a_pys3
ai_ax_pxs2 = oo2zeta*ax_pxs2
ax_dxxpx1 = PBx*ax_dxxs1 - PCx*ax_dxxs2 + 2.0d0*ai_ax_pxs1 - 2.0d0*ai_ax_pxs2 + a_dxxs2
ai_ay_pxs2 = oo2zeta*ay_pxs2
ay_dxxpx1 = PBx*ay_dxxs1 - PCx*ay_dxxs2 + 2.0d0*ai_ay_pxs1 - 2.0d0*ai_ay_pxs2
ai_az_pxs2 = oo2zeta*az_pxs2
az_dxxpx1 = PBx*az_dxxs1 - PCx*az_dxxs2 + 2.0d0*ai_az_pxs1 - 2.0d0*ai_az_pxs2
ax_dxxpy1 = PBy*ax_dxxs1 - PCy*ax_dxxs2
ay_dxxpy1 = PBy*ay_dxxs1 - PCy*ay_dxxs2 + a_dxxs2
az_dxxpy1 = PBy*az_dxxs1 - PCy*az_dxxs2
ax_dxxpz1 = PBz*ax_dxxs1 - PCz*ax_dxxs2
ay_dxxpz1 = PBz*ay_dxxs1 - PCz*ay_dxxs2
az_dxxpz1 = PBz*az_dxxs1 - PCz*az_dxxs2 + a_dxxs2
ax_dyypx1 = PBx*ax_dyys1 - PCx*ax_dyys2 + a_dyys2
ay_dyypx1 = PBx*ay_dyys1 - PCx*ay_dyys2
az_dyypx1 = PBx*az_dyys1 - PCx*az_dyys2
ai_ax_pys2 = oo2zeta*ax_pys2
ax_dyypy1 = PBy*ax_dyys1 - PCy*ax_dyys2 + 2.0d0*ai_ax_pys1 - 2.0d0*ai_ax_pys2
ai_ay_pys2 = oo2zeta*ay_pys2
ay_dyypy1 = PBy*ay_dyys1 - PCy*ay_dyys2 + 2.0d0*ai_ay_pys1 - 2.0d0*ai_ay_pys2 + a_dyys2
ai_az_pys2 = oo2zeta*az_pys2
az_dyypy1 = PBy*az_dyys1 - PCy*az_dyys2 + 2.0d0*ai_az_pys1 - 2.0d0*ai_az_pys2
ax_dyypz1 = PBz*ax_dyys1 - PCz*ax_dyys2
ay_dyypz1 = PBz*ay_dyys1 - PCz*ay_dyys2
az_dyypz1 = PBz*az_dyys1 - PCz*az_dyys2 + a_dyys2
ax_dzzpx1 = PBx*ax_dzzs1 - PCx*ax_dzzs2 + a_dzzs2
ay_dzzpx1 = PBx*ay_dzzs1 - PCx*ay_dzzs2
az_dzzpx1 = PBx*az_dzzs1 - PCx*az_dzzs2
ax_dzzpy1 = PBy*ax_dzzs1 - PCy*ax_dzzs2
ay_dzzpy1 = PBy*ay_dzzs1 - PCy*ay_dzzs2 + a_dzzs2
az_dzzpy1 = PBy*az_dzzs1 - PCy*az_dzzs2
ai_ax_pzs2 = oo2zeta*ax_pzs2
ax_dzzpz1 = PBz*ax_dzzs1 - PCz*ax_dzzs2 + 2.0d0*ai_ax_pzs1 - 2.0d0*ai_ax_pzs2
ai_ay_pzs2 = oo2zeta*ay_pzs2
ay_dzzpz1 = PBz*ay_dzzs1 - PCz*ay_dzzs2 + 2.0d0*ai_ay_pzs1 - 2.0d0*ai_ay_pzs2
ai_az_pzs2 = oo2zeta*az_pzs2
az_dzzpz1 = PBz*az_dzzs1 - PCz*az_dzzs2 + 2.0d0*ai_az_pzs1 - 2.0d0*ai_az_pzs2 + a_dzzs2
ax_dxypx1 = PBx*ax_dxys1 - PCx*ax_dxys2 + ai_ax_pys1 - ai_ax_pys2 + a_dxys2
ay_dxypx1 = PBx*ay_dxys1 - PCx*ay_dxys2 + ai_ay_pys1 - ai_ay_pys2
az_dxypx1 = PBx*az_dxys1 - PCx*az_dxys2 + ai_az_pys1 - ai_az_pys2
ax_dxypy1 = PBy*ax_dxys1 - PCy*ax_dxys2 + ai_ax_pxs1 - ai_ax_pxs2
ay_dxypy1 = PBy*ay_dxys1 - PCy*ay_dxys2 + ai_ay_pxs1 - ai_ay_pxs2 + a_dxys2
az_dxypy1 = PBy*az_dxys1 - PCy*az_dxys2 + ai_az_pxs1 - ai_az_pxs2
ax_dxypz1 = PBz*ax_dxys1 - PCz*ax_dxys2
ay_dxypz1 = PBz*ay_dxys1 - PCz*ay_dxys2
az_dxypz1 = PBz*az_dxys1 - PCz*az_dxys2 + a_dxys2
ax_dxzpx1 = PBx*ax_dxzs1 - PCx*ax_dxzs2 + ai_ax_pzs1 - ai_ax_pzs2 + a_dxzs2
ay_dxzpx1 = PBx*ay_dxzs1 - PCx*ay_dxzs2 + ai_ay_pzs1 - ai_ay_pzs2
az_dxzpx1 = PBx*az_dxzs1 - PCx*az_dxzs2 + ai_az_pzs1 - ai_az_pzs2
ax_dxzpy1 = PBy*ax_dxzs1 - PCy*ax_dxzs2
ay_dxzpy1 = PBy*ay_dxzs1 - PCy*ay_dxzs2 + a_dxzs2
az_dxzpy1 = PBy*az_dxzs1 - PCy*az_dxzs2
ax_dxzpz1 = PBz*ax_dxzs1 - PCz*ax_dxzs2 + ai_ax_pxs1 - ai_ax_pxs2
ay_dxzpz1 = PBz*ay_dxzs1 - PCz*ay_dxzs2 + ai_ay_pxs1 - ai_ay_pxs2
az_dxzpz1 = PBz*az_dxzs1 - PCz*az_dxzs2 + ai_az_pxs1 - ai_az_pxs2 + a_dxzs2
ax_dyzpx1 = PBx*ax_dyzs1 - PCx*ax_dyzs2 + a_dyzs2
ay_dyzpx1 = PBx*ay_dyzs1 - PCx*ay_dyzs2
az_dyzpx1 = PBx*az_dyzs1 - PCx*az_dyzs2
ax_dyzpy1 = PBy*ax_dyzs1 - PCy*ax_dyzs2 + ai_ax_pzs1 - ai_ax_pzs2
ay_dyzpy1 = PBy*ay_dyzs1 - PCy*ay_dyzs2 + ai_ay_pzs1 - ai_ay_pzs2 + a_dyzs2
az_dyzpy1 = PBy*az_dyzs1 - PCy*az_dyzs2 + ai_az_pzs1 - ai_az_pzs2
ax_dyzpz1 = PBz*ax_dyzs1 - PCz*ax_dyzs2 + ai_ax_pys1 - ai_ax_pys2
ay_dyzpz1 = PBz*ay_dyzs1 - PCz*ay_dyzs2 + ai_ay_pys1 - ai_ay_pys2
az_dyzpz1 = PBz*az_dyzs1 - PCz*az_dyzs2 + ai_az_pys1 - ai_az_pys2 + a_dyzs2
ax_pxpx0 = PBx*ax_pxs0 - PCx*ax_pxs1 + ai_ax_ss0 - ai_ax_ss1 + a_pxs1
ay_pxpx0 = PBx*ay_pxs0 - PCx*ay_pxs1 + ai_ay_ss0 - ai_ay_ss1
az_pxpx0 = PBx*az_pxs0 - PCx*az_pxs1 + ai_az_ss0 - ai_az_ss1
ax_pxpy0 = PBy*ax_pxs0 - PCy*ax_pxs1
ay_pxpy0 = PBy*ay_pxs0 - PCy*ay_pxs1 + a_pxs1
az_pxpy0 = PBy*az_pxs0 - PCy*az_pxs1
ax_pxpz0 = PBz*ax_pxs0 - PCz*ax_pxs1
ay_pxpz0 = PBz*ay_pxs0 - PCz*ay_pxs1
az_pxpz0 = PBz*az_pxs0 - PCz*az_pxs1 + a_pxs1
ax_pypx0 = PBx*ax_pys0 - PCx*ax_pys1 + a_pys1
ay_pypx0 = PBx*ay_pys0 - PCx*ay_pys1
az_pypx0 = PBx*az_pys0 - PCx*az_pys1
ax_pypy0 = PBy*ax_pys0 - PCy*ax_pys1 + ai_ax_ss0 - ai_ax_ss1
ay_pypy0 = PBy*ay_pys0 - PCy*ay_pys1 + ai_ay_ss0 - ai_ay_ss1 + a_pys1
az_pypy0 = PBy*az_pys0 - PCy*az_pys1 + ai_az_ss0 - ai_az_ss1
ax_pypz0 = PBz*ax_pys0 - PCz*ax_pys1
ay_pypz0 = PBz*ay_pys0 - PCz*ay_pys1
az_pypz0 = PBz*az_pys0 - PCz*az_pys1 + a_pys1
ax_pzpx0 = PBx*ax_pzs0 - PCx*ax_pzs1 + a_pzs1
ay_pzpx0 = PBx*ay_pzs0 - PCx*ay_pzs1
az_pzpx0 = PBx*az_pzs0 - PCx*az_pzs1
ax_pzpy0 = PBy*ax_pzs0 - PCy*ax_pzs1
ay_pzpy0 = PBy*ay_pzs0 - PCy*ay_pzs1 + a_pzs1
az_pzpy0 = PBy*az_pzs0 - PCy*az_pzs1
ax_pzpz0 = PBz*ax_pzs0 - PCz*ax_pzs1 + ai_ax_ss0 - ai_ax_ss1
ay_pzpz0 = PBz*ay_pzs0 - PCz*ay_pzs1 + ai_ay_ss0 - ai_ay_ss1
az_pzpz0 = PBz*az_pzs0 - PCz*az_pzs1 + ai_az_ss0 - ai_az_ss1 + a_pzs1
ax_pxpx1 = PBx*ax_pxs1 - PCx*ax_pxs2 + ai_ax_ss1 - ai_ax_ss2 + a_pxs2
ay_pxpx1 = PBx*ay_pxs1 - PCx*ay_pxs2 + ai_ay_ss1 - ai_ay_ss2
az_pxpx1 = PBx*az_pxs1 - PCx*az_pxs2 + ai_az_ss1 - ai_az_ss2
ax_pxpy1 = PBy*ax_pxs1 - PCy*ax_pxs2
ay_pxpy1 = PBy*ay_pxs1 - PCy*ay_pxs2 + a_pxs2
az_pxpy1 = PBy*az_pxs1 - PCy*az_pxs2
ax_pxpz1 = PBz*ax_pxs1 - PCz*ax_pxs2
ay_pxpz1 = PBz*ay_pxs1 - PCz*ay_pxs2
az_pxpz1 = PBz*az_pxs1 - PCz*az_pxs2 + a_pxs2
ax_pypx1 = PBx*ax_pys1 - PCx*ax_pys2 + a_pys2
ay_pypx1 = PBx*ay_pys1 - PCx*ay_pys2
az_pypx1 = PBx*az_pys1 - PCx*az_pys2
ax_pypy1 = PBy*ax_pys1 - PCy*ax_pys2 + ai_ax_ss1 - ai_ax_ss2
ay_pypy1 = PBy*ay_pys1 - PCy*ay_pys2 + ai_ay_ss1 - ai_ay_ss2 + a_pys2
az_pypy1 = PBy*az_pys1 - PCy*az_pys2 + ai_az_ss1 - ai_az_ss2
ax_pypz1 = PBz*ax_pys1 - PCz*ax_pys2
ay_pypz1 = PBz*ay_pys1 - PCz*ay_pys2
az_pypz1 = PBz*az_pys1 - PCz*az_pys2 + a_pys2
ax_pzpx1 = PBx*ax_pzs1 - PCx*ax_pzs2 + a_pzs2
ay_pzpx1 = PBx*ay_pzs1 - PCx*ay_pzs2
az_pzpx1 = PBx*az_pzs1 - PCx*az_pzs2
ax_pzpy1 = PBy*ax_pzs1 - PCy*ax_pzs2
ay_pzpy1 = PBy*ay_pzs1 - PCy*ay_pzs2 + a_pzs2
az_pzpy1 = PBy*az_pzs1 - PCy*az_pzs2
ax_pzpz1 = PBz*ax_pzs1 - PCz*ax_pzs2 + ai_ax_ss1 - ai_ax_ss2
ay_pzpz1 = PBz*ay_pzs1 - PCz*ay_pzs2 + ai_ay_ss1 - ai_ay_ss2
az_pzpz1 = PBz*az_pzs1 - PCz*az_pzs2 + ai_az_ss1 - ai_az_ss2 + a_pzs2
ai_a_pxs1 = oo2zeta*a_pxs1
ai_a_pxs2 = oo2zeta*a_pxs2
a_dxxpx1 = PBx*a_dxxs1 - PCx*a_dxxs2 + 2.0d0*ai_a_pxs1 - 2.0d0*ai_a_pxs2
a_dxxpy1 = PBy*a_dxxs1 - PCy*a_dxxs2
a_dxxpz1 = PBz*a_dxxs1 - PCz*a_dxxs2
a_dyypx1 = PBx*a_dyys1 - PCx*a_dyys2
ai_a_pys1 = oo2zeta*a_pys1
ai_a_pys2 = oo2zeta*a_pys2
a_dyypy1 = PBy*a_dyys1 - PCy*a_dyys2 + 2.0d0*ai_a_pys1 - 2.0d0*ai_a_pys2
a_dyypz1 = PBz*a_dyys1 - PCz*a_dyys2
a_dzzpx1 = PBx*a_dzzs1 - PCx*a_dzzs2
a_dzzpy1 = PBy*a_dzzs1 - PCy*a_dzzs2
ai_a_pzs1 = oo2zeta*a_pzs1
ai_a_pzs2 = oo2zeta*a_pzs2
a_dzzpz1 = PBz*a_dzzs1 - PCz*a_dzzs2 + 2.0d0*ai_a_pzs1 - 2.0d0*ai_a_pzs2
a_dxypx1 = PBx*a_dxys1 - PCx*a_dxys2 + ai_a_pys1 - ai_a_pys2
a_dxypy1 = PBy*a_dxys1 - PCy*a_dxys2 + ai_a_pxs1 - ai_a_pxs2
a_dxypz1 = PBz*a_dxys1 - PCz*a_dxys2
a_dxzpx1 = PBx*a_dxzs1 - PCx*a_dxzs2 + ai_a_pzs1 - ai_a_pzs2
a_dxzpy1 = PBy*a_dxzs1 - PCy*a_dxzs2
a_dxzpz1 = PBz*a_dxzs1 - PCz*a_dxzs2 + ai_a_pxs1 - ai_a_pxs2
a_dyzpx1 = PBx*a_dyzs1 - PCx*a_dyzs2
a_dyzpy1 = PBy*a_dyzs1 - PCy*a_dyzs2 + ai_a_pzs1 - ai_a_pzs2
a_dyzpz1 = PBz*a_dyzs1 - PCz*a_dyzs2 + ai_a_pys1 - ai_a_pys2
ai_ax_pxpx0 = oo2zeta*ax_pxpx0
ai_ax_pxpx1 = oo2zeta*ax_pxpx1
bi_ax_dxxs0 = oo2zeta*ax_dxxs0
bi_ax_dxxs1 = oo2zeta*ax_dxxs1
ax_dxxdxx0 = PBx*ax_dxxpx0 - PCx*ax_dxxpx1 + 2.0d0*ai_ax_pxpx0 - 2.0d0*ai_ax_pxpx1 + bi_ax_dxxs0 - bi_ax_dxxs1 + a_dxxpx1
ai_ay_pxpx0 = oo2zeta*ay_pxpx0
ai_ay_pxpx1 = oo2zeta*ay_pxpx1
bi_ay_dxxs0 = oo2zeta*ay_dxxs0
bi_ay_dxxs1 = oo2zeta*ay_dxxs1
ay_dxxdxx0 = PBx*ay_dxxpx0 - PCx*ay_dxxpx1 + 2.0d0*ai_ay_pxpx0 - 2.0d0*ai_ay_pxpx1 + bi_ay_dxxs0 - bi_ay_dxxs1
ai_az_pxpx0 = oo2zeta*az_pxpx0
ai_az_pxpx1 = oo2zeta*az_pxpx1
bi_az_dxxs0 = oo2zeta*az_dxxs0
bi_az_dxxs1 = oo2zeta*az_dxxs1
az_dxxdxx0 = PBx*az_dxxpx0 - PCx*az_dxxpx1 + 2.0d0*ai_az_pxpx0 - 2.0d0*ai_az_pxpx1 + bi_az_dxxs0 - bi_az_dxxs1
ax_dxxdyy0 = PBy*ax_dxxpy0 - PCy*ax_dxxpy1 + bi_ax_dxxs0 - bi_ax_dxxs1
ay_dxxdyy0 = PBy*ay_dxxpy0 - PCy*ay_dxxpy1 + bi_ay_dxxs0 - bi_ay_dxxs1 + a_dxxpy1
az_dxxdyy0 = PBy*az_dxxpy0 - PCy*az_dxxpy1 + bi_az_dxxs0 - bi_az_dxxs1
ax_dxxdzz0 = PBz*ax_dxxpz0 - PCz*ax_dxxpz1 + bi_ax_dxxs0 - bi_ax_dxxs1
ay_dxxdzz0 = PBz*ay_dxxpz0 - PCz*ay_dxxpz1 + bi_ay_dxxs0 - bi_ay_dxxs1
az_dxxdzz0 = PBz*az_dxxpz0 - PCz*az_dxxpz1 + bi_az_dxxs0 - bi_az_dxxs1 + a_dxxpz1
ax_dxxdxy0 = PBy*ax_dxxpx0 - PCy*ax_dxxpx1
ay_dxxdxy0 = PBy*ay_dxxpx0 - PCy*ay_dxxpx1 + a_dxxpx1
az_dxxdxy0 = PBy*az_dxxpx0 - PCy*az_dxxpx1
ax_dxxdxz0 = PBz*ax_dxxpx0 - PCz*ax_dxxpx1
ay_dxxdxz0 = PBz*ay_dxxpx0 - PCz*ay_dxxpx1
az_dxxdxz0 = PBz*az_dxxpx0 - PCz*az_dxxpx1 + a_dxxpx1
ax_dxxdyz0 = PBz*ax_dxxpy0 - PCz*ax_dxxpy1
ay_dxxdyz0 = PBz*ay_dxxpy0 - PCz*ay_dxxpy1
az_dxxdyz0 = PBz*az_dxxpy0 - PCz*az_dxxpy1 + a_dxxpy1
bi_ax_dyys0 = oo2zeta*ax_dyys0
bi_ax_dyys1 = oo2zeta*ax_dyys1
ax_dyydxx0 = PBx*ax_dyypx0 - PCx*ax_dyypx1 + bi_ax_dyys0 - bi_ax_dyys1 + a_dyypx1
bi_ay_dyys0 = oo2zeta*ay_dyys0
bi_ay_dyys1 = oo2zeta*ay_dyys1
ay_dyydxx0 = PBx*ay_dyypx0 - PCx*ay_dyypx1 + bi_ay_dyys0 - bi_ay_dyys1
bi_az_dyys0 = oo2zeta*az_dyys0
bi_az_dyys1 = oo2zeta*az_dyys1
az_dyydxx0 = PBx*az_dyypx0 - PCx*az_dyypx1 + bi_az_dyys0 - bi_az_dyys1
ai_ax_pypy0 = oo2zeta*ax_pypy0
ai_ax_pypy1 = oo2zeta*ax_pypy1
ax_dyydyy0 = PBy*ax_dyypy0 - PCy*ax_dyypy1 + 2.0d0*ai_ax_pypy0 - 2.0d0*ai_ax_pypy1 + bi_ax_dyys0 - bi_ax_dyys1
ai_ay_pypy0 = oo2zeta*ay_pypy0
ai_ay_pypy1 = oo2zeta*ay_pypy1
ay_dyydyy0 = PBy*ay_dyypy0 - PCy*ay_dyypy1 + 2.0d0*ai_ay_pypy0 - 2.0d0*ai_ay_pypy1 + bi_ay_dyys0 - bi_ay_dyys1 + a_dyypy1
ai_az_pypy0 = oo2zeta*az_pypy0
ai_az_pypy1 = oo2zeta*az_pypy1
az_dyydyy0 = PBy*az_dyypy0 - PCy*az_dyypy1 + 2.0d0*ai_az_pypy0 - 2.0d0*ai_az_pypy1 + bi_az_dyys0 - bi_az_dyys1
ax_dyydzz0 = PBz*ax_dyypz0 - PCz*ax_dyypz1 + bi_ax_dyys0 - bi_ax_dyys1
ay_dyydzz0 = PBz*ay_dyypz0 - PCz*ay_dyypz1 + bi_ay_dyys0 - bi_ay_dyys1
az_dyydzz0 = PBz*az_dyypz0 - PCz*az_dyypz1 + bi_az_dyys0 - bi_az_dyys1 + a_dyypz1
ai_ax_pypx0 = oo2zeta*ax_pypx0
ai_ax_pypx1 = oo2zeta*ax_pypx1
ax_dyydxy0 = PBy*ax_dyypx0 - PCy*ax_dyypx1 + 2.0d0*ai_ax_pypx0 - 2.0d0*ai_ax_pypx1
ai_ay_pypx0 = oo2zeta*ay_pypx0
ai_ay_pypx1 = oo2zeta*ay_pypx1
ay_dyydxy0 = PBy*ay_dyypx0 - PCy*ay_dyypx1 + 2.0d0*ai_ay_pypx0 - 2.0d0*ai_ay_pypx1 + a_dyypx1
ai_az_pypx0 = oo2zeta*az_pypx0
ai_az_pypx1 = oo2zeta*az_pypx1
az_dyydxy0 = PBy*az_dyypx0 - PCy*az_dyypx1 + 2.0d0*ai_az_pypx0 - 2.0d0*ai_az_pypx1
ax_dyydxz0 = PBz*ax_dyypx0 - PCz*ax_dyypx1
ay_dyydxz0 = PBz*ay_dyypx0 - PCz*ay_dyypx1
az_dyydxz0 = PBz*az_dyypx0 - PCz*az_dyypx1 + a_dyypx1
ax_dyydyz0 = PBz*ax_dyypy0 - PCz*ax_dyypy1
ay_dyydyz0 = PBz*ay_dyypy0 - PCz*ay_dyypy1
az_dyydyz0 = PBz*az_dyypy0 - PCz*az_dyypy1 + a_dyypy1
bi_ax_dzzs0 = oo2zeta*ax_dzzs0
bi_ax_dzzs1 = oo2zeta*ax_dzzs1
ax_dzzdxx0 = PBx*ax_dzzpx0 - PCx*ax_dzzpx1 + bi_ax_dzzs0 - bi_ax_dzzs1 + a_dzzpx1
bi_ay_dzzs0 = oo2zeta*ay_dzzs0
bi_ay_dzzs1 = oo2zeta*ay_dzzs1
ay_dzzdxx0 = PBx*ay_dzzpx0 - PCx*ay_dzzpx1 + bi_ay_dzzs0 - bi_ay_dzzs1
bi_az_dzzs0 = oo2zeta*az_dzzs0
bi_az_dzzs1 = oo2zeta*az_dzzs1
az_dzzdxx0 = PBx*az_dzzpx0 - PCx*az_dzzpx1 + bi_az_dzzs0 - bi_az_dzzs1
ax_dzzdyy0 = PBy*ax_dzzpy0 - PCy*ax_dzzpy1 + bi_ax_dzzs0 - bi_ax_dzzs1
ay_dzzdyy0 = PBy*ay_dzzpy0 - PCy*ay_dzzpy1 + bi_ay_dzzs0 - bi_ay_dzzs1 + a_dzzpy1
az_dzzdyy0 = PBy*az_dzzpy0 - PCy*az_dzzpy1 + bi_az_dzzs0 - bi_az_dzzs1
ai_ax_pzpz0 = oo2zeta*ax_pzpz0
ai_ax_pzpz1 = oo2zeta*ax_pzpz1
ax_dzzdzz0 = PBz*ax_dzzpz0 - PCz*ax_dzzpz1 + 2.0d0*ai_ax_pzpz0 - 2.0d0*ai_ax_pzpz1 + bi_ax_dzzs0 - bi_ax_dzzs1
ai_ay_pzpz0 = oo2zeta*ay_pzpz0
ai_ay_pzpz1 = oo2zeta*ay_pzpz1
ay_dzzdzz0 = PBz*ay_dzzpz0 - PCz*ay_dzzpz1 + 2.0d0*ai_ay_pzpz0 - 2.0d0*ai_ay_pzpz1 + bi_ay_dzzs0 - bi_ay_dzzs1
ai_az_pzpz0 = oo2zeta*az_pzpz0
ai_az_pzpz1 = oo2zeta*az_pzpz1
az_dzzdzz0 = PBz*az_dzzpz0 - PCz*az_dzzpz1 + 2.0d0*ai_az_pzpz0 - 2.0d0*ai_az_pzpz1 + bi_az_dzzs0 - bi_az_dzzs1 + a_dzzpz1
ax_dzzdxy0 = PBy*ax_dzzpx0 - PCy*ax_dzzpx1
ay_dzzdxy0 = PBy*ay_dzzpx0 - PCy*ay_dzzpx1 + a_dzzpx1
az_dzzdxy0 = PBy*az_dzzpx0 - PCy*az_dzzpx1
ai_ax_pzpx0 = oo2zeta*ax_pzpx0
ai_ax_pzpx1 = oo2zeta*ax_pzpx1
ax_dzzdxz0 = PBz*ax_dzzpx0 - PCz*ax_dzzpx1 + 2.0d0*ai_ax_pzpx0 - 2.0d0*ai_ax_pzpx1
ai_ay_pzpx0 = oo2zeta*ay_pzpx0
ai_ay_pzpx1 = oo2zeta*ay_pzpx1
ay_dzzdxz0 = PBz*ay_dzzpx0 - PCz*ay_dzzpx1 + 2.0d0*ai_ay_pzpx0 - 2.0d0*ai_ay_pzpx1
ai_az_pzpx0 = oo2zeta*az_pzpx0
ai_az_pzpx1 = oo2zeta*az_pzpx1
az_dzzdxz0 = PBz*az_dzzpx0 - PCz*az_dzzpx1 + 2.0d0*ai_az_pzpx0 - 2.0d0*ai_az_pzpx1 + a_dzzpx1
ai_ax_pzpy0 = oo2zeta*ax_pzpy0
ai_ax_pzpy1 = oo2zeta*ax_pzpy1
ax_dzzdyz0 = PBz*ax_dzzpy0 - PCz*ax_dzzpy1 + 2.0d0*ai_ax_pzpy0 - 2.0d0*ai_ax_pzpy1
ai_ay_pzpy0 = oo2zeta*ay_pzpy0
ai_ay_pzpy1 = oo2zeta*ay_pzpy1
ay_dzzdyz0 = PBz*ay_dzzpy0 - PCz*ay_dzzpy1 + 2.0d0*ai_ay_pzpy0 - 2.0d0*ai_ay_pzpy1
ai_az_pzpy0 = oo2zeta*az_pzpy0
ai_az_pzpy1 = oo2zeta*az_pzpy1
az_dzzdyz0 = PBz*az_dzzpy0 - PCz*az_dzzpy1 + 2.0d0*ai_az_pzpy0 - 2.0d0*ai_az_pzpy1 + a_dzzpy1
bi_ax_dxys0 = oo2zeta*ax_dxys0
bi_ax_dxys1 = oo2zeta*ax_dxys1
ax_dxydxx0 = PBx*ax_dxypx0 - PCx*ax_dxypx1 + ai_ax_pypx0 - ai_ax_pypx1 + bi_ax_dxys0 - bi_ax_dxys1 + a_dxypx1
bi_ay_dxys0 = oo2zeta*ay_dxys0
bi_ay_dxys1 = oo2zeta*ay_dxys1
ay_dxydxx0 = PBx*ay_dxypx0 - PCx*ay_dxypx1 + ai_ay_pypx0 - ai_ay_pypx1 + bi_ay_dxys0 - bi_ay_dxys1
bi_az_dxys0 = oo2zeta*az_dxys0
bi_az_dxys1 = oo2zeta*az_dxys1
az_dxydxx0 = PBx*az_dxypx0 - PCx*az_dxypx1 + ai_az_pypx0 - ai_az_pypx1 + bi_az_dxys0 - bi_az_dxys1
ai_ax_pxpy0 = oo2zeta*ax_pxpy0
ai_ax_pxpy1 = oo2zeta*ax_pxpy1
ax_dxydyy0 = PBy*ax_dxypy0 - PCy*ax_dxypy1 + ai_ax_pxpy0 - ai_ax_pxpy1 + bi_ax_dxys0 - bi_ax_dxys1
ai_ay_pxpy0 = oo2zeta*ay_pxpy0
ai_ay_pxpy1 = oo2zeta*ay_pxpy1
ay_dxydyy0 = PBy*ay_dxypy0 - PCy*ay_dxypy1 + ai_ay_pxpy0 - ai_ay_pxpy1 + bi_ay_dxys0 - bi_ay_dxys1 + a_dxypy1
ai_az_pxpy0 = oo2zeta*az_pxpy0
ai_az_pxpy1 = oo2zeta*az_pxpy1
az_dxydyy0 = PBy*az_dxypy0 - PCy*az_dxypy1 + ai_az_pxpy0 - ai_az_pxpy1 + bi_az_dxys0 - bi_az_dxys1
ax_dxydzz0 = PBz*ax_dxypz0 - PCz*ax_dxypz1 + bi_ax_dxys0 - bi_ax_dxys1
ay_dxydzz0 = PBz*ay_dxypz0 - PCz*ay_dxypz1 + bi_ay_dxys0 - bi_ay_dxys1
az_dxydzz0 = PBz*az_dxypz0 - PCz*az_dxypz1 + bi_az_dxys0 - bi_az_dxys1 + a_dxypz1
ax_dxydxy0 = PBy*ax_dxypx0 - PCy*ax_dxypx1 + ai_ax_pxpx0 - ai_ax_pxpx1
ay_dxydxy0 = PBy*ay_dxypx0 - PCy*ay_dxypx1 + ai_ay_pxpx0 - ai_ay_pxpx1 + a_dxypx1
az_dxydxy0 = PBy*az_dxypx0 - PCy*az_dxypx1 + ai_az_pxpx0 - ai_az_pxpx1
ax_dxydxz0 = PBz*ax_dxypx0 - PCz*ax_dxypx1
ay_dxydxz0 = PBz*ay_dxypx0 - PCz*ay_dxypx1
az_dxydxz0 = PBz*az_dxypx0 - PCz*az_dxypx1 + a_dxypx1
ax_dxydyz0 = PBz*ax_dxypy0 - PCz*ax_dxypy1
ay_dxydyz0 = PBz*ay_dxypy0 - PCz*ay_dxypy1
az_dxydyz0 = PBz*az_dxypy0 - PCz*az_dxypy1 + a_dxypy1
bi_ax_dxzs0 = oo2zeta*ax_dxzs0
bi_ax_dxzs1 = oo2zeta*ax_dxzs1
ax_dxzdxx0 = PBx*ax_dxzpx0 - PCx*ax_dxzpx1 + ai_ax_pzpx0 - ai_ax_pzpx1 + bi_ax_dxzs0 - bi_ax_dxzs1 + a_dxzpx1
bi_ay_dxzs0 = oo2zeta*ay_dxzs0
bi_ay_dxzs1 = oo2zeta*ay_dxzs1
ay_dxzdxx0 = PBx*ay_dxzpx0 - PCx*ay_dxzpx1 + ai_ay_pzpx0 - ai_ay_pzpx1 + bi_ay_dxzs0 - bi_ay_dxzs1
bi_az_dxzs0 = oo2zeta*az_dxzs0
bi_az_dxzs1 = oo2zeta*az_dxzs1
az_dxzdxx0 = PBx*az_dxzpx0 - PCx*az_dxzpx1 + ai_az_pzpx0 - ai_az_pzpx1 + bi_az_dxzs0 - bi_az_dxzs1
ax_dxzdyy0 = PBy*ax_dxzpy0 - PCy*ax_dxzpy1 + bi_ax_dxzs0 - bi_ax_dxzs1
ay_dxzdyy0 = PBy*ay_dxzpy0 - PCy*ay_dxzpy1 + bi_ay_dxzs0 - bi_ay_dxzs1 + a_dxzpy1
az_dxzdyy0 = PBy*az_dxzpy0 - PCy*az_dxzpy1 + bi_az_dxzs0 - bi_az_dxzs1
ai_ax_pxpz0 = oo2zeta*ax_pxpz0
ai_ax_pxpz1 = oo2zeta*ax_pxpz1
ax_dxzdzz0 = PBz*ax_dxzpz0 - PCz*ax_dxzpz1 + ai_ax_pxpz0 - ai_ax_pxpz1 + bi_ax_dxzs0 - bi_ax_dxzs1
ai_ay_pxpz0 = oo2zeta*ay_pxpz0
ai_ay_pxpz1 = oo2zeta*ay_pxpz1
ay_dxzdzz0 = PBz*ay_dxzpz0 - PCz*ay_dxzpz1 + ai_ay_pxpz0 - ai_ay_pxpz1 + bi_ay_dxzs0 - bi_ay_dxzs1
ai_az_pxpz0 = oo2zeta*az_pxpz0
ai_az_pxpz1 = oo2zeta*az_pxpz1
az_dxzdzz0 = PBz*az_dxzpz0 - PCz*az_dxzpz1 + ai_az_pxpz0 - ai_az_pxpz1 + bi_az_dxzs0 - bi_az_dxzs1 + a_dxzpz1
ax_dxzdxy0 = PBy*ax_dxzpx0 - PCy*ax_dxzpx1
ay_dxzdxy0 = PBy*ay_dxzpx0 - PCy*ay_dxzpx1 + a_dxzpx1
az_dxzdxy0 = PBy*az_dxzpx0 - PCy*az_dxzpx1
ax_dxzdxz0 = PBz*ax_dxzpx0 - PCz*ax_dxzpx1 + ai_ax_pxpx0 - ai_ax_pxpx1
ay_dxzdxz0 = PBz*ay_dxzpx0 - PCz*ay_dxzpx1 + ai_ay_pxpx0 - ai_ay_pxpx1
az_dxzdxz0 = PBz*az_dxzpx0 - PCz*az_dxzpx1 + ai_az_pxpx0 - ai_az_pxpx1 + a_dxzpx1
ax_dxzdyz0 = PBz*ax_dxzpy0 - PCz*ax_dxzpy1 + ai_ax_pxpy0 - ai_ax_pxpy1
ay_dxzdyz0 = PBz*ay_dxzpy0 - PCz*ay_dxzpy1 + ai_ay_pxpy0 - ai_ay_pxpy1
az_dxzdyz0 = PBz*az_dxzpy0 - PCz*az_dxzpy1 + ai_az_pxpy0 - ai_az_pxpy1 + a_dxzpy1
bi_ax_dyzs0 = oo2zeta*ax_dyzs0
bi_ax_dyzs1 = oo2zeta*ax_dyzs1
ax_dyzdxx0 = PBx*ax_dyzpx0 - PCx*ax_dyzpx1 + bi_ax_dyzs0 - bi_ax_dyzs1 + a_dyzpx1
bi_ay_dyzs0 = oo2zeta*ay_dyzs0
bi_ay_dyzs1 = oo2zeta*ay_dyzs1
ay_dyzdxx0 = PBx*ay_dyzpx0 - PCx*ay_dyzpx1 + bi_ay_dyzs0 - bi_ay_dyzs1
bi_az_dyzs0 = oo2zeta*az_dyzs0
bi_az_dyzs1 = oo2zeta*az_dyzs1
az_dyzdxx0 = PBx*az_dyzpx0 - PCx*az_dyzpx1 + bi_az_dyzs0 - bi_az_dyzs1
ax_dyzdyy0 = PBy*ax_dyzpy0 - PCy*ax_dyzpy1 + ai_ax_pzpy0 - ai_ax_pzpy1 + bi_ax_dyzs0 - bi_ax_dyzs1
ay_dyzdyy0 = PBy*ay_dyzpy0 - PCy*ay_dyzpy1 + ai_ay_pzpy0 - ai_ay_pzpy1 + bi_ay_dyzs0 - bi_ay_dyzs1 + a_dyzpy1
az_dyzdyy0 = PBy*az_dyzpy0 - PCy*az_dyzpy1 + ai_az_pzpy0 - ai_az_pzpy1 + bi_az_dyzs0 - bi_az_dyzs1
ai_ax_pypz0 = oo2zeta*ax_pypz0
ai_ax_pypz1 = oo2zeta*ax_pypz1
ax_dyzdzz0 = PBz*ax_dyzpz0 - PCz*ax_dyzpz1 + ai_ax_pypz0 - ai_ax_pypz1 + bi_ax_dyzs0 - bi_ax_dyzs1
ai_ay_pypz0 = oo2zeta*ay_pypz0
ai_ay_pypz1 = oo2zeta*ay_pypz1
ay_dyzdzz0 = PBz*ay_dyzpz0 - PCz*ay_dyzpz1 + ai_ay_pypz0 - ai_ay_pypz1 + bi_ay_dyzs0 - bi_ay_dyzs1
ai_az_pypz0 = oo2zeta*az_pypz0
ai_az_pypz1 = oo2zeta*az_pypz1
az_dyzdzz0 = PBz*az_dyzpz0 - PCz*az_dyzpz1 + ai_az_pypz0 - ai_az_pypz1 + bi_az_dyzs0 - bi_az_dyzs1 + a_dyzpz1
ax_dyzdxy0 = PBy*ax_dyzpx0 - PCy*ax_dyzpx1 + ai_ax_pzpx0 - ai_ax_pzpx1
ay_dyzdxy0 = PBy*ay_dyzpx0 - PCy*ay_dyzpx1 + ai_ay_pzpx0 - ai_ay_pzpx1 + a_dyzpx1
az_dyzdxy0 = PBy*az_dyzpx0 - PCy*az_dyzpx1 + ai_az_pzpx0 - ai_az_pzpx1
ax_dyzdxz0 = PBz*ax_dyzpx0 - PCz*ax_dyzpx1 + ai_ax_pypx0 - ai_ax_pypx1
ay_dyzdxz0 = PBz*ay_dyzpx0 - PCz*ay_dyzpx1 + ai_ay_pypx0 - ai_ay_pypx1
az_dyzdxz0 = PBz*az_dyzpx0 - PCz*az_dyzpx1 + ai_az_pypx0 - ai_az_pypx1 + a_dyzpx1
ax_dyzdyz0 = PBz*ax_dyzpy0 - PCz*ax_dyzpy1 + ai_ax_pypy0 - ai_ax_pypy1
ay_dyzdyz0 = PBz*ay_dyzpy0 - PCz*ay_dyzpy1 + ai_ay_pypy0 - ai_ay_pypy1
az_dyzdyz0 = PBz*az_dyzpy0 - PCz*az_dyzpy1 + ai_az_pypy0 - ai_az_pypy1 + a_dyzpy1

v(1,1,k) = v(1,1,k) + ax_dxxdxx0
v(2,1,k) = v(2,1,k) + ax_dxxdyy0
v(3,1,k) = v(3,1,k) + ax_dxxdzz0
v(4,1,k) = v(4,1,k) + ax_dxxdxy0*root3
v(5,1,k) = v(5,1,k) + ax_dxxdxz0*root3
v(6,1,k) = v(6,1,k) + ax_dxxdyz0*root3
v(7,1,k) = v(7,1,k) + ax_dyydxx0
v(8,1,k) = v(8,1,k) + ax_dyydyy0
v(9,1,k) = v(9,1,k) + ax_dyydzz0
v(10,1,k) = v(10,1,k) + ax_dyydxy0*root3
v(11,1,k) = v(11,1,k) + ax_dyydxz0*root3
v(12,1,k) = v(12,1,k) + ax_dyydyz0*root3
v(13,1,k) = v(13,1,k) + ax_dzzdxx0
v(14,1,k) = v(14,1,k) + ax_dzzdyy0
v(15,1,k) = v(15,1,k) + ax_dzzdzz0
v(16,1,k) = v(16,1,k) + ax_dzzdxy0*root3
v(17,1,k) = v(17,1,k) + ax_dzzdxz0*root3
v(18,1,k) = v(18,1,k) + ax_dzzdyz0*root3
v(19,1,k) = v(19,1,k) + ax_dxydxx0*root3
v(20,1,k) = v(20,1,k) + ax_dxydyy0*root3
v(21,1,k) = v(21,1,k) + ax_dxydzz0*root3
v(22,1,k) = v(22,1,k) + ax_dxydxy0*root3x2
v(23,1,k) = v(23,1,k) + ax_dxydxz0*root3x2
v(24,1,k) = v(24,1,k) + ax_dxydyz0*root3x2
v(25,1,k) = v(25,1,k) + ax_dxzdxx0*root3
v(26,1,k) = v(26,1,k) + ax_dxzdyy0*root3
v(27,1,k) = v(27,1,k) + ax_dxzdzz0*root3
v(28,1,k) = v(28,1,k) + ax_dxzdxy0*root3x2
v(29,1,k) = v(29,1,k) + ax_dxzdxz0*root3x2
v(30,1,k) = v(30,1,k) + ax_dxzdyz0*root3x2
v(31,1,k) = v(31,1,k) + ax_dyzdxx0*root3
v(32,1,k) = v(32,1,k) + ax_dyzdyy0*root3
v(33,1,k) = v(33,1,k) + ax_dyzdzz0*root3
v(34,1,k) = v(34,1,k) + ax_dyzdxy0*root3x2
v(35,1,k) = v(35,1,k) + ax_dyzdxz0*root3x2
v(36,1,k) = v(36,1,k) + ax_dyzdyz0*root3x2
v(1,2,k) = v(1,2,k) + ay_dxxdxx0
v(2,2,k) = v(2,2,k) + ay_dxxdyy0
v(3,2,k) = v(3,2,k) + ay_dxxdzz0
v(4,2,k) = v(4,2,k) + ay_dxxdxy0*root3
v(5,2,k) = v(5,2,k) + ay_dxxdxz0*root3
v(6,2,k) = v(6,2,k) + ay_dxxdyz0*root3
v(7,2,k) = v(7,2,k) + ay_dyydxx0
v(8,2,k) = v(8,2,k) + ay_dyydyy0
v(9,2,k) = v(9,2,k) + ay_dyydzz0
v(10,2,k) = v(10,2,k) + ay_dyydxy0*root3
v(11,2,k) = v(11,2,k) + ay_dyydxz0*root3
v(12,2,k) = v(12,2,k) + ay_dyydyz0*root3
v(13,2,k) = v(13,2,k) + ay_dzzdxx0
v(14,2,k) = v(14,2,k) + ay_dzzdyy0
v(15,2,k) = v(15,2,k) + ay_dzzdzz0
v(16,2,k) = v(16,2,k) + ay_dzzdxy0*root3
v(17,2,k) = v(17,2,k) + ay_dzzdxz0*root3
v(18,2,k) = v(18,2,k) + ay_dzzdyz0*root3
v(19,2,k) = v(19,2,k) + ay_dxydxx0*root3
v(20,2,k) = v(20,2,k) + ay_dxydyy0*root3
v(21,2,k) = v(21,2,k) + ay_dxydzz0*root3
v(22,2,k) = v(22,2,k) + ay_dxydxy0*root3x2
v(23,2,k) = v(23,2,k) + ay_dxydxz0*root3x2
v(24,2,k) = v(24,2,k) + ay_dxydyz0*root3x2
v(25,2,k) = v(25,2,k) + ay_dxzdxx0*root3
v(26,2,k) = v(26,2,k) + ay_dxzdyy0*root3
v(27,2,k) = v(27,2,k) + ay_dxzdzz0*root3
v(28,2,k) = v(28,2,k) + ay_dxzdxy0*root3x2
v(29,2,k) = v(29,2,k) + ay_dxzdxz0*root3x2
v(30,2,k) = v(30,2,k) + ay_dxzdyz0*root3x2
v(31,2,k) = v(31,2,k) + ay_dyzdxx0*root3
v(32,2,k) = v(32,2,k) + ay_dyzdyy0*root3
v(33,2,k) = v(33,2,k) + ay_dyzdzz0*root3
v(34,2,k) = v(34,2,k) + ay_dyzdxy0*root3x2
v(35,2,k) = v(35,2,k) + ay_dyzdxz0*root3x2
v(36,2,k) = v(36,2,k) + ay_dyzdyz0*root3x2
v(1,3,k) = v(1,3,k) + az_dxxdxx0
v(2,3,k) = v(2,3,k) + az_dxxdyy0
v(3,3,k) = v(3,3,k) + az_dxxdzz0
v(4,3,k) = v(4,3,k) + az_dxxdxy0*root3
v(5,3,k) = v(5,3,k) + az_dxxdxz0*root3
v(6,3,k) = v(6,3,k) + az_dxxdyz0*root3
v(7,3,k) = v(7,3,k) + az_dyydxx0
v(8,3,k) = v(8,3,k) + az_dyydyy0
v(9,3,k) = v(9,3,k) + az_dyydzz0
v(10,3,k) = v(10,3,k) + az_dyydxy0*root3
v(11,3,k) = v(11,3,k) + az_dyydxz0*root3
v(12,3,k) = v(12,3,k) + az_dyydyz0*root3
v(13,3,k) = v(13,3,k) + az_dzzdxx0
v(14,3,k) = v(14,3,k) + az_dzzdyy0
v(15,3,k) = v(15,3,k) + az_dzzdzz0
v(16,3,k) = v(16,3,k) + az_dzzdxy0*root3
v(17,3,k) = v(17,3,k) + az_dzzdxz0*root3
v(18,3,k) = v(18,3,k) + az_dzzdyz0*root3
v(19,3,k) = v(19,3,k) + az_dxydxx0*root3
v(20,3,k) = v(20,3,k) + az_dxydyy0*root3
v(21,3,k) = v(21,3,k) + az_dxydzz0*root3
v(22,3,k) = v(22,3,k) + az_dxydxy0*root3x2
v(23,3,k) = v(23,3,k) + az_dxydxz0*root3x2
v(24,3,k) = v(24,3,k) + az_dxydyz0*root3x2
v(25,3,k) = v(25,3,k) + az_dxzdxx0*root3
v(26,3,k) = v(26,3,k) + az_dxzdyy0*root3
v(27,3,k) = v(27,3,k) + az_dxzdzz0*root3
v(28,3,k) = v(28,3,k) + az_dxzdxy0*root3x2
v(29,3,k) = v(29,3,k) + az_dxzdxz0*root3x2
v(30,3,k) = v(30,3,k) + az_dxzdyz0*root3x2
v(31,3,k) = v(31,3,k) + az_dyzdxx0*root3
v(32,3,k) = v(32,3,k) + az_dyzdyy0*root3
v(33,3,k) = v(33,3,k) + az_dyzdzz0*root3
v(34,3,k) = v(34,3,k) + az_dyzdxy0*root3x2
v(35,3,k) = v(35,3,k) + az_dyzdxz0*root3x2
v(36,3,k) = v(36,3,k) + az_dyzdyz0*root3x2
        end do

     end do

!!$     do k = 1, nc
!!$        write(6,'(''v(dd)='',i4,108f12.7)') k,((v(ip,iq,k),ip=1,36),iq=1,3)
!!$     end do

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do jx = jao, jao + 6-1
           do ix = jx, iao + 6-1
              key = ( ix-iao ) * 6 + jx-jao +1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              do ixyz = 1, 3
                 do k = 1, nc
                    g(ixyz,k) = g(ixyz,k) + dm(ix,jx)*v(key,ixyz,k)*ci
                 end do
              end do
           end do
        end do
     else
        do jx = jao, jao + 6-1
           do ix = iao, iao + 6-1
              key = ( ix-iao ) * 6 + jx-jao +1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              do ixyz = 1, 3
                 do k = 1, nc
                    g(ixyz,k) = g(ixyz,k) + dm(ix,jx)*v(key,ixyz,k)*ci
                 end do
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_hfi_dd_v2
