! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> kinetic energy integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_kei_s1(t,nao,tables,atoms)
  use integral_tables_class
  use atoms_class
  use pl_prof_class
  integer,intent(in)::nao
  real(8),intent(inout)::t(nao,nao)
  type(tables_type),intent(in)::tables
  type(atoms_type),intent(in)::atoms

!$omp parallel&
!$omp& default(private)&
!$omp& shared(t,nao,tables,atoms)
  call pl_prof_region_begin('os_kei_ss_s1')
  call os_kei_ss_s1(&
       t,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,tables%xsi,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_kei_ss_s1')

  call pl_prof_region_begin('os_kei_ps_s1')
  call os_kei_ps_s1(&
       t,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,tables%xsi,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_kei_ps_s1')

  call pl_prof_region_begin('os_kei_pp_s1')
  call os_kei_pp_s1(&
       t,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,tables%xsi,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_kei_pp_s1')

  call pl_prof_region_begin('os_kei_ds_s1')
  call os_kei_ds_s1(&
       t,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,tables%xsi,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_kei_ds_s1')

  call pl_prof_region_begin('os_kei_dp_s1')
  call os_kei_dp_s1(&
       t,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,tables%xsi,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_kei_dp_s1')

  call pl_prof_region_begin('os_kei_dd_s1')
  call os_kei_dd_s1(&
       t,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,tables%xsi,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_kei_dd_s1')
!$omp end parallel

end subroutine os_kei_s1

!------------------------------------------------------------------------------
!> ss kinetic energy integrals
!
subroutine os_kei_ss_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,xi,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp),xi(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(1)

!$omp do schedule(dynamic)
  do ij = 1, ishp(1)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

     tss = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xi_ = xi(irs)
        xi2 = 2.d0 * xi_
        tss = tss + oss * xi_ * ( 3.d0 - xi2 * AB2 )

     end do

     !v(1) = tss

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     !val(iao,jao) = v(1)
     val(iao,jao) = tss
  end do
!$omp end do nowait
end subroutine os_kei_ss_s1

!------------------------------------------------------------------------------
!> ps kinetic energy integrals
!
subroutine os_kei_ps_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,xi,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp),xi(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(3)

!$omp do schedule(dynamic)
  do ij = ishp(1)+1, ishp(2)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

     tpxs = 0.0d0
     tpys = 0.0d0
     tpzs = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xi_ = xi(irs)
        xi2 = 2.d0 * xi_
        tss = oss * xi_ * ( 3.d0 - xi2 * AB2 )
        xioza = xiozetaa(irs)
        xiozb = 1.0d0 - xioza
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz

        opzs = oss*PAz
        opys = oss*PAy
        opxs = oss*PAx
        tpzs = tpzs + tss*PAz + opzs*xi2
        tpys = tpys + tss*PAy + opys*xi2
        tpxs = tpxs + tss*PAx + opxs*xi2
     end do

     !v(1) = tpxs
     !v(2) = tpys
     !v(3) = tpzs

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        val(jao,iao  ) = tpxs
        val(jao,iao+1) = tpys
        val(jao,iao+2) = tpzs
     else
        val(iao  ,jao) = tpxs
        val(iao+1,jao) = tpys
        val(iao+2,jao) = tpzs
     end if
  end do
!$omp end do nowait
end subroutine os_kei_ps_s1

!------------------------------------------------------------------------------
!> pp kinetic energy integrals
!
subroutine os_kei_pp_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,xi,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp),xi(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(9)

!$omp do schedule(dynamic)
  do ij = ishp(2)+1, ishp(3)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

     tpxpx = 0.0d0
     tpxpy = 0.0d0
     tpxpz = 0.0d0
     tpypx = 0.0d0
     tpypy = 0.0d0
     tpypz = 0.0d0
     tpzpx = 0.0d0
     tpzpy = 0.0d0
     tpzpz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xi_ = xi(irs)
        xi2 = 2.d0 * xi_
        tss = oss * xi_ * ( 3.d0 - xi2 * AB2 )
        xioza = xiozetaa(irs)
        xiozb = 1.0d0 - xioza
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz

        ai_oss = oo2zeta*oss
        opzs = oss*PAz
        opzpz = opzs*PBz + ai_oss
        opzpy = opzs*PBy
        opzpx = opzs*PBx
        opys = oss*PAy
        opypz = opys*PBz
        opypy = opys*PBy + ai_oss
        opypx = opys*PBx
        opxs = oss*PAx
        opxpz = opxs*PBz
        opxpy = opxs*PBy
        opxpx = opxs*PBx + ai_oss
        ai_tss = oo2zeta*tss
        tpzs = tss*PAz + opzs*xi2
        tpzpz = tpzpz + tpzs*PBz + opzpz*xi2 + ai_tss
        tpzpy = tpzpy + tpzs*PBy + opzpy*xi2
        tpzpx = tpzpx + tpzs*PBx + opzpx*xi2
        tpys = tss*PAy + opys*xi2
        tpypz = tpypz + tpys*PBz + opypz*xi2
        tpypy = tpypy + tpys*PBy + opypy*xi2 + ai_tss
        tpypx = tpypx + tpys*PBx + opypx*xi2
        tpxs = tss*PAx + opxs*xi2
        tpxpz = tpxpz + tpxs*PBz + opxpz*xi2
        tpxpy = tpxpy + tpxs*PBy + opxpy*xi2
        tpxpx = tpxpx + tpxs*PBx + opxpx*xi2 + ai_tss
     end do

     v(1) = tpxpx
     v(2) = tpxpy
     v(3) = tpxpz
     v(4) = tpypx
     v(5) = tpypy
     v(6) = tpypz
     v(7) = tpzpx
     v(8) = tpzpy
     v(9) = tpzpz

     iao = nsh2ao(i)
     jao = nsh2ao(j)

    if (iao == jao) then
      do jx = jao,jao + 3-1
        do ix = jx,iao + 3-1
          key = (ix-iao)*3 + jx-jao + 1
          val(ix,jx) =  v(key)
        end do
      end do
    else
      do jx = jao,jao + 3-1
        do ix = iao,iao + 3-1
          key = (ix-iao)*3 + jx-jao + 1
          val(ix,jx) =  v(key)
        end do
      end do
    end if
  end do
!$omp end do nowait
end subroutine os_kei_pp_s1

!------------------------------------------------------------------------------
!> ds kinetic energy integrals
!
subroutine os_kei_ds_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,xi,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp),xi(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(6)

!$omp do schedule(dynamic)
  do ij = ishp(3)+1, ishp(4)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

     tdxxs = 0.0d0
     tdyys = 0.0d0
     tdzzs = 0.0d0
     tdxys = 0.0d0
     tdxzs = 0.0d0
     tdyzs = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xi_ = xi(irs)
        xi2 = 2.d0 * xi_
        tss = oss * xi_ * ( 3.d0 - xi2 * AB2 )
        xioza = xiozetaa(irs)
        xiozb = 1.0d0 - xioza
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz

        ai_oss = oo2zeta*oss
        opzs = oss*PAz
        odzzs = opzs*PAz + ai_oss
        opys = oss*PAy
        odyzs = opys*PAz
        odyys = opys*PAy + ai_oss
        opxs = oss*PAx
        odxzs = opxs*PAz
        odxys = opxs*PAy
        odxxs = opxs*PAx + ai_oss
        xioza_oss = xioza*oss
        ai_tss = oo2zeta*tss
        tpzs = tss*PAz + opzs*xi2
        tdzzs = tdzzs + tpzs*PAz + odzzs*xi2 + ai_tss - xioza_oss
        tpys = tss*PAy + opys*xi2
        tdyzs = tdyzs + tpys*PAz + odyzs*xi2
        tdyys = tdyys + tpys*PAy + odyys*xi2 + ai_tss - xioza_oss
        tpxs = tss*PAx + opxs*xi2
        tdxzs = tdxzs + tpxs*PAz + odxzs*xi2
        tdxys = tdxys + tpxs*PAy + odxys*xi2
        tdxxs = tdxxs + tpxs*PAx + odxxs*xi2 + ai_tss - xioza_oss
     end do

     v(1) = tdxxs
     v(2) = tdyys
     v(3) = tdzzs
     v(4) = tdxys*root3
     v(5) = tdxzs*root3
     v(6) = tdyzs*root3

     iao = nsh2ao(i)
     jao = nsh2ao(j)

    if (i < j) then
      do ix = iao,iao + 6-1
        key = ix-iao + 1
        val(jao,ix) =  v(key)
      end do
    else
      do ix = iao,iao + 6-1
        key = ix-iao + 1
        val(ix,jao) =  v(key)
      end do
    end if
  end do
!$omp end do nowait
end subroutine os_kei_ds_s1

!------------------------------------------------------------------------------
!> dp kinetic energy integrals
!
subroutine os_kei_dp_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,xi,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp),xi(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(18)

!$omp do schedule(dynamic)
  do ij = ishp(4)+1, ishp(5)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

     tdxxpx = 0.0d0
     tdxxpy = 0.0d0
     tdxxpz = 0.0d0
     tdyypx = 0.0d0
     tdyypy = 0.0d0
     tdyypz = 0.0d0
     tdzzpx = 0.0d0
     tdzzpy = 0.0d0
     tdzzpz = 0.0d0
     tdxypx = 0.0d0
     tdxypy = 0.0d0
     tdxypz = 0.0d0
     tdxzpx = 0.0d0
     tdxzpy = 0.0d0
     tdxzpz = 0.0d0
     tdyzpx = 0.0d0
     tdyzpy = 0.0d0
     tdyzpz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xi_ = xi(irs)
        xi2 = 2.d0 * xi_
        tss = oss * xi_ * ( 3.d0 - xi2 * AB2 )
        xioza = xiozetaa(irs)
        xiozb = 1.0d0 - xioza
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz

        ai_oss = oo2zeta*oss
        opzs = oss*PAz
        ai_opzs = oo2zeta*opzs
        odzzs = opzs*PAz + ai_oss
        odzzpz = odzzs*PBz + ai_opzs*2.0d0
        odzzpy = odzzs*PBy
        odzzpx = odzzs*PBx
        opys = oss*PAy
        ai_opys = oo2zeta*opys
        odyzs = opys*PAz
        odyzpz = odyzs*PBz + ai_opys
        odyzpy = odyzs*PBy + ai_opzs
        odyzpx = odyzs*PBx
        odyys = opys*PAy + ai_oss
        odyypz = odyys*PBz
        odyypy = odyys*PBy + ai_opys*2.0d0
        odyypx = odyys*PBx
        opxs = oss*PAx
        ai_opxs = oo2zeta*opxs
        odxzs = opxs*PAz
        odxzpz = odxzs*PBz + ai_opxs
        odxzpy = odxzs*PBy
        odxzpx = odxzs*PBx + ai_opzs
        odxys = opxs*PAy
        odxypz = odxys*PBz
        odxypy = odxys*PBy + ai_opxs
        odxypx = odxys*PBx + ai_opys
        odxxs = opxs*PAx + ai_oss
        odxxpz = odxxs*PBz
        odxxpy = odxxs*PBy
        odxxpx = odxxs*PBx + ai_opxs*2.0d0
        xioza_oss = xioza*oss
        ai_tss = oo2zeta*tss
        tpzs = tss*PAz + opzs*xi2
        ai_tpzs = oo2zeta*tpzs
        tdzzs = tpzs*PAz + odzzs*xi2 + ai_tss - xioza_oss
        tdzzpz = tdzzpz + tdzzs*PBz + odzzpz*xi2 + ai_tpzs*2.0d0
        tdzzpy = tdzzpy + tdzzs*PBy + odzzpy*xi2
        tdzzpx = tdzzpx + tdzzs*PBx + odzzpx*xi2
        tpys = tss*PAy + opys*xi2
        ai_tpys = oo2zeta*tpys
        tdyzs = tpys*PAz + odyzs*xi2
        tdyzpz = tdyzpz + tdyzs*PBz + odyzpz*xi2 + ai_tpys
        tdyzpy = tdyzpy + tdyzs*PBy + odyzpy*xi2 + ai_tpzs
        tdyzpx = tdyzpx + tdyzs*PBx + odyzpx*xi2
        tdyys = tpys*PAy + odyys*xi2 + ai_tss - xioza_oss
        tdyypz = tdyypz + tdyys*PBz + odyypz*xi2
        tdyypy = tdyypy + tdyys*PBy + odyypy*xi2 + ai_tpys*2.0d0
        tdyypx = tdyypx + tdyys*PBx + odyypx*xi2
        tpxs = tss*PAx + opxs*xi2
        ai_tpxs = oo2zeta*tpxs
        tdxzs = tpxs*PAz + odxzs*xi2
        tdxzpz = tdxzpz + tdxzs*PBz + odxzpz*xi2 + ai_tpxs
        tdxzpy = tdxzpy + tdxzs*PBy + odxzpy*xi2
        tdxzpx = tdxzpx + tdxzs*PBx + odxzpx*xi2 + ai_tpzs
        tdxys = tpxs*PAy + odxys*xi2
        tdxypz = tdxypz + tdxys*PBz + odxypz*xi2
        tdxypy = tdxypy + tdxys*PBy + odxypy*xi2 + ai_tpxs
        tdxypx = tdxypx + tdxys*PBx + odxypx*xi2 + ai_tpys
        tdxxs = tpxs*PAx + odxxs*xi2 + ai_tss - xioza_oss
        tdxxpz = tdxxpz + tdxxs*PBz + odxxpz*xi2
        tdxxpy = tdxxpy + tdxxs*PBy + odxxpy*xi2
        tdxxpx = tdxxpx + tdxxs*PBx + odxxpx*xi2 + ai_tpxs*2.0d0
     end do

     v(1) = tdxxpx
     v(2) = tdxxpy
     v(3) = tdxxpz
     v(4) = tdyypx
     v(5) = tdyypy
     v(6) = tdyypz
     v(7) = tdzzpx
     v(8) = tdzzpy
     v(9) = tdzzpz
     v(10) = tdxypx*root3
     v(11) = tdxypy*root3
     v(12) = tdxypz*root3
     v(13) = tdxzpx*root3
     v(14) = tdxzpy*root3
     v(15) = tdxzpz*root3
     v(16) = tdyzpx*root3
     v(17) = tdyzpy*root3
     v(18) = tdyzpz*root3

     iao = nsh2ao(i)
     jao = nsh2ao(j)

    if (i < j) then
      do jx = jao,jao + 3-1
        do ix = iao,iao + 6-1
          key = ( ix-iao ) * 3 + jx-jao +1
          val(jx,ix) =  v(key)
        end do
      end do
    else
      do jx = jao,jao + 3-1
        do ix = iao,iao + 6-1
          key = ( ix-iao ) * 3 + jx-jao +1
          val(ix,jx) =  v(key)
        end do
      end do
    end if
  end do
!$omp end do nowait
end subroutine os_kei_dp_s1

!------------------------------------------------------------------------------
!> dd kinetic energy integrals
!
subroutine os_kei_dd_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,xi,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp),xi(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(36)

!$omp do schedule(dynamic)
  do ij = ishp(5)+1, ishp(6)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

     tdxxdxx = 0.0d0
     tdxxdyy = 0.0d0
     tdxxdzz = 0.0d0
     tdxxdxy = 0.0d0
     tdxxdxz = 0.0d0
     tdxxdyz = 0.0d0
     tdyydxx = 0.0d0
     tdyydyy = 0.0d0
     tdyydzz = 0.0d0
     tdyydxy = 0.0d0
     tdyydxz = 0.0d0
     tdyydyz = 0.0d0
     tdzzdxx = 0.0d0
     tdzzdyy = 0.0d0
     tdzzdzz = 0.0d0
     tdzzdxy = 0.0d0
     tdzzdxz = 0.0d0
     tdzzdyz = 0.0d0
     tdxydxx = 0.0d0
     tdxydyy = 0.0d0
     tdxydzz = 0.0d0
     tdxydxy = 0.0d0
     tdxydxz = 0.0d0
     tdxydyz = 0.0d0
     tdxzdxx = 0.0d0
     tdxzdyy = 0.0d0
     tdxzdzz = 0.0d0
     tdxzdxy = 0.0d0
     tdxzdxz = 0.0d0
     tdxzdyz = 0.0d0
     tdyzdxx = 0.0d0
     tdyzdyy = 0.0d0
     tdyzdzz = 0.0d0
     tdyzdxy = 0.0d0
     tdyzdxz = 0.0d0
     tdyzdyz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xi_ = xi(irs)
        xi2 = 2.d0 * xi_
        tss = oss * xi_ * ( 3.d0 - xi2 * AB2 )
        xioza = xiozetaa(irs)
        xiozb = 1.0d0 - xioza
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz

        ai_oss = oo2zeta*oss
        opzs = oss*PAz
        ai_opzs = oo2zeta*opzs
        odzzs = opzs*PAz + ai_oss
        bj_odzzs = oo2zeta*odzzs
        odzzpz = odzzs*PBz + ai_opzs*2.0d0
        odzzpy = odzzs*PBy
        odzzdyy = odzzpy*PBy + bj_odzzs
        odzzpx = odzzs*PBx
        odzzdxy = odzzpx*PBy
        odzzdxx = odzzpx*PBx + bj_odzzs
        xiozb_odzzs = xiozb*odzzs
        opzpz = opzs*PBz + ai_oss
        ai_opzpz = oo2zeta*opzpz
        odzzdzz = odzzpz*PBz + bj_odzzs + ai_opzpz*2.0d0
        opzpy = opzs*PBy
        ai_opzpy = oo2zeta*opzpy
        odzzdyz = odzzpy*PBz + ai_opzpy*2.0d0
        opzpx = opzs*PBx
        ai_opzpx = oo2zeta*opzpx
        odzzdxz = odzzpx*PBz + ai_opzpx*2.0d0
        opys = oss*PAy
        ai_opys = oo2zeta*opys
        odyzs = opys*PAz
        bj_odyzs = oo2zeta*odyzs
        odyzpz = odyzs*PBz + ai_opys
        odyzpy = odyzs*PBy + ai_opzs
        odyzdyy = odyzpy*PBy + bj_odyzs + ai_opzpy
        odyzpx = odyzs*PBx
        odyzdxy = odyzpx*PBy + ai_opzpx
        odyzdxx = odyzpx*PBx + bj_odyzs
        xiozb_odyzs = xiozb*odyzs
        odyys = opys*PAy + ai_oss
        bj_odyys = oo2zeta*odyys
        odyypz = odyys*PBz
        odyydzz = odyypz*PBz + bj_odyys
        odyypy = odyys*PBy + ai_opys*2.0d0
        odyydyz = odyypy*PBz
        odyypx = odyys*PBx
        odyydxz = odyypx*PBz
        odyydxx = odyypx*PBx + bj_odyys
        xiozb_odyys = xiozb*odyys
        opypz = opys*PBz
        ai_opypz = oo2zeta*opypz
        odyzdzz = odyzpz*PBz + bj_odyzs + ai_opypz
        opypy = opys*PBy + ai_oss
        ai_opypy = oo2zeta*opypy
        odyzdyz = odyzpy*PBz + ai_opypy
        odyydyy = odyypy*PBy + bj_odyys + ai_opypy*2.0d0
        opypx = opys*PBx
        ai_opypx = oo2zeta*opypx
        odyzdxz = odyzpx*PBz + ai_opypx
        odyydxy = odyypx*PBy + ai_opypx*2.0d0
        opxs = oss*PAx
        ai_opxs = oo2zeta*opxs
        odxzs = opxs*PAz
        bj_odxzs = oo2zeta*odxzs
        odxzpz = odxzs*PBz + ai_opxs
        odxzpy = odxzs*PBy
        odxzdyy = odxzpy*PBy + bj_odxzs
        odxzpx = odxzs*PBx + ai_opzs
        odxzdxy = odxzpx*PBy
        odxzdxx = odxzpx*PBx + bj_odxzs + ai_opzpx
        xiozb_odxzs = xiozb*odxzs
        odxys = opxs*PAy
        bj_odxys = oo2zeta*odxys
        odxypz = odxys*PBz
        odxydzz = odxypz*PBz + bj_odxys
        odxypy = odxys*PBy + ai_opxs
        odxydyz = odxypy*PBz
        odxypx = odxys*PBx + ai_opys
        odxydxz = odxypx*PBz
        odxydxx = odxypx*PBx + bj_odxys + ai_opypx
        xiozb_odxys = xiozb*odxys
        odxxs = opxs*PAx + ai_oss
        bj_odxxs = oo2zeta*odxxs
        odxxpz = odxxs*PBz
        odxxdzz = odxxpz*PBz + bj_odxxs
        odxxpy = odxxs*PBy
        odxxdyz = odxxpy*PBz
        odxxdyy = odxxpy*PBy + bj_odxxs
        odxxpx = odxxs*PBx + ai_opxs*2.0d0
        odxxdxz = odxxpx*PBz
        odxxdxy = odxxpx*PBy
        xiozb_odxxs = xiozb*odxxs
        opxpz = opxs*PBz
        ai_opxpz = oo2zeta*opxpz
        odxzdzz = odxzpz*PBz + bj_odxzs + ai_opxpz
        opxpy = opxs*PBy
        ai_opxpy = oo2zeta*opxpy
        odxzdyz = odxzpy*PBz + ai_opxpy
        odxydyy = odxypy*PBy + bj_odxys + ai_opxpy
        opxpx = opxs*PBx + ai_oss
        ai_opxpx = oo2zeta*opxpx
        odxzdxz = odxzpx*PBz + ai_opxpx
        odxydxy = odxypx*PBy + ai_opxpx
        odxxdxx = odxxpx*PBx + bj_odxxs + ai_opxpx*2.0d0
        xioza_oss = xioza*oss
        ai_tss = oo2zeta*tss
        tpzs = tss*PAz + opzs*xi2
        ai_tpzs = oo2zeta*tpzs
        tdzzs = tpzs*PAz + odzzs*xi2 + ai_tss - xioza_oss
        bj_tdzzs = oo2zeta*tdzzs
        tdzzpz = tdzzs*PBz + odzzpz*xi2 + ai_tpzs*2.0d0
        tdzzpy = tdzzs*PBy + odzzpy*xi2
        tdzzdyy = tdzzdyy + tdzzpy*PBy + odzzdyy*xi2 + bj_tdzzs - xiozb_odzzs
        tdzzpx = tdzzs*PBx + odzzpx*xi2
        tdzzdxy = tdzzdxy + tdzzpx*PBy + odzzdxy*xi2
        tdzzdxx = tdzzdxx + tdzzpx*PBx + odzzdxx*xi2 + bj_tdzzs - xiozb_odzzs
        tpzpz = tpzs*PBz + opzpz*xi2 + ai_tss
        ai_tpzpz = oo2zeta*tpzpz
        tdzzdzz = tdzzdzz + tdzzpz*PBz + odzzdzz*xi2 + bj_tdzzs - xiozb_odzzs + ai_tpzpz*2.0d0
        tpzpy = tpzs*PBy + opzpy*xi2
        ai_tpzpy = oo2zeta*tpzpy
        tdzzdyz = tdzzdyz + tdzzpy*PBz + odzzdyz*xi2 + ai_tpzpy*2.0d0
        tpzpx = tpzs*PBx + opzpx*xi2
        ai_tpzpx = oo2zeta*tpzpx
        tdzzdxz = tdzzdxz + tdzzpx*PBz + odzzdxz*xi2 + ai_tpzpx*2.0d0
        tpys = tss*PAy + opys*xi2
        ai_tpys = oo2zeta*tpys
        tdyzs = tpys*PAz + odyzs*xi2
        bj_tdyzs = oo2zeta*tdyzs
        tdyzpz = tdyzs*PBz + odyzpz*xi2 + ai_tpys
        tdyzpy = tdyzs*PBy + odyzpy*xi2 + ai_tpzs
        tdyzdyy = tdyzdyy + tdyzpy*PBy + odyzdyy*xi2 + bj_tdyzs - xiozb_odyzs + ai_tpzpy
        tdyzpx = tdyzs*PBx + odyzpx*xi2
        tdyzdxy = tdyzdxy + tdyzpx*PBy + odyzdxy*xi2 + ai_tpzpx
        tdyzdxx = tdyzdxx + tdyzpx*PBx + odyzdxx*xi2 + bj_tdyzs - xiozb_odyzs
        tdyys = tpys*PAy + odyys*xi2 + ai_tss - xioza_oss
        bj_tdyys = oo2zeta*tdyys
        tdyypz = tdyys*PBz + odyypz*xi2
        tdyydzz = tdyydzz + tdyypz*PBz + odyydzz*xi2 + bj_tdyys - xiozb_odyys
        tdyypy = tdyys*PBy + odyypy*xi2 + ai_tpys*2.0d0
        tdyydyz = tdyydyz + tdyypy*PBz + odyydyz*xi2
        tdyypx = tdyys*PBx + odyypx*xi2
        tdyydxz = tdyydxz + tdyypx*PBz + odyydxz*xi2
        tdyydxx = tdyydxx + tdyypx*PBx + odyydxx*xi2 + bj_tdyys - xiozb_odyys
        tpypz = tpys*PBz + opypz*xi2
        ai_tpypz = oo2zeta*tpypz
        tdyzdzz = tdyzdzz + tdyzpz*PBz + odyzdzz*xi2 + bj_tdyzs - xiozb_odyzs + ai_tpypz
        tpypy = tpys*PBy + opypy*xi2 + ai_tss
        ai_tpypy = oo2zeta*tpypy
        tdyzdyz = tdyzdyz + tdyzpy*PBz + odyzdyz*xi2 + ai_tpypy
        tdyydyy = tdyydyy + tdyypy*PBy + odyydyy*xi2 + bj_tdyys - xiozb_odyys + ai_tpypy*2.0d0
        tpypx = tpys*PBx + opypx*xi2
        ai_tpypx = oo2zeta*tpypx
        tdyzdxz = tdyzdxz + tdyzpx*PBz + odyzdxz*xi2 + ai_tpypx
        tdyydxy = tdyydxy + tdyypx*PBy + odyydxy*xi2 + ai_tpypx*2.0d0
        tpxs = tss*PAx + opxs*xi2
        ai_tpxs = oo2zeta*tpxs
        tdxzs = tpxs*PAz + odxzs*xi2
        bj_tdxzs = oo2zeta*tdxzs
        tdxzpz = tdxzs*PBz + odxzpz*xi2 + ai_tpxs
        tdxzpy = tdxzs*PBy + odxzpy*xi2
        tdxzdyy = tdxzdyy + tdxzpy*PBy + odxzdyy*xi2 + bj_tdxzs - xiozb_odxzs
        tdxzpx = tdxzs*PBx + odxzpx*xi2 + ai_tpzs
        tdxzdxy = tdxzdxy + tdxzpx*PBy + odxzdxy*xi2
        tdxzdxx = tdxzdxx + tdxzpx*PBx + odxzdxx*xi2 + bj_tdxzs - xiozb_odxzs + ai_tpzpx
        tdxys = tpxs*PAy + odxys*xi2
        bj_tdxys = oo2zeta*tdxys
        tdxypz = tdxys*PBz + odxypz*xi2
        tdxydzz = tdxydzz + tdxypz*PBz + odxydzz*xi2 + bj_tdxys - xiozb_odxys
        tdxypy = tdxys*PBy + odxypy*xi2 + ai_tpxs
        tdxydyz = tdxydyz + tdxypy*PBz + odxydyz*xi2
        tdxypx = tdxys*PBx + odxypx*xi2 + ai_tpys
        tdxydxz = tdxydxz + tdxypx*PBz + odxydxz*xi2
        tdxydxx = tdxydxx + tdxypx*PBx + odxydxx*xi2 + bj_tdxys - xiozb_odxys + ai_tpypx
        tdxxs = tpxs*PAx + odxxs*xi2 + ai_tss - xioza_oss
        bj_tdxxs = oo2zeta*tdxxs
        tdxxpz = tdxxs*PBz + odxxpz*xi2
        tdxxdzz = tdxxdzz + tdxxpz*PBz + odxxdzz*xi2 + bj_tdxxs - xiozb_odxxs
        tdxxpy = tdxxs*PBy + odxxpy*xi2
        tdxxdyz = tdxxdyz + tdxxpy*PBz + odxxdyz*xi2
        tdxxdyy = tdxxdyy + tdxxpy*PBy + odxxdyy*xi2 + bj_tdxxs - xiozb_odxxs
        tdxxpx = tdxxs*PBx + odxxpx*xi2 + ai_tpxs*2.0d0
        tdxxdxz = tdxxdxz + tdxxpx*PBz + odxxdxz*xi2
        tdxxdxy = tdxxdxy + tdxxpx*PBy + odxxdxy*xi2
        tpxpz = tpxs*PBz + opxpz*xi2
        ai_tpxpz = oo2zeta*tpxpz
        tdxzdzz = tdxzdzz + tdxzpz*PBz + odxzdzz*xi2 + bj_tdxzs - xiozb_odxzs + ai_tpxpz
        tpxpy = tpxs*PBy + opxpy*xi2
        ai_tpxpy = oo2zeta*tpxpy
        tdxzdyz = tdxzdyz + tdxzpy*PBz + odxzdyz*xi2 + ai_tpxpy
        tdxydyy = tdxydyy + tdxypy*PBy + odxydyy*xi2 + bj_tdxys - xiozb_odxys + ai_tpxpy
        tpxpx = tpxs*PBx + opxpx*xi2 + ai_tss
        ai_tpxpx = oo2zeta*tpxpx
        tdxzdxz = tdxzdxz + tdxzpx*PBz + odxzdxz*xi2 + ai_tpxpx
        tdxydxy = tdxydxy + tdxypx*PBy + odxydxy*xi2 + ai_tpxpx
        tdxxdxx = tdxxdxx + tdxxpx*PBx + odxxdxx*xi2 + bj_tdxxs - xiozb_odxxs + ai_tpxpx*2.0d0
     end do

     v(1) = tdxxdxx
     v(2) = tdxxdyy
     v(3) = tdxxdzz
     v(4) = tdxxdxy*root3
     v(5) = tdxxdxz*root3
     v(6) = tdxxdyz*root3
     v(7) = tdyydxx
     v(8) = tdyydyy
     v(9) = tdyydzz
     v(10) = tdyydxy*root3
     v(11) = tdyydxz*root3
     v(12) = tdyydyz*root3
     v(13) = tdzzdxx
     v(14) = tdzzdyy
     v(15) = tdzzdzz
     v(16) = tdzzdxy*root3
     v(17) = tdzzdxz*root3
     v(18) = tdzzdyz*root3
     v(19) = tdxydxx*root3
     v(20) = tdxydyy*root3
     v(21) = tdxydzz*root3
     v(22) = tdxydxy*root3x2
     v(23) = tdxydxz*root3x2
     v(24) = tdxydyz*root3x2
     v(25) = tdxzdxx*root3
     v(26) = tdxzdyy*root3
     v(27) = tdxzdzz*root3
     v(28) = tdxzdxy*root3x2
     v(29) = tdxzdxz*root3x2
     v(30) = tdxzdyz*root3x2
     v(31) = tdyzdxx*root3
     v(32) = tdyzdyy*root3
     v(33) = tdyzdzz*root3
     v(34) = tdyzdxy*root3x2
     v(35) = tdyzdxz*root3x2
     v(36) = tdyzdyz*root3x2

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do jx = jao,jao + 6-1
           do ix = jx,iao + 6-1
              key = ( ix-iao ) * 6 + jx-jao +1
              val(ix,jx) =  v(key)
           end do
        end do
     else
        do jx = jao,jao + 6-1
           do ix = iao,iao + 6-1
              key = ( ix-iao ) * 6 + jx-jao +1
              val(ix,jx) =  v(key)
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_kei_dd_s1
