! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!--------------------------------------------------------------------
module pl_prof_class
  use omp_lib !< OpenMP
  implicit none

  private
  public::pl_prof_initialize
  public::pl_prof_finalize
  public::pl_prof_region_begin
  public::pl_prof_region_end
  public::pl_prof_print

  character(len=12),parameter::PL_PROF_ENV_NAME = 'PL_PROF_PAPI' !< 環境変数名
  integer,parameter::MAX_ENV_VALUE_LEN = 32 !< 環境変数の値の最大文字列長
  integer,parameter::MAX_REGION_NAME_LEN = 256 !< 測定区間名の最大文字列長
  integer,parameter::MAX_REGIONS = 2048 !< 測定区間の最大数
  integer,parameter::MAX_EVENTS  = 10   !< PAPIイベントの最大数
  integer,parameter::MAX_DEPTH   = 10   !< ネストの深さ(0始まり)の最大数

  !> 測定区間開始時点の測定値
  type::region_beginning_type
     character(len=MAX_REGION_NAME_LEN)::name !< 測定区間名
     real(8)::wtime                !< 経過時間
     integer(8)::real_usec         !< 経過時間 (精度:マイクロ秒)
     !integer(8)::real_cyc          !< clock cycles
     integer(8)::counts(MAX_EVENTS) !< 各PAPIイベントのカウント値
  end type region_beginning_type

  !> 測定区間情報
  type::region_type
     character(len=MAX_REGION_NAME_LEN)::name !< 測定区間名
     integer::depth       !< ネストの深さ (0始まり)
     integer::num_called  !< 呼び出された回数
     logical::has_begun   !< 測定開始フラグ (不要??)
  end type region_type

  !> 測定値の累積
  type::value_type
     real(8)::wtime                !< 経過時間
     integer(8)::real_usec         !< 経過時間 (精度:マイクロ秒)
     !integer(8)::real_cyc          !< clock cycles
     integer(8)::counts(MAX_EVENTS) !< 各PAPIイベントのカウント値
  end type value_type

  ! SAVE属性
  integer,save::ldebug = 0  !< デバッグレベル (開発時使用; 1:on, 0:off )
  integer,save,allocatable::num_regions(:) !< その時点の測定区間数
  integer,save,allocatable::depth(:)       !< その時点のネストの深さ
  integer,save::num_events  !< PAPIイベント数
  integer,save,allocatable::event_sets(:) !< PAPIイベントセット
  integer,save::clockrate   !< Processor clockrate in MHz
  type(region_beginning_type),save,allocatable&
       ::region_beginnings(:,:) !< 測定区間開始時点の測定値
  type(region_type),save,allocatable::regions(:,:) !< 測定区間情報
  type(value_type),save,allocatable::values(:,:)   !< 測定値の累積
  integer(8),save,allocatable::counts(:,:) !< 各PAPIイベントのカウント値(tmp)
  integer,save::max_threads !< スレッド数の最大値 (omp_get_max_threads)
  logical,save,allocatable::thread_runnings(:) !< 各スレッドの状態
  character(len=MAX_ENV_VALUE_LEN),&
       save::env_val !< 環境変数(PL_PROF_ENV_NAME)の値

  !> 測定状態フラグ
  !!  - .false. : pl_prof_initializeのcall前およびpl_prof_finalizeのcall後
  !!  - .true.  : pl_prof_initializeのcall後からpl_prof_finalizeのcall前
  logical,save::is_running = .false.
  !> 測定実施フラグ
  !!  - .false. : 測定を行わない
  !!  - .true.  : 測定を行う [既定値]
  logical,save::do_prof
  !> PAPI使用フラグ
  !!  - .false. : 使用しない (do_profが.true.ならば経過時間の測定を行う)
  !!  - .true.  : 使用する [既定値]
  logical,save::use_papi

contains
  !--------------------------------------------------------------------
  !> 初期処理
  subroutine pl_prof_initialize(do_prof_, use_papi_)
    logical,optional,intent(in)::do_prof_  !< 測定実施フラグ [既定値:.true.]
    logical,optional,intent(in)::use_papi_ !< PAPI使用フラグ [既定値:.true.]
    ! local
    logical,parameter::do_prof_default  = .true.
    logical,parameter::use_papi_default = .true.
    integer::istat, i, thread_id

    ! 測定状態フラグの確認
    if (is_running)&
         call error_abort('pl_prof has already been running @initialize')
    is_running = .true.

    ! 測定実施フラグの設定
    if (present(do_prof_)) then
       do_prof = do_prof_
    else
       do_prof = do_prof_default
    end if
    if (.not. do_prof) return  ! 測定せず戻る

    ! PAPI使用フラグの設定
    if (present(use_papi_)) then
       use_papi = use_papi_
    else
       use_papi = use_papi_default
    end if

    max_threads = max(omp_get_max_threads(), omp_get_num_procs())
    !write(6,*) 'max_threads=',max_threads,omp_get_num_procs()

    write(6,'(''Maximum number of threads :'',i3)') max_threads
    allocate(&
         region_beginnings(MAX_DEPTH,max_threads),&
         regions(MAX_REGIONS,max_threads),&
         values(MAX_REGIONS,max_threads),&
         num_regions(max_threads),&
         depth(max_threads),&
         thread_runnings(max_threads),&
         counts(MAX_EVENTS,max_threads),&
         stat=istat)
    if (istat /= 0) call error_abort('allocate error @pl_prof')

    num_regions(:)     = 0
    depth(:)           = 0
    thread_runnings(:) = .false.

  end subroutine pl_prof_initialize

  !--------------------------------------------------------------------
  !> 終了処理
  subroutine pl_prof_finalize()
    ! local
    integer::istat, thread_id

    if (.not. is_running)& ! 測定状態フラグの確認
         call error_abort('pl_prof is not running @finalize')
    if (.not. do_prof) return  ! 測定せず戻る

    deallocate(region_beginnings, regions, values,&
         num_regions, depth, thread_runnings, counts, stat=istat)
    is_running = .false.

  end subroutine pl_prof_finalize

  !--------------------------------------------------------------------
  !> 測定区間の開始
  subroutine pl_prof_region_begin(region_name)
    character(len=*),intent(in)::region_name
    ! local
    integer(8)::real_usec, real_cyc
    integer::idx, i, istat, thread_id, depth_

    if (.not. is_running)& ! 測定状態フラグの確認
         call error_abort(&
         'pl_prof is not running @region_begin : '//trim(region_name))
    if (.not. do_prof) return  ! 測定せず戻る

    thread_id = omp_get_thread_num() + 1 ! 1始まり
    !print *, 'thread_id=',thread_id,omp_get_num_threads(),omp_get_num_procs()

    idx = get_region_index(&
         regions(1,thread_id), num_regions(thread_id), region_name)
    !print *, 'idx=',idx

    if (idx == 0) then
       num_regions(thread_id) = num_regions(thread_id) + 1
       !print *, 'num_regions=',num_regions
       if (num_regions(thread_id) > MAX_REGIONS) call error_abort(&
            'region over @ region_begin : region = ' // trim(region_name))
       idx = num_regions(thread_id)
       regions(idx,thread_id)%num_called = 1
       regions(idx,thread_id)%depth      = depth(thread_id)
       regions(idx,thread_id)%name       = region_name
       values(idx,thread_id)%wtime       = 0.0d0
       values(idx,thread_id)%real_usec   = 0
       !values(idx,thread_id)%real_cyc    = 0
       values(idx,thread_id)%counts(:)   = 0
    else
       if (regions(idx,thread_id)%has_begun) call error_abort(&
            'region has already begun @ region_begin : '//trim(region_name))
       regions(idx,thread_id)%num_called = regions(idx,thread_id)%num_called + 1
    endif
    !print *, 'num_regions=',num_regions

    regions(idx,thread_id)%has_begun = .true.
    depth(thread_id) = depth(thread_id) + 1
    depth_ = depth(thread_id)
    region_beginnings(depth_,thread_id)%name = trim(region_name)
    !print *, 'begin=',trim(region_beginnings(depth_,thread_id)%name),':',depth_

    !print *, 'thread_id=',thread_id,thread_runnings(thread_id)
    region_beginnings(depth_,thread_id)%wtime = omp_get_wtime()

  end subroutine pl_prof_region_begin

  !--------------------------------------------------------------------
  !> 測定区間の終了
  subroutine pl_prof_region_end(region_name)
    character(len=*),intent(in)::region_name
    ! local
    integer::idx, istat
    integer(8)::real_usec_end, real_usec, real_cyc_end, real_cyc
    integer::thread_id, depth_
    real(8)::wtime_end

#ifdef HAVE_KPROFDETAIL
       call stop_collection(region_name)
#endif
    if (.not. is_running)& ! 測定状態フラグの確認
         call error_abort(&
         'pl_prof is not running @region_end : '//trim(region_name))
    if (.not. do_prof) return  ! 測定せず戻る

    thread_id = omp_get_thread_num() + 1 ! 1始まり
    if (num_regions(thread_id) == 0) call error_abort(&
         'region has not begun yet @ region_end : '//trim(region_name))

    depth_ = depth(thread_id)
    !print *, 'end  =',trim(region_beginnings(depth_,thread_id)%name),':',depth_
    idx = get_region_index(&
         regions(1,thread_id), num_regions(thread_id), region_name)
    if (idx == 0) call error_abort(&
         'region has not begun yet @ region_end : '//trim(region_name))
    if (trim(region_beginnings(depth_,thread_id)%name) /= trim(region_name))&
         call error_abort(&
         'region has not begun yet @ region_end : '//trim(region_name))

    wtime_end = omp_get_wtime()
    values(idx,thread_id)%wtime&
         = values(idx,thread_id)%wtime&
         + ( wtime_end - region_beginnings(depth_,thread_id)%wtime )
    !print *, 'end(real_usec)=',&
    !     trim(region_beginnings(depth_,thread_id)%name),':',&
    !     values(idx,thread_id)%real_usec

    regions(idx,thread_id)%has_begun = .false.
    depth(thread_id) = depth(thread_id) - 1

  end subroutine pl_prof_region_end

  !--------------------------------------------------------------------
  !> 測定結果の表示
  subroutine pl_prof_print()
    integer::event_set

    if (.not. is_running)& ! 測定状態フラグの確認
         call error_abort('pl_prof is not running @print')
    if (.not. do_prof) return  ! 測定せず戻る

!$omp master
    call print_summary_presets()
    call print_summary_derived()
    call print_summary_usec()
!$omp end master

  end subroutine pl_prof_print

  !--------------------------------------------------------------------
  !> ハードウェア情報
  subroutine pl_prof_get_hardware_info()
  end subroutine pl_prof_get_hardware_info

  !====================================================================
  ! private
  !--------------------------------------------------------------------
  !> PAPIイベントセットの設定
  subroutine set_event(event_set, keyword)
    integer,intent(inout)::event_set !< PAPIイベントセット
    character(len=*),intent(in)::keyword !< キーワード
  end subroutine set_event

  !--------------------------------------------------------------------
  !> PAPIイベントセットの解放
  subroutine clear_events(event_set)
    integer,intent(inout)::event_set !< PAPIイベントセット
  end subroutine clear_events

  !--------------------------------------------------------------------
  !> PAPI状態の表示
  subroutine print_state(event_set)
    integer,intent(in)::event_set !< PAPIイベントセット
  end subroutine print_state

  !--------------------------------------------------------------------
  !> イベントコードの取得
  subroutine get_event_codes(event_set, event_codes, event_names, num_events)
    integer,intent(in)::event_set
    integer,intent(inout)::event_codes(:)
    character(len=*),intent(inout)::event_names(:)
    integer,intent(out)::num_events
    num_events = 0
  end subroutine get_event_codes

  !--------------------------------------------------------------------
  !> 計測結果の表示
  subroutine print_summary_presets()
  end subroutine print_summary_presets

  !--------------------------------------------------------------------
  !> 計測結果の表示
  subroutine print_summary_derived()
    ! local
    integer,parameter::COLUMN_WIDTH = 15
    integer,parameter::INDENT_WIDTH = 2
    integer,parameter::MAX_DERIVED_NAME_LEN = COLUMN_WIDTH
    integer,parameter::MAX_DERIVED_ITEMS = 32
    integer::num_events
    integer(8)::real_usec
    real(8)::wtime
    character(len=MAX_DERIVED_NAME_LEN)::derived_names(MAX_DERIVED_ITEMS)
    character(len=MAX_DERIVED_NAME_LEN)::derived_name
    real(8)::derived_values(MAX_DERIVED_ITEMS)
    real(8)::derived_value
    integer::iregion, thread_id, ievent, depth_, num_derived, iderived
    character(len=16)::fmtt, fmtv, fmtd
    logical::flag

    thread_id = 1
    iregion   = 1
    real_usec = 1
    wtime     = 1.0d0
    num_derived = 0

    if (num_derived == 0) then
       fmtt = ''
       fmtv = ''
    else
       write(fmtt,'(i2,''a'',i2)')        num_derived, COLUMN_WIDTH
       write(fmtv,'(i2,''f'',i2,''.6'')') num_derived, COLUMN_WIDTH
    end if
    !print *,fmtt; print *,fmtv

    write(6,'(a1)') '#'
    do thread_id = 1, max_threads
       if (num_regions(thread_id) == 0) cycle
       write(6,'(''# Thread_id ='',i3)') thread_id - 1

       write(6,'(a1)',  advance='no') '#'
       write(6,'(' // fmtt // ')', advance='no')&
            (trim(derived_names(iderived)),iderived=1,num_derived)
       write(6,'(1x)',  advance='no')
       write(6,'(a15)', advance='no') 'Wtime(sec)'
       write(6,'(a8)',  advance='no') 'Called'
       write(6,'(a6)',  advance='no') 'Depth'
       write(6,'(3x)',  advance='no')
       write(6,'(a)',   advance='no') 'Name'
       write(6,'(2x)',  advance='no')
       write(6,*)

       do iregion = 1, num_regions(thread_id)
          num_derived = 0

          write(6,'(''#'')', advance='no')
          write(6,'(' // fmtv // ')', advance='no')&
               (derived_values(iderived),iderived=1,num_derived)
          write(6,'(1x)', advance='no')
          write(6,'(f15.6)', advance='no') values(iregion,thread_id)%wtime
          write(6,'(i8)', advance='no') regions(iregion,thread_id)%num_called
          depth_ = regions(iregion,thread_id)%depth
          write(6,'(i6)', advance='no') depth_
          fmtd = ''
          if (depth_ > 0) write(fmtd,'('','',i3,''x'')') depth_*INDENT_WIDTH
          write(6,'(3x' // fmtd // ')', advance='no')
          write(6,'(a)', advance='no') trim(regions(iregion,thread_id)%name)
          write(6,*)
       end do
    end do

  end subroutine print_summary_derived

  !--------------------------------------------------------------------
  !> 計測結果の表示
  subroutine print_summary_usec()
    ! local
    integer,parameter::COLUMN_WIDTH = 15
    integer,parameter::INDENT_WIDTH = 2
    integer::iregion, thread_id, ievent, depth_
    integer(8)::i8
    real(8)::r8
    character(len=16)::fmtd
    integer::max_regions,idx,nt
    integer(8),allocatable::max_usecs(:)
    integer(8),allocatable::min_usecs(:)
    real(8),allocatable::max_wtime(:)
    real(8),allocatable::min_wtime(:)
    integer,allocatable::num_thrds(:)
    integer,allocatable::max_tid(:)
    integer,allocatable::min_tid(:)

    write(6,*) 'num_regions(1)=',num_regions(1)
    max_regions = num_regions(1)
       allocate(max_wtime(max_regions), min_wtime(max_regions))
    allocate(max_tid(max_regions), min_tid(max_regions), num_thrds(max_regions))

       max_wtime(:) = 0
       min_wtime(:) = Huge(r8)
    num_thrds(:) = 0
    max_tid(:) = 0
    min_tid(:) = 0
    do thread_id = 1, max_threads
       do iregion = 1, num_regions(thread_id)
          !write(6,*) 'td=',thread_id,iregion
          idx = get_region_index(&
               regions(1,1), num_regions(1), regions(iregion,thread_id)%name)
          !write(6,*) 'name=',trim(regions(iregion,thread_id)%name),idx
          num_thrds(idx) = num_thrds(idx) + 1
             if (values(iregion,thread_id)%wtime > max_wtime(idx)) then
                max_wtime(idx) = values(iregion,thread_id)%wtime
                max_tid(idx) = thread_id
             end if
             if (values(iregion,thread_id)%wtime < min_wtime(idx)) then
                min_wtime(idx) = values(iregion,thread_id)%wtime
                min_tid(idx) = thread_id
             end if
       end do
    end do

    write(6,*) 'max_threads=',max_threads
    write(6,'(a1)',  advance='no') '#'
    write(6,'(a15)', advance='no') 'num_threads'
       write(6,'(a20)', advance='no') 'max_Wtime(sec)'
    write(6,'(a5)', advance='no') 'tid'
       write(6,'(a20)', advance='no') 'min_Wtime(sec)'
    write(6,'(a5)', advance='no') 'tid'
    !write(6,'(a15)', advance='no') 'Wtime(sec)'
    write(6,'(3x)',  advance='no')
    write(6,'(a)',   advance='no') 'Name'
    write(6,'(2x)',  advance='no')
    write(6,*)

    thread_id = 1
    do iregion = 1, max_regions
       write(6,'(''#'')', advance='no')
       write(6,'(i15)', advance='no') num_thrds(iregion)
          write(6,'(f20.6,i5)', advance='no')&
               max_wtime(iregion), max_tid(iregion)
       if (num_thrds(iregion) > 1) then
             write(6,'(f20.6,i5)', advance='no')&
                  min_wtime(iregion), min_tid(iregion)
       else
          write(6,'(25x)', advance='no')
       end if
!!$       !write(6,'(f15.6)', advance='no') values(iregion,thread_id)%wtime
       depth_ = regions(iregion,thread_id)%depth
       fmtd = ''
       if (depth_ > 0) write(fmtd,'('','',i3,''x'')') depth_*INDENT_WIDTH
       write(6,'(3x' // fmtd // ')', advance='no')
       write(6,'(a)', advance='no') trim(regions(iregion,thread_id)%name)
       write(6,*)
    end do
!!$#endif
  end subroutine print_summary_usec

  !--------------------------------------------------------------------
  !> 配列格納位置を得る
  function get_region_index(regions, num_regions, region_name) result(idx)
    integer,intent(in)::num_regions
    type(region_type),intent(in)::regions(num_regions)
    character(len=*),intent(in)::region_name
    integer::idx
    ! local
    integer::i

    idx = 0
    do i = 1, num_regions
       if (regions(i)%name == region_name) then
          idx = i
          return
       end if
    end do

  end function get_region_index

  !--------------------------------------------------------
  !>
  function get_index(codes, num_codes, code) result(idx)
    integer,intent(in)::num_codes
    integer,intent(in)::codes(num_codes)
    integer,intent(in)::code
    integer::idx

    integer::i

    idx = 0
    do i = 1, num_codes
       if (codes(i) == code) then
          idx = i
          return
       end if
    end do

  end function get_index

  !--------------------------------------------------------------------
  !> エラーメッセージ表示
  subroutine error_abort(message)
    character(len=*),intent(in)::message
    print *, 'Error : ',trim(message)
    call abort()
  end subroutine error_abort

  !--------------------------------------------------------------------
end module pl_prof_class
