! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C======================================================================
C T = S^{-1/2} ... cholesky factorization.
C======================================================================
      subroutine rhf_t_matrix(
     O     t,
     I     s,
     I     nao,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension t(nao,nao)
      dimension s(nao,nao)

c     W = S(upper triangular part)
      t = 0.0d0
      do j = 1, nao
         do i = 1, j
            t(i,j) = s(i,j)
         enddo
      enddo

c     W = L * L^transpose
      call dpotrf( 'U', nao, t, nao, ierr )
      if ( ierr.ne.0 ) go to 9999

c     T = L^{-1}
      call dtrtri( 'U', 'N', nao, t, nao, ierr )
      if ( ierr.ne.0 ) go to 9999

      if ( ldebug >= 2 ) then
         call ummtw( 'T matrix', t, nao, nao, nao, 8, 6, ierr )
      endif

 9999 return
      end
C======================================================================
C T = S^{-1/2} ... cannonical orthogonalization
C======================================================================
      subroutine rhf_t_matrix_cannonical(
     O     t,
     I     s,
     I     nao,
     O     nindep,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension t(nao,nao)
      dimension s(nao,nao)

      real(8),allocatable::e(:)

      nindep = 0

      if ( ldebug >= 2 ) then
         call ummtw( 'S', s, nao, nao, nao, 8, 6, ierr )
      endif

c     W = S(lower triangular part)
      t = 0.0d0
      do j = 1, nao
         do i = 1, nao
            t(i,j) = s(i,j)
         enddo
      enddo

c     W U = U E
      allocate(e(nao))
      call eigen( t, e, nao, ierr )
      if ( ierr.ne.0 ) go to 9999
      if ( ldebug >= 2 ) then
         call ummtw2( 'W U = U E', t, e, nao, nao, nao, 8, 6, ierr )
      endif

c     T = U e^{-1/2}
      do j = 1, nao
         x = 1.0d0/sqrt(e(j))
         do i = 1, nao
            t(i,j) = t(i,j)*x
         enddo
      enddo

      deallocate(e)
      if ( ldebug >= 2 ) then
         call ummtw( 'T matrix', t, nao, nao, nao, 8, 6, ierr )
      endif

 9999 return
      end
C======================================================================
C  Similarity transformation
C     B = U^transpose * A * U
C======================================================================
      subroutine similar_trans(
     O     b,
     I     a, u,
     I     n,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension b(n,n)
      dimension a(n,n)
      dimension u(n,n)

      real(8),allocatable :: w(:,:)

      allocate(w(n,n))

      if ( ldebug >= 2 ) then
         call ummtw( 'u', u, n, n, n, 8, 6, ierr )
         call ummtw( 'a', a, n, n, n, 8, 6, ierr )
      endif

      call dsymm( 'l', 'l', n, n, 1.0d0, a, n, u, n, 0.0d0, w, n )
      if ( ldebug >= 2 ) then
         call ummtw( 'au', w, n, n, n, 8, 6, ierr )
      endif
      call dgemm( 't', 'n', n, n, n, 1.0d0, u, n, w, n, 0.0d0, b, n )

      if ( ldebug >= 2 ) then
         call ummtw( 'B = U^t A U', b, n, n, n, 8, 6, ierr )
      endif

      deallocate(w)

      return
      end
C======================================================================
C     D = B * A * C^transpose
C     (m x n) = (m x k) * (k x l) * (n x l)^t
C======================================================================
      subroutine mxmxm_BACt(
     O     D,
     I     ld, A, la, B, lb, C, lc,
     I     m, n, k, l,
     I     ldebug,
     O     ierr)
      implicit real(8) (a-h,o-z)

      dimension d(ld,n) ! m x n
      dimension a(la,l) ! k x l
      dimension b(lb,k) ! m x k
      dimension c(lc,l) ! n x l

      real(8),allocatable::W(:,:)
      integer::lw

      if (ldebug >= 2) then
         call ummtw('A', A, la, k, l, 10, 6, ierr)
         call ummtw('B', B, lb, m, k, 10, 6, ierr)
         call ummtw('C', C, lc, n, l, 10, 6, ierr)
      endif

      lw = m
      allocate(W(lw,l))

      call dgemm('N', 'N', m, l, k, 1.0d0, B, lb, A, la, 0.0d0, W, lw)
      call dgemm('N', 'T', m, n, l, 1.0d0, W, lw, C, lc, 0.0d0, D, ld)

      if (ldebug >= 2) then
         call ummtw('D = B * A * C^transpose', D, ld, m, n, 10, 6, ierr)
      endif

      deallocate(W)

      return
      end
C======================================================================
C     D = B^transpose * A * C
C     (m x n) = (k x m)^t * (k x l) * (l x n)
C======================================================================
      subroutine mxmxm_BtAC(
     O     D,
     I     ld, A, la, B, lb, C, lc,
     I     m, n, k, l,
     I     ldebug,
     O     ierr)
      implicit real(8) (a-h,o-z)

      dimension d(ld,n) ! m x n
      dimension a(la,l) ! k x l
      dimension b(lb,m) ! k x m
      dimension c(lc,n) ! l x n

      real(8),allocatable::W(:,:)
      integer::lw

      if (ldebug >= 2) then
         call ummtw('A', A, la, k, l, 10, 6, ierr)
         call ummtw('B', B, lb, k, m, 10, 6, ierr)
         call ummtw('C', C, lc, l, n, 10, 6, ierr)
      endif

      lw = k
      allocate(W(k,n))

      call dgemm('N', 'N', k, n, l, 1.0d0, A, la, C, lc, 0.0d0, W, lw)
      call dgemm('T', 'N', m, n, k, 1.0d0, B, lb, W, lw, 0.0d0, D, ld)

      if (ldebug >= 2) then
         call ummtw('D = B^transpose * A * C', D, ld, m, n, 10, 6, ierr)
      endif

      deallocate(W)

      return
      end
C======================================================================
C  Modified Gram-Schmidt orthogonalization
C======================================================================
      subroutine modified_gram_schmidt(
     $     v, ld, m, n, thresh, indep, nindep, ldebug, istat)
      implicit real(8) (a-h,o-z)

      dimension v(ld,n)  !> v(m,n)
      dimension indep(n)

      if (ldebug > 0) then
         call ummtw('v:mgs 0', v, ld, m, n, 10, 6, istat)
      end if

      nindep = 0
      do i = 1, n
         x = dnrm2(m, v(1,i), 1)
         if (ldebug > 0) write(6,*) 'dnrm2=',i,x
         if (dabs(x) > thresh) then
            indep(i) = 1
            nindep = nindep + 1
            x = 1.0d0/x
            do k = 1, m
               v(k,i) = v(k,i) * x
            enddo
            do j = i+1, n
               x = -ddot(m, v(1,i), 1, v(1,j), 1)
               if (ldebug > 0) write(6,*) 'ddot=',i,j,x
               call daxpy(m, x, v(1,i), 1, v(1,j), 1)
            enddo
         else
            indep(i) = 0
         endif
      enddo

      if (ldebug > 0) then
         call ummtw('v:mgs 1', v, ld, m, n, 10, 6, istat)
         write(6,*) 'indep=', (indep(i), i = 1, n)
      end if

      istat = 0
      return
      end
C======================================================================
C  Modified Gram-Schmidt orthogonalization
C======================================================================
      subroutine modified_gram_schmidts(
     $     v, ld, m, n, l, thresh, indep, nindep, ldebug, istat)
      implicit real(8) (a-h,o-z)

      dimension v(ld,n)  !> v(m,n)
      dimension indep(n)

      if (ldebug > 0) then
         call ummtw('v:mgs 0', v, ld, m, n, 10, 6, istat)
      end if

      nindep = 0
      do i = 1, l
         x = dnrm2(m, v(1,i), 1)
         if (ldebug > 0) write(6,*) 'dnrm2=',i,x
         if (dabs(x) > thresh) then
            indep(i) = 1
            nindep = nindep + 1
            x = 1.0d0/x
            do k = 1, m
               v(k,i) = v(k,i) * x
            enddo
         else
            indep(i) = 0
         endif
      enddo
      do j = l+1, n
         x = dnrm2(m, v(1,j), 1)
         if (ldebug > 0) write(6,*) 'dnrm2j=',j,x
         if (dabs(x) > thresh) then
            indep(j) = 1
            nindep = nindep + 1
            x = 1.0d0/x
            do k = 1, m
               v(k,j) = v(k,j) * x
            enddo
            do i = 1, l
               x = -ddot(m, v(1,i), 1, v(1,j), 1)
               if (ldebug > 0) write(6,*) 'ddot=',i,j,x
               call daxpy(m, x, v(1,i), 1, v(1,j), 1)
            enddo
         else
            indep(j) = 0
         endif
      enddo

      if (ldebug > 0) then
         call ummtw('v:mgs 1', v, ld, m, n, 10, 6, istat)
         write(6,*) 'indep=', (indep(i), i = 1, n)
      end if

      istat = 0
      return
      end
C======================================================================
C
C======================================================================
      subroutine rhf_new_mo(
     O     c, oen,
     I     f, t,
     I     nao, nmo,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension c(nao,nmo), oen(nmo)
      dimension f(nao,nao)
      dimension t(nao,nao)

      real(8),allocatable :: wm1(:,:)
      allocate(wm1(nao,nao))

      if ( ldebug >= 2 ) then
         call ummtw( 't', t, nao, nao, nao, 8, 6, ierr )
         call ummtw( 'f', f, nao, nao, nao, 8, 6, ierr )
      endif

c     W = T^transpose * F * T
      call similar_trans(
     O     wm1,
     I     f, t,
     I     nao,
     I     ldebug,
     O     ierr )
      if ( ierr.ne.0 ) go to 9901

      if ( ldebug >= 2 ) then
         call ummtw( 'T^t F T', wm1, nao, nao, nao, 8, 6, ierr )
      endif

c     W * U = U * E
      call eigen(
     U     wm1,
     O     oen,
     I     nao,
     O     ierr )
      if ( ierr.ne.0 ) go to 9901

      if ( ldebug >= 2 ) then
         call ummtw2( 'U', wm1, oen, nao, nao, nao, 8, 6, ierr )
      endif

c     C = T * U
      call dgemm( 'n', 'n',
     $     nao, nao, nao, 1.0d0, t, nao, wm1, nao, 0.0d0, c, nao )

      if ( ldebug >= 2 ) then
         call ummtw2( 'new mo', c, oen, nao, nao, nmo, 8, 6, ierr )
      endif

 9901 deallocate(wm1)
      return
      end
C======================================================================
C
C======================================================================
      subroutine eigen(
     U     a,
     O     e,
     I     n,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension a(n,n), e(n)

      dimension dummy(1)
      dimension idummy(1)
      real(8),allocatable :: w(:)
      integer,allocatable :: iw(:)

      !call ummtw( 'A', a, n, n, n, 8, 6, ierr )

      lwork  = -1
      liwork = -1
      call dsyevd(
     $     'v', 'l', n, a, n, e, dummy, lwork, idummy, liwork, ierr )
      if ( ierr.ne.0 ) go to 9901

      lwork  = int(dummy(1))
      liwork = int(dummy(1))
      allocate(w(lwork), iw(liwork))

      call dsyevd(
     $     'v', 'l', n, a, n, e, w, lwork, iw, liwork, ierr )
      if ( ierr.ne.0 ) go to 9900

 9900 deallocate(w, iw)

 9901 return
      end
C======================================================================
C
C======================================================================
      subroutine density_matrix(
     O     d,
     I     c,
     I     nao, nclosed,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension d(nao,nao)
      dimension c(nao,nclosed)

      d = 0.0d0

      call dgemm( 'n', 't',
     $     nao, nao, nclosed, 1.0d0, c, nao, c, nao, 0.0d0, d, nao )

      if ( ldebug >= 2 ) then
         call ummtw( 'density matrix', d, nao, nao, nao, 8, 6, ierr )
      endif

      return
      end
C======================================================================
C
C======================================================================
      subroutine diis(
     O     f_new,
     I     f,
     U     diis_vec, diis_err, diis_b,
     I     d, s, t,
     I     nao, mxdiis, idiis_curpos, idiis_cursize,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension f_new(nao,nao)
      dimension f(nao,nao)
      dimension d(nao,nao)
      dimension s(nao,nao)
      dimension t(nao,nao)
      dimension diis_vec(nao*(nao+1)/2,mxdiis)
      dimension diis_err(nao*(nao+1)/2,mxdiis)
      dimension diis_b(mxdiis,mxdiis)

      real(8),allocatable :: wm1(:,:)
      real(8),allocatable :: wm2(:,:)
      real(8),allocatable :: wm3(:,:)

      real(8),allocatable :: x(:)
      real(8),allocatable :: b(:,:)
      real(8),allocatable :: x_tmp(:)
      real(8),allocatable :: b_tmp(:,:)
      integer,allocatable :: iw(:)

      allocate(wm1(nao,nao))
      allocate(wm2(nao,nao))
      allocate(wm3(nao,nao))

      allocate(x(idiis_cursize+1))
      allocate(b(idiis_cursize+1,idiis_cursize+1))
      allocate(x_tmp(idiis_cursize+1))
      allocate(b_tmp(idiis_cursize+1,idiis_cursize+1))
      allocate(iw(idiis_cursize+1))

      if ( ldebug >= 2 ) then
         write(6,*) 'idiis_cursize = ',idiis_cursize
         write(6,*) 'idiis_curpos  = ',idiis_curpos
      endif

c     store diis-vector
      call sq2tri(
     O     diis_vec(1,idiis_curpos),
     I     f, nao )

c     e = FDS - SDF
      call diis_fds_sdf(
     O     wm1,
     I     f, d, s, t, nao,
     W     wm2, wm3,
     I     ldebug,
     O     ierr)

c     store diis-error-vector
      call sq2tri(
     O     diis_err(1,idiis_curpos),
     I     wm1, nao )

c     set B and x
      do i = 1, idiis_cursize
         val = ddot(
     $        nao*(nao+1)/2,
     $        diis_err(1,idiis_curpos), 1, diis_err(1,i), 1 )
         diis_b(idiis_curpos,i) = val
         diis_b(i,idiis_curpos) = val
      enddo

      do j = 1, idiis_cursize
         do i = 1, idiis_cursize
            b(i,j) = diis_b(i,j)
         enddo
      enddo

      do i = 1, idiis_cursize
         x(i)                 =  0.0
         b(idiis_cursize+1,i) = -1.0d0
         b(i,idiis_cursize+1) = -1.0d0
      enddo
      x(idiis_cursize+1)                 = -1.0d0
      b(idiis_cursize+1,idiis_cursize+1) =  0.0d0

      if ( ldebug >= 2 ) then
         call ummtw( 'B',
     $        b, idiis_cursize+1, idiis_cursize+1, idiis_cursize+1,
     $        8, 6, ierr )
         call ummtw( 'a',
     $        x, idiis_cursize+1, idiis_cursize+1, 1, 8, 6, ierr )
      endif

c     solve linear equations, Bx = a
      b_tmp = b
      x_tmp = x
      call dgesv( idiis_cursize+1, 1,
     $     b, idiis_cursize+1, iw, x, idiis_cursize+1, ierr )
      if ( ierr.ne.0 ) then
         ierr = 0
         b = b_tmp
         x = x_tmp
         call solveEigen( x, b, idiis_cursize+1, ldebug, ierr )
         if ( ierr.ne.0 ) goto 9900
      endif

      if ( ldebug >= 2 ) then
         call ummtw( 'x',
     $        x, idiis_cursize+1, idiis_cursize+1, 1, 8, 6, ierr )
      endif

c     scaling
      val = 0.0d0
      do i = 1, idiis_cursize
         val = val + x(i)
      enddo

      thres_val = 1.0d-15
      if ( dabs(val) < thres_val ) then
         do i = 1, idiis_cursize
            x(i) = 0.0d0
         enddo
         x(idiis_curpos) = 1.0d0
      else
         do i = 1, idiis_cursize
            x(i) = x(i)/val
         enddo
      endif

      if ( ldebug >= 2 ) then
         call ummtw( 'scaled x',
     $        x, idiis_cursize+1, idiis_cursize+1, 1, 8, 6, ierr )
      endif

c     extrapolate
      wm1 = 0.0d0
      do i = 1, idiis_cursize
         call daxpy( nao*(nao+1)/2, x(i), diis_vec(1,i), 1, wm1, 1 )
      enddo
      call tri2sq(
     O     f_new,
     I     wm1, nao )

      if ( ldebug >= 2 ) then
         call ummtw( 'DIIS, fock matrix',
     $        f_new, nao, nao, nao, 8, 6, ierr )
      endif

 9900 deallocate(wm1,wm2,wm3)
      deallocate(x,b,x_tmp,b_tmp,iw)

      return
      end
C======================================================================
C
C======================================================================
      subroutine diis_fds_sdf(
     O     fds_sdf,
     I     f, d, s, t, nao,
     W     wm2, wm3,
     I     ldebug,
     O     ierr)
      implicit real(8) (a-h,o-z)

      dimension fds_sdf(nao,nao)

      dimension f(nao,nao)
      dimension d(nao,nao)
      dimension s(nao,nao)
      dimension t(nao,nao)

      dimension wm2(nao,nao), wm3(nao,nao)

c     FDS - SDF
c       W := DS
      call dsymm( 'r', 'u',
     $     nao, nao, 1.0d0, s, nao, d, nao, 0.0d0, fds_sdf, nao )

c       FDS := FW
      call dgemm( 'n', 'n',
     $     nao, nao, nao, 1.0d0, f, nao, fds_sdf, nao, 0.0d0, wm2, nao )

c       W := SD
      call dsymm( 'l', 'u',
     $     nao, nao, 1.0d0, s, nao, d, nao, 0.0d0, fds_sdf, nao )

c       SDF := FW
      call dgemm( 'n', 'n',
     $     nao, nao, nao, 1.0d0, fds_sdf, nao, f, nao, 0.0d0, wm3, nao )

c       e = FDS - SDF
      do j = 1, nao
         do i = 1, nao
            wm2(i,j) = wm2(i,j) - wm3(i,j)
         enddo
      enddo

      if ( ldebug >= 2 ) then
         call ummtw( 'FDS-SDF', wm2, nao, nao, nao, 8, 6, ierr )
      endif

c     e' = T^transpose e T
      call similar_trans(
     O     fds_sdf,
     I     wm2, t,
     I     nao,
     I     ldebug,
     O     ierr )

      if ( ldebug >= 2 ) then
         call ummtw( 'T^transpose(FDS-SDF)T',
     $        fds_sdf, nao, nao, nao, 8, 6, ierr )
      endif

      return
      end
C======================================================================
C
C======================================================================
      subroutine solveEigen(
     U     x,
     I     b, n,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension x(n)
      dimension b(n,n)

      real(8),allocatable :: w1(:)
      real(8),allocatable :: w2(:)
      real(8),allocatable :: wm1(:)

      dimension dummy(1)

      ierr = 0
      thres = 1.0d-15

      allocate(w1(n))

      if ( ldebug >= 2 ) then
         call ummtw2( 'Ax=b', b, x, n, n, n, 8, 6, ierr )
      endif

      lwork = -1
      call dsyev( 'v', 'u', n, b, n, w1, dummy, lwork, ierr )
      if ( ierr.ne.0 ) return

      lwork = int(dummy(1))
      allocate( wm1(lwork) )
      call dsyev( 'v', 'u', n, b, n, w1, wm1, lwork, ierr )
      if ( ierr.ne.0 ) return

      if ( ldebug >= 2 ) then
         call ummtw2( 'UAU=e', b, w1, n, n, n, 8, 6, ierr )
      endif

!     x' = U *  e^{-1} * U^t * x
      allocate(w2(n))

!     e^{-1}
      do i = 1, n
         if ( dabs( w1(i) ) > thres ) then
            w1(i) = 1.0d0 / w1(i)
         else
            w1(i) = 0.0d0
            if ( ldebug > 0 ) then
               write(6,*) 'warning!! : DIIS/dsyev : ', dabs(w1(i))
            endif
         endif
      enddo

      do i = 1, n
         w2(i) = 0.0d0
         do j = 1, n
            w2(i) = w2(i) + b(j,i) * x(j)
         enddo
         w2(i) = w2(i) * w1(i)
      enddo

      do i = 1, n
         x(i) = 0.0d0
         do j = 1, n
            x(i) = x(i) + b(i,j) * w2(j)
         enddo
      enddo

      if ( ldebug >= 2 ) then
         call ummtw( 'x', x, n, n, n, 8, 6, ierr )
      endif

      deallocate(w1, w2, wm1)

      return
      end
C======================================================================
C
C======================================================================
      subroutine sq2tri(
     O     tri,
     I     sq, n )
      implicit real(8) (a-h,o-z)

      dimension tri(n*(n+1)/2)
      dimension sq(n,n)

      ipos = 0
      do i = 1, n
         do j = 1, i
            ipos = ipos + 1
            tri(ipos) = sq(i,j)
         enddo
      enddo

      return
      end
C======================================================================
C
C======================================================================
      subroutine tri2sq(
     O     sq,
     I     tri, n )
      implicit real(8) (a-h,o-z)

      dimension tri(n*(n+1)/2)
      dimension sq(n,n)

      ipos = 0
      do i = 1, n
         do j = 1, i
            ipos = ipos + 1
            sq(i,j) = tri(ipos)
            sq(j,i) = tri(ipos)
         enddo
      enddo

      return
      end
C======================================================================
C
C======================================================================
      subroutine convergence(
     O     conv_d_max, conv_d_rms,
     I     d, d_new,
     I     nao )
      implicit real(8) (a-h,o-z)

      dimension d(nao,nao)
      dimension d_new(nao,nao)

      conv_d_max = 0.0d0
      conv_d_rms = 0.0d0

      do j = 1, nao
         do i = 1, nao
            diff = dabs( d(i,j) - d_new(i,j) )
            conv_d_max = max( diff, conv_d_max )
            conv_d_rms = conv_d_rms + diff*diff
         enddo
      enddo
      conv_d_rms = dsqrt( conv_d_rms / (nao*nao) )

      return
      end
C======================================================================
C
C======================================================================
      subroutine lagrangian_matrix(
     O     eij,
     I     f, d, nao,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension eij(nao,nao)
      dimension f(nao,nao)
      dimension d(nao,nao)

      real(8),allocatable :: wm(:,:)

      allocate(wm(nao,nao))

c     Lagrangian = D F D
      call dsymm( 'L', 'U',
     $     nao, nao, 1.0d0, f, nao, d, nao, 0.0d0, wm, nao )

      call dgemm( 'T', 'N',
     $     nao, nao, nao,
     $     1.0d0, d, nao, wm, nao, 0.0d0, eij, nao )

      if ( ldebug >= 2 ) then
         call ummtw( 'lagrangian', eij, nao, nao, nao, 8, 6, ierr )
      endif

      deallocate(wm)

      return
      end
C======================================================================
C
C======================================================================
      subroutine interaction_energies_zd(
     O     e_sum,
     O     e_nucrep, e_nuc_gc, e_nuc_pc, e_nuc_fsw,
     O     e_gc_gc, e_gc_pc, e_gc_fsw,
     O     e_pc_pc, e_pc_fsw,
     O     e_fsw_fsw,
     I     xyz, charge, natom,
     I     xyz_gc, charge_gc, rc2v, ngc,
     I     xyz_pc, charge_pc, npc,
     I     xyz_fsw, charge_fsw, rc2v_fsw, nfsw,
     I     rcut, alpha, method_zd,
     I     ldebug,
     O     ierr)
      implicit real(8) (a-h,o-z)

      dimension xyz(3,natom)
      dimension charge(natom)

      dimension xyz_gc(3,ngc)
      dimension charge_gc(ngc)
      dimension rc2v(ngc)

      dimension xyz_pc(3,npc)
      dimension charge_pc(npc)

      dimension xyz_fsw(3,nfsw)
      dimension charge_fsw(nfsw)
      dimension rc2v_fsw(nfsw)

      e_nucrep = 0.0d0
      e_nuc_gc = 0.0d0
      e_nuc_pc = 0.0d0
      e_gc_gc  = 0.0d0
      e_gc_pc  = 0.0d0
      e_pc_pc  = 0.0d0
      e_nuc_fsw = 0.0d0
      e_pc_fsw  = 0.0d0
      e_gc_fsw  = 0.0d0
      e_fsw_fsw = 0.0d0

      if (ldebug > 0) then
         write(6,*) 'method_zd@interaction_energies_zd=',method_zd
      end if

      ! nuclear - nuclear
      call nuclear_repulsion_energy(
     O     e_nucrep,
     I     xyz, charge,
     I     natom,
     I     ldebug,
     O     ierr )
      if ( ierr.ne.0 ) go to 9999

      ! nuclear - gauss charge
      call nuclear_gauss_charge_energy(
     O     e_nuc_gc,
     I     xyz, charge, xyz_gc, charge_gc, rc2v,
     I     natom, ngc,
     I     ldebug,
     O     ierr )
      if ( ierr.ne.0 ) go to 9999

      ! nuclear - point charge
c$$$      if (method_zd == 0) then
         call nuclear_point_charge_energy(
     O        e_nuc_pc,
     I        xyz, charge, xyz_pc, charge_pc,
     I        natom, npc,
     I        ldebug,
     O        ierr )
         if ( ierr.ne.0 ) go to 9999
c$$$      else if (method_zd == 1 .or. method_zd == 2) then
c$$$         call nuclear_point_charge_energy_zd1(
c$$$     O        e_nuc_pc,
c$$$     I        xyz, charge, xyz_pc, charge_pc,
c$$$     I        natom, npc, rcut,
c$$$     I        ldebug,
c$$$     O        ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$      else if (method_zd == 3) then
c$$$         call nuclear_point_charge_energy_zd3(
c$$$     O        e_nuc_pc,
c$$$     I        xyz, charge, xyz_pc, charge_pc,
c$$$     I        natom, npc, rcut,
c$$$     I        ldebug,
c$$$     O        ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$      end if

      ! nuclear - Force-Switching Wolf
      call nuclear_fswolf_energy(
     O     e_nuc_fsw,
     I     xyz, charge, xyz_fsw, charge_fsw, rc2v_fsw,
     I     natom, nfsw,
     I     ldebug,
     O     ierr )
      if ( ierr.ne.0 ) go to 9999

! Omit the energy
      ! gauss charge - gauss charge
      call gauss_charge_repulsion_energy(
     O     e_gc_gc,
     I     xyz_gc, charge_gc, rc2v,
     I     ngc,
     I     ldebug,
     O     ierr )
      if ( ierr.ne.0 ) go to 9999
! Omit the energy End

! Omit the energy
      ! gauss charge - point charge
      call nuclear_gauss_charge_energy(
     O     e_gc_pc,
     I     xyz_pc, charge_pc, xyz_gc, charge_gc, rc2v,
     I     npc, ngc,
     I     ldebug,
     O     ierr )
      if ( ierr.ne.0 ) go to 9999
! Omit the energy End

! Omit the energy
      ! point charge - point charge
c$$$      if (method_zd == 0) then
         call nuclear_repulsion_energy(
     O        e_pc_pc,
     I        xyz_pc, charge_pc,
     I        npc,
     I        ldebug,
     O        ierr )
         if ( ierr.ne.0 ) go to 9999
c$$$      else if (method_zd == 1 .or. method_zd == 2) then
c$$$         call nuclear_repulsion_energy_zd1(
c$$$     O        e_pc_pc,
c$$$     I        xyz, xyz_pc, charge_pc,
c$$$     I        natom, npc, rcut,
c$$$     I        ldebug,
c$$$     O        ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$      else if (method_zd == 3) then
c$$$         call nuclear_repulsion_energy_zd3(
c$$$     O        e_pc_pc,
c$$$     I        xyz, xyz_pc, charge_pc,
c$$$     I        natom, npc, rcut,
c$$$     I        ldebug,
c$$$     O        ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$      end if
! Omit the energy End

      e_sum =
     $     e_nucrep + e_nuc_gc + e_nuc_pc + e_nuc_fsw
     $     + e_gc_gc + e_gc_pc + e_gc_fsw
     $     + e_pc_pc + e_pc_fsw
     $     + e_fsw_fsw

      if( ldebug > 0 ) then
         write(6,'(1x)')
         write(6,'('' Nuclear repulsion energy       ='',f20.10)')
     $        e_nucrep
         write(6,'('' Nuclear-GaussCharge energy     ='',f20.10)')
     $        e_nuc_gc
         write(6,'('' Nuclear-PointCharge energy     ='',f20.10)')
     $        e_nuc_pc
         write(6,'('' Nuclear-FSWolf energy          ='',f20.10)')
     $        e_nuc_fsw
         write(6,'('' GaussCharge-GaussCharge energy ='',f20.10)')
     $        e_gc_gc
         write(6,'('' GaussCharge-PointCharge energy ='',f20.10)')
     $        e_gc_pc
         write(6,'('' PointCharge-PointCharge energy ='',f20.10)')
     $        e_pc_pc
         write(6,'(''                            Sum :'',f20.10)')
     $        e_sum
      endif

 9999 return
      end
C======================================================================
C
C======================================================================
      subroutine interaction_forces_atoms(
     O     force,
     I     xyz, charge, natom,
     I     xyz_gc, charge_gc, rc2v, ngc,
     I     xyz_pc, charge_pc, npc,
     I     xyz_fsw, charge_fsw, rc2v_fsw, nfsw,
     I     icomm, nproc, myrank,
     $     ldebug,
     O     ierr )
      use pl_prof_class
c$$$      use mpi
      implicit real(8)(a-h,o-z)

      include 'mpif.h'

      dimension force(3,natom)

      dimension xyz(3,natom)
      dimension charge(natom)

      dimension xyz_gc(3,ngc)
      dimension charge_gc(ngc)
      dimension rc2v(ngc)

      dimension xyz_pc(3,npc)
      dimension charge_pc(npc)

      dimension xyz_fsw(3,nfsw)
      dimension charge_fsw(nfsw)
      dimension rc2v_fsw(nfsw)

! =========================================================
! working arrays
      real(8),allocatable :: gw(:,:)
! =========================================================

      if ( myrank == 0 ) then
         if ( ldebug > 0 ) then
            write(6,'(/''Coordinates :'')')
            if ( natom > 0 ) then
               write(6,'(''  Atoms :'')')
               do i = 1, natom
                  write(6,'(i6,f14.6,3f18.10)')
     $                 i, charge(i), (xyz(j,i), j = 1, 3)
               enddo
            endif
            if ( ngc > 0 ) then
               write(6,'(/''  Gauss charges :'')')
               do i = 1, ngc
                  write(6,'(i6,f14.6,4f18.10)')
     $                 i, charge_gc(i), (xyz_gc(j,i), j = 1, 3), rc2v(i)
               enddo
            endif
            if ( npc > 0 ) then
               write(6,'(/''  Point charges :'')')
               do i = 1, npc
                  write(6,'(i6,f14.6,3f18.10)')
     $                 i, charge_pc(i), (xyz_pc(j,i), j = 1, 3)
               enddo
            endif
         endif
      endif

! QM atoms (excluding electron<-{electron,nuclear,gauss charge,point charge})
      call mpi_barrier(icomm, istat)
      call pl_prof_region_begin('s:dNuRep_atom')
      if ( natom > 0 ) then
         allocate(
     $        gw(3,natom),
     $        stat=ierr)
         if ( ierr.ne.0 ) go to 9900

         gw(:,:) = 0.d0

         if ( myrank == 0 ) then
            ! nuclear <- nuclear
            call gconrg(gw,charge,xyz,natom,lprint,ldebug)
            call gsum_gnr(force,gw,natom)

            ! nuclear <- gauss charge
            call gconrg_nuc_gc(
     $           gw,charge,xyz,natom,
     $           xyz_gc, charge_gc, rc2v, ngc,
     $           lprint,ldebug)
            call gsum_gnr(force,gw,natom)

c$$$            ! nuclear <- point charge
c$$$            call gconrg_pc(
c$$$     $           gw,charge,xyz,natom,
c$$$     $           xyz_pc, charge_pc, npc,
c$$$     $           lprint,ldebug)
c$$$            call gsum_gnr(force,gw,natom)

            ! nuclear <- Force-Switching Wolf
            call gconrg_nuc_fsw(
     $           gw,charge,xyz,natom,
     $           xyz_fsw, charge_fsw, rc2v_fsw, nfsw,
     $           lprint,ldebug)
            call gsum_gnr(force,gw,natom)

         endif

         deallocate(gw)

      endif
      call mpi_barrier(icomm, istat)
      call pl_prof_region_end('s:dNuRep_atom')

      if ( myrank == 0 ) then
         if ( ldebug > 0 ) then
            write(6,'(1x)')
            write(6,'(''Forces :'')')
            if ( natom > 0 ) then
               write(6,'(''  Atoms :'')')
               do i = 1, natom
                  write(6,'(i6,3f18.10)') i, (force(j,i), j = 1, 3)
               enddo
            endif
         endif
      endif

      go to 9999
 9900 write(6,'(''Error : Array not allocated'')')
      ierr = 1
      go to 9999

 9999 return
      end
C======================================================================
C
C======================================================================
      subroutine interaction_forces(
     O     force,
     O     force_gc,
     O     force_pc,
     O     force_fsw,
     I     xyz, charge, natom,
     I     xyz_gc, charge_gc, rc2v, ngc,
     I     xyz_pc, charge_pc, npc,
     I     xyz_fsw, charge_fsw, rc2v_fsw, nfsw,
     $     rcut, method_zd,
     I     icomm, nproc, myrank,
     $     ldebug,
     O     ierr )
      use pl_prof_class
c$$$      use mpi
      implicit real(8)(a-h,o-z)

      include 'mpif.h'

      dimension force(3,natom)
      dimension force_pc(3,npc)
      dimension force_gc(3,ngc)
      dimension force_fsw(3,nfsw)

      dimension xyz(3,natom)
      dimension charge(natom)

      dimension xyz_gc(3,ngc)
      dimension charge_gc(ngc)
      dimension rc2v(ngc)

      dimension xyz_pc(3,npc)
      dimension charge_pc(npc)

      dimension xyz_fsw(3,nfsw)
      dimension charge_fsw(nfsw)
      dimension rc2v_fsw(nfsw)

! =========================================================
! working arrays
      real(8),allocatable :: gw(:,:)
! =========================================================

      if ( myrank == 0 ) then
         if ( ldebug > 0 ) then
            write(6,'(/''Coordinates :'')')
            if ( natom > 0 ) then
               write(6,'(''  Atoms :'')')
               do i = 1, natom
                  write(6,'(i6,f14.6,3f18.10)')
     $                 i, charge(i), (xyz(j,i), j = 1, 3)
               enddo
            endif
            if ( ngc > 0 ) then
               write(6,'(/''  Gauss charges :'')')
               do i = 1, ngc
                  write(6,'(i6,f14.6,4f18.10)')
     $                 i, charge_gc(i), (xyz_gc(j,i), j = 1, 3), rc2v(i)
               enddo
            endif
            if ( npc > 0 ) then
               write(6,'(/''  Point charges :'')')
               do i = 1, npc
                  write(6,'(i6,f14.6,3f18.10)')
     $                 i, charge_pc(i), (xyz_pc(j,i), j = 1, 3)
               enddo
            endif
            if ( nfsw > 0 ) then
               write(6,'(/''  Force-Switching Wolf :'')')
               do i = 1, nfsw
                  write(6,'(i6,f14.6,4f18.10)')
     $                 i,charge_fsw(i),(xyz_fsw(j,i),j=1,3),rc2v_fsw(i)
               enddo
            endif
         endif
      endif

! QM atoms (excluding electron<-{electron,nuclear,gauss charge,point charge})
      !force(:,:) = 0.0d0

      call mpi_barrier(icomm, istat)
      call pl_prof_region_begin('s:dNuRep_atom')
      if ( natom > 0 ) then
         allocate(
     $        gw(3,natom),
     $        stat=ierr)
         if ( ierr.ne.0 ) go to 9900

         gw(:,:) = 0.d0

         if ( myrank == 0 ) then
            ! nuclear <- nuclear
            !write(6,*) 'nuclear <- nuclear'
            call gconrg(gw,charge,xyz,natom,lprint,ldebug)
            call gsum_gnr(force,gw,natom)

            ! nuclear <- gauss charge
            call gconrg_nuc_gc(
     $           gw,charge,xyz,natom,
     $           xyz_gc, charge_gc, rc2v, ngc,
     $           lprint,ldebug)
            call gsum_gnr(force,gw,natom)

            ! nuclear <- point charge
c$$$            if (method_zd == 0) then
               call gconrg_pc(
     $              gw,charge,xyz,natom,
     $              xyz_pc, charge_pc, npc,
     $              lprint,ldebug)
               call gsum_gnr(force,gw,natom)
c$$$            else if (method_zd == 1 .or. method_zd == 2) then
c$$$               call gconrg_zd1(
c$$$     $              gw, xyz, charge, natom, rcut,
c$$$     $              xyz_pc, charge_pc, npc, ldebug)
c$$$               call gsum_gnr(force,gw,natom)
c$$$            else if (method_zd == 3) then
c$$$               call gconrg_zd3(
c$$$     $              gw, xyz, charge, natom, rcut,
c$$$     $              xyz_pc, charge_pc, npc, ldebug)
c$$$               call gsum_gnr(force,gw,natom)
c$$$            end if


            ! nuclear <- Force-Switching Wolf
            call gconrg_nuc_fsw(
     $           gw,charge,xyz,natom,
     $           xyz_fsw, charge_fsw, rc2v_fsw, nfsw,
     $           lprint,ldebug)
            call gsum_gnr(force,gw,natom)
         endif

         deallocate(gw)

      endif
      call mpi_barrier(icomm, istat)
      call pl_prof_region_end('s:dNuRep_atom')

! Gauss charges
      if ( ngc > 0 ) then
         call mpi_barrier(icomm, istat)
         call pl_prof_region_begin('s:dNuRep_gc')
         allocate(
     $        gw(3,ngc),
     $        stat=ierr)
         if ( ierr.ne.0 ) go to 9900

         gw(:,:) = 0.d0

         if ( myrank == 0 ) then
! Omit Gauss charge interaction
            ! gauss charge <- nuclear
            call gconrg_gc_nuc(
     $           gw,charge_gc,xyz_gc,rc2v,ngc,
     $           xyz, charge, natom,
     $           lprint,ldebug)
            call gsum_gnr(force_gc,gw,ngc)

            ! gauss charge <- gauss charge
            call gconrg_gc_gc(
     $           gw,charge_gc,xyz_gc,rc2v,ngc,lprint,ldebug)
            call gsum_gnr(force_gc,gw,ngc)

            ! gauss charge <- point charge
            call gconrg_gc_nuc(
     $           gw,charge_gc,xyz_gc,rc2v,ngc,
     $           xyz_pc, charge_pc, npc,
     $           lprint,ldebug)
            call gsum_gnr(force_gc,gw,ngc)
! Omit Gauss charge interaction
         endif

         deallocate(gw)

c$$$         call mpi_bcast(
c$$$     $        force_gc, 3*ngc, MPI_DOUBLE_PRECISION,0,icomm, ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$
c$$$         call dscal( 3*ngc, -1.0d0, force_gc, 1 )

         call mpi_barrier(icomm, istat)
         call pl_prof_region_end('s:dNuRep_gc')
      endif

! Point charges
      if ( npc > 0 ) then
         call mpi_barrier(icomm, istat)
         call pl_prof_region_begin('s:dNuRep_pc')
         allocate(
     $        gw(3,npc),
     $        stat=ierr)
         if ( ierr.ne.0 ) go to 9900

         gw(:,:) = 0.d0

         if ( myrank == 0 ) then
! Omit Point charge interaction
            ! point charge <- nuclear
c$$$            if (method_zd == 0) then
               call gconrg_pc(
     $              gw,charge_pc,xyz_pc,npc,
     $              xyz, charge, natom,
     $              lprint,ldebug)
               call gsum_gnr(force_pc,gw,npc)
c$$$            else if (method_zd == 1 .or. method_zd == 2) then
c$$$               call gconrg_on_zd1(
c$$$     $              gw,xyz_pc,charge_pc,npc,rcut,
c$$$     $              xyz,charge,natom,ldebug)
c$$$               call gsum_gnr(force_pc,gw,npc)
c$$$            else if (method_zd == 3) then
c$$$               call gconrg_on_zd3(
c$$$     $              gw,xyz_pc,charge_pc,npc,rcut,
c$$$     $              xyz,charge,natom,ldebug)
c$$$               call gsum_gnr(force_pc,gw,npc)
c$$$            end if

            ! point charge <- gauss charge
            call gconrg_nuc_gc(
     $           gw,charge_pc,xyz_pc,npc,
     $           xyz_gc, charge_gc, rc2v, ngc,
     $           lprint,ldebug)
            call gsum_gnr(force_pc,gw,npc)

            ! point charge <- point charge
c$$$            if (method_zd == 0) then
               call gconrg(gw,charge_pc,xyz_pc,npc,lprint,ldebug)
               call gsum_gnr(force_pc,gw,npc)
c$$$            else if (method_zd == 1 .or. method_zd == 2) then
c$$$               call gconrg_pc_zd1(
c$$$     $              gw,charge_pc,xyz_pc,npc,
c$$$     $              xyz,natom,rcut,lprint,ldebug)
c$$$               call gsum_gnr(force_pc,gw,npc)
c$$$            else if (method_zd == 3) then
c$$$               call gconrg_pc_zd3(
c$$$     $              gw,charge_pc,xyz_pc,npc,
c$$$     $              xyz,natom,rcut,lprint,ldebug)
c$$$               call gsum_gnr(force_pc,gw,npc)
c$$$            end if
! Omit Point charge interaction
         endif

         deallocate(gw)

c$$$         call mpi_bcast(
c$$$     $        force_pc, 3*npc, MPI_DOUBLE_PRECISION,0,icomm, ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$
c$$$         call dscal( 3*npc,   -1.0d0, force_pc, 1 )

         call mpi_barrier(icomm, istat)
         call pl_prof_region_end('s:dNuRep_pc')
      endif

      if ( myrank == 0 ) then
         if ( ldebug > 0 ) then
            write(6,'(1x)')
            write(6,'(''Forces @interaction_forces:'')')
            if ( natom > 0 ) then
               write(6,'(''  Atoms :'')')
               do i = 1, natom
                  write(6,'(i6,3f18.10)') i, (force(j,i), j = 1, 3)
               enddo
            endif

            if ( ngc > 0 ) then
               write(6,'(1x)')
               write(6,'(''  Gauss charges :'')')
               do i = 1, ngc
                  write(6,'(i6,3f18.10)') i, (force_gc(j,i), j = 1, 3)
               enddo
            endif

            if ( npc > 0 ) then
               write(6,'(1x)')
               write(6,'(''  Point charges :'')')
               do i = 1, npc
                  write(6,'(i6,3f18.10)') i, (force_pc(j,i), j = 1, 3)
               enddo
            endif
         endif
      endif

      go to 9999
 9900 write(6,'(''Error : Array not allocated'')')
      ierr = 1
      go to 9999

 9999 return
      end
C======================================================================
C
C======================================================================
      subroutine nuclear_repulsion_energy(
     O     e_nucrep,
     I     xyz, charge,
     I     natom,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension xyz(3,natom)
      dimension charge(natom)

      thresh = 1.0d-6
      e_nucrep = 0.0d0

      if ( natom == 0 ) return

      do i = 1, natom
         e = 0.0d0
         do j = i+1, natom
            d = dsqrt(
     $             (xyz(1,i) - xyz(1,j)) * (xyz(1,i) - xyz(1,j))
     $           + (xyz(2,i) - xyz(2,j)) * (xyz(2,i) - xyz(2,j))
     $           + (xyz(3,i) - xyz(3,j)) * (xyz(3,i) - xyz(3,j))
     $           )
            if ( d > thresh ) e = e + charge(j)/d
         enddo
         e_nucrep = e_nucrep + charge(i)*e
      enddo

      if ( ldebug >= 2 ) then
         write(6,*) 'e_nucrep =', e_nucrep
      endif

      return
      end
c$$$C======================================================================
c$$$C
c$$$C======================================================================
c$$$      subroutine nuclear_repulsion_energy_zd1(
c$$$     O     e_nucrep,
c$$$     I     xyz, xyz_pc, charge_pc,
c$$$     I     natom, npc, rcut,
c$$$     I     ldebug,
c$$$     O     ierr )
c$$$      implicit real(8) (a-h,o-z)
c$$$
c$$$      dimension xyz(3,natom)
c$$$      dimension xyz_pc(3,npc)
c$$$      dimension charge_pc(npc)
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$      e_nucrep = 0.0d0
c$$$
c$$$      write(6,*) 'rcut@nuclear_repulsion_energy_zd1=',rcut,natom,npc
c$$$      if ( npc == 0 ) return
c$$$
c$$$      rcut2 = rcut**2
c$$$      rcut3 = rcut**3
c$$$
c$$$      e_nucrep1 = 0.0d0
c$$$      e_nucrep2 = 0.0d0
c$$$      e_nucrep3 = 0.0d0
c$$$      e_nucrep4 = 0.0d0
c$$$      do i = 1, npc
c$$$         e1 = 0.0d0
c$$$         e2 = 0.0d0
c$$$         e3 = 0.0d0
c$$$         do j = i+1, npc
c$$$            dd =
c$$$     $         (xyz_pc(1,i) - xyz_pc(1,j)) * (xyz_pc(1,i) - xyz_pc(1,j))
c$$$     $       + (xyz_pc(2,i) - xyz_pc(2,j)) * (xyz_pc(2,i) - xyz_pc(2,j))
c$$$     $       + (xyz_pc(3,i) - xyz_pc(3,j)) * (xyz_pc(3,i) - xyz_pc(3,j))
c$$$            d = dsqrt(dd)
c$$$            if ( d > rcut ) cycle
c$$$            if ( d > thresh ) then
c$$$               if (rcut > 0.0d0) then
c$$$                  e1 = e1 + charge_pc(j) / d
c$$$                  e2 = e2 + charge_pc(j) * dd
c$$$                  e3 = e3 + charge_pc(j)
c$$$               else
c$$$                  e1 = e1 + charge_pc(j) / d
c$$$               end if
c$$$            end if
c$$$         enddo
c$$$         if (rcut > 0.0d0) then
c$$$            e_nucrep1 = e_nucrep1 + charge_pc(i)*e1
c$$$            e_nucrep2 = e_nucrep2 + charge_pc(i)*e2
c$$$            e_nucrep3 = e_nucrep3 + charge_pc(i)*e3
c$$$            e_nucrep4 = e_nucrep4
c$$$     $           - 3.0d0/(4.0d0*rcut)*charge_pc(i)*charge_pc(i)
c$$$         else
c$$$            e_nucrep1 = e_nucrep1 + charge_pc(i)*e1
c$$$         end if
c$$$      enddo
c$$$      e_nucrep2 = e_nucrep2 / (2.0d0*rcut3)
c$$$      e_nucrep3 = e_nucrep3
c$$$     $     *(
c$$$     $     - 1.0d0/rcut
c$$$     $     - 1.0d0/(2.0d0*rcut)
c$$$     $     )
c$$$      e_nucrep = e_nucrep1 + e_nucrep2 + e_nucrep3 + e_nucrep4
c$$$      if ( ldebug >= 2 ) then
c$$$         write(6,'(a,f20.10)') 'e_pc_pc1 =', e_nucrep1
c$$$         write(6,'(a,f20.10)') 'e_pc_pc2 =', e_nucrep2
c$$$         write(6,'(a,f20.10)') 'e_pc_pc3 =', e_nucrep3
c$$$         write(6,'(a,f20.10)') 'e_pc_pc4 =', e_nucrep4
c$$$         write(6,'(a,f20.10)') 'e_pc_pc  =', e_nucrep
c$$$      endif
c$$$
c$$$      return
c$$$      end
c$$$C======================================================================
c$$$C
c$$$C======================================================================
c$$$      subroutine nuclear_repulsion_energy_zd3(
c$$$     O     e_nucrep,
c$$$     I     xyz, xyz_pc, charge_pc,
c$$$     I     natom, npc, rcut,
c$$$     I     ldebug,
c$$$     O     ierr )
c$$$      implicit real(8) (a-h,o-z)
c$$$
c$$$      dimension xyz(3,natom)
c$$$      dimension xyz_pc(3,npc)
c$$$      dimension charge_pc(npc)
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$      e_nucrep = 0.0d0
c$$$
c$$$      if ( ldebug >= 2 ) then
c$$$         write(6,*) 'rcut@nuclear_repulsion_energy_zd3=',rcut,natom,npc
c$$$      endif
c$$$      if ( npc == 0 ) return
c$$$
c$$$      rcut2 = rcut**2
c$$$      rcut3 = rcut**3
c$$$
c$$$      lskip(:) = .true.
c$$$      do k = 1, npc
c$$$         Cx = xyz_pc(1,k)
c$$$         Cy = xyz_pc(2,k)
c$$$         Cz = xyz_pc(3,k)
c$$$         flag = .false.
c$$$         do iatom = 1, natom
c$$$            Ax = xyz(1,iatom)
c$$$            Ay = xyz(2,iatom)
c$$$            Az = xyz(3,iatom)
c$$$            ACx = Ax - Cx
c$$$            ACy = Ay - Cy
c$$$            ACz = Az - Cz
c$$$            AC2 = ACx*ACx + ACy*ACy + ACz*ACz
c$$$            if (AC2 < rcut2) then
c$$$               flag = .true.
c$$$               go to 1
c$$$            end if
c$$$         end do
c$$$ 1       if (flag) lskip(k) = .false.
c$$$      end do
c$$$
c$$$      e_nucrep1 = 0.0d0
c$$$      e_nucrep2 = 0.0d0
c$$$      e_nucrep3 = 0.0d0
c$$$      e_nucrep4 = 0.0d0
c$$$      do i = 1, npc
c$$$         if (lskip(i)) cycle
c$$$         e1 = 0.0d0
c$$$         e2 = 0.0d0
c$$$         e3 = 0.0d0
c$$$         do j = i+1, npc
c$$$            if (lskip(j)) cycle
c$$$            dd =
c$$$     $         (xyz_pc(1,i) - xyz_pc(1,j)) * (xyz_pc(1,i) - xyz_pc(1,j))
c$$$     $       + (xyz_pc(2,i) - xyz_pc(2,j)) * (xyz_pc(2,i) - xyz_pc(2,j))
c$$$     $       + (xyz_pc(3,i) - xyz_pc(3,j)) * (xyz_pc(3,i) - xyz_pc(3,j))
c$$$            d = dsqrt(dd)
c$$$            if ( d > thresh ) then
c$$$               if (rcut > 0.0d0) then
c$$$                  e1 = e1 + charge_pc(j) / d
c$$$                  e2 = e2 + charge_pc(j) * dd
c$$$                  e3 = e3 + charge_pc(j)
c$$$               else
c$$$                  e1 = e1 + charge_pc(j) / d
c$$$               end if
c$$$            end if
c$$$         enddo
c$$$         if (rcut > 0.0d0) then
c$$$            e_nucrep1 = e_nucrep1 + charge_pc(i)*e1
c$$$            e_nucrep2 = e_nucrep2 + charge_pc(i)*e2
c$$$            e_nucrep3 = e_nucrep3 + charge_pc(i)*e3
c$$$            e_nucrep4 = e_nucrep4
c$$$     $           - 3.0d0/(4.0d0*rcut)*charge_pc(i)*charge_pc(i)
c$$$         else
c$$$            e_nucrep1 = e_nucrep1 + charge_pc(i)*e1
c$$$         end if
c$$$      enddo
c$$$      e_nucrep2 = e_nucrep2 / (2.0d0*rcut3)
c$$$      e_nucrep3 = e_nucrep3
c$$$     $     *(
c$$$     $     - 1.0d0/rcut
c$$$     $     - 1.0d0/(2.0d0*rcut)
c$$$     $     )
c$$$      e_nucrep = e_nucrep1 + e_nucrep2 + e_nucrep3 + e_nucrep4
c$$$      if ( ldebug >= 2 ) then
c$$$         write(6,'(a,f20.10)') 'e_pc_pc1 =', e_nucrep1
c$$$         write(6,'(a,f20.10)') 'e_pc_pc2 =', e_nucrep2
c$$$         write(6,'(a,f20.10)') 'e_pc_pc3 =', e_nucrep3
c$$$         write(6,'(a,f20.10)') 'e_pc_pc4 =', e_nucrep4
c$$$         write(6,'(a,f20.10)') 'e_pc_pc  =', e_nucrep
c$$$      endif
c$$$
c$$$      return
c$$$      end
C======================================================================
C
C======================================================================
      subroutine gauss_charge_repulsion_energy(
     O     e_nucrep,
     I     xyz_gc, charge_gc, rc2v,
     I     ngc,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension xyz_gc(3,ngc)
      dimension charge_gc(ngc)
      dimension rc2v(ngc)

      thresh = 1.0d-6
      e_nucrep = 0.0d0

      if ( ngc == 0 ) return

      do i = 1, ngc
         e = 0.0d0
         do j = i+1, ngc
            d = dsqrt(
     $             (xyz_gc(1,i)-xyz_gc(1,j)) * (xyz_gc(1,i)-xyz_gc(1,j))
     $           + (xyz_gc(2,i)-xyz_gc(2,j)) * (xyz_gc(2,i)-xyz_gc(2,j))
     $           + (xyz_gc(3,i)-xyz_gc(3,j)) * (xyz_gc(3,i)-xyz_gc(3,j))
     $           )
            if( rc2v(i) < 0.0d0 .or. rc2v(j) < 0.0d0 ) then
               write(6,'(" rc2v must not to be negative. ")') 
               stop 
            endif
            rc = dsqrt(rc2v(i))
            if( rc > thresh ) then
               charge_gc_i = charge_gc(i)*derf( d/rc )
            elseif( 0.0d0 <= rc .and. rc <= thresh ) then
               charge_gc_i = charge_gc(i)
            endif
            rc = dsqrt(rc2v(j))
            if( rc > thresh ) then
               charge_gc_j = charge_gc(j)*derf( d/rc )
            elseif( 0.0d0 <= rc .and. rc <= thresh ) then
               charge_gc_j = charge_gc(j)
            endif
            if ( d > thresh ) then
               e = e + charge_gc_i * charge_gc_j / d
            endif
         enddo
         e_nucrep = e_nucrep + e
      enddo

      if ( ldebug >= 2 ) then
         write(6,*) 'e_nucrep =', e_nucrep
      endif

      return
      end
C======================================================================
C
C======================================================================
      subroutine fswolf_repulsion_energy(
     O     e_nucrep,
     I     xyz_fsw, charge_fsw, rc2v_fsw,
     I     nfsw,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension xyz_fsw(3,nfsw)
      dimension charge_fsw(nfsw)
      dimension rc2v_fsw(nfsw)

      thresh = 1.0d-6
      e_nucrep = 0.0d0

      if ( nfsw == 0 ) return

      do i = 1, nfsw
         e = 0.0d0
         do j = i+1, nfsw
            d = dsqrt(
     $         (xyz_fsw(1,i)-xyz_fsw(1,j)) * (xyz_fsw(1,i)-xyz_fsw(1,j))
     $       + (xyz_fsw(2,i)-xyz_fsw(2,j)) * (xyz_fsw(2,i)-xyz_fsw(2,j))
     $       + (xyz_fsw(3,i)-xyz_fsw(3,j)) * (xyz_fsw(3,i)-xyz_fsw(3,j))
     $           )
            if( rc2v_fsw(i) < 0.0d0 .or. rc2v_fsw(j) < 0.0d0 ) then
               write(6,'(" rc2v must not to be negative. ")') 
               stop 
            endif
            rc = dsqrt(rc2v_fsw(i))
            if( rc > thresh ) then
               charge_fsw_i = charge_fsw(i)*derfc( d/rc )
            elseif( 0.0d0 <= rc .and. rc <= thresh ) then
               charge_fsw_i = charge_fsw(i)
            endif
            rc = dsqrt(rc2v_fsw(j))
            if( rc > thresh ) then
               charge_fsw_j = charge_fsw(j)*derfc( d/rc )
            elseif( 0.0d0 <= rc .and. rc <= thresh ) then
               charge_fsw_j = charge_fsw(j)
            endif
            if ( d > thresh ) then
               e = e + charge_fsw_i * charge_fsw_j / d
            endif
         enddo
         e_nucrep = e_nucrep + e
      enddo

      if ( ldebug >= 2 ) then
         write(6,*) 'e_nucrep =', e_nucrep
      endif

      return
      end
C======================================================================
C
C======================================================================
      subroutine nuclear_point_charge_energy(
     O     e_nuc_pc,
     I     xyz, charge, xyz_pc, charge_pc,
     I     natom, npc,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension xyz(3,natom)
      dimension charge(natom)
      dimension xyz_pc(3,npc)
      dimension charge_pc(npc)

      thresh = 1.0d-6
      e_nuc_pc = 0.0d0

      if ( npc == 0 ) return

      do i = 1, natom
         e = 0.0d0
         do j = 1, npc
            d = dsqrt(
     $        (xyz(1,i) - xyz_pc(1,j)) * (xyz(1,i) - xyz_pc(1,j))
     $      + (xyz(2,i) - xyz_pc(2,j)) * (xyz(2,i) - xyz_pc(2,j))
     $      + (xyz(3,i) - xyz_pc(3,j)) * (xyz(3,i) - xyz_pc(3,j))
     $           )
            if ( d > thresh ) e = e + charge_pc(j)/d
         enddo
         e_nuc_pc = e_nuc_pc + charge(i)*e
      enddo

      if ( ldebug >= 2 ) then
         write(6,*) 'e_nuc_pc =', e_nuc_pc
      endif

      return
      end
c$$$C======================================================================
c$$$C
c$$$C======================================================================
c$$$      subroutine nuclear_point_charge_energy_zd1(
c$$$     O     e_nuc_pc,
c$$$     I     xyz, charge, xyz_pc, charge_pc,
c$$$     I     natom, npc, rcut,
c$$$     I     ldebug,
c$$$     O     ierr )
c$$$      implicit real(8) (a-h,o-z)
c$$$
c$$$      dimension xyz(3,natom)
c$$$      dimension charge(natom)
c$$$      dimension xyz_pc(3,npc)
c$$$      dimension charge_pc(npc)
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$      e_nuc_pc = 0.0d0
c$$$
c$$$      if ( npc == 0 ) return
c$$$
c$$$      !rcut = 16.0d0/0.5291772083d0
c$$$      dmax = 0.0d0
c$$$      nmax = 0
c$$$      if ( ldebug >= 2 ) then
c$$$         write(6,*) 'rcut@nuclear_point_charge_energy_zd1=',rcut
c$$$      endif
c$$$      rcut2 = rcut**2
c$$$      rcut3 = rcut**3
c$$$
c$$$      e_nuc_pc1 = 0.0d0
c$$$      e_nuc_pc2 = 0.0d0
c$$$      e_nuc_pc3 = 0.0d0
c$$$      e_nuc_pc4 = 0.0d0
c$$$      do i = 1, natom
c$$$         e1 = 0.0d0
c$$$         e2 = 0.0d0
c$$$         e3 = 0.0d0
c$$$         n = 0
c$$$         do j = 1, npc
c$$$            dd =
c$$$     $        (xyz(1,i) - xyz_pc(1,j)) * (xyz(1,i) - xyz_pc(1,j))
c$$$     $      + (xyz(2,i) - xyz_pc(2,j)) * (xyz(2,i) - xyz_pc(2,j))
c$$$     $      + (xyz(3,i) - xyz_pc(3,j)) * (xyz(3,i) - xyz_pc(3,j))
c$$$            d = dsqrt(dd)
c$$$            dmax = dmax1(dmax, d)
c$$$            if ( d > rcut ) cycle
c$$$            n = n + 1
c$$$            if ( d > thresh ) then
c$$$               if (rcut > 0.0d0) then
c$$$                  e1 = e1 + charge_pc(j) / d
c$$$                  e2 = e2 + charge_pc(j) * dd
c$$$                  e3 = e3 + charge_pc(j)
c$$$               else
c$$$                  e1 = e1 + charge_pc(j) / d
c$$$               end if
c$$$            end if
c$$$         enddo
c$$$         nmax = max0(nmax, n)
c$$$         if (rcut > 0.0d0) then
c$$$            e_nuc_pc1 = e_nuc_pc1 + charge(i)*e1
c$$$            e_nuc_pc2 = e_nuc_pc2 + charge(i)*e2
c$$$            e_nuc_pc3 = e_nuc_pc3 + charge(i)*e3
c$$$            e_nuc_pc4 = e_nuc_pc4
c$$$     $           - 3.0d0/(4.0d0*rcut)*charge(i)*charge(i)
c$$$         else
c$$$            e_nuc_pc1 = e_nuc_pc1 + charge(i)*e1
c$$$         end if
c$$$      enddo
c$$$      e_nuc_pc2 = e_nuc_pc2 / (2.0d0*rcut3)
c$$$      e_nuc_pc3 = e_nuc_pc3
c$$$     $     *(
c$$$     $     - 1.0d0/rcut
c$$$     $     - 1.0d0/(2.0d0*rcut)
c$$$     $     )
c$$$      e_nuc_pc = e_nuc_pc1 + e_nuc_pc2 + e_nuc_pc3 + e_nuc_pc4
c$$$      if ( ldebug >= 2 ) then
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc1 =', e_nuc_pc1
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc2 =', e_nuc_pc2
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc3 =', e_nuc_pc3
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc4 =', e_nuc_pc4
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc  =', e_nuc_pc
c$$$         write(6,*) 'ZD: dmax=',dmax, ',', nmax, '/', npc
c$$$      endif
c$$$
c$$$      return
c$$$      end
c$$$C======================================================================
c$$$C
c$$$C======================================================================
c$$$      subroutine nuclear_point_charge_energy_zd3(
c$$$     O     e_nuc_pc,
c$$$     I     xyz, charge, xyz_pc, charge_pc,
c$$$     I     natom, npc, rcut,
c$$$     I     ldebug,
c$$$     O     ierr )
c$$$      implicit real(8) (a-h,o-z)
c$$$
c$$$      dimension xyz(3,natom)
c$$$      dimension charge(natom)
c$$$      dimension xyz_pc(3,npc)
c$$$      dimension charge_pc(npc)
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$      e_nuc_pc = 0.0d0
c$$$
c$$$      if ( npc == 0 ) return
c$$$
c$$$      !rcut = 16.0d0/0.5291772083d0
c$$$      if ( ldebug >= 2 ) then
c$$$         write(6,*) 'rcut@nuclear_point_charge_energy_zd3=',rcut
c$$$      endif
c$$$      rcut2 = rcut**2
c$$$      rcut3 = rcut**3
c$$$
c$$$      lskip(:) = .true.
c$$$      do k = 1, npc
c$$$         Cx = xyz_pc(1,k)
c$$$         Cy = xyz_pc(2,k)
c$$$         Cz = xyz_pc(3,k)
c$$$         flag = .false.
c$$$         do iatom = 1, natom
c$$$            Ax = xyz(1,iatom)
c$$$            Ay = xyz(2,iatom)
c$$$            Az = xyz(3,iatom)
c$$$            ACx = Ax - Cx
c$$$            ACy = Ay - Cy
c$$$            ACz = Az - Cz
c$$$            AC2 = ACx*ACx + ACy*ACy + ACz*ACz
c$$$            if (AC2 < rcut2) then
c$$$               flag = .true.
c$$$               go to 1
c$$$            end if
c$$$         end do
c$$$ 1       if (flag) lskip(k) = .false.
c$$$      end do
c$$$
c$$$      dmax = 0.0d0
c$$$      nmax = 0
c$$$      e_nuc_pc1 = 0.0d0
c$$$      e_nuc_pc2 = 0.0d0
c$$$      e_nuc_pc3 = 0.0d0
c$$$      e_nuc_pc4 = 0.0d0
c$$$      do i = 1, natom
c$$$         e1 = 0.0d0
c$$$         e2 = 0.0d0
c$$$         e3 = 0.0d0
c$$$         n = 0
c$$$         do j = 1, npc
c$$$            if (lskip(j)) cycle
c$$$            dd =
c$$$     $        (xyz(1,i) - xyz_pc(1,j)) * (xyz(1,i) - xyz_pc(1,j))
c$$$     $      + (xyz(2,i) - xyz_pc(2,j)) * (xyz(2,i) - xyz_pc(2,j))
c$$$     $      + (xyz(3,i) - xyz_pc(3,j)) * (xyz(3,i) - xyz_pc(3,j))
c$$$            d = dsqrt(dd)
c$$$            dmax = dmax1(dmax, d)
c$$$            n = n + 1
c$$$            if ( d > thresh ) then
c$$$               if (rcut > 0.0d0) then
c$$$                  e1 = e1 + charge_pc(j) / d
c$$$                  e2 = e2 + charge_pc(j) * dd
c$$$                  e3 = e3 + charge_pc(j)
c$$$               else
c$$$                  e1 = e1 + charge_pc(j) / d
c$$$               end if
c$$$            end if
c$$$         enddo
c$$$         nmax = max0(nmax, n)
c$$$         if (rcut > 0.0d0) then
c$$$            e_nuc_pc1 = e_nuc_pc1 + charge(i)*e1
c$$$            e_nuc_pc2 = e_nuc_pc2 + charge(i)*e2
c$$$            e_nuc_pc3 = e_nuc_pc3 + charge(i)*e3
c$$$            e_nuc_pc4 = e_nuc_pc4
c$$$     $           - 3.0d0/(4.0d0*rcut)*charge(i)*charge(i)
c$$$         else
c$$$            e_nuc_pc1 = e_nuc_pc1 + charge(i)*e1
c$$$         end if
c$$$      enddo
c$$$      e_nuc_pc2 = e_nuc_pc2 / (2.0d0*rcut3)
c$$$      e_nuc_pc3 = e_nuc_pc3
c$$$     $     *(
c$$$     $     - 1.0d0/rcut
c$$$     $     - 1.0d0/(2.0d0*rcut)
c$$$     $     )
c$$$      e_nuc_pc = e_nuc_pc1 + e_nuc_pc2 + e_nuc_pc3 + e_nuc_pc4
c$$$      if ( ldebug >= 2 ) then
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc1 =', e_nuc_pc1
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc2 =', e_nuc_pc2
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc3 =', e_nuc_pc3
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc4 =', e_nuc_pc4
c$$$         write(6,'(a,f20.10)') 'e_nuc_pc  =', e_nuc_pc
c$$$         write(6,*) 'ZD: dmax=',dmax, ',', nmax, '/', npc
c$$$      endif
c$$$
c$$$      return
c$$$      end
C======================================================================
C
C======================================================================
      subroutine nuclear_gauss_charge_energy(
     O     e_nuc_gc,
     I     xyz, charge, xyz_gc, charge_gc, rc2v,
     I     natom, ngc,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension xyz(3,natom)
      dimension charge(natom)
      dimension xyz_gc(3,ngc)
      dimension charge_gc(ngc)
      dimension rc2v(ngc)

      thresh = 1.0d-6
      e_nuc_gc = 0.0d0

      if ( ngc == 0 ) return

      do i = 1, natom
         e = 0.0d0
         do j = 1, ngc
            d = dsqrt(
     $        (xyz(1,i) - xyz_gc(1,j)) * (xyz(1,i) - xyz_gc(1,j))
     $      + (xyz(2,i) - xyz_gc(2,j)) * (xyz(2,i) - xyz_gc(2,j))
     $      + (xyz(3,i) - xyz_gc(3,j)) * (xyz(3,i) - xyz_gc(3,j))
     $           )
!     V = Q q erf(r/rc)/r
            if ( d > thresh ) then
               if( rc2v(j) < 0.0d0 ) then
                  write(6,'(" rc2v must not to be negative. ")') 
                  stop 
               endif
               rc = dsqrt(rc2v(j))
               if( rc > thresh ) then
                  e = e + charge_gc(j) * derf( d/rc ) / d
               else
                  e = e + charge_gc(j) / d
               endif
            endif
         enddo
         e_nuc_gc = e_nuc_gc + charge(i)*e
      enddo

      if ( ldebug >= 2 ) then
         write(6,*) 'e_nuc_gc =', e_nuc_gc
      endif

      return
      end
C======================================================================
C
C======================================================================
      subroutine nuclear_fswolf_energy(
     O     e_nuc_fsw,
     I     xyz, charge, xyz_fsw, charge_fsw, rc2v_fsw,
     I     natom, nfsw,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension xyz(3,natom)
      dimension charge(natom)
      dimension xyz_fsw(3,nfsw)
      dimension charge_fsw(nfsw)
      dimension rc2v_fsw(nfsw)

      thresh = 1.0d-6
      e_nuc_fsw = 0.0d0

      if ( nfsw == 0 ) return

      do i = 1, natom
         e = 0.0d0
         do j = 1, nfsw
            d = dsqrt(
     $        (xyz(1,i) - xyz_fsw(1,j)) * (xyz(1,i) - xyz_fsw(1,j))
     $      + (xyz(2,i) - xyz_fsw(2,j)) * (xyz(2,i) - xyz_fsw(2,j))
     $      + (xyz(3,i) - xyz_fsw(3,j)) * (xyz(3,i) - xyz_fsw(3,j))
     $           )
!     V = Q q erfc(r/rc)/r
            if ( d > thresh ) then
               if( rc2v_fsw(j) < 0.0d0 ) then
                  write(6,'(" rc2v must not to be negative. ")') 
                  stop 
               endif
               rc = dsqrt(rc2v_fsw(j))
               if( rc > thresh ) then
                  e = e + charge_fsw(j) * derfc( d/rc ) / d
               else
                  e = e + charge_fsw(j) / d
               endif
            endif
         enddo
         e_nuc_fsw = e_nuc_fsw + charge(i)*e
      enddo

      if ( ldebug >= 2 ) then
         write(6,*) 'e_nuc_fsw =', e_nuc_fsw
      endif

      return
      end
C======================================================================
C
C======================================================================
      subroutine orthogonalize_mo(
     O     c,
     I     s,
     I     nao, nmo,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      parameter ( thresh = 1.0d-8 )

      dimension c(nao,nmo)
      dimension s(nao,nao)

      real(8),allocatable :: work(:)
      real(8),allocatable :: wv1(:)
      real(8),allocatable :: wm1(:,:)
      real(8),allocatable :: wm2(:,:)

      if ( ldebug >= 1 ) then
         call ummtw( 'C', c, nao, nao, nmo, 8, 6, ierr )
      endif

      allocate(
     $     wv1(nmo),
     $     wm1(nmo, nmo),
     $     wm2(nao, nmo),
     $     stat=ierr)
      if ( ierr.ne.0 ) go to 9900

!     W = C^t * S * C
!     S C
      call dsymm(
     $     'L', 'U',
     $     nao, nmo,
     $     1.0d0, s, nao, c, nao, 0.0d0, wm2, nao )
      call dgemm(
     $     'T', 'N',
     $     nmo, nmo, nao,
     $     1.0d0, c, nao, wm2, nao, 0.0d0, wm1, nmo )

      if ( ldebug >= 1 ) then
         call ummtw( 'C^t*S*C', wm1, nmo, nmo, nmo, 8, 6, ierr )
      endif

!     if ( W == I ) return 0
      iflag = 0
      do i = 1, nmo
         if ( dabs(1.0d0 - wm1(i,i)) > thresh ) iflag = 1
      enddo

      do i = 1, nmo
         do j = i+1, nmo
            if ( dabs(wm1(i,j)) > thresh ) iflag = 1
            if ( dabs(wm1(j,i)) > thresh ) iflag = 1
         enddo
      enddo
      if ( iflag == 0 ) go to 9999
      
!     W * U = U * E
      lwork = -1
      call dsyev( 'V', 'U', nmo, wm1, nmo, wv1, dummy, lwork, info )
      if ( info /= 0 ) go to 9901

      lwork = int(dummy)
      if ( ldebug >= 1 ) then
         write(6,*) 'lwork = ', lwork
      endif
      allocate(
     $     work(lwork),
     $     stat=ierr)
      if ( ierr.ne.0 ) go to 9900

      call dsyev( 'V', 'U', nmo, wm1, nmo, wv1, work, lwork, info )
      if ( info /= 0 ) go to 9901

      deallocate(work)

      if ( ldebug >= 1 ) then
         call ummtw2( 'W*U=U*E', wm1, wv1, nmo, nmo, nmo, 8, 6, ierr )
      endif

!     C = C * U * E^{-1/2} * U^transpose
!     W2 = C * U
      call dgemm(
     $     'N', 'N',
     $     nao, nmo, nmo,
     $     1.0d0, c, nao, wm1, nmo, 0.0d0, wm2, nao )

!     W1 = U * E^{-1/2}
      do j = 1, nmo
         if ( wv1(j) > 1.0e-10 ) then
            do i = 1, nmo
               wm1(i,j) = wm1(i,j)/dsqrt(wv1(j))
            enddo
         endif
      enddo

!     W2 * W1^t
      call dgemm(
     $   'N', 'T',
     $   nao, nmo, nmo,
     $   1.0d0, wm2, nao, wm1, nmo, 0.0d0, c, nao )

      deallocate(wv1, wm1, wm2)

      if ( ldebug >= 1 ) then
         call ummtw( 'New C', c, nao, nao, nmo, 8, 6, ierr )
      endif

      go to 9999
 9900 write(6,'(''Error : Array not allocated'')')
      ierr = 1
      go to 9999

 9901 write(6,'(''Error : dsyev'')')
      deallocate(wv1, wm1, wm2)
      ierr = 1
      go to 9999

 9999 return
      end
C======================================================================
C
C======================================================================
      subroutine ao2mo(
     O     fmo,
     I     fao, c, nao, nmo,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension fmo(nmo,nmo)
      dimension fao(nao,nao)
      dimension c(nao,nmo)

      real(8),allocatable :: wm(:,:)

      if ( ldebug >= 2 ) then
         call ummtw( 'fao', fao, nao, nao, nao, 8, 6, ierr )
         call ummtw( 'c',   c,   nao, nao, nmo, 8, 6, ierr )
      endif

      allocate(
     $     wm(nao,nao),
     $     stat=ierr)
      if ( ierr.ne.0 ) go to 9900

      ! fmo = c^t  fao  c
      !  w = fao  c
      call dsymm( 'L', 'U',
     $     nao, nmo, 1.0d0, fao, nao, c, nao, 0.0d0, wm, nao )
      !  fmo = c^t  w
      call dgemm( 'T', 'N',
     $     nmo, nmo, nao, 1.0d0, c, nao, wm, nao, 0.0d0, fmo, nmo )

      if ( ldebug >= 2 ) then
         call ummtw( 'fmo', fmo, nmo, nmo, nmo, 8, 6, ierr )
      endif

      deallocate(wm)

      go to 9999
 9900 write(6,'(''Error : Array not allocated'')')
      ierr = 1
      go to 9999

 9999 return
      end
C======================================================================
C
C======================================================================
      subroutine block_eig(
     U     a,
     O     b,
     I     n,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension a(n,n)
      dimension b(n)

      real(8),allocatable :: work(:)

      lwork = -1
      call dsyev( 'V', 'U', n, a, n, b, dummy, lwork, ierr )
      if ( ierr.ne.0 ) go to 9999

      lwork = int(dummy)
      if ( ldebug >= 1 ) then
         write(6,*) 'lwork = ', lwork
      endif
      allocate(
     $     work(lwork),
     $     stat=ierr)
      if ( ierr.ne.0 ) go to 9900

      call dsyev( 'V', 'U', n, a, n, b, work, lwork, info )
      if ( ierr.ne.0 ) go to 9999

      deallocate(work)

      if ( ldebug >= 1 ) then
         call ummtw2( 'W*U=U*E', a, b, n, n, n, 8, 6, ierr )
      endif

      go to 9999
 9900 write(6,'(''Error : Array not allocated'')')
      ierr = 1
      go to 9999

 9999 return
      end
C======================================================================
C Mulliken Population Analysis
C======================================================================
      subroutine population_mulliken(
     O     pop_atom,
     I     s, d, atoms, natom, nao,
     I     lprint, ldebug,
     O     ierr )
      use atoms_class
      implicit real(8) (a-h,o-z)

      type(atoms_type)::atoms

      dimension pop_atom(natom)
      dimension s(nao,nao)
      dimension d(nao,nao)

      real(8),allocatable::pop_ao(:)
      real(8)::ddot

      allocate(pop_ao(nao), stat=ierr)
      if (ierr .ne.0) go to 9999

      do iao = 1, nao
         pop_ao(iao) = ddot(nao, d(1,iao), 1, s(1,iao), 1)
      enddo

      ifrom = 1
      do iatom = 1, natom
         ibasis   = atoms%atom2basis(iatom)
         nao_atom = atoms%basis%ntao(ibasis)
         sum = 0.0d0
         ito = ifrom + nao_atom - 1
         do iao = ifrom, ito
            sum = sum + pop_ao(iao)
         enddo
         ifrom = ito + 1
         pop_atom(iatom) = sum
      enddo
         
      deallocate(pop_ao, stat=ierr)
      if (ierr .ne.0) go to 9999

      if (lprint > 0) then
         write(6,'(/'' Mulliken population'')')
         write(6,'(13x,''Atomic pop.'',10x,''Charges'')')
         sum = 0.0d0
         do iatom = 1, natom
            write(6,'(i5,1x,2f18.10)')
     $           iatom,pop_atom(iatom),atoms%chg(iatom)-pop_atom(iatom)
            sum = sum + pop_atom(iatom)
         end do
         write(6,'('' Total'',f18.10)') sum
      end if

      pop_atom(1:natom) = atoms%chg(1:natom)-pop_atom(1:natom)

 9999 return
      end
C======================================================================
C     
C======================================================================
