! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> nuclear attraction integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_dnai_v2(g_,d,nao,tables,atoms,fmt,myrank,nproc)
  use integral_tables_class
  use atoms_class
  use integral_fmt_class
  use pl_prof_class
  integer,intent(in)::nao
  real(8),intent(in)::d(nao,nao)
  type(tables_type),intent(in)::tables
  type(atoms_type),intent(in)::atoms
  real(8),intent(inout)::g_(3,atoms%natom)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),allocatable::g(:,:)

  allocate(g(3,atoms%natom))
  g(:,:) = 0.0d0

!$omp parallel&
!$omp& default(private)&
!$omp& shared(d,nao,tables,atoms,fmt,myrank,nproc)&
!$omp& reduction(+:g)
  call pl_prof_region_begin('os_dnai_ss_v2')
  call os_dnai_ss_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_dnai_ss_v2')
  if (tables%mang == 1) go to 9

  call pl_prof_region_begin('os_dnai_ps_v2')
  call os_dnai_ps_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_dnai_ps_v2')

  call pl_prof_region_begin('os_dnai_pp_v2')
  call os_dnai_pp_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_dnai_pp_v2')
  if (tables%mang == 2) go to 9

  call pl_prof_region_begin('os_dnai_ds_v2')
  call os_dnai_ds_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_dnai_ds_v2')

  call pl_prof_region_begin('os_dnai_dp_v2')
  call os_dnai_dp_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_dnai_dp_v2')

  call pl_prof_region_begin('os_dnai_dd_v2')
  call os_dnai_dd_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       atoms%xyz,atoms%chg,atoms%natom,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_dnai_dd_v2')
9 continue
!$omp end parallel

  g_(:,:) = g_(:,:) + g(:,:)

end subroutine os_dnai_v2

!------------------------------------------------------------------------------
!> ss nuclear attraction integrals
!
subroutine os_dnai_ss_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(1,3,2)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:1,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = myrank+1, ishp(1), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

a_ss_dAx = 0.0d0
a_ss_dAy = 0.0d0
a_ss_dAz = 0.0d0
a_ss_dBx = 0.0d0
a_ss_dBy = 0.0d0
a_ss_dBz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(2)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(8,it)*dt6 + fmt%fdat(7,it))*dt5&
                  + fmt%fdat(6,it))*dt4 + fmt%fdat(5,it))*dt3&
                  + fmt%fdat(4,it))*dt2 + fmt%fdat(3,it))*dt + fmt%fdat(2,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(2)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss1 = ft * ossx
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
           end if

a_ss0 = ssss0
a_ss1 = ssss1
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
a_pxs0 = PAx*a_ss0 - PCx*a_ss1
a_pys0 = PAy*a_ss0 - PCy*a_ss1
a_pzs0 = PAz*a_ss0 - PCz*a_ss1
a_spx0 = PBx*a_ss0 - PCx*a_ss1
a_spy0 = PBy*a_ss0 - PCy*a_ss1
a_spz0 = PBz*a_ss0 - PCz*a_ss1
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
a_ss_dAx = a_ss_dAx + alpha2*a_pxs0
a_ss_dAy = a_ss_dAy + alpha2*a_pys0
a_ss_dAz = a_ss_dAz + alpha2*a_pzs0
beta2 = 2.0d0*beta
a_ss_dBx = a_ss_dBx + beta2*a_spx0
a_ss_dBy = a_ss_dBy + beta2*a_spy0
a_ss_dBz = a_ss_dBz + beta2*a_spz0
        end do

     end do

v(1,1,1) = a_ss_dAx
v(1,2,1) = a_ss_dAy
v(1,3,1) = a_ss_dAz
v(1,1,2) = a_ss_dBx
v(1,2,2) = a_ss_dBy
v(1,3,2) = a_ss_dBz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     ci = 2.0d0
     if (iao == jao) ci = 1.0d0
     if (iao >= jao) then
        do ixyz = 1, 3
           g(ixyz,jatom) = g(ixyz,jatom) + dm(iao,jao)*v(1,ixyz,2)*ci
        end do
        do ixyz = 1, 3
           g(ixyz,iatom) = g(ixyz,iatom) + dm(iao,jao)*v(1,ixyz,1)*ci
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dnai_ss_v2

!------------------------------------------------------------------------------
!> ps nuclear attraction integrals
!
subroutine os_dnai_ps_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(3,3,2)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:2,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(1)+myrank+1, ishp(2), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

a_pxs_dAx = 0.0d0
a_pys_dAx = 0.0d0
a_pzs_dAx = 0.0d0
a_pxs_dAy = 0.0d0
a_pys_dAy = 0.0d0
a_pzs_dAy = 0.0d0
a_pxs_dAz = 0.0d0
a_pys_dAz = 0.0d0
a_pzs_dAz = 0.0d0
a_pxs_dBx = 0.0d0
a_pys_dBx = 0.0d0
a_pzs_dBx = 0.0d0
a_pxs_dBy = 0.0d0
a_pys_dBy = 0.0d0
a_pzs_dBy = 0.0d0
a_pxs_dBz = 0.0d0
a_pys_dBz = 0.0d0
a_pzs_dBz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(3)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(9,it)*dt6 + fmt%fdat(8,it))*dt5&
                  + fmt%fdat(7,it))*dt4 + fmt%fdat(6,it))*dt3&
                  + fmt%fdat(5,it))*dt2 + fmt%fdat(4,it))*dt + fmt%fdat(3,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(3)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss2 = ft * ossx
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
           end if

a_ss0 = ssss0
a_ss1 = ssss1
a_ss2 = ssss2
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
a_pxs0 = PAx*a_ss0 - PCx*a_ss1
a_pys0 = PAy*a_ss0 - PCy*a_ss1
a_pzs0 = PAz*a_ss0 - PCz*a_ss1
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_a_ss0 = oo2zeta*a_ss0
ai_a_ss1 = oo2zeta*a_ss1
a_dxxs0 = PAx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_dyys0 = PAy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_dzzs0 = PAz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
a_dxys0 = PAy*a_pxs0 - PCy*a_pxs1
a_dxzs0 = PAz*a_pxs0 - PCz*a_pxs1
a_dyzs0 = PAz*a_pys0 - PCz*a_pys1
a_pxpx0 = PBx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_pxpy0 = PBy*a_pxs0 - PCy*a_pxs1
a_pxpz0 = PBz*a_pxs0 - PCz*a_pxs1
a_pypx0 = PBx*a_pys0 - PCx*a_pys1
a_pypy0 = PBy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_pypz0 = PBz*a_pys0 - PCz*a_pys1
a_pzpx0 = PBx*a_pzs0 - PCx*a_pzs1
a_pzpy0 = PBy*a_pzs0 - PCy*a_pzs1
a_pzpz0 = PBz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
a_pxs_dAx = a_pxs_dAx + alpha2*a_dxxs0 - a_ss0
a_pys_dAx = a_pys_dAx + alpha2*a_dxys0
a_pzs_dAx = a_pzs_dAx + alpha2*a_dxzs0
a_pxs_dAy = a_pxs_dAy + alpha2*a_dxys0
a_pys_dAy = a_pys_dAy + alpha2*a_dyys0 - a_ss0
a_pzs_dAy = a_pzs_dAy + alpha2*a_dyzs0
a_pxs_dAz = a_pxs_dAz + alpha2*a_dxzs0
a_pys_dAz = a_pys_dAz + alpha2*a_dyzs0
a_pzs_dAz = a_pzs_dAz + alpha2*a_dzzs0 - a_ss0
beta2 = 2.0d0*beta
a_pxs_dBx = a_pxs_dBx + beta2*a_pxpx0
a_pys_dBx = a_pys_dBx + beta2*a_pypx0
a_pzs_dBx = a_pzs_dBx + beta2*a_pzpx0
a_pxs_dBy = a_pxs_dBy + beta2*a_pxpy0
a_pys_dBy = a_pys_dBy + beta2*a_pypy0
a_pzs_dBy = a_pzs_dBy + beta2*a_pzpy0
a_pxs_dBz = a_pxs_dBz + beta2*a_pxpz0
a_pys_dBz = a_pys_dBz + beta2*a_pypz0
a_pzs_dBz = a_pzs_dBz + beta2*a_pzpz0
        end do

     end do

v(1,1,1) = a_pxs_dAx
v(2,1,1) = a_pys_dAx
v(3,1,1) = a_pzs_dAx
v(1,2,1) = a_pxs_dAy
v(2,2,1) = a_pys_dAy
v(3,2,1) = a_pzs_dAy
v(1,3,1) = a_pxs_dAz
v(2,3,1) = a_pys_dAz
v(3,3,1) = a_pzs_dAz
v(1,1,2) = a_pxs_dBx
v(2,1,2) = a_pys_dBx
v(3,1,2) = a_pzs_dBx
v(1,2,2) = a_pxs_dBy
v(2,2,2) = a_pys_dBy
v(3,2,2) = a_pzs_dBy
v(1,3,2) = a_pxs_dBz
v(2,3,2) = a_pys_dBz
v(3,3,2) = a_pzs_dBz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     ci = 2.0d0
     if (i < j) then
        do ix = iao, iao + 3-1
           key = ix-iao + 1
           do ixyz = 1, 3
              g(ixyz,iatom) = g(ixyz,iatom) + dm(jao,ix)*v(key,ixyz,1)*ci
           end do
        end do
        do ix = iao, iao + 3-1
           key = ix-iao + 1
           do ixyz = 1, 3
              g(ixyz,jatom) = g(ixyz,jatom) + dm(jao,ix)*v(key,ixyz,2)*ci
           end do
        end do
     else
        do ix = iao, iao + 3-1
           key = ix-iao + 1
           do ixyz = 1, 3
              g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jao)*v(key,ixyz,1)*ci
           end do
        end do
        do ix = iao, iao + 3-1
           key = ix-iao + 1
           do ixyz = 1, 3
              g(ixyz,jatom) = g(ixyz,jatom) + dm(ix,jao)*v(key,ixyz,2)*ci
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dnai_ps_v2

!------------------------------------------------------------------------------
!> pp nuclear attraction integrals
!
subroutine os_dnai_pp_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(9,3,2)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:3,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(2)+myrank+1, ishp(3), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

a_pxpx_dAx = 0.0d0
a_pxpy_dAx = 0.0d0
a_pxpz_dAx = 0.0d0
a_pypx_dAx = 0.0d0
a_pypy_dAx = 0.0d0
a_pypz_dAx = 0.0d0
a_pzpx_dAx = 0.0d0
a_pzpy_dAx = 0.0d0
a_pzpz_dAx = 0.0d0
a_pxpx_dAy = 0.0d0
a_pxpy_dAy = 0.0d0
a_pxpz_dAy = 0.0d0
a_pypx_dAy = 0.0d0
a_pypy_dAy = 0.0d0
a_pypz_dAy = 0.0d0
a_pzpx_dAy = 0.0d0
a_pzpy_dAy = 0.0d0
a_pzpz_dAy = 0.0d0
a_pxpx_dAz = 0.0d0
a_pxpy_dAz = 0.0d0
a_pxpz_dAz = 0.0d0
a_pypx_dAz = 0.0d0
a_pypy_dAz = 0.0d0
a_pypz_dAz = 0.0d0
a_pzpx_dAz = 0.0d0
a_pzpy_dAz = 0.0d0
a_pzpz_dAz = 0.0d0
a_pxpx_dBx = 0.0d0
a_pxpy_dBx = 0.0d0
a_pxpz_dBx = 0.0d0
a_pypx_dBx = 0.0d0
a_pypy_dBx = 0.0d0
a_pypz_dBx = 0.0d0
a_pzpx_dBx = 0.0d0
a_pzpy_dBx = 0.0d0
a_pzpz_dBx = 0.0d0
a_pxpx_dBy = 0.0d0
a_pxpy_dBy = 0.0d0
a_pxpz_dBy = 0.0d0
a_pypx_dBy = 0.0d0
a_pypy_dBy = 0.0d0
a_pypz_dBy = 0.0d0
a_pzpx_dBy = 0.0d0
a_pzpy_dBy = 0.0d0
a_pzpz_dBy = 0.0d0
a_pxpx_dBz = 0.0d0
a_pxpy_dBz = 0.0d0
a_pxpz_dBz = 0.0d0
a_pypx_dBz = 0.0d0
a_pypy_dBz = 0.0d0
a_pypz_dBz = 0.0d0
a_pzpx_dBz = 0.0d0
a_pzpy_dBz = 0.0d0
a_pzpz_dBz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(4)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(10,it)*dt6 + fmt%fdat(9,it))*dt5&
                  + fmt%fdat(8,it))*dt4 + fmt%fdat(7,it))*dt3&
                  + fmt%fdat(6,it))*dt2 + fmt%fdat(5,it))*dt + fmt%fdat(4,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(4)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss3 = ft * ossx
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
           end if

a_ss0 = ssss0
a_ss1 = ssss1
a_ss2 = ssss2
a_ss3 = ssss3
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
a_pxs0 = PAx*a_ss0 - PCx*a_ss1
a_pys0 = PAy*a_ss0 - PCy*a_ss1
a_pzs0 = PAz*a_ss0 - PCz*a_ss1
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_a_ss0 = oo2zeta*a_ss0
ai_a_ss1 = oo2zeta*a_ss1
a_dxxs0 = PAx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_dyys0 = PAy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_dzzs0 = PAz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
a_dxys0 = PAy*a_pxs0 - PCy*a_pxs1
a_dxzs0 = PAz*a_pxs0 - PCz*a_pxs1
a_dyzs0 = PAz*a_pys0 - PCz*a_pys1
a_pxs2 = PAx*a_ss2 - PCx*a_ss3
a_pys2 = PAy*a_ss2 - PCy*a_ss3
a_pzs2 = PAz*a_ss2 - PCz*a_ss3
ai_a_ss2 = oo2zeta*a_ss2
a_dxxs1 = PAx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_dyys1 = PAy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_dzzs1 = PAz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
a_dxys1 = PAy*a_pxs1 - PCy*a_pxs2
a_dxzs1 = PAz*a_pxs1 - PCz*a_pxs2
a_dyzs1 = PAz*a_pys1 - PCz*a_pys2
ai_a_pxs0 = oo2zeta*a_pxs0
ai_a_pxs1 = oo2zeta*a_pxs1
a_dxxpx0 = PBx*a_dxxs0 - PCx*a_dxxs1 + 2.0d0*ai_a_pxs0 - 2.0d0*ai_a_pxs1
a_dxxpy0 = PBy*a_dxxs0 - PCy*a_dxxs1
a_dxxpz0 = PBz*a_dxxs0 - PCz*a_dxxs1
a_dyypx0 = PBx*a_dyys0 - PCx*a_dyys1
ai_a_pys0 = oo2zeta*a_pys0
ai_a_pys1 = oo2zeta*a_pys1
a_dyypy0 = PBy*a_dyys0 - PCy*a_dyys1 + 2.0d0*ai_a_pys0 - 2.0d0*ai_a_pys1
a_dyypz0 = PBz*a_dyys0 - PCz*a_dyys1
a_dzzpx0 = PBx*a_dzzs0 - PCx*a_dzzs1
a_dzzpy0 = PBy*a_dzzs0 - PCy*a_dzzs1
ai_a_pzs0 = oo2zeta*a_pzs0
ai_a_pzs1 = oo2zeta*a_pzs1
a_dzzpz0 = PBz*a_dzzs0 - PCz*a_dzzs1 + 2.0d0*ai_a_pzs0 - 2.0d0*ai_a_pzs1
a_dxypx0 = PBx*a_dxys0 - PCx*a_dxys1 + ai_a_pys0 - ai_a_pys1
a_dxypy0 = PBy*a_dxys0 - PCy*a_dxys1 + ai_a_pxs0 - ai_a_pxs1
a_dxypz0 = PBz*a_dxys0 - PCz*a_dxys1
a_dxzpx0 = PBx*a_dxzs0 - PCx*a_dxzs1 + ai_a_pzs0 - ai_a_pzs1
a_dxzpy0 = PBy*a_dxzs0 - PCy*a_dxzs1
a_dxzpz0 = PBz*a_dxzs0 - PCz*a_dxzs1 + ai_a_pxs0 - ai_a_pxs1
a_dyzpx0 = PBx*a_dyzs0 - PCx*a_dyzs1
a_dyzpy0 = PBy*a_dyzs0 - PCy*a_dyzs1 + ai_a_pzs0 - ai_a_pzs1
a_dyzpz0 = PBz*a_dyzs0 - PCz*a_dyzs1 + ai_a_pys0 - ai_a_pys1
a_spx0 = PBx*a_ss0 - PCx*a_ss1
a_spy0 = PBy*a_ss0 - PCy*a_ss1
a_spz0 = PBz*a_ss0 - PCz*a_ss1
a_pxpx0 = PBx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_pxpy0 = PBy*a_pxs0 - PCy*a_pxs1
a_pxpz0 = PBz*a_pxs0 - PCz*a_pxs1
a_pypx0 = PBx*a_pys0 - PCx*a_pys1
a_pypy0 = PBy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_pypz0 = PBz*a_pys0 - PCz*a_pys1
a_pzpx0 = PBx*a_pzs0 - PCx*a_pzs1
a_pzpy0 = PBy*a_pzs0 - PCy*a_pzs1
a_pzpz0 = PBz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
a_pxpx1 = PBx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_pxpy1 = PBy*a_pxs1 - PCy*a_pxs2
a_pxpz1 = PBz*a_pxs1 - PCz*a_pxs2
a_pypx1 = PBx*a_pys1 - PCx*a_pys2
a_pypy1 = PBy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_pypz1 = PBz*a_pys1 - PCz*a_pys2
a_pzpx1 = PBx*a_pzs1 - PCx*a_pzs2
a_pzpy1 = PBy*a_pzs1 - PCy*a_pzs2
a_pzpz1 = PBz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
a_spx1 = PBx*a_ss1 - PCx*a_ss2
a_spy1 = PBy*a_ss1 - PCy*a_ss2
a_spz1 = PBz*a_ss1 - PCz*a_ss2
ai_a_spx0 = oo2zeta*a_spx0
ai_a_spx1 = oo2zeta*a_spx1
bi_a_pxs0 = oo2zeta*a_pxs0
bi_a_pxs1 = oo2zeta*a_pxs1
a_pxdxx0 = PBx*a_pxpx0 - PCx*a_pxpx1 + ai_a_spx0 - ai_a_spx1 + bi_a_pxs0 - bi_a_pxs1
a_pxdyy0 = PBy*a_pxpy0 - PCy*a_pxpy1 + bi_a_pxs0 - bi_a_pxs1
a_pxdzz0 = PBz*a_pxpz0 - PCz*a_pxpz1 + bi_a_pxs0 - bi_a_pxs1
a_pxdxy0 = PBy*a_pxpx0 - PCy*a_pxpx1
a_pxdxz0 = PBz*a_pxpx0 - PCz*a_pxpx1
a_pxdyz0 = PBz*a_pxpy0 - PCz*a_pxpy1
bi_a_pys0 = oo2zeta*a_pys0
bi_a_pys1 = oo2zeta*a_pys1
a_pydxx0 = PBx*a_pypx0 - PCx*a_pypx1 + bi_a_pys0 - bi_a_pys1
ai_a_spy0 = oo2zeta*a_spy0
ai_a_spy1 = oo2zeta*a_spy1
a_pydyy0 = PBy*a_pypy0 - PCy*a_pypy1 + ai_a_spy0 - ai_a_spy1 + bi_a_pys0 - bi_a_pys1
a_pydzz0 = PBz*a_pypz0 - PCz*a_pypz1 + bi_a_pys0 - bi_a_pys1
a_pydxy0 = PBy*a_pypx0 - PCy*a_pypx1 + ai_a_spx0 - ai_a_spx1
a_pydxz0 = PBz*a_pypx0 - PCz*a_pypx1
a_pydyz0 = PBz*a_pypy0 - PCz*a_pypy1
bi_a_pzs0 = oo2zeta*a_pzs0
bi_a_pzs1 = oo2zeta*a_pzs1
a_pzdxx0 = PBx*a_pzpx0 - PCx*a_pzpx1 + bi_a_pzs0 - bi_a_pzs1
a_pzdyy0 = PBy*a_pzpy0 - PCy*a_pzpy1 + bi_a_pzs0 - bi_a_pzs1
ai_a_spz0 = oo2zeta*a_spz0
ai_a_spz1 = oo2zeta*a_spz1
a_pzdzz0 = PBz*a_pzpz0 - PCz*a_pzpz1 + ai_a_spz0 - ai_a_spz1 + bi_a_pzs0 - bi_a_pzs1
a_pzdxy0 = PBy*a_pzpx0 - PCy*a_pzpx1
a_pzdxz0 = PBz*a_pzpx0 - PCz*a_pzpx1 + ai_a_spx0 - ai_a_spx1
a_pzdyz0 = PBz*a_pzpy0 - PCz*a_pzpy1 + ai_a_spy0 - ai_a_spy1
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
a_pxpx_dAx = a_pxpx_dAx + alpha2*a_dxxpx0 - a_spx0
a_pxpy_dAx = a_pxpy_dAx + alpha2*a_dxxpy0 - a_spy0
a_pxpz_dAx = a_pxpz_dAx + alpha2*a_dxxpz0 - a_spz0
a_pypx_dAx = a_pypx_dAx + alpha2*a_dxypx0
a_pypy_dAx = a_pypy_dAx + alpha2*a_dxypy0
a_pypz_dAx = a_pypz_dAx + alpha2*a_dxypz0
a_pzpx_dAx = a_pzpx_dAx + alpha2*a_dxzpx0
a_pzpy_dAx = a_pzpy_dAx + alpha2*a_dxzpy0
a_pzpz_dAx = a_pzpz_dAx + alpha2*a_dxzpz0
a_pxpx_dAy = a_pxpx_dAy + alpha2*a_dxypx0
a_pxpy_dAy = a_pxpy_dAy + alpha2*a_dxypy0
a_pxpz_dAy = a_pxpz_dAy + alpha2*a_dxypz0
a_pypx_dAy = a_pypx_dAy + alpha2*a_dyypx0 - a_spx0
a_pypy_dAy = a_pypy_dAy + alpha2*a_dyypy0 - a_spy0
a_pypz_dAy = a_pypz_dAy + alpha2*a_dyypz0 - a_spz0
a_pzpx_dAy = a_pzpx_dAy + alpha2*a_dyzpx0
a_pzpy_dAy = a_pzpy_dAy + alpha2*a_dyzpy0
a_pzpz_dAy = a_pzpz_dAy + alpha2*a_dyzpz0
a_pxpx_dAz = a_pxpx_dAz + alpha2*a_dxzpx0
a_pxpy_dAz = a_pxpy_dAz + alpha2*a_dxzpy0
a_pxpz_dAz = a_pxpz_dAz + alpha2*a_dxzpz0
a_pypx_dAz = a_pypx_dAz + alpha2*a_dyzpx0
a_pypy_dAz = a_pypy_dAz + alpha2*a_dyzpy0
a_pypz_dAz = a_pypz_dAz + alpha2*a_dyzpz0
a_pzpx_dAz = a_pzpx_dAz + alpha2*a_dzzpx0 - a_spx0
a_pzpy_dAz = a_pzpy_dAz + alpha2*a_dzzpy0 - a_spy0
a_pzpz_dAz = a_pzpz_dAz + alpha2*a_dzzpz0 - a_spz0
beta2 = 2.0d0*beta
a_pxpx_dBx = a_pxpx_dBx + beta2*a_pxdxx0 - a_pxs0
a_pxpy_dBx = a_pxpy_dBx + beta2*a_pxdxy0
a_pxpz_dBx = a_pxpz_dBx + beta2*a_pxdxz0
a_pypx_dBx = a_pypx_dBx + beta2*a_pydxx0 - a_pys0
a_pypy_dBx = a_pypy_dBx + beta2*a_pydxy0
a_pypz_dBx = a_pypz_dBx + beta2*a_pydxz0
a_pzpx_dBx = a_pzpx_dBx + beta2*a_pzdxx0 - a_pzs0
a_pzpy_dBx = a_pzpy_dBx + beta2*a_pzdxy0
a_pzpz_dBx = a_pzpz_dBx + beta2*a_pzdxz0
a_pxpx_dBy = a_pxpx_dBy + beta2*a_pxdxy0
a_pxpy_dBy = a_pxpy_dBy + beta2*a_pxdyy0 - a_pxs0
a_pxpz_dBy = a_pxpz_dBy + beta2*a_pxdyz0
a_pypx_dBy = a_pypx_dBy + beta2*a_pydxy0
a_pypy_dBy = a_pypy_dBy + beta2*a_pydyy0 - a_pys0
a_pypz_dBy = a_pypz_dBy + beta2*a_pydyz0
a_pzpx_dBy = a_pzpx_dBy + beta2*a_pzdxy0
a_pzpy_dBy = a_pzpy_dBy + beta2*a_pzdyy0 - a_pzs0
a_pzpz_dBy = a_pzpz_dBy + beta2*a_pzdyz0
a_pxpx_dBz = a_pxpx_dBz + beta2*a_pxdxz0
a_pxpy_dBz = a_pxpy_dBz + beta2*a_pxdyz0
a_pxpz_dBz = a_pxpz_dBz + beta2*a_pxdzz0 - a_pxs0
a_pypx_dBz = a_pypx_dBz + beta2*a_pydxz0
a_pypy_dBz = a_pypy_dBz + beta2*a_pydyz0
a_pypz_dBz = a_pypz_dBz + beta2*a_pydzz0 - a_pys0
a_pzpx_dBz = a_pzpx_dBz + beta2*a_pzdxz0
a_pzpy_dBz = a_pzpy_dBz + beta2*a_pzdyz0
a_pzpz_dBz = a_pzpz_dBz + beta2*a_pzdzz0 - a_pzs0
        end do

     end do

v(1,1,1) = a_pxpx_dAx
v(2,1,1) = a_pxpy_dAx
v(3,1,1) = a_pxpz_dAx
v(4,1,1) = a_pypx_dAx
v(5,1,1) = a_pypy_dAx
v(6,1,1) = a_pypz_dAx
v(7,1,1) = a_pzpx_dAx
v(8,1,1) = a_pzpy_dAx
v(9,1,1) = a_pzpz_dAx
v(1,2,1) = a_pxpx_dAy
v(2,2,1) = a_pxpy_dAy
v(3,2,1) = a_pxpz_dAy
v(4,2,1) = a_pypx_dAy
v(5,2,1) = a_pypy_dAy
v(6,2,1) = a_pypz_dAy
v(7,2,1) = a_pzpx_dAy
v(8,2,1) = a_pzpy_dAy
v(9,2,1) = a_pzpz_dAy
v(1,3,1) = a_pxpx_dAz
v(2,3,1) = a_pxpy_dAz
v(3,3,1) = a_pxpz_dAz
v(4,3,1) = a_pypx_dAz
v(5,3,1) = a_pypy_dAz
v(6,3,1) = a_pypz_dAz
v(7,3,1) = a_pzpx_dAz
v(8,3,1) = a_pzpy_dAz
v(9,3,1) = a_pzpz_dAz
v(1,1,2) = a_pxpx_dBx
v(2,1,2) = a_pxpy_dBx
v(3,1,2) = a_pxpz_dBx
v(4,1,2) = a_pypx_dBx
v(5,1,2) = a_pypy_dBx
v(6,1,2) = a_pypz_dBx
v(7,1,2) = a_pzpx_dBx
v(8,1,2) = a_pzpy_dBx
v(9,1,2) = a_pzpz_dBx
v(1,2,2) = a_pxpx_dBy
v(2,2,2) = a_pxpy_dBy
v(3,2,2) = a_pxpz_dBy
v(4,2,2) = a_pypx_dBy
v(5,2,2) = a_pypy_dBy
v(6,2,2) = a_pypz_dBy
v(7,2,2) = a_pzpx_dBy
v(8,2,2) = a_pzpy_dBy
v(9,2,2) = a_pzpz_dBy
v(1,3,2) = a_pxpx_dBz
v(2,3,2) = a_pxpy_dBz
v(3,3,2) = a_pxpz_dBz
v(4,3,2) = a_pypx_dBz
v(5,3,2) = a_pypy_dBz
v(6,3,2) = a_pypz_dBz
v(7,3,2) = a_pzpx_dBz
v(8,3,2) = a_pzpy_dBz
v(9,3,2) = a_pzpz_dBz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do jx = jao, jao + 3-1
           do ix = jx, iao + 3-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = (ix-iao)*3 + jx-jao + 1
              do ixyz = 1, 3
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz,1)*ci
              end do
           end do
        end do
        do jx = jao, jao + 3-1
           do ix = jx, iao + 3-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = (ix-iao)*3 + jx-jao + 1
              do ixyz = 1, 3
                 g(ixyz,jatom) = g(ixyz,jatom) + dm(ix,jx)*v(key,ixyz,2)*ci
              end do
           end do
        end do
     else
        do jx = jao, jao + 3-1
           do ix = iao, iao + 3-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = (ix-iao)*3 + jx-jao + 1
              do ixyz = 1, 3
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz,1)*ci
              end do
           end do
        end do
        do jx = jao, jao + 3-1
           do ix = iao, iao + 3-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = (ix-iao)*3 + jx-jao + 1
              do ixyz = 1, 3
                 g(ixyz,jatom) = g(ixyz,jatom) + dm(ix,jx)*v(key,ixyz,2)*ci
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dnai_pp_v2

!------------------------------------------------------------------------------
!> ds nuclear attraction integrals
!
subroutine os_dnai_ds_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(6,3,2)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:3,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(3)+myrank+1, ishp(4), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

a_dxxs_dAx = 0.0d0
a_dyys_dAx = 0.0d0
a_dzzs_dAx = 0.0d0
a_dxys_dAx = 0.0d0
a_dxzs_dAx = 0.0d0
a_dyzs_dAx = 0.0d0
a_dxxs_dAy = 0.0d0
a_dyys_dAy = 0.0d0
a_dzzs_dAy = 0.0d0
a_dxys_dAy = 0.0d0
a_dxzs_dAy = 0.0d0
a_dyzs_dAy = 0.0d0
a_dxxs_dAz = 0.0d0
a_dyys_dAz = 0.0d0
a_dzzs_dAz = 0.0d0
a_dxys_dAz = 0.0d0
a_dxzs_dAz = 0.0d0
a_dyzs_dAz = 0.0d0
a_dxxs_dBx = 0.0d0
a_dyys_dBx = 0.0d0
a_dzzs_dBx = 0.0d0
a_dxys_dBx = 0.0d0
a_dxzs_dBx = 0.0d0
a_dyzs_dBx = 0.0d0
a_dxxs_dBy = 0.0d0
a_dyys_dBy = 0.0d0
a_dzzs_dBy = 0.0d0
a_dxys_dBy = 0.0d0
a_dxzs_dBy = 0.0d0
a_dyzs_dBy = 0.0d0
a_dxxs_dBz = 0.0d0
a_dyys_dBz = 0.0d0
a_dzzs_dBz = 0.0d0
a_dxys_dBz = 0.0d0
a_dxzs_dBz = 0.0d0
a_dyzs_dBz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(4)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(10,it)*dt6 + fmt%fdat(9,it))*dt5&
                  + fmt%fdat(8,it))*dt4 + fmt%fdat(7,it))*dt3&
                  + fmt%fdat(6,it))*dt2 + fmt%fdat(5,it))*dt + fmt%fdat(4,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(4)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss3 = ft * ossx
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
           end if

a_ss0 = ssss0
a_ss1 = ssss1
a_ss2 = ssss2
a_ss3 = ssss3
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
a_pxs0 = PAx*a_ss0 - PCx*a_ss1
a_pys0 = PAy*a_ss0 - PCy*a_ss1
a_pzs0 = PAz*a_ss0 - PCz*a_ss1
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_a_ss0 = oo2zeta*a_ss0
ai_a_ss1 = oo2zeta*a_ss1
a_dxxs0 = PAx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_dyys0 = PAy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_dzzs0 = PAz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
a_dxys0 = PAy*a_pxs0 - PCy*a_pxs1
a_dxzs0 = PAz*a_pxs0 - PCz*a_pxs1
a_dyzs0 = PAz*a_pys0 - PCz*a_pys1
a_pxs2 = PAx*a_ss2 - PCx*a_ss3
a_pys2 = PAy*a_ss2 - PCy*a_ss3
a_pzs2 = PAz*a_ss2 - PCz*a_ss3
ai_a_ss2 = oo2zeta*a_ss2
a_dxxs1 = PAx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_dyys1 = PAy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_dzzs1 = PAz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
a_dxys1 = PAy*a_pxs1 - PCy*a_pxs2
a_dxzs1 = PAz*a_pxs1 - PCz*a_pxs2
a_dyzs1 = PAz*a_pys1 - PCz*a_pys2
ai_a_pxs0 = oo2zeta*a_pxs0
ai_a_pxs1 = oo2zeta*a_pxs1
a_fxxxs0 = PAx*a_dxxs0 - PCx*a_dxxs1 + 2.0d0*ai_a_pxs0 - 2.0d0*ai_a_pxs1
ai_a_pys0 = oo2zeta*a_pys0
ai_a_pys1 = oo2zeta*a_pys1
a_fyyys0 = PAy*a_dyys0 - PCy*a_dyys1 + 2.0d0*ai_a_pys0 - 2.0d0*ai_a_pys1
ai_a_pzs0 = oo2zeta*a_pzs0
ai_a_pzs1 = oo2zeta*a_pzs1
a_fzzzs0 = PAz*a_dzzs0 - PCz*a_dzzs1 + 2.0d0*ai_a_pzs0 - 2.0d0*ai_a_pzs1
a_fxxys0 = PAy*a_dxxs0 - PCy*a_dxxs1
a_fxxzs0 = PAz*a_dxxs0 - PCz*a_dxxs1
a_fyyxs0 = PAx*a_dyys0 - PCx*a_dyys1
a_fyyzs0 = PAz*a_dyys0 - PCz*a_dyys1
a_fzzxs0 = PAx*a_dzzs0 - PCx*a_dzzs1
a_fzzys0 = PAy*a_dzzs0 - PCy*a_dzzs1
a_fxyzs0 = PAz*a_dxys0 - PCz*a_dxys1
a_dxxpx0 = PBx*a_dxxs0 - PCx*a_dxxs1 + 2.0d0*ai_a_pxs0 - 2.0d0*ai_a_pxs1
a_dxxpy0 = PBy*a_dxxs0 - PCy*a_dxxs1
a_dxxpz0 = PBz*a_dxxs0 - PCz*a_dxxs1
a_dyypx0 = PBx*a_dyys0 - PCx*a_dyys1
a_dyypy0 = PBy*a_dyys0 - PCy*a_dyys1 + 2.0d0*ai_a_pys0 - 2.0d0*ai_a_pys1
a_dyypz0 = PBz*a_dyys0 - PCz*a_dyys1
a_dzzpx0 = PBx*a_dzzs0 - PCx*a_dzzs1
a_dzzpy0 = PBy*a_dzzs0 - PCy*a_dzzs1
a_dzzpz0 = PBz*a_dzzs0 - PCz*a_dzzs1 + 2.0d0*ai_a_pzs0 - 2.0d0*ai_a_pzs1
a_dxypx0 = PBx*a_dxys0 - PCx*a_dxys1 + ai_a_pys0 - ai_a_pys1
a_dxypy0 = PBy*a_dxys0 - PCy*a_dxys1 + ai_a_pxs0 - ai_a_pxs1
a_dxypz0 = PBz*a_dxys0 - PCz*a_dxys1
a_dxzpx0 = PBx*a_dxzs0 - PCx*a_dxzs1 + ai_a_pzs0 - ai_a_pzs1
a_dxzpy0 = PBy*a_dxzs0 - PCy*a_dxzs1
a_dxzpz0 = PBz*a_dxzs0 - PCz*a_dxzs1 + ai_a_pxs0 - ai_a_pxs1
a_dyzpx0 = PBx*a_dyzs0 - PCx*a_dyzs1
a_dyzpy0 = PBy*a_dyzs0 - PCy*a_dyzs1 + ai_a_pzs0 - ai_a_pzs1
a_dyzpz0 = PBz*a_dyzs0 - PCz*a_dyzs1 + ai_a_pys0 - ai_a_pys1
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
a_dxxs_dAx = a_dxxs_dAx + alpha2*a_fxxxs0 - 2.0d0*a_pxs0
a_dyys_dAx = a_dyys_dAx + alpha2*a_fyyxs0
a_dzzs_dAx = a_dzzs_dAx + alpha2*a_fzzxs0
a_dxys_dAx = a_dxys_dAx + alpha2*a_fxxys0 - a_pys0
a_dxzs_dAx = a_dxzs_dAx + alpha2*a_fxxzs0 - a_pzs0
a_dyzs_dAx = a_dyzs_dAx + alpha2*a_fxyzs0
a_dxxs_dAy = a_dxxs_dAy + alpha2*a_fxxys0
a_dyys_dAy = a_dyys_dAy + alpha2*a_fyyys0 - 2.0d0*a_pys0
a_dzzs_dAy = a_dzzs_dAy + alpha2*a_fzzys0
a_dxys_dAy = a_dxys_dAy + alpha2*a_fyyxs0 - a_pxs0
a_dxzs_dAy = a_dxzs_dAy + alpha2*a_fxyzs0
a_dyzs_dAy = a_dyzs_dAy + alpha2*a_fyyzs0 - a_pzs0
a_dxxs_dAz = a_dxxs_dAz + alpha2*a_fxxzs0
a_dyys_dAz = a_dyys_dAz + alpha2*a_fyyzs0
a_dzzs_dAz = a_dzzs_dAz + alpha2*a_fzzzs0 - 2.0d0*a_pzs0
a_dxys_dAz = a_dxys_dAz + alpha2*a_fxyzs0
a_dxzs_dAz = a_dxzs_dAz + alpha2*a_fzzxs0 - a_pxs0
a_dyzs_dAz = a_dyzs_dAz + alpha2*a_fzzys0 - a_pys0
beta2 = 2.0d0*beta
a_dxxs_dBx = a_dxxs_dBx + beta2*a_dxxpx0
a_dyys_dBx = a_dyys_dBx + beta2*a_dyypx0
a_dzzs_dBx = a_dzzs_dBx + beta2*a_dzzpx0
a_dxys_dBx = a_dxys_dBx + beta2*a_dxypx0
a_dxzs_dBx = a_dxzs_dBx + beta2*a_dxzpx0
a_dyzs_dBx = a_dyzs_dBx + beta2*a_dyzpx0
a_dxxs_dBy = a_dxxs_dBy + beta2*a_dxxpy0
a_dyys_dBy = a_dyys_dBy + beta2*a_dyypy0
a_dzzs_dBy = a_dzzs_dBy + beta2*a_dzzpy0
a_dxys_dBy = a_dxys_dBy + beta2*a_dxypy0
a_dxzs_dBy = a_dxzs_dBy + beta2*a_dxzpy0
a_dyzs_dBy = a_dyzs_dBy + beta2*a_dyzpy0
a_dxxs_dBz = a_dxxs_dBz + beta2*a_dxxpz0
a_dyys_dBz = a_dyys_dBz + beta2*a_dyypz0
a_dzzs_dBz = a_dzzs_dBz + beta2*a_dzzpz0
a_dxys_dBz = a_dxys_dBz + beta2*a_dxypz0
a_dxzs_dBz = a_dxzs_dBz + beta2*a_dxzpz0
a_dyzs_dBz = a_dyzs_dBz + beta2*a_dyzpz0
        end do

     end do
v(1,1,1) = a_dxxs_dAx
v(2,1,1) = a_dyys_dAx
v(3,1,1) = a_dzzs_dAx
v(4,1,1) = a_dxys_dAx*root3
v(5,1,1) = a_dxzs_dAx*root3
v(6,1,1) = a_dyzs_dAx*root3
v(1,2,1) = a_dxxs_dAy
v(2,2,1) = a_dyys_dAy
v(3,2,1) = a_dzzs_dAy
v(4,2,1) = a_dxys_dAy*root3
v(5,2,1) = a_dxzs_dAy*root3
v(6,2,1) = a_dyzs_dAy*root3
v(1,3,1) = a_dxxs_dAz
v(2,3,1) = a_dyys_dAz
v(3,3,1) = a_dzzs_dAz
v(4,3,1) = a_dxys_dAz*root3
v(5,3,1) = a_dxzs_dAz*root3
v(6,3,1) = a_dyzs_dAz*root3
v(1,1,2) = a_dxxs_dBx
v(2,1,2) = a_dyys_dBx
v(3,1,2) = a_dzzs_dBx
v(4,1,2) = a_dxys_dBx*root3
v(5,1,2) = a_dxzs_dBx*root3
v(6,1,2) = a_dyzs_dBx*root3
v(1,2,2) = a_dxxs_dBy
v(2,2,2) = a_dyys_dBy
v(3,2,2) = a_dzzs_dBy
v(4,2,2) = a_dxys_dBy*root3
v(5,2,2) = a_dxzs_dBy*root3
v(6,2,2) = a_dyzs_dBy*root3
v(1,3,2) = a_dxxs_dBz
v(2,3,2) = a_dyys_dBz
v(3,3,2) = a_dzzs_dBz
v(4,3,2) = a_dxys_dBz*root3
v(5,3,2) = a_dxzs_dBz*root3
v(6,3,2) = a_dyzs_dBz*root3

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     ci = 2.0d0
     if (i < j) then
        do ix = iao, iao + 6-1
           key = ix-iao + 1
           do ixyz = 1, 3
              g(ixyz,iatom) = g(ixyz,iatom) + dm(jao,ix)*v(key,ixyz,1)*ci
           end do
        end do
        do ix = iao, iao + 6-1
           key = ix-iao + 1
           do ixyz = 1, 3
              g(ixyz,jatom) = g(ixyz,jatom) + dm(jao,ix)*v(key,ixyz,2)*ci
           end do
        end do
     else
        do ix = iao, iao + 6-1
           key = ix-iao + 1
           do ixyz = 1, 3
              g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jao)*v(key,ixyz,1)*ci
           end do
        end do
        do ix = iao, iao + 6-1
           key = ix-iao + 1
           do ixyz = 1, 3
              g(ixyz,jatom) = g(ixyz,jatom) + dm(ix,jao)*v(key,ixyz,2)*ci
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dnai_ds_v2

!------------------------------------------------------------------------------
!> dp nuclear attraction integrals
!
subroutine os_dnai_dp_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(18,3,2)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:4,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(4)+myrank+1, ishp(5), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

a_dxxpx_dAx = 0.0d0
a_dxxpy_dAx = 0.0d0
a_dxxpz_dAx = 0.0d0
a_dyypx_dAx = 0.0d0
a_dyypy_dAx = 0.0d0
a_dyypz_dAx = 0.0d0
a_dzzpx_dAx = 0.0d0
a_dzzpy_dAx = 0.0d0
a_dzzpz_dAx = 0.0d0
a_dxypx_dAx = 0.0d0
a_dxypy_dAx = 0.0d0
a_dxypz_dAx = 0.0d0
a_dxzpx_dAx = 0.0d0
a_dxzpy_dAx = 0.0d0
a_dxzpz_dAx = 0.0d0
a_dyzpx_dAx = 0.0d0
a_dyzpy_dAx = 0.0d0
a_dyzpz_dAx = 0.0d0
a_dxxpx_dAy = 0.0d0
a_dxxpy_dAy = 0.0d0
a_dxxpz_dAy = 0.0d0
a_dyypx_dAy = 0.0d0
a_dyypy_dAy = 0.0d0
a_dyypz_dAy = 0.0d0
a_dzzpx_dAy = 0.0d0
a_dzzpy_dAy = 0.0d0
a_dzzpz_dAy = 0.0d0
a_dxypx_dAy = 0.0d0
a_dxypy_dAy = 0.0d0
a_dxypz_dAy = 0.0d0
a_dxzpx_dAy = 0.0d0
a_dxzpy_dAy = 0.0d0
a_dxzpz_dAy = 0.0d0
a_dyzpx_dAy = 0.0d0
a_dyzpy_dAy = 0.0d0
a_dyzpz_dAy = 0.0d0
a_dxxpx_dAz = 0.0d0
a_dxxpy_dAz = 0.0d0
a_dxxpz_dAz = 0.0d0
a_dyypx_dAz = 0.0d0
a_dyypy_dAz = 0.0d0
a_dyypz_dAz = 0.0d0
a_dzzpx_dAz = 0.0d0
a_dzzpy_dAz = 0.0d0
a_dzzpz_dAz = 0.0d0
a_dxypx_dAz = 0.0d0
a_dxypy_dAz = 0.0d0
a_dxypz_dAz = 0.0d0
a_dxzpx_dAz = 0.0d0
a_dxzpy_dAz = 0.0d0
a_dxzpz_dAz = 0.0d0
a_dyzpx_dAz = 0.0d0
a_dyzpy_dAz = 0.0d0
a_dyzpz_dAz = 0.0d0
a_dxxpx_dBx = 0.0d0
a_dxxpy_dBx = 0.0d0
a_dxxpz_dBx = 0.0d0
a_dyypx_dBx = 0.0d0
a_dyypy_dBx = 0.0d0
a_dyypz_dBx = 0.0d0
a_dzzpx_dBx = 0.0d0
a_dzzpy_dBx = 0.0d0
a_dzzpz_dBx = 0.0d0
a_dxypx_dBx = 0.0d0
a_dxypy_dBx = 0.0d0
a_dxypz_dBx = 0.0d0
a_dxzpx_dBx = 0.0d0
a_dxzpy_dBx = 0.0d0
a_dxzpz_dBx = 0.0d0
a_dyzpx_dBx = 0.0d0
a_dyzpy_dBx = 0.0d0
a_dyzpz_dBx = 0.0d0
a_dxxpx_dBy = 0.0d0
a_dxxpy_dBy = 0.0d0
a_dxxpz_dBy = 0.0d0
a_dyypx_dBy = 0.0d0
a_dyypy_dBy = 0.0d0
a_dyypz_dBy = 0.0d0
a_dzzpx_dBy = 0.0d0
a_dzzpy_dBy = 0.0d0
a_dzzpz_dBy = 0.0d0
a_dxypx_dBy = 0.0d0
a_dxypy_dBy = 0.0d0
a_dxypz_dBy = 0.0d0
a_dxzpx_dBy = 0.0d0
a_dxzpy_dBy = 0.0d0
a_dxzpz_dBy = 0.0d0
a_dyzpx_dBy = 0.0d0
a_dyzpy_dBy = 0.0d0
a_dyzpz_dBy = 0.0d0
a_dxxpx_dBz = 0.0d0
a_dxxpy_dBz = 0.0d0
a_dxxpz_dBz = 0.0d0
a_dyypx_dBz = 0.0d0
a_dyypy_dBz = 0.0d0
a_dyypz_dBz = 0.0d0
a_dzzpx_dBz = 0.0d0
a_dzzpy_dBz = 0.0d0
a_dzzpz_dBz = 0.0d0
a_dxypx_dBz = 0.0d0
a_dxypy_dBz = 0.0d0
a_dxypz_dBz = 0.0d0
a_dxzpx_dBz = 0.0d0
a_dxzpy_dBz = 0.0d0
a_dxzpz_dBz = 0.0d0
a_dyzpx_dBz = 0.0d0
a_dyzpy_dBz = 0.0d0
a_dyzpz_dBz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(5)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(11,it)*dt6 + fmt%fdat(10,it))*dt5&
                  + fmt%fdat(9,it))*dt4 + fmt%fdat(8,it))*dt3&
                  + fmt%fdat(7,it))*dt2 + fmt%fdat(6,it))*dt + fmt%fdat(5,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(5)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss4 = ft * ossx
              ssss3 = (ssss4 * t2 + etcn) * rminv(4)
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
              ssss4 = ssss3 * tinv * 3.5d0
           end if

a_ss0 = ssss0
a_ss1 = ssss1
a_ss2 = ssss2
a_ss3 = ssss3
a_ss4 = ssss4
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
a_pxs0 = PAx*a_ss0 - PCx*a_ss1
a_pys0 = PAy*a_ss0 - PCy*a_ss1
a_pzs0 = PAz*a_ss0 - PCz*a_ss1
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_a_ss0 = oo2zeta*a_ss0
ai_a_ss1 = oo2zeta*a_ss1
a_dxxs0 = PAx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_dyys0 = PAy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_dzzs0 = PAz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
a_dxys0 = PAy*a_pxs0 - PCy*a_pxs1
a_dxzs0 = PAz*a_pxs0 - PCz*a_pxs1
a_dyzs0 = PAz*a_pys0 - PCz*a_pys1
a_pxs2 = PAx*a_ss2 - PCx*a_ss3
a_pys2 = PAy*a_ss2 - PCy*a_ss3
a_pzs2 = PAz*a_ss2 - PCz*a_ss3
ai_a_ss2 = oo2zeta*a_ss2
a_dxxs1 = PAx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_dyys1 = PAy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_dzzs1 = PAz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
a_dxys1 = PAy*a_pxs1 - PCy*a_pxs2
a_dxzs1 = PAz*a_pxs1 - PCz*a_pxs2
a_dyzs1 = PAz*a_pys1 - PCz*a_pys2
ai_a_pxs0 = oo2zeta*a_pxs0
ai_a_pxs1 = oo2zeta*a_pxs1
a_fxxxs0 = PAx*a_dxxs0 - PCx*a_dxxs1 + 2.0d0*ai_a_pxs0 - 2.0d0*ai_a_pxs1
ai_a_pys0 = oo2zeta*a_pys0
ai_a_pys1 = oo2zeta*a_pys1
a_fyyys0 = PAy*a_dyys0 - PCy*a_dyys1 + 2.0d0*ai_a_pys0 - 2.0d0*ai_a_pys1
ai_a_pzs0 = oo2zeta*a_pzs0
ai_a_pzs1 = oo2zeta*a_pzs1
a_fzzzs0 = PAz*a_dzzs0 - PCz*a_dzzs1 + 2.0d0*ai_a_pzs0 - 2.0d0*ai_a_pzs1
a_fxxys0 = PAy*a_dxxs0 - PCy*a_dxxs1
a_fxxzs0 = PAz*a_dxxs0 - PCz*a_dxxs1
a_fyyxs0 = PAx*a_dyys0 - PCx*a_dyys1
a_fyyzs0 = PAz*a_dyys0 - PCz*a_dyys1
a_fzzxs0 = PAx*a_dzzs0 - PCx*a_dzzs1
a_fzzys0 = PAy*a_dzzs0 - PCy*a_dzzs1
a_fxyzs0 = PAz*a_dxys0 - PCz*a_dxys1
a_pxs3 = PAx*a_ss3 - PCx*a_ss4
a_pys3 = PAy*a_ss3 - PCy*a_ss4
a_pzs3 = PAz*a_ss3 - PCz*a_ss4
ai_a_ss3 = oo2zeta*a_ss3
a_dxxs2 = PAx*a_pxs2 - PCx*a_pxs3 + ai_a_ss2 - ai_a_ss3
a_dyys2 = PAy*a_pys2 - PCy*a_pys3 + ai_a_ss2 - ai_a_ss3
a_dzzs2 = PAz*a_pzs2 - PCz*a_pzs3 + ai_a_ss2 - ai_a_ss3
a_dxys2 = PAy*a_pxs2 - PCy*a_pxs3
a_dxzs2 = PAz*a_pxs2 - PCz*a_pxs3
a_dyzs2 = PAz*a_pys2 - PCz*a_pys3
ai_a_pxs2 = oo2zeta*a_pxs2
a_fxxxs1 = PAx*a_dxxs1 - PCx*a_dxxs2 + 2.0d0*ai_a_pxs1 - 2.0d0*ai_a_pxs2
ai_a_pys2 = oo2zeta*a_pys2
a_fyyys1 = PAy*a_dyys1 - PCy*a_dyys2 + 2.0d0*ai_a_pys1 - 2.0d0*ai_a_pys2
ai_a_pzs2 = oo2zeta*a_pzs2
a_fzzzs1 = PAz*a_dzzs1 - PCz*a_dzzs2 + 2.0d0*ai_a_pzs1 - 2.0d0*ai_a_pzs2
a_fxxys1 = PAy*a_dxxs1 - PCy*a_dxxs2
a_fxxzs1 = PAz*a_dxxs1 - PCz*a_dxxs2
a_fyyxs1 = PAx*a_dyys1 - PCx*a_dyys2
a_fyyzs1 = PAz*a_dyys1 - PCz*a_dyys2
a_fzzxs1 = PAx*a_dzzs1 - PCx*a_dzzs2
a_fzzys1 = PAy*a_dzzs1 - PCy*a_dzzs2
a_fxyzs1 = PAz*a_dxys1 - PCz*a_dxys2
ai_a_dxxs0 = oo2zeta*a_dxxs0
ai_a_dxxs1 = oo2zeta*a_dxxs1
a_fxxxpx0 = PBx*a_fxxxs0 - PCx*a_fxxxs1 + 3.0d0*ai_a_dxxs0 - 3.0d0*ai_a_dxxs1
a_fxxxpy0 = PBy*a_fxxxs0 - PCy*a_fxxxs1
a_fxxxpz0 = PBz*a_fxxxs0 - PCz*a_fxxxs1
a_fyyypx0 = PBx*a_fyyys0 - PCx*a_fyyys1
ai_a_dyys0 = oo2zeta*a_dyys0
ai_a_dyys1 = oo2zeta*a_dyys1
a_fyyypy0 = PBy*a_fyyys0 - PCy*a_fyyys1 + 3.0d0*ai_a_dyys0 - 3.0d0*ai_a_dyys1
a_fyyypz0 = PBz*a_fyyys0 - PCz*a_fyyys1
a_fzzzpx0 = PBx*a_fzzzs0 - PCx*a_fzzzs1
a_fzzzpy0 = PBy*a_fzzzs0 - PCy*a_fzzzs1
ai_a_dzzs0 = oo2zeta*a_dzzs0
ai_a_dzzs1 = oo2zeta*a_dzzs1
a_fzzzpz0 = PBz*a_fzzzs0 - PCz*a_fzzzs1 + 3.0d0*ai_a_dzzs0 - 3.0d0*ai_a_dzzs1
ai_a_dxys0 = oo2zeta*a_dxys0
ai_a_dxys1 = oo2zeta*a_dxys1
a_fxxypx0 = PBx*a_fxxys0 - PCx*a_fxxys1 + 2.0d0*ai_a_dxys0 - 2.0d0*ai_a_dxys1
a_fxxypy0 = PBy*a_fxxys0 - PCy*a_fxxys1 + ai_a_dxxs0 - ai_a_dxxs1
a_fxxypz0 = PBz*a_fxxys0 - PCz*a_fxxys1
ai_a_dxzs0 = oo2zeta*a_dxzs0
ai_a_dxzs1 = oo2zeta*a_dxzs1
a_fxxzpx0 = PBx*a_fxxzs0 - PCx*a_fxxzs1 + 2.0d0*ai_a_dxzs0 - 2.0d0*ai_a_dxzs1
a_fxxzpy0 = PBy*a_fxxzs0 - PCy*a_fxxzs1
a_fxxzpz0 = PBz*a_fxxzs0 - PCz*a_fxxzs1 + ai_a_dxxs0 - ai_a_dxxs1
a_fyyxpx0 = PBx*a_fyyxs0 - PCx*a_fyyxs1 + ai_a_dyys0 - ai_a_dyys1
a_fyyxpy0 = PBy*a_fyyxs0 - PCy*a_fyyxs1 + 2.0d0*ai_a_dxys0 - 2.0d0*ai_a_dxys1
a_fyyxpz0 = PBz*a_fyyxs0 - PCz*a_fyyxs1
a_fyyzpx0 = PBx*a_fyyzs0 - PCx*a_fyyzs1
ai_a_dyzs0 = oo2zeta*a_dyzs0
ai_a_dyzs1 = oo2zeta*a_dyzs1
a_fyyzpy0 = PBy*a_fyyzs0 - PCy*a_fyyzs1 + 2.0d0*ai_a_dyzs0 - 2.0d0*ai_a_dyzs1
a_fyyzpz0 = PBz*a_fyyzs0 - PCz*a_fyyzs1 + ai_a_dyys0 - ai_a_dyys1
a_fzzxpx0 = PBx*a_fzzxs0 - PCx*a_fzzxs1 + ai_a_dzzs0 - ai_a_dzzs1
a_fzzxpy0 = PBy*a_fzzxs0 - PCy*a_fzzxs1
a_fzzxpz0 = PBz*a_fzzxs0 - PCz*a_fzzxs1 + 2.0d0*ai_a_dxzs0 - 2.0d0*ai_a_dxzs1
a_fzzypx0 = PBx*a_fzzys0 - PCx*a_fzzys1
a_fzzypy0 = PBy*a_fzzys0 - PCy*a_fzzys1 + ai_a_dzzs0 - ai_a_dzzs1
a_fzzypz0 = PBz*a_fzzys0 - PCz*a_fzzys1 + 2.0d0*ai_a_dyzs0 - 2.0d0*ai_a_dyzs1
a_fxyzpx0 = PBx*a_fxyzs0 - PCx*a_fxyzs1 + ai_a_dyzs0 - ai_a_dyzs1
a_fxyzpy0 = PBy*a_fxyzs0 - PCy*a_fxyzs1 + ai_a_dxzs0 - ai_a_dxzs1
a_fxyzpz0 = PBz*a_fxyzs0 - PCz*a_fxyzs1 + ai_a_dxys0 - ai_a_dxys1
a_pxpx0 = PBx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_pxpy0 = PBy*a_pxs0 - PCy*a_pxs1
a_pxpz0 = PBz*a_pxs0 - PCz*a_pxs1
a_pypx0 = PBx*a_pys0 - PCx*a_pys1
a_pypy0 = PBy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_pypz0 = PBz*a_pys0 - PCz*a_pys1
a_pzpx0 = PBx*a_pzs0 - PCx*a_pzs1
a_pzpy0 = PBy*a_pzs0 - PCy*a_pzs1
a_pzpz0 = PBz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
a_dxxpx0 = PBx*a_dxxs0 - PCx*a_dxxs1 + 2.0d0*ai_a_pxs0 - 2.0d0*ai_a_pxs1
a_dxxpy0 = PBy*a_dxxs0 - PCy*a_dxxs1
a_dxxpz0 = PBz*a_dxxs0 - PCz*a_dxxs1
a_dyypx0 = PBx*a_dyys0 - PCx*a_dyys1
a_dyypy0 = PBy*a_dyys0 - PCy*a_dyys1 + 2.0d0*ai_a_pys0 - 2.0d0*ai_a_pys1
a_dyypz0 = PBz*a_dyys0 - PCz*a_dyys1
a_dzzpx0 = PBx*a_dzzs0 - PCx*a_dzzs1
a_dzzpy0 = PBy*a_dzzs0 - PCy*a_dzzs1
a_dzzpz0 = PBz*a_dzzs0 - PCz*a_dzzs1 + 2.0d0*ai_a_pzs0 - 2.0d0*ai_a_pzs1
a_dxypx0 = PBx*a_dxys0 - PCx*a_dxys1 + ai_a_pys0 - ai_a_pys1
a_dxypy0 = PBy*a_dxys0 - PCy*a_dxys1 + ai_a_pxs0 - ai_a_pxs1
a_dxypz0 = PBz*a_dxys0 - PCz*a_dxys1
a_dxzpx0 = PBx*a_dxzs0 - PCx*a_dxzs1 + ai_a_pzs0 - ai_a_pzs1
a_dxzpy0 = PBy*a_dxzs0 - PCy*a_dxzs1
a_dxzpz0 = PBz*a_dxzs0 - PCz*a_dxzs1 + ai_a_pxs0 - ai_a_pxs1
a_dyzpx0 = PBx*a_dyzs0 - PCx*a_dyzs1
a_dyzpy0 = PBy*a_dyzs0 - PCy*a_dyzs1 + ai_a_pzs0 - ai_a_pzs1
a_dyzpz0 = PBz*a_dyzs0 - PCz*a_dyzs1 + ai_a_pys0 - ai_a_pys1
a_dxxpx1 = PBx*a_dxxs1 - PCx*a_dxxs2 + 2.0d0*ai_a_pxs1 - 2.0d0*ai_a_pxs2
a_dxxpy1 = PBy*a_dxxs1 - PCy*a_dxxs2
a_dxxpz1 = PBz*a_dxxs1 - PCz*a_dxxs2
a_dyypx1 = PBx*a_dyys1 - PCx*a_dyys2
a_dyypy1 = PBy*a_dyys1 - PCy*a_dyys2 + 2.0d0*ai_a_pys1 - 2.0d0*ai_a_pys2
a_dyypz1 = PBz*a_dyys1 - PCz*a_dyys2
a_dzzpx1 = PBx*a_dzzs1 - PCx*a_dzzs2
a_dzzpy1 = PBy*a_dzzs1 - PCy*a_dzzs2
a_dzzpz1 = PBz*a_dzzs1 - PCz*a_dzzs2 + 2.0d0*ai_a_pzs1 - 2.0d0*ai_a_pzs2
a_dxypx1 = PBx*a_dxys1 - PCx*a_dxys2 + ai_a_pys1 - ai_a_pys2
a_dxypy1 = PBy*a_dxys1 - PCy*a_dxys2 + ai_a_pxs1 - ai_a_pxs2
a_dxypz1 = PBz*a_dxys1 - PCz*a_dxys2
a_dxzpx1 = PBx*a_dxzs1 - PCx*a_dxzs2 + ai_a_pzs1 - ai_a_pzs2
a_dxzpy1 = PBy*a_dxzs1 - PCy*a_dxzs2
a_dxzpz1 = PBz*a_dxzs1 - PCz*a_dxzs2 + ai_a_pxs1 - ai_a_pxs2
a_dyzpx1 = PBx*a_dyzs1 - PCx*a_dyzs2
a_dyzpy1 = PBy*a_dyzs1 - PCy*a_dyzs2 + ai_a_pzs1 - ai_a_pzs2
a_dyzpz1 = PBz*a_dyzs1 - PCz*a_dyzs2 + ai_a_pys1 - ai_a_pys2
a_pxpx1 = PBx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_pxpy1 = PBy*a_pxs1 - PCy*a_pxs2
a_pxpz1 = PBz*a_pxs1 - PCz*a_pxs2
a_pypx1 = PBx*a_pys1 - PCx*a_pys2
a_pypy1 = PBy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_pypz1 = PBz*a_pys1 - PCz*a_pys2
a_pzpx1 = PBx*a_pzs1 - PCx*a_pzs2
a_pzpy1 = PBy*a_pzs1 - PCy*a_pzs2
a_pzpz1 = PBz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
ai_a_pxpx0 = oo2zeta*a_pxpx0
ai_a_pxpx1 = oo2zeta*a_pxpx1
bi_a_dxxs0 = oo2zeta*a_dxxs0
bi_a_dxxs1 = oo2zeta*a_dxxs1
a_dxxdxx0 = PBx*a_dxxpx0 - PCx*a_dxxpx1 + 2.0d0*ai_a_pxpx0 - 2.0d0*ai_a_pxpx1 + bi_a_dxxs0 - bi_a_dxxs1
a_dxxdyy0 = PBy*a_dxxpy0 - PCy*a_dxxpy1 + bi_a_dxxs0 - bi_a_dxxs1
a_dxxdzz0 = PBz*a_dxxpz0 - PCz*a_dxxpz1 + bi_a_dxxs0 - bi_a_dxxs1
a_dxxdxy0 = PBy*a_dxxpx0 - PCy*a_dxxpx1
a_dxxdxz0 = PBz*a_dxxpx0 - PCz*a_dxxpx1
a_dxxdyz0 = PBz*a_dxxpy0 - PCz*a_dxxpy1
bi_a_dyys0 = oo2zeta*a_dyys0
bi_a_dyys1 = oo2zeta*a_dyys1
a_dyydxx0 = PBx*a_dyypx0 - PCx*a_dyypx1 + bi_a_dyys0 - bi_a_dyys1
ai_a_pypy0 = oo2zeta*a_pypy0
ai_a_pypy1 = oo2zeta*a_pypy1
a_dyydyy0 = PBy*a_dyypy0 - PCy*a_dyypy1 + 2.0d0*ai_a_pypy0 - 2.0d0*ai_a_pypy1 + bi_a_dyys0 - bi_a_dyys1
a_dyydzz0 = PBz*a_dyypz0 - PCz*a_dyypz1 + bi_a_dyys0 - bi_a_dyys1
ai_a_pypx0 = oo2zeta*a_pypx0
ai_a_pypx1 = oo2zeta*a_pypx1
a_dyydxy0 = PBy*a_dyypx0 - PCy*a_dyypx1 + 2.0d0*ai_a_pypx0 - 2.0d0*ai_a_pypx1
a_dyydxz0 = PBz*a_dyypx0 - PCz*a_dyypx1
a_dyydyz0 = PBz*a_dyypy0 - PCz*a_dyypy1
bi_a_dzzs0 = oo2zeta*a_dzzs0
bi_a_dzzs1 = oo2zeta*a_dzzs1
a_dzzdxx0 = PBx*a_dzzpx0 - PCx*a_dzzpx1 + bi_a_dzzs0 - bi_a_dzzs1
a_dzzdyy0 = PBy*a_dzzpy0 - PCy*a_dzzpy1 + bi_a_dzzs0 - bi_a_dzzs1
ai_a_pzpz0 = oo2zeta*a_pzpz0
ai_a_pzpz1 = oo2zeta*a_pzpz1
a_dzzdzz0 = PBz*a_dzzpz0 - PCz*a_dzzpz1 + 2.0d0*ai_a_pzpz0 - 2.0d0*ai_a_pzpz1 + bi_a_dzzs0 - bi_a_dzzs1
a_dzzdxy0 = PBy*a_dzzpx0 - PCy*a_dzzpx1
ai_a_pzpx0 = oo2zeta*a_pzpx0
ai_a_pzpx1 = oo2zeta*a_pzpx1
a_dzzdxz0 = PBz*a_dzzpx0 - PCz*a_dzzpx1 + 2.0d0*ai_a_pzpx0 - 2.0d0*ai_a_pzpx1
ai_a_pzpy0 = oo2zeta*a_pzpy0
ai_a_pzpy1 = oo2zeta*a_pzpy1
a_dzzdyz0 = PBz*a_dzzpy0 - PCz*a_dzzpy1 + 2.0d0*ai_a_pzpy0 - 2.0d0*ai_a_pzpy1
bi_a_dxys0 = oo2zeta*a_dxys0
bi_a_dxys1 = oo2zeta*a_dxys1
a_dxydxx0 = PBx*a_dxypx0 - PCx*a_dxypx1 + ai_a_pypx0 - ai_a_pypx1 + bi_a_dxys0 - bi_a_dxys1
ai_a_pxpy0 = oo2zeta*a_pxpy0
ai_a_pxpy1 = oo2zeta*a_pxpy1
a_dxydyy0 = PBy*a_dxypy0 - PCy*a_dxypy1 + ai_a_pxpy0 - ai_a_pxpy1 + bi_a_dxys0 - bi_a_dxys1
a_dxydzz0 = PBz*a_dxypz0 - PCz*a_dxypz1 + bi_a_dxys0 - bi_a_dxys1
a_dxydxy0 = PBy*a_dxypx0 - PCy*a_dxypx1 + ai_a_pxpx0 - ai_a_pxpx1
a_dxydxz0 = PBz*a_dxypx0 - PCz*a_dxypx1
a_dxydyz0 = PBz*a_dxypy0 - PCz*a_dxypy1
bi_a_dxzs0 = oo2zeta*a_dxzs0
bi_a_dxzs1 = oo2zeta*a_dxzs1
a_dxzdxx0 = PBx*a_dxzpx0 - PCx*a_dxzpx1 + ai_a_pzpx0 - ai_a_pzpx1 + bi_a_dxzs0 - bi_a_dxzs1
a_dxzdyy0 = PBy*a_dxzpy0 - PCy*a_dxzpy1 + bi_a_dxzs0 - bi_a_dxzs1
ai_a_pxpz0 = oo2zeta*a_pxpz0
ai_a_pxpz1 = oo2zeta*a_pxpz1
a_dxzdzz0 = PBz*a_dxzpz0 - PCz*a_dxzpz1 + ai_a_pxpz0 - ai_a_pxpz1 + bi_a_dxzs0 - bi_a_dxzs1
a_dxzdxy0 = PBy*a_dxzpx0 - PCy*a_dxzpx1
a_dxzdxz0 = PBz*a_dxzpx0 - PCz*a_dxzpx1 + ai_a_pxpx0 - ai_a_pxpx1
a_dxzdyz0 = PBz*a_dxzpy0 - PCz*a_dxzpy1 + ai_a_pxpy0 - ai_a_pxpy1
bi_a_dyzs0 = oo2zeta*a_dyzs0
bi_a_dyzs1 = oo2zeta*a_dyzs1
a_dyzdxx0 = PBx*a_dyzpx0 - PCx*a_dyzpx1 + bi_a_dyzs0 - bi_a_dyzs1
a_dyzdyy0 = PBy*a_dyzpy0 - PCy*a_dyzpy1 + ai_a_pzpy0 - ai_a_pzpy1 + bi_a_dyzs0 - bi_a_dyzs1
ai_a_pypz0 = oo2zeta*a_pypz0
ai_a_pypz1 = oo2zeta*a_pypz1
a_dyzdzz0 = PBz*a_dyzpz0 - PCz*a_dyzpz1 + ai_a_pypz0 - ai_a_pypz1 + bi_a_dyzs0 - bi_a_dyzs1
a_dyzdxy0 = PBy*a_dyzpx0 - PCy*a_dyzpx1 + ai_a_pzpx0 - ai_a_pzpx1
a_dyzdxz0 = PBz*a_dyzpx0 - PCz*a_dyzpx1 + ai_a_pypx0 - ai_a_pypx1
a_dyzdyz0 = PBz*a_dyzpy0 - PCz*a_dyzpy1 + ai_a_pypy0 - ai_a_pypy1
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
a_dxxpx_dAx = a_dxxpx_dAx + alpha2*a_fxxxpx0 - 2.0d0*a_pxpx0
a_dxxpy_dAx = a_dxxpy_dAx + alpha2*a_fxxxpy0 - 2.0d0*a_pxpy0
a_dxxpz_dAx = a_dxxpz_dAx + alpha2*a_fxxxpz0 - 2.0d0*a_pxpz0
a_dyypx_dAx = a_dyypx_dAx + alpha2*a_fyyxpx0
a_dyypy_dAx = a_dyypy_dAx + alpha2*a_fyyxpy0
a_dyypz_dAx = a_dyypz_dAx + alpha2*a_fyyxpz0
a_dzzpx_dAx = a_dzzpx_dAx + alpha2*a_fzzxpx0
a_dzzpy_dAx = a_dzzpy_dAx + alpha2*a_fzzxpy0
a_dzzpz_dAx = a_dzzpz_dAx + alpha2*a_fzzxpz0
a_dxypx_dAx = a_dxypx_dAx + alpha2*a_fxxypx0 - a_pypx0
a_dxypy_dAx = a_dxypy_dAx + alpha2*a_fxxypy0 - a_pypy0
a_dxypz_dAx = a_dxypz_dAx + alpha2*a_fxxypz0 - a_pypz0
a_dxzpx_dAx = a_dxzpx_dAx + alpha2*a_fxxzpx0 - a_pzpx0
a_dxzpy_dAx = a_dxzpy_dAx + alpha2*a_fxxzpy0 - a_pzpy0
a_dxzpz_dAx = a_dxzpz_dAx + alpha2*a_fxxzpz0 - a_pzpz0
a_dyzpx_dAx = a_dyzpx_dAx + alpha2*a_fxyzpx0
a_dyzpy_dAx = a_dyzpy_dAx + alpha2*a_fxyzpy0
a_dyzpz_dAx = a_dyzpz_dAx + alpha2*a_fxyzpz0
a_dxxpx_dAy = a_dxxpx_dAy + alpha2*a_fxxypx0
a_dxxpy_dAy = a_dxxpy_dAy + alpha2*a_fxxypy0
a_dxxpz_dAy = a_dxxpz_dAy + alpha2*a_fxxypz0
a_dyypx_dAy = a_dyypx_dAy + alpha2*a_fyyypx0 - 2.0d0*a_pypx0
a_dyypy_dAy = a_dyypy_dAy + alpha2*a_fyyypy0 - 2.0d0*a_pypy0
a_dyypz_dAy = a_dyypz_dAy + alpha2*a_fyyypz0 - 2.0d0*a_pypz0
a_dzzpx_dAy = a_dzzpx_dAy + alpha2*a_fzzypx0
a_dzzpy_dAy = a_dzzpy_dAy + alpha2*a_fzzypy0
a_dzzpz_dAy = a_dzzpz_dAy + alpha2*a_fzzypz0
a_dxypx_dAy = a_dxypx_dAy + alpha2*a_fyyxpx0 - a_pxpx0
a_dxypy_dAy = a_dxypy_dAy + alpha2*a_fyyxpy0 - a_pxpy0
a_dxypz_dAy = a_dxypz_dAy + alpha2*a_fyyxpz0 - a_pxpz0
a_dxzpx_dAy = a_dxzpx_dAy + alpha2*a_fxyzpx0
a_dxzpy_dAy = a_dxzpy_dAy + alpha2*a_fxyzpy0
a_dxzpz_dAy = a_dxzpz_dAy + alpha2*a_fxyzpz0
a_dyzpx_dAy = a_dyzpx_dAy + alpha2*a_fyyzpx0 - a_pzpx0
a_dyzpy_dAy = a_dyzpy_dAy + alpha2*a_fyyzpy0 - a_pzpy0
a_dyzpz_dAy = a_dyzpz_dAy + alpha2*a_fyyzpz0 - a_pzpz0
a_dxxpx_dAz = a_dxxpx_dAz + alpha2*a_fxxzpx0
a_dxxpy_dAz = a_dxxpy_dAz + alpha2*a_fxxzpy0
a_dxxpz_dAz = a_dxxpz_dAz + alpha2*a_fxxzpz0
a_dyypx_dAz = a_dyypx_dAz + alpha2*a_fyyzpx0
a_dyypy_dAz = a_dyypy_dAz + alpha2*a_fyyzpy0
a_dyypz_dAz = a_dyypz_dAz + alpha2*a_fyyzpz0
a_dzzpx_dAz = a_dzzpx_dAz + alpha2*a_fzzzpx0 - 2.0d0*a_pzpx0
a_dzzpy_dAz = a_dzzpy_dAz + alpha2*a_fzzzpy0 - 2.0d0*a_pzpy0
a_dzzpz_dAz = a_dzzpz_dAz + alpha2*a_fzzzpz0 - 2.0d0*a_pzpz0
a_dxypx_dAz = a_dxypx_dAz + alpha2*a_fxyzpx0
a_dxypy_dAz = a_dxypy_dAz + alpha2*a_fxyzpy0
a_dxypz_dAz = a_dxypz_dAz + alpha2*a_fxyzpz0
a_dxzpx_dAz = a_dxzpx_dAz + alpha2*a_fzzxpx0 - a_pxpx0
a_dxzpy_dAz = a_dxzpy_dAz + alpha2*a_fzzxpy0 - a_pxpy0
a_dxzpz_dAz = a_dxzpz_dAz + alpha2*a_fzzxpz0 - a_pxpz0
a_dyzpx_dAz = a_dyzpx_dAz + alpha2*a_fzzypx0 - a_pypx0
a_dyzpy_dAz = a_dyzpy_dAz + alpha2*a_fzzypy0 - a_pypy0
a_dyzpz_dAz = a_dyzpz_dAz + alpha2*a_fzzypz0 - a_pypz0
beta2 = 2.0d0*beta
a_dxxpx_dBx = a_dxxpx_dBx + beta2*a_dxxdxx0 - a_dxxs0
a_dxxpy_dBx = a_dxxpy_dBx + beta2*a_dxxdxy0
a_dxxpz_dBx = a_dxxpz_dBx + beta2*a_dxxdxz0
a_dyypx_dBx = a_dyypx_dBx + beta2*a_dyydxx0 - a_dyys0
a_dyypy_dBx = a_dyypy_dBx + beta2*a_dyydxy0
a_dyypz_dBx = a_dyypz_dBx + beta2*a_dyydxz0
a_dzzpx_dBx = a_dzzpx_dBx + beta2*a_dzzdxx0 - a_dzzs0
a_dzzpy_dBx = a_dzzpy_dBx + beta2*a_dzzdxy0
a_dzzpz_dBx = a_dzzpz_dBx + beta2*a_dzzdxz0
a_dxypx_dBx = a_dxypx_dBx + beta2*a_dxydxx0 - a_dxys0
a_dxypy_dBx = a_dxypy_dBx + beta2*a_dxydxy0
a_dxypz_dBx = a_dxypz_dBx + beta2*a_dxydxz0
a_dxzpx_dBx = a_dxzpx_dBx + beta2*a_dxzdxx0 - a_dxzs0
a_dxzpy_dBx = a_dxzpy_dBx + beta2*a_dxzdxy0
a_dxzpz_dBx = a_dxzpz_dBx + beta2*a_dxzdxz0
a_dyzpx_dBx = a_dyzpx_dBx + beta2*a_dyzdxx0 - a_dyzs0
a_dyzpy_dBx = a_dyzpy_dBx + beta2*a_dyzdxy0
a_dyzpz_dBx = a_dyzpz_dBx + beta2*a_dyzdxz0
a_dxxpx_dBy = a_dxxpx_dBy + beta2*a_dxxdxy0
a_dxxpy_dBy = a_dxxpy_dBy + beta2*a_dxxdyy0 - a_dxxs0
a_dxxpz_dBy = a_dxxpz_dBy + beta2*a_dxxdyz0
a_dyypx_dBy = a_dyypx_dBy + beta2*a_dyydxy0
a_dyypy_dBy = a_dyypy_dBy + beta2*a_dyydyy0 - a_dyys0
a_dyypz_dBy = a_dyypz_dBy + beta2*a_dyydyz0
a_dzzpx_dBy = a_dzzpx_dBy + beta2*a_dzzdxy0
a_dzzpy_dBy = a_dzzpy_dBy + beta2*a_dzzdyy0 - a_dzzs0
a_dzzpz_dBy = a_dzzpz_dBy + beta2*a_dzzdyz0
a_dxypx_dBy = a_dxypx_dBy + beta2*a_dxydxy0
a_dxypy_dBy = a_dxypy_dBy + beta2*a_dxydyy0 - a_dxys0
a_dxypz_dBy = a_dxypz_dBy + beta2*a_dxydyz0
a_dxzpx_dBy = a_dxzpx_dBy + beta2*a_dxzdxy0
a_dxzpy_dBy = a_dxzpy_dBy + beta2*a_dxzdyy0 - a_dxzs0
a_dxzpz_dBy = a_dxzpz_dBy + beta2*a_dxzdyz0
a_dyzpx_dBy = a_dyzpx_dBy + beta2*a_dyzdxy0
a_dyzpy_dBy = a_dyzpy_dBy + beta2*a_dyzdyy0 - a_dyzs0
a_dyzpz_dBy = a_dyzpz_dBy + beta2*a_dyzdyz0
a_dxxpx_dBz = a_dxxpx_dBz + beta2*a_dxxdxz0
a_dxxpy_dBz = a_dxxpy_dBz + beta2*a_dxxdyz0
a_dxxpz_dBz = a_dxxpz_dBz + beta2*a_dxxdzz0 - a_dxxs0
a_dyypx_dBz = a_dyypx_dBz + beta2*a_dyydxz0
a_dyypy_dBz = a_dyypy_dBz + beta2*a_dyydyz0
a_dyypz_dBz = a_dyypz_dBz + beta2*a_dyydzz0 - a_dyys0
a_dzzpx_dBz = a_dzzpx_dBz + beta2*a_dzzdxz0
a_dzzpy_dBz = a_dzzpy_dBz + beta2*a_dzzdyz0
a_dzzpz_dBz = a_dzzpz_dBz + beta2*a_dzzdzz0 - a_dzzs0
a_dxypx_dBz = a_dxypx_dBz + beta2*a_dxydxz0
a_dxypy_dBz = a_dxypy_dBz + beta2*a_dxydyz0
a_dxypz_dBz = a_dxypz_dBz + beta2*a_dxydzz0 - a_dxys0
a_dxzpx_dBz = a_dxzpx_dBz + beta2*a_dxzdxz0
a_dxzpy_dBz = a_dxzpy_dBz + beta2*a_dxzdyz0
a_dxzpz_dBz = a_dxzpz_dBz + beta2*a_dxzdzz0 - a_dxzs0
a_dyzpx_dBz = a_dyzpx_dBz + beta2*a_dyzdxz0
a_dyzpy_dBz = a_dyzpy_dBz + beta2*a_dyzdyz0
a_dyzpz_dBz = a_dyzpz_dBz + beta2*a_dyzdzz0 - a_dyzs0
        end do

     end do
v(1,1,1) = a_dxxpx_dAx
v(2,1,1) = a_dxxpy_dAx
v(3,1,1) = a_dxxpz_dAx
v(4,1,1) = a_dyypx_dAx
v(5,1,1) = a_dyypy_dAx
v(6,1,1) = a_dyypz_dAx
v(7,1,1) = a_dzzpx_dAx
v(8,1,1) = a_dzzpy_dAx
v(9,1,1) = a_dzzpz_dAx
v(10,1,1) = a_dxypx_dAx*root3
v(11,1,1) = a_dxypy_dAx*root3
v(12,1,1) = a_dxypz_dAx*root3
v(13,1,1) = a_dxzpx_dAx*root3
v(14,1,1) = a_dxzpy_dAx*root3
v(15,1,1) = a_dxzpz_dAx*root3
v(16,1,1) = a_dyzpx_dAx*root3
v(17,1,1) = a_dyzpy_dAx*root3
v(18,1,1) = a_dyzpz_dAx*root3
v(1,2,1) = a_dxxpx_dAy
v(2,2,1) = a_dxxpy_dAy
v(3,2,1) = a_dxxpz_dAy
v(4,2,1) = a_dyypx_dAy
v(5,2,1) = a_dyypy_dAy
v(6,2,1) = a_dyypz_dAy
v(7,2,1) = a_dzzpx_dAy
v(8,2,1) = a_dzzpy_dAy
v(9,2,1) = a_dzzpz_dAy
v(10,2,1) = a_dxypx_dAy*root3
v(11,2,1) = a_dxypy_dAy*root3
v(12,2,1) = a_dxypz_dAy*root3
v(13,2,1) = a_dxzpx_dAy*root3
v(14,2,1) = a_dxzpy_dAy*root3
v(15,2,1) = a_dxzpz_dAy*root3
v(16,2,1) = a_dyzpx_dAy*root3
v(17,2,1) = a_dyzpy_dAy*root3
v(18,2,1) = a_dyzpz_dAy*root3
v(1,3,1) = a_dxxpx_dAz
v(2,3,1) = a_dxxpy_dAz
v(3,3,1) = a_dxxpz_dAz
v(4,3,1) = a_dyypx_dAz
v(5,3,1) = a_dyypy_dAz
v(6,3,1) = a_dyypz_dAz
v(7,3,1) = a_dzzpx_dAz
v(8,3,1) = a_dzzpy_dAz
v(9,3,1) = a_dzzpz_dAz
v(10,3,1) = a_dxypx_dAz*root3
v(11,3,1) = a_dxypy_dAz*root3
v(12,3,1) = a_dxypz_dAz*root3
v(13,3,1) = a_dxzpx_dAz*root3
v(14,3,1) = a_dxzpy_dAz*root3
v(15,3,1) = a_dxzpz_dAz*root3
v(16,3,1) = a_dyzpx_dAz*root3
v(17,3,1) = a_dyzpy_dAz*root3
v(18,3,1) = a_dyzpz_dAz*root3
v(1,1,2) = a_dxxpx_dBx
v(2,1,2) = a_dxxpy_dBx
v(3,1,2) = a_dxxpz_dBx
v(4,1,2) = a_dyypx_dBx
v(5,1,2) = a_dyypy_dBx
v(6,1,2) = a_dyypz_dBx
v(7,1,2) = a_dzzpx_dBx
v(8,1,2) = a_dzzpy_dBx
v(9,1,2) = a_dzzpz_dBx
v(10,1,2) = a_dxypx_dBx*root3
v(11,1,2) = a_dxypy_dBx*root3
v(12,1,2) = a_dxypz_dBx*root3
v(13,1,2) = a_dxzpx_dBx*root3
v(14,1,2) = a_dxzpy_dBx*root3
v(15,1,2) = a_dxzpz_dBx*root3
v(16,1,2) = a_dyzpx_dBx*root3
v(17,1,2) = a_dyzpy_dBx*root3
v(18,1,2) = a_dyzpz_dBx*root3
v(1,2,2) = a_dxxpx_dBy
v(2,2,2) = a_dxxpy_dBy
v(3,2,2) = a_dxxpz_dBy
v(4,2,2) = a_dyypx_dBy
v(5,2,2) = a_dyypy_dBy
v(6,2,2) = a_dyypz_dBy
v(7,2,2) = a_dzzpx_dBy
v(8,2,2) = a_dzzpy_dBy
v(9,2,2) = a_dzzpz_dBy
v(10,2,2) = a_dxypx_dBy*root3
v(11,2,2) = a_dxypy_dBy*root3
v(12,2,2) = a_dxypz_dBy*root3
v(13,2,2) = a_dxzpx_dBy*root3
v(14,2,2) = a_dxzpy_dBy*root3
v(15,2,2) = a_dxzpz_dBy*root3
v(16,2,2) = a_dyzpx_dBy*root3
v(17,2,2) = a_dyzpy_dBy*root3
v(18,2,2) = a_dyzpz_dBy*root3
v(1,3,2) = a_dxxpx_dBz
v(2,3,2) = a_dxxpy_dBz
v(3,3,2) = a_dxxpz_dBz
v(4,3,2) = a_dyypx_dBz
v(5,3,2) = a_dyypy_dBz
v(6,3,2) = a_dyypz_dBz
v(7,3,2) = a_dzzpx_dBz
v(8,3,2) = a_dzzpy_dBz
v(9,3,2) = a_dzzpz_dBz
v(10,3,2) = a_dxypx_dBz*root3
v(11,3,2) = a_dxypy_dBz*root3
v(12,3,2) = a_dxypz_dBz*root3
v(13,3,2) = a_dxzpx_dBz*root3
v(14,3,2) = a_dxzpy_dBz*root3
v(15,3,2) = a_dxzpz_dBz*root3
v(16,3,2) = a_dyzpx_dBz*root3
v(17,3,2) = a_dyzpy_dBz*root3
v(18,3,2) = a_dyzpz_dBz*root3

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     ci = 2.0d0
     if (i < j) then
        do jx = jao, jao + 3-1
           do ix = iao, iao + 6-1
              key = ( ix-iao ) * 3 + jx-jao +1
              do ixyz = 1, 3
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(jx,ix)*v(key,ixyz,1)*ci
              end do
           end do
        end do
        do jx = jao, jao + 3-1
           do ix = iao, iao + 6-1
              key = ( ix-iao ) * 3 + jx-jao +1
              do ixyz = 1, 3
                 g(ixyz,jatom) = g(ixyz,jatom) + dm(jx,ix)*v(key,ixyz,2)*ci
              end do
           end do
        end do
     else
        do jx = jao, jao + 3-1
           do ix = iao, iao + 6-1
              key = ( ix-iao ) * 3 + jx-jao +1
              do ixyz = 1, 3
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz,1)*ci
              end do
           end do
        end do
        do jx = jao, jao + 3-1
           do ix = iao, iao + 6-1
              key = ( ix-iao ) * 3 + jx-jao +1
              do ixyz = 1, 3
                 g(ixyz,jatom) = g(ixyz,jatom) + dm(ix,jx)*v(key,ixyz,2)*ci
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dnai_dp_v2

!------------------------------------------------------------------------------
!> dd nuclear attraction integrals
!
subroutine os_dnai_dd_v2(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(36,3,2)

  real(8),allocatable::ssssx(:,:)
  real(8),allocatable::wkk(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:5,nc))
  allocate(wkk(6,nc))

!$omp do schedule(dynamic)
  do ij = ishp(5)+myrank+1, ishp(6), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

a_dxxdxx_dAx = 0.0d0
a_dxxdyy_dAx = 0.0d0
a_dxxdzz_dAx = 0.0d0
a_dxxdxy_dAx = 0.0d0
a_dxxdxz_dAx = 0.0d0
a_dxxdyz_dAx = 0.0d0
a_dyydxx_dAx = 0.0d0
a_dyydyy_dAx = 0.0d0
a_dyydzz_dAx = 0.0d0
a_dyydxy_dAx = 0.0d0
a_dyydxz_dAx = 0.0d0
a_dyydyz_dAx = 0.0d0
a_dzzdxx_dAx = 0.0d0
a_dzzdyy_dAx = 0.0d0
a_dzzdzz_dAx = 0.0d0
a_dzzdxy_dAx = 0.0d0
a_dzzdxz_dAx = 0.0d0
a_dzzdyz_dAx = 0.0d0
a_dxydxx_dAx = 0.0d0
a_dxydyy_dAx = 0.0d0
a_dxydzz_dAx = 0.0d0
a_dxydxy_dAx = 0.0d0
a_dxydxz_dAx = 0.0d0
a_dxydyz_dAx = 0.0d0
a_dxzdxx_dAx = 0.0d0
a_dxzdyy_dAx = 0.0d0
a_dxzdzz_dAx = 0.0d0
a_dxzdxy_dAx = 0.0d0
a_dxzdxz_dAx = 0.0d0
a_dxzdyz_dAx = 0.0d0
a_dyzdxx_dAx = 0.0d0
a_dyzdyy_dAx = 0.0d0
a_dyzdzz_dAx = 0.0d0
a_dyzdxy_dAx = 0.0d0
a_dyzdxz_dAx = 0.0d0
a_dyzdyz_dAx = 0.0d0
a_dxxdxx_dAy = 0.0d0
a_dxxdyy_dAy = 0.0d0
a_dxxdzz_dAy = 0.0d0
a_dxxdxy_dAy = 0.0d0
a_dxxdxz_dAy = 0.0d0
a_dxxdyz_dAy = 0.0d0
a_dyydxx_dAy = 0.0d0
a_dyydyy_dAy = 0.0d0
a_dyydzz_dAy = 0.0d0
a_dyydxy_dAy = 0.0d0
a_dyydxz_dAy = 0.0d0
a_dyydyz_dAy = 0.0d0
a_dzzdxx_dAy = 0.0d0
a_dzzdyy_dAy = 0.0d0
a_dzzdzz_dAy = 0.0d0
a_dzzdxy_dAy = 0.0d0
a_dzzdxz_dAy = 0.0d0
a_dzzdyz_dAy = 0.0d0
a_dxydxx_dAy = 0.0d0
a_dxydyy_dAy = 0.0d0
a_dxydzz_dAy = 0.0d0
a_dxydxy_dAy = 0.0d0
a_dxydxz_dAy = 0.0d0
a_dxydyz_dAy = 0.0d0
a_dxzdxx_dAy = 0.0d0
a_dxzdyy_dAy = 0.0d0
a_dxzdzz_dAy = 0.0d0
a_dxzdxy_dAy = 0.0d0
a_dxzdxz_dAy = 0.0d0
a_dxzdyz_dAy = 0.0d0
a_dyzdxx_dAy = 0.0d0
a_dyzdyy_dAy = 0.0d0
a_dyzdzz_dAy = 0.0d0
a_dyzdxy_dAy = 0.0d0
a_dyzdxz_dAy = 0.0d0
a_dyzdyz_dAy = 0.0d0
a_dxxdxx_dAz = 0.0d0
a_dxxdyy_dAz = 0.0d0
a_dxxdzz_dAz = 0.0d0
a_dxxdxy_dAz = 0.0d0
a_dxxdxz_dAz = 0.0d0
a_dxxdyz_dAz = 0.0d0
a_dyydxx_dAz = 0.0d0
a_dyydyy_dAz = 0.0d0
a_dyydzz_dAz = 0.0d0
a_dyydxy_dAz = 0.0d0
a_dyydxz_dAz = 0.0d0
a_dyydyz_dAz = 0.0d0
a_dzzdxx_dAz = 0.0d0
a_dzzdyy_dAz = 0.0d0
a_dzzdzz_dAz = 0.0d0
a_dzzdxy_dAz = 0.0d0
a_dzzdxz_dAz = 0.0d0
a_dzzdyz_dAz = 0.0d0
a_dxydxx_dAz = 0.0d0
a_dxydyy_dAz = 0.0d0
a_dxydzz_dAz = 0.0d0
a_dxydxy_dAz = 0.0d0
a_dxydxz_dAz = 0.0d0
a_dxydyz_dAz = 0.0d0
a_dxzdxx_dAz = 0.0d0
a_dxzdyy_dAz = 0.0d0
a_dxzdzz_dAz = 0.0d0
a_dxzdxy_dAz = 0.0d0
a_dxzdxz_dAz = 0.0d0
a_dxzdyz_dAz = 0.0d0
a_dyzdxx_dAz = 0.0d0
a_dyzdyy_dAz = 0.0d0
a_dyzdzz_dAz = 0.0d0
a_dyzdxy_dAz = 0.0d0
a_dyzdxz_dAz = 0.0d0
a_dyzdyz_dAz = 0.0d0
a_dxxdxx_dBx = 0.0d0
a_dxxdyy_dBx = 0.0d0
a_dxxdzz_dBx = 0.0d0
a_dxxdxy_dBx = 0.0d0
a_dxxdxz_dBx = 0.0d0
a_dxxdyz_dBx = 0.0d0
a_dyydxx_dBx = 0.0d0
a_dyydyy_dBx = 0.0d0
a_dyydzz_dBx = 0.0d0
a_dyydxy_dBx = 0.0d0
a_dyydxz_dBx = 0.0d0
a_dyydyz_dBx = 0.0d0
a_dzzdxx_dBx = 0.0d0
a_dzzdyy_dBx = 0.0d0
a_dzzdzz_dBx = 0.0d0
a_dzzdxy_dBx = 0.0d0
a_dzzdxz_dBx = 0.0d0
a_dzzdyz_dBx = 0.0d0
a_dxydxx_dBx = 0.0d0
a_dxydyy_dBx = 0.0d0
a_dxydzz_dBx = 0.0d0
a_dxydxy_dBx = 0.0d0
a_dxydxz_dBx = 0.0d0
a_dxydyz_dBx = 0.0d0
a_dxzdxx_dBx = 0.0d0
a_dxzdyy_dBx = 0.0d0
a_dxzdzz_dBx = 0.0d0
a_dxzdxy_dBx = 0.0d0
a_dxzdxz_dBx = 0.0d0
a_dxzdyz_dBx = 0.0d0
a_dyzdxx_dBx = 0.0d0
a_dyzdyy_dBx = 0.0d0
a_dyzdzz_dBx = 0.0d0
a_dyzdxy_dBx = 0.0d0
a_dyzdxz_dBx = 0.0d0
a_dyzdyz_dBx = 0.0d0
a_dxxdxx_dBy = 0.0d0
a_dxxdyy_dBy = 0.0d0
a_dxxdzz_dBy = 0.0d0
a_dxxdxy_dBy = 0.0d0
a_dxxdxz_dBy = 0.0d0
a_dxxdyz_dBy = 0.0d0
a_dyydxx_dBy = 0.0d0
a_dyydyy_dBy = 0.0d0
a_dyydzz_dBy = 0.0d0
a_dyydxy_dBy = 0.0d0
a_dyydxz_dBy = 0.0d0
a_dyydyz_dBy = 0.0d0
a_dzzdxx_dBy = 0.0d0
a_dzzdyy_dBy = 0.0d0
a_dzzdzz_dBy = 0.0d0
a_dzzdxy_dBy = 0.0d0
a_dzzdxz_dBy = 0.0d0
a_dzzdyz_dBy = 0.0d0
a_dxydxx_dBy = 0.0d0
a_dxydyy_dBy = 0.0d0
a_dxydzz_dBy = 0.0d0
a_dxydxy_dBy = 0.0d0
a_dxydxz_dBy = 0.0d0
a_dxydyz_dBy = 0.0d0
a_dxzdxx_dBy = 0.0d0
a_dxzdyy_dBy = 0.0d0
a_dxzdzz_dBy = 0.0d0
a_dxzdxy_dBy = 0.0d0
a_dxzdxz_dBy = 0.0d0
a_dxzdyz_dBy = 0.0d0
a_dyzdxx_dBy = 0.0d0
a_dyzdyy_dBy = 0.0d0
a_dyzdzz_dBy = 0.0d0
a_dyzdxy_dBy = 0.0d0
a_dyzdxz_dBy = 0.0d0
a_dyzdyz_dBy = 0.0d0
a_dxxdxx_dBz = 0.0d0
a_dxxdyy_dBz = 0.0d0
a_dxxdzz_dBz = 0.0d0
a_dxxdxy_dBz = 0.0d0
a_dxxdxz_dBz = 0.0d0
a_dxxdyz_dBz = 0.0d0
a_dyydxx_dBz = 0.0d0
a_dyydyy_dBz = 0.0d0
a_dyydzz_dBz = 0.0d0
a_dyydxy_dBz = 0.0d0
a_dyydxz_dBz = 0.0d0
a_dyydyz_dBz = 0.0d0
a_dzzdxx_dBz = 0.0d0
a_dzzdyy_dBz = 0.0d0
a_dzzdzz_dBz = 0.0d0
a_dzzdxy_dBz = 0.0d0
a_dzzdxz_dBz = 0.0d0
a_dzzdyz_dBz = 0.0d0
a_dxydxx_dBz = 0.0d0
a_dxydyy_dBz = 0.0d0
a_dxydzz_dBz = 0.0d0
a_dxydxy_dBz = 0.0d0
a_dxydxz_dBz = 0.0d0
a_dxydyz_dBz = 0.0d0
a_dxzdxx_dBz = 0.0d0
a_dxzdyy_dBz = 0.0d0
a_dxzdzz_dBz = 0.0d0
a_dxzdxy_dBz = 0.0d0
a_dxzdxz_dBz = 0.0d0
a_dxzdyz_dBz = 0.0d0
a_dyzdxx_dBz = 0.0d0
a_dyzdyy_dBz = 0.0d0
a_dyzdzz_dBz = 0.0d0
a_dyzdxy_dBz = 0.0d0
a_dyzdxz_dBz = 0.0d0
a_dyzdyz_dBz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)

           wkk(1,k) = PCx
           wkk(2,k) = PCy
           wkk(3,k) = PCz
           wkk(4,k) = t
        end do

!ocl nosimd
        do k = 1, nc
           t = wkk(4,k)

           if (t <= tf(6)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(12,it)*dt6 + fmt%fdat(11,it))*dt5&
                   + fmt%fdat(10,it))*dt4 + fmt%fdat(9,it))*dt3&
                   + fmt%fdat(8,it))*dt2 + fmt%fdat(7,it))*dt + fmt%fdat(6,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                   + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              wkk(5,k) = ft
              wkk(6,k) = et
           else
              wkk(5,k) = 0.0d0
              wkk(6,k) = 0.0d0
           end if
!!$        end do
!!$
!!$!ocl simd
!!$        do k = 1, nc
           PCx = wkk(1,k)
           PCy = wkk(2,k)
           PCz = wkk(3,k)
           t   = wkk(4,k)
           ft  = wkk(5,k)
           et  = wkk(6,k)
           ossx = -chg_c(k) * osstwosqrtzopi

           if (t <= tf(6)) then
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss5 = ft * ossx
              ssss4 = (ssss5 * t2 + etcn) * rminv(5)
              ssss3 = (ssss4 * t2 + etcn) * rminv(4)
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
              ssss4 = ssss3 * tinv * 3.5d0
              ssss5 = ssss4 * tinv * 4.5d0
           end if

a_ss0 = ssss0
a_ss1 = ssss1
a_ss2 = ssss2
a_ss3 = ssss3
a_ss4 = ssss4
a_ss5 = ssss5
PCx = Px - C(1,k)
PCy = Py - C(2,k)
PCz = Pz - C(3,k)
a_pxs0 = PAx*a_ss0 - PCx*a_ss1
a_pys0 = PAy*a_ss0 - PCy*a_ss1
a_pzs0 = PAz*a_ss0 - PCz*a_ss1
a_pxs1 = PAx*a_ss1 - PCx*a_ss2
a_pys1 = PAy*a_ss1 - PCy*a_ss2
a_pzs1 = PAz*a_ss1 - PCz*a_ss2
ai_a_ss0 = oo2zeta*a_ss0
ai_a_ss1 = oo2zeta*a_ss1
a_dxxs0 = PAx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_dyys0 = PAy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_dzzs0 = PAz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
a_dxys0 = PAy*a_pxs0 - PCy*a_pxs1
a_dxzs0 = PAz*a_pxs0 - PCz*a_pxs1
a_dyzs0 = PAz*a_pys0 - PCz*a_pys1
a_pxs2 = PAx*a_ss2 - PCx*a_ss3
a_pys2 = PAy*a_ss2 - PCy*a_ss3
a_pzs2 = PAz*a_ss2 - PCz*a_ss3
ai_a_ss2 = oo2zeta*a_ss2
a_dxxs1 = PAx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_dyys1 = PAy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_dzzs1 = PAz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
a_dxys1 = PAy*a_pxs1 - PCy*a_pxs2
a_dxzs1 = PAz*a_pxs1 - PCz*a_pxs2
a_dyzs1 = PAz*a_pys1 - PCz*a_pys2
ai_a_pxs0 = oo2zeta*a_pxs0
ai_a_pxs1 = oo2zeta*a_pxs1
a_fxxxs0 = PAx*a_dxxs0 - PCx*a_dxxs1 + 2.0d0*ai_a_pxs0 - 2.0d0*ai_a_pxs1
ai_a_pys0 = oo2zeta*a_pys0
ai_a_pys1 = oo2zeta*a_pys1
a_fyyys0 = PAy*a_dyys0 - PCy*a_dyys1 + 2.0d0*ai_a_pys0 - 2.0d0*ai_a_pys1
ai_a_pzs0 = oo2zeta*a_pzs0
ai_a_pzs1 = oo2zeta*a_pzs1
a_fzzzs0 = PAz*a_dzzs0 - PCz*a_dzzs1 + 2.0d0*ai_a_pzs0 - 2.0d0*ai_a_pzs1
a_fxxys0 = PAy*a_dxxs0 - PCy*a_dxxs1
a_fxxzs0 = PAz*a_dxxs0 - PCz*a_dxxs1
a_fyyxs0 = PAx*a_dyys0 - PCx*a_dyys1
a_fyyzs0 = PAz*a_dyys0 - PCz*a_dyys1
a_fzzxs0 = PAx*a_dzzs0 - PCx*a_dzzs1
a_fzzys0 = PAy*a_dzzs0 - PCy*a_dzzs1
a_fxyzs0 = PAz*a_dxys0 - PCz*a_dxys1
a_pxs3 = PAx*a_ss3 - PCx*a_ss4
a_pys3 = PAy*a_ss3 - PCy*a_ss4
a_pzs3 = PAz*a_ss3 - PCz*a_ss4
ai_a_ss3 = oo2zeta*a_ss3
a_dxxs2 = PAx*a_pxs2 - PCx*a_pxs3 + ai_a_ss2 - ai_a_ss3
a_dyys2 = PAy*a_pys2 - PCy*a_pys3 + ai_a_ss2 - ai_a_ss3
a_dzzs2 = PAz*a_pzs2 - PCz*a_pzs3 + ai_a_ss2 - ai_a_ss3
a_dxys2 = PAy*a_pxs2 - PCy*a_pxs3
a_dxzs2 = PAz*a_pxs2 - PCz*a_pxs3
a_dyzs2 = PAz*a_pys2 - PCz*a_pys3
ai_a_pxs2 = oo2zeta*a_pxs2
a_fxxxs1 = PAx*a_dxxs1 - PCx*a_dxxs2 + 2.0d0*ai_a_pxs1 - 2.0d0*ai_a_pxs2
ai_a_pys2 = oo2zeta*a_pys2
a_fyyys1 = PAy*a_dyys1 - PCy*a_dyys2 + 2.0d0*ai_a_pys1 - 2.0d0*ai_a_pys2
ai_a_pzs2 = oo2zeta*a_pzs2
a_fzzzs1 = PAz*a_dzzs1 - PCz*a_dzzs2 + 2.0d0*ai_a_pzs1 - 2.0d0*ai_a_pzs2
a_fxxys1 = PAy*a_dxxs1 - PCy*a_dxxs2
a_fxxzs1 = PAz*a_dxxs1 - PCz*a_dxxs2
a_fyyxs1 = PAx*a_dyys1 - PCx*a_dyys2
a_fyyzs1 = PAz*a_dyys1 - PCz*a_dyys2
a_fzzxs1 = PAx*a_dzzs1 - PCx*a_dzzs2
a_fzzys1 = PAy*a_dzzs1 - PCy*a_dzzs2
a_fxyzs1 = PAz*a_dxys1 - PCz*a_dxys2
ai_a_dxxs0 = oo2zeta*a_dxxs0
ai_a_dxxs1 = oo2zeta*a_dxxs1
a_fxxxpx0 = PBx*a_fxxxs0 - PCx*a_fxxxs1 + 3.0d0*ai_a_dxxs0 - 3.0d0*ai_a_dxxs1
a_fxxxpy0 = PBy*a_fxxxs0 - PCy*a_fxxxs1
a_fxxxpz0 = PBz*a_fxxxs0 - PCz*a_fxxxs1
a_fyyypx0 = PBx*a_fyyys0 - PCx*a_fyyys1
ai_a_dyys0 = oo2zeta*a_dyys0
ai_a_dyys1 = oo2zeta*a_dyys1
a_fyyypy0 = PBy*a_fyyys0 - PCy*a_fyyys1 + 3.0d0*ai_a_dyys0 - 3.0d0*ai_a_dyys1
a_fyyypz0 = PBz*a_fyyys0 - PCz*a_fyyys1
a_fzzzpx0 = PBx*a_fzzzs0 - PCx*a_fzzzs1
a_fzzzpy0 = PBy*a_fzzzs0 - PCy*a_fzzzs1
ai_a_dzzs0 = oo2zeta*a_dzzs0
ai_a_dzzs1 = oo2zeta*a_dzzs1
a_fzzzpz0 = PBz*a_fzzzs0 - PCz*a_fzzzs1 + 3.0d0*ai_a_dzzs0 - 3.0d0*ai_a_dzzs1
ai_a_dxys0 = oo2zeta*a_dxys0
ai_a_dxys1 = oo2zeta*a_dxys1
a_fxxypx0 = PBx*a_fxxys0 - PCx*a_fxxys1 + 2.0d0*ai_a_dxys0 - 2.0d0*ai_a_dxys1
a_fxxypy0 = PBy*a_fxxys0 - PCy*a_fxxys1 + ai_a_dxxs0 - ai_a_dxxs1
a_fxxypz0 = PBz*a_fxxys0 - PCz*a_fxxys1
ai_a_dxzs0 = oo2zeta*a_dxzs0
ai_a_dxzs1 = oo2zeta*a_dxzs1
a_fxxzpx0 = PBx*a_fxxzs0 - PCx*a_fxxzs1 + 2.0d0*ai_a_dxzs0 - 2.0d0*ai_a_dxzs1
a_fxxzpy0 = PBy*a_fxxzs0 - PCy*a_fxxzs1
a_fxxzpz0 = PBz*a_fxxzs0 - PCz*a_fxxzs1 + ai_a_dxxs0 - ai_a_dxxs1
a_fyyxpx0 = PBx*a_fyyxs0 - PCx*a_fyyxs1 + ai_a_dyys0 - ai_a_dyys1
a_fyyxpy0 = PBy*a_fyyxs0 - PCy*a_fyyxs1 + 2.0d0*ai_a_dxys0 - 2.0d0*ai_a_dxys1
a_fyyxpz0 = PBz*a_fyyxs0 - PCz*a_fyyxs1
a_fyyzpx0 = PBx*a_fyyzs0 - PCx*a_fyyzs1
ai_a_dyzs0 = oo2zeta*a_dyzs0
ai_a_dyzs1 = oo2zeta*a_dyzs1
a_fyyzpy0 = PBy*a_fyyzs0 - PCy*a_fyyzs1 + 2.0d0*ai_a_dyzs0 - 2.0d0*ai_a_dyzs1
a_fyyzpz0 = PBz*a_fyyzs0 - PCz*a_fyyzs1 + ai_a_dyys0 - ai_a_dyys1
a_fzzxpx0 = PBx*a_fzzxs0 - PCx*a_fzzxs1 + ai_a_dzzs0 - ai_a_dzzs1
a_fzzxpy0 = PBy*a_fzzxs0 - PCy*a_fzzxs1
a_fzzxpz0 = PBz*a_fzzxs0 - PCz*a_fzzxs1 + 2.0d0*ai_a_dxzs0 - 2.0d0*ai_a_dxzs1
a_fzzypx0 = PBx*a_fzzys0 - PCx*a_fzzys1
a_fzzypy0 = PBy*a_fzzys0 - PCy*a_fzzys1 + ai_a_dzzs0 - ai_a_dzzs1
a_fzzypz0 = PBz*a_fzzys0 - PCz*a_fzzys1 + 2.0d0*ai_a_dyzs0 - 2.0d0*ai_a_dyzs1
a_fxyzpx0 = PBx*a_fxyzs0 - PCx*a_fxyzs1 + ai_a_dyzs0 - ai_a_dyzs1
a_fxyzpy0 = PBy*a_fxyzs0 - PCy*a_fxyzs1 + ai_a_dxzs0 - ai_a_dxzs1
a_fxyzpz0 = PBz*a_fxyzs0 - PCz*a_fxyzs1 + ai_a_dxys0 - ai_a_dxys1
a_pxs4 = PAx*a_ss4 - PCx*a_ss5
a_pys4 = PAy*a_ss4 - PCy*a_ss5
a_pzs4 = PAz*a_ss4 - PCz*a_ss5
ai_a_ss4 = oo2zeta*a_ss4
a_dxxs3 = PAx*a_pxs3 - PCx*a_pxs4 + ai_a_ss3 - ai_a_ss4
a_dyys3 = PAy*a_pys3 - PCy*a_pys4 + ai_a_ss3 - ai_a_ss4
a_dzzs3 = PAz*a_pzs3 - PCz*a_pzs4 + ai_a_ss3 - ai_a_ss4
a_dxys3 = PAy*a_pxs3 - PCy*a_pxs4
a_dxzs3 = PAz*a_pxs3 - PCz*a_pxs4
a_dyzs3 = PAz*a_pys3 - PCz*a_pys4
ai_a_pxs3 = oo2zeta*a_pxs3
a_fxxxs2 = PAx*a_dxxs2 - PCx*a_dxxs3 + 2.0d0*ai_a_pxs2 - 2.0d0*ai_a_pxs3
ai_a_pys3 = oo2zeta*a_pys3
a_fyyys2 = PAy*a_dyys2 - PCy*a_dyys3 + 2.0d0*ai_a_pys2 - 2.0d0*ai_a_pys3
ai_a_pzs3 = oo2zeta*a_pzs3
a_fzzzs2 = PAz*a_dzzs2 - PCz*a_dzzs3 + 2.0d0*ai_a_pzs2 - 2.0d0*ai_a_pzs3
a_fxxys2 = PAy*a_dxxs2 - PCy*a_dxxs3
a_fxxzs2 = PAz*a_dxxs2 - PCz*a_dxxs3
a_fyyxs2 = PAx*a_dyys2 - PCx*a_dyys3
a_fyyzs2 = PAz*a_dyys2 - PCz*a_dyys3
a_fzzxs2 = PAx*a_dzzs2 - PCx*a_dzzs3
a_fzzys2 = PAy*a_dzzs2 - PCy*a_dzzs3
a_fxyzs2 = PAz*a_dxys2 - PCz*a_dxys3
ai_a_dxxs2 = oo2zeta*a_dxxs2
a_fxxxpx1 = PBx*a_fxxxs1 - PCx*a_fxxxs2 + 3.0d0*ai_a_dxxs1 - 3.0d0*ai_a_dxxs2
a_fxxxpy1 = PBy*a_fxxxs1 - PCy*a_fxxxs2
a_fxxxpz1 = PBz*a_fxxxs1 - PCz*a_fxxxs2
a_fyyypx1 = PBx*a_fyyys1 - PCx*a_fyyys2
ai_a_dyys2 = oo2zeta*a_dyys2
a_fyyypy1 = PBy*a_fyyys1 - PCy*a_fyyys2 + 3.0d0*ai_a_dyys1 - 3.0d0*ai_a_dyys2
a_fyyypz1 = PBz*a_fyyys1 - PCz*a_fyyys2
a_fzzzpx1 = PBx*a_fzzzs1 - PCx*a_fzzzs2
a_fzzzpy1 = PBy*a_fzzzs1 - PCy*a_fzzzs2
ai_a_dzzs2 = oo2zeta*a_dzzs2
a_fzzzpz1 = PBz*a_fzzzs1 - PCz*a_fzzzs2 + 3.0d0*ai_a_dzzs1 - 3.0d0*ai_a_dzzs2
ai_a_dxys2 = oo2zeta*a_dxys2
a_fxxypx1 = PBx*a_fxxys1 - PCx*a_fxxys2 + 2.0d0*ai_a_dxys1 - 2.0d0*ai_a_dxys2
a_fxxypy1 = PBy*a_fxxys1 - PCy*a_fxxys2 + ai_a_dxxs1 - ai_a_dxxs2
a_fxxypz1 = PBz*a_fxxys1 - PCz*a_fxxys2
ai_a_dxzs2 = oo2zeta*a_dxzs2
a_fxxzpx1 = PBx*a_fxxzs1 - PCx*a_fxxzs2 + 2.0d0*ai_a_dxzs1 - 2.0d0*ai_a_dxzs2
a_fxxzpy1 = PBy*a_fxxzs1 - PCy*a_fxxzs2
a_fxxzpz1 = PBz*a_fxxzs1 - PCz*a_fxxzs2 + ai_a_dxxs1 - ai_a_dxxs2
a_fyyxpx1 = PBx*a_fyyxs1 - PCx*a_fyyxs2 + ai_a_dyys1 - ai_a_dyys2
a_fyyxpy1 = PBy*a_fyyxs1 - PCy*a_fyyxs2 + 2.0d0*ai_a_dxys1 - 2.0d0*ai_a_dxys2
a_fyyxpz1 = PBz*a_fyyxs1 - PCz*a_fyyxs2
a_fyyzpx1 = PBx*a_fyyzs1 - PCx*a_fyyzs2
ai_a_dyzs2 = oo2zeta*a_dyzs2
a_fyyzpy1 = PBy*a_fyyzs1 - PCy*a_fyyzs2 + 2.0d0*ai_a_dyzs1 - 2.0d0*ai_a_dyzs2
a_fyyzpz1 = PBz*a_fyyzs1 - PCz*a_fyyzs2 + ai_a_dyys1 - ai_a_dyys2
a_fzzxpx1 = PBx*a_fzzxs1 - PCx*a_fzzxs2 + ai_a_dzzs1 - ai_a_dzzs2
a_fzzxpy1 = PBy*a_fzzxs1 - PCy*a_fzzxs2
a_fzzxpz1 = PBz*a_fzzxs1 - PCz*a_fzzxs2 + 2.0d0*ai_a_dxzs1 - 2.0d0*ai_a_dxzs2
a_fzzypx1 = PBx*a_fzzys1 - PCx*a_fzzys2
a_fzzypy1 = PBy*a_fzzys1 - PCy*a_fzzys2 + ai_a_dzzs1 - ai_a_dzzs2
a_fzzypz1 = PBz*a_fzzys1 - PCz*a_fzzys2 + 2.0d0*ai_a_dyzs1 - 2.0d0*ai_a_dyzs2
a_fxyzpx1 = PBx*a_fxyzs1 - PCx*a_fxyzs2 + ai_a_dyzs1 - ai_a_dyzs2
a_fxyzpy1 = PBy*a_fxyzs1 - PCy*a_fxyzs2 + ai_a_dxzs1 - ai_a_dxzs2
a_fxyzpz1 = PBz*a_fxyzs1 - PCz*a_fxyzs2 + ai_a_dxys1 - ai_a_dxys2
a_dxxpx0 = PBx*a_dxxs0 - PCx*a_dxxs1 + 2.0d0*ai_a_pxs0 - 2.0d0*ai_a_pxs1
a_dxxpy0 = PBy*a_dxxs0 - PCy*a_dxxs1
a_dxxpz0 = PBz*a_dxxs0 - PCz*a_dxxs1
a_dyypx0 = PBx*a_dyys0 - PCx*a_dyys1
a_dyypy0 = PBy*a_dyys0 - PCy*a_dyys1 + 2.0d0*ai_a_pys0 - 2.0d0*ai_a_pys1
a_dyypz0 = PBz*a_dyys0 - PCz*a_dyys1
a_dzzpx0 = PBx*a_dzzs0 - PCx*a_dzzs1
a_dzzpy0 = PBy*a_dzzs0 - PCy*a_dzzs1
a_dzzpz0 = PBz*a_dzzs0 - PCz*a_dzzs1 + 2.0d0*ai_a_pzs0 - 2.0d0*ai_a_pzs1
a_dxypx0 = PBx*a_dxys0 - PCx*a_dxys1 + ai_a_pys0 - ai_a_pys1
a_dxypy0 = PBy*a_dxys0 - PCy*a_dxys1 + ai_a_pxs0 - ai_a_pxs1
a_dxypz0 = PBz*a_dxys0 - PCz*a_dxys1
a_dxzpx0 = PBx*a_dxzs0 - PCx*a_dxzs1 + ai_a_pzs0 - ai_a_pzs1
a_dxzpy0 = PBy*a_dxzs0 - PCy*a_dxzs1
a_dxzpz0 = PBz*a_dxzs0 - PCz*a_dxzs1 + ai_a_pxs0 - ai_a_pxs1
a_dyzpx0 = PBx*a_dyzs0 - PCx*a_dyzs1
a_dyzpy0 = PBy*a_dyzs0 - PCy*a_dyzs1 + ai_a_pzs0 - ai_a_pzs1
a_dyzpz0 = PBz*a_dyzs0 - PCz*a_dyzs1 + ai_a_pys0 - ai_a_pys1
a_dxxpx1 = PBx*a_dxxs1 - PCx*a_dxxs2 + 2.0d0*ai_a_pxs1 - 2.0d0*ai_a_pxs2
a_dxxpy1 = PBy*a_dxxs1 - PCy*a_dxxs2
a_dxxpz1 = PBz*a_dxxs1 - PCz*a_dxxs2
a_dyypx1 = PBx*a_dyys1 - PCx*a_dyys2
a_dyypy1 = PBy*a_dyys1 - PCy*a_dyys2 + 2.0d0*ai_a_pys1 - 2.0d0*ai_a_pys2
a_dyypz1 = PBz*a_dyys1 - PCz*a_dyys2
a_dzzpx1 = PBx*a_dzzs1 - PCx*a_dzzs2
a_dzzpy1 = PBy*a_dzzs1 - PCy*a_dzzs2
a_dzzpz1 = PBz*a_dzzs1 - PCz*a_dzzs2 + 2.0d0*ai_a_pzs1 - 2.0d0*ai_a_pzs2
a_dxypx1 = PBx*a_dxys1 - PCx*a_dxys2 + ai_a_pys1 - ai_a_pys2
a_dxypy1 = PBy*a_dxys1 - PCy*a_dxys2 + ai_a_pxs1 - ai_a_pxs2
a_dxypz1 = PBz*a_dxys1 - PCz*a_dxys2
a_dxzpx1 = PBx*a_dxzs1 - PCx*a_dxzs2 + ai_a_pzs1 - ai_a_pzs2
a_dxzpy1 = PBy*a_dxzs1 - PCy*a_dxzs2
a_dxzpz1 = PBz*a_dxzs1 - PCz*a_dxzs2 + ai_a_pxs1 - ai_a_pxs2
a_dyzpx1 = PBx*a_dyzs1 - PCx*a_dyzs2
a_dyzpy1 = PBy*a_dyzs1 - PCy*a_dyzs2 + ai_a_pzs1 - ai_a_pzs2
a_dyzpz1 = PBz*a_dyzs1 - PCz*a_dyzs2 + ai_a_pys1 - ai_a_pys2
ai_a_dxxpx0 = oo2zeta*a_dxxpx0
ai_a_dxxpx1 = oo2zeta*a_dxxpx1
bi_a_fxxxs0 = oo2zeta*a_fxxxs0
bi_a_fxxxs1 = oo2zeta*a_fxxxs1
a_fxxxdxx0 = PBx*a_fxxxpx0 - PCx*a_fxxxpx1 + 3.0d0*ai_a_dxxpx0 - 3.0d0*ai_a_dxxpx1 + bi_a_fxxxs0 - bi_a_fxxxs1
a_fxxxdyy0 = PBy*a_fxxxpy0 - PCy*a_fxxxpy1 + bi_a_fxxxs0 - bi_a_fxxxs1
a_fxxxdzz0 = PBz*a_fxxxpz0 - PCz*a_fxxxpz1 + bi_a_fxxxs0 - bi_a_fxxxs1
a_fxxxdxy0 = PBy*a_fxxxpx0 - PCy*a_fxxxpx1
a_fxxxdxz0 = PBz*a_fxxxpx0 - PCz*a_fxxxpx1
a_fxxxdyz0 = PBz*a_fxxxpy0 - PCz*a_fxxxpy1
bi_a_fyyys0 = oo2zeta*a_fyyys0
bi_a_fyyys1 = oo2zeta*a_fyyys1
a_fyyydxx0 = PBx*a_fyyypx0 - PCx*a_fyyypx1 + bi_a_fyyys0 - bi_a_fyyys1
ai_a_dyypy0 = oo2zeta*a_dyypy0
ai_a_dyypy1 = oo2zeta*a_dyypy1
a_fyyydyy0 = PBy*a_fyyypy0 - PCy*a_fyyypy1 + 3.0d0*ai_a_dyypy0 - 3.0d0*ai_a_dyypy1 + bi_a_fyyys0 - bi_a_fyyys1
a_fyyydzz0 = PBz*a_fyyypz0 - PCz*a_fyyypz1 + bi_a_fyyys0 - bi_a_fyyys1
ai_a_dyypx0 = oo2zeta*a_dyypx0
ai_a_dyypx1 = oo2zeta*a_dyypx1
a_fyyydxy0 = PBy*a_fyyypx0 - PCy*a_fyyypx1 + 3.0d0*ai_a_dyypx0 - 3.0d0*ai_a_dyypx1
a_fyyydxz0 = PBz*a_fyyypx0 - PCz*a_fyyypx1
a_fyyydyz0 = PBz*a_fyyypy0 - PCz*a_fyyypy1
bi_a_fzzzs0 = oo2zeta*a_fzzzs0
bi_a_fzzzs1 = oo2zeta*a_fzzzs1
a_fzzzdxx0 = PBx*a_fzzzpx0 - PCx*a_fzzzpx1 + bi_a_fzzzs0 - bi_a_fzzzs1
a_fzzzdyy0 = PBy*a_fzzzpy0 - PCy*a_fzzzpy1 + bi_a_fzzzs0 - bi_a_fzzzs1
ai_a_dzzpz0 = oo2zeta*a_dzzpz0
ai_a_dzzpz1 = oo2zeta*a_dzzpz1
a_fzzzdzz0 = PBz*a_fzzzpz0 - PCz*a_fzzzpz1 + 3.0d0*ai_a_dzzpz0 - 3.0d0*ai_a_dzzpz1 + bi_a_fzzzs0 - bi_a_fzzzs1
a_fzzzdxy0 = PBy*a_fzzzpx0 - PCy*a_fzzzpx1
ai_a_dzzpx0 = oo2zeta*a_dzzpx0
ai_a_dzzpx1 = oo2zeta*a_dzzpx1
a_fzzzdxz0 = PBz*a_fzzzpx0 - PCz*a_fzzzpx1 + 3.0d0*ai_a_dzzpx0 - 3.0d0*ai_a_dzzpx1
ai_a_dzzpy0 = oo2zeta*a_dzzpy0
ai_a_dzzpy1 = oo2zeta*a_dzzpy1
a_fzzzdyz0 = PBz*a_fzzzpy0 - PCz*a_fzzzpy1 + 3.0d0*ai_a_dzzpy0 - 3.0d0*ai_a_dzzpy1
ai_a_dxypx0 = oo2zeta*a_dxypx0
ai_a_dxypx1 = oo2zeta*a_dxypx1
bi_a_fxxys0 = oo2zeta*a_fxxys0
bi_a_fxxys1 = oo2zeta*a_fxxys1
a_fxxydxx0 = PBx*a_fxxypx0 - PCx*a_fxxypx1 + 2.0d0*ai_a_dxypx0 - 2.0d0*ai_a_dxypx1 + bi_a_fxxys0 - bi_a_fxxys1
ai_a_dxxpy0 = oo2zeta*a_dxxpy0
ai_a_dxxpy1 = oo2zeta*a_dxxpy1
a_fxxydyy0 = PBy*a_fxxypy0 - PCy*a_fxxypy1 + ai_a_dxxpy0 - ai_a_dxxpy1 + bi_a_fxxys0 - bi_a_fxxys1
a_fxxydzz0 = PBz*a_fxxypz0 - PCz*a_fxxypz1 + bi_a_fxxys0 - bi_a_fxxys1
a_fxxydxy0 = PBy*a_fxxypx0 - PCy*a_fxxypx1 + ai_a_dxxpx0 - ai_a_dxxpx1
a_fxxydxz0 = PBz*a_fxxypx0 - PCz*a_fxxypx1
a_fxxydyz0 = PBz*a_fxxypy0 - PCz*a_fxxypy1
ai_a_dxzpx0 = oo2zeta*a_dxzpx0
ai_a_dxzpx1 = oo2zeta*a_dxzpx1
bi_a_fxxzs0 = oo2zeta*a_fxxzs0
bi_a_fxxzs1 = oo2zeta*a_fxxzs1
a_fxxzdxx0 = PBx*a_fxxzpx0 - PCx*a_fxxzpx1 + 2.0d0*ai_a_dxzpx0 - 2.0d0*ai_a_dxzpx1 + bi_a_fxxzs0 - bi_a_fxxzs1
a_fxxzdyy0 = PBy*a_fxxzpy0 - PCy*a_fxxzpy1 + bi_a_fxxzs0 - bi_a_fxxzs1
ai_a_dxxpz0 = oo2zeta*a_dxxpz0
ai_a_dxxpz1 = oo2zeta*a_dxxpz1
a_fxxzdzz0 = PBz*a_fxxzpz0 - PCz*a_fxxzpz1 + ai_a_dxxpz0 - ai_a_dxxpz1 + bi_a_fxxzs0 - bi_a_fxxzs1
a_fxxzdxy0 = PBy*a_fxxzpx0 - PCy*a_fxxzpx1
a_fxxzdxz0 = PBz*a_fxxzpx0 - PCz*a_fxxzpx1 + ai_a_dxxpx0 - ai_a_dxxpx1
a_fxxzdyz0 = PBz*a_fxxzpy0 - PCz*a_fxxzpy1 + ai_a_dxxpy0 - ai_a_dxxpy1
bi_a_fyyxs0 = oo2zeta*a_fyyxs0
bi_a_fyyxs1 = oo2zeta*a_fyyxs1
a_fyyxdxx0 = PBx*a_fyyxpx0 - PCx*a_fyyxpx1 + ai_a_dyypx0 - ai_a_dyypx1 + bi_a_fyyxs0 - bi_a_fyyxs1
ai_a_dxypy0 = oo2zeta*a_dxypy0
ai_a_dxypy1 = oo2zeta*a_dxypy1
a_fyyxdyy0 = PBy*a_fyyxpy0 - PCy*a_fyyxpy1 + 2.0d0*ai_a_dxypy0 - 2.0d0*ai_a_dxypy1 + bi_a_fyyxs0 - bi_a_fyyxs1
a_fyyxdzz0 = PBz*a_fyyxpz0 - PCz*a_fyyxpz1 + bi_a_fyyxs0 - bi_a_fyyxs1
a_fyyxdxy0 = PBy*a_fyyxpx0 - PCy*a_fyyxpx1 + 2.0d0*ai_a_dxypx0 - 2.0d0*ai_a_dxypx1
a_fyyxdxz0 = PBz*a_fyyxpx0 - PCz*a_fyyxpx1
a_fyyxdyz0 = PBz*a_fyyxpy0 - PCz*a_fyyxpy1
bi_a_fyyzs0 = oo2zeta*a_fyyzs0
bi_a_fyyzs1 = oo2zeta*a_fyyzs1
a_fyyzdxx0 = PBx*a_fyyzpx0 - PCx*a_fyyzpx1 + bi_a_fyyzs0 - bi_a_fyyzs1
ai_a_dyzpy0 = oo2zeta*a_dyzpy0
ai_a_dyzpy1 = oo2zeta*a_dyzpy1
a_fyyzdyy0 = PBy*a_fyyzpy0 - PCy*a_fyyzpy1 + 2.0d0*ai_a_dyzpy0 - 2.0d0*ai_a_dyzpy1 + bi_a_fyyzs0 - bi_a_fyyzs1
ai_a_dyypz0 = oo2zeta*a_dyypz0
ai_a_dyypz1 = oo2zeta*a_dyypz1
a_fyyzdzz0 = PBz*a_fyyzpz0 - PCz*a_fyyzpz1 + ai_a_dyypz0 - ai_a_dyypz1 + bi_a_fyyzs0 - bi_a_fyyzs1
ai_a_dyzpx0 = oo2zeta*a_dyzpx0
ai_a_dyzpx1 = oo2zeta*a_dyzpx1
a_fyyzdxy0 = PBy*a_fyyzpx0 - PCy*a_fyyzpx1 + 2.0d0*ai_a_dyzpx0 - 2.0d0*ai_a_dyzpx1
a_fyyzdxz0 = PBz*a_fyyzpx0 - PCz*a_fyyzpx1 + ai_a_dyypx0 - ai_a_dyypx1
a_fyyzdyz0 = PBz*a_fyyzpy0 - PCz*a_fyyzpy1 + ai_a_dyypy0 - ai_a_dyypy1
bi_a_fzzxs0 = oo2zeta*a_fzzxs0
bi_a_fzzxs1 = oo2zeta*a_fzzxs1
a_fzzxdxx0 = PBx*a_fzzxpx0 - PCx*a_fzzxpx1 + ai_a_dzzpx0 - ai_a_dzzpx1 + bi_a_fzzxs0 - bi_a_fzzxs1
a_fzzxdyy0 = PBy*a_fzzxpy0 - PCy*a_fzzxpy1 + bi_a_fzzxs0 - bi_a_fzzxs1
ai_a_dxzpz0 = oo2zeta*a_dxzpz0
ai_a_dxzpz1 = oo2zeta*a_dxzpz1
a_fzzxdzz0 = PBz*a_fzzxpz0 - PCz*a_fzzxpz1 + 2.0d0*ai_a_dxzpz0 - 2.0d0*ai_a_dxzpz1 + bi_a_fzzxs0 - bi_a_fzzxs1
a_fzzxdxy0 = PBy*a_fzzxpx0 - PCy*a_fzzxpx1
a_fzzxdxz0 = PBz*a_fzzxpx0 - PCz*a_fzzxpx1 + 2.0d0*ai_a_dxzpx0 - 2.0d0*ai_a_dxzpx1
ai_a_dxzpy0 = oo2zeta*a_dxzpy0
ai_a_dxzpy1 = oo2zeta*a_dxzpy1
a_fzzxdyz0 = PBz*a_fzzxpy0 - PCz*a_fzzxpy1 + 2.0d0*ai_a_dxzpy0 - 2.0d0*ai_a_dxzpy1
bi_a_fzzys0 = oo2zeta*a_fzzys0
bi_a_fzzys1 = oo2zeta*a_fzzys1
a_fzzydxx0 = PBx*a_fzzypx0 - PCx*a_fzzypx1 + bi_a_fzzys0 - bi_a_fzzys1
a_fzzydyy0 = PBy*a_fzzypy0 - PCy*a_fzzypy1 + ai_a_dzzpy0 - ai_a_dzzpy1 + bi_a_fzzys0 - bi_a_fzzys1
ai_a_dyzpz0 = oo2zeta*a_dyzpz0
ai_a_dyzpz1 = oo2zeta*a_dyzpz1
a_fzzydzz0 = PBz*a_fzzypz0 - PCz*a_fzzypz1 + 2.0d0*ai_a_dyzpz0 - 2.0d0*ai_a_dyzpz1 + bi_a_fzzys0 - bi_a_fzzys1
a_fzzydxy0 = PBy*a_fzzypx0 - PCy*a_fzzypx1 + ai_a_dzzpx0 - ai_a_dzzpx1
a_fzzydxz0 = PBz*a_fzzypx0 - PCz*a_fzzypx1 + 2.0d0*ai_a_dyzpx0 - 2.0d0*ai_a_dyzpx1
a_fzzydyz0 = PBz*a_fzzypy0 - PCz*a_fzzypy1 + 2.0d0*ai_a_dyzpy0 - 2.0d0*ai_a_dyzpy1
bi_a_fxyzs0 = oo2zeta*a_fxyzs0
bi_a_fxyzs1 = oo2zeta*a_fxyzs1
a_fxyzdxx0 = PBx*a_fxyzpx0 - PCx*a_fxyzpx1 + ai_a_dyzpx0 - ai_a_dyzpx1 + bi_a_fxyzs0 - bi_a_fxyzs1
a_fxyzdyy0 = PBy*a_fxyzpy0 - PCy*a_fxyzpy1 + ai_a_dxzpy0 - ai_a_dxzpy1 + bi_a_fxyzs0 - bi_a_fxyzs1
ai_a_dxypz0 = oo2zeta*a_dxypz0
ai_a_dxypz1 = oo2zeta*a_dxypz1
a_fxyzdzz0 = PBz*a_fxyzpz0 - PCz*a_fxyzpz1 + ai_a_dxypz0 - ai_a_dxypz1 + bi_a_fxyzs0 - bi_a_fxyzs1
a_fxyzdxy0 = PBy*a_fxyzpx0 - PCy*a_fxyzpx1 + ai_a_dxzpx0 - ai_a_dxzpx1
a_fxyzdxz0 = PBz*a_fxyzpx0 - PCz*a_fxyzpx1 + ai_a_dxypx0 - ai_a_dxypx1
a_fxyzdyz0 = PBz*a_fxyzpy0 - PCz*a_fxyzpy1 + ai_a_dxypy0 - ai_a_dxypy1
a_pxpx0 = PBx*a_pxs0 - PCx*a_pxs1 + ai_a_ss0 - ai_a_ss1
a_pxpy0 = PBy*a_pxs0 - PCy*a_pxs1
a_pxpz0 = PBz*a_pxs0 - PCz*a_pxs1
a_pypx0 = PBx*a_pys0 - PCx*a_pys1
a_pypy0 = PBy*a_pys0 - PCy*a_pys1 + ai_a_ss0 - ai_a_ss1
a_pypz0 = PBz*a_pys0 - PCz*a_pys1
a_pzpx0 = PBx*a_pzs0 - PCx*a_pzs1
a_pzpy0 = PBy*a_pzs0 - PCy*a_pzs1
a_pzpz0 = PBz*a_pzs0 - PCz*a_pzs1 + ai_a_ss0 - ai_a_ss1
a_pxpx1 = PBx*a_pxs1 - PCx*a_pxs2 + ai_a_ss1 - ai_a_ss2
a_pxpy1 = PBy*a_pxs1 - PCy*a_pxs2
a_pxpz1 = PBz*a_pxs1 - PCz*a_pxs2
a_pypx1 = PBx*a_pys1 - PCx*a_pys2
a_pypy1 = PBy*a_pys1 - PCy*a_pys2 + ai_a_ss1 - ai_a_ss2
a_pypz1 = PBz*a_pys1 - PCz*a_pys2
a_pzpx1 = PBx*a_pzs1 - PCx*a_pzs2
a_pzpy1 = PBy*a_pzs1 - PCy*a_pzs2
a_pzpz1 = PBz*a_pzs1 - PCz*a_pzs2 + ai_a_ss1 - ai_a_ss2
a_spx0 = PBx*a_ss0 - PCx*a_ss1
a_spy0 = PBy*a_ss0 - PCy*a_ss1
a_spz0 = PBz*a_ss0 - PCz*a_ss1
a_spx1 = PBx*a_ss1 - PCx*a_ss2
a_spy1 = PBy*a_ss1 - PCy*a_ss2
a_spz1 = PBz*a_ss1 - PCz*a_ss2
ai_a_spx0 = oo2zeta*a_spx0
ai_a_spx1 = oo2zeta*a_spx1
bi_a_pxs0 = oo2zeta*a_pxs0
bi_a_pxs1 = oo2zeta*a_pxs1
a_pxdxx0 = PBx*a_pxpx0 - PCx*a_pxpx1 + ai_a_spx0 - ai_a_spx1 + bi_a_pxs0 - bi_a_pxs1
a_pxdyy0 = PBy*a_pxpy0 - PCy*a_pxpy1 + bi_a_pxs0 - bi_a_pxs1
a_pxdzz0 = PBz*a_pxpz0 - PCz*a_pxpz1 + bi_a_pxs0 - bi_a_pxs1
a_pxdxy0 = PBy*a_pxpx0 - PCy*a_pxpx1
a_pxdxz0 = PBz*a_pxpx0 - PCz*a_pxpx1
a_pxdyz0 = PBz*a_pxpy0 - PCz*a_pxpy1
bi_a_pys0 = oo2zeta*a_pys0
bi_a_pys1 = oo2zeta*a_pys1
a_pydxx0 = PBx*a_pypx0 - PCx*a_pypx1 + bi_a_pys0 - bi_a_pys1
ai_a_spy0 = oo2zeta*a_spy0
ai_a_spy1 = oo2zeta*a_spy1
a_pydyy0 = PBy*a_pypy0 - PCy*a_pypy1 + ai_a_spy0 - ai_a_spy1 + bi_a_pys0 - bi_a_pys1
a_pydzz0 = PBz*a_pypz0 - PCz*a_pypz1 + bi_a_pys0 - bi_a_pys1
a_pydxy0 = PBy*a_pypx0 - PCy*a_pypx1 + ai_a_spx0 - ai_a_spx1
a_pydxz0 = PBz*a_pypx0 - PCz*a_pypx1
a_pydyz0 = PBz*a_pypy0 - PCz*a_pypy1
bi_a_pzs0 = oo2zeta*a_pzs0
bi_a_pzs1 = oo2zeta*a_pzs1
a_pzdxx0 = PBx*a_pzpx0 - PCx*a_pzpx1 + bi_a_pzs0 - bi_a_pzs1
a_pzdyy0 = PBy*a_pzpy0 - PCy*a_pzpy1 + bi_a_pzs0 - bi_a_pzs1
ai_a_spz0 = oo2zeta*a_spz0
ai_a_spz1 = oo2zeta*a_spz1
a_pzdzz0 = PBz*a_pzpz0 - PCz*a_pzpz1 + ai_a_spz0 - ai_a_spz1 + bi_a_pzs0 - bi_a_pzs1
a_pzdxy0 = PBy*a_pzpx0 - PCy*a_pzpx1
a_pzdxz0 = PBz*a_pzpx0 - PCz*a_pzpx1 + ai_a_spx0 - ai_a_spx1
a_pzdyz0 = PBz*a_pzpy0 - PCz*a_pzpy1 + ai_a_spy0 - ai_a_spy1
ai_a_pxpx0 = oo2zeta*a_pxpx0
ai_a_pxpx1 = oo2zeta*a_pxpx1
bi_a_dxxs0 = oo2zeta*a_dxxs0
bi_a_dxxs1 = oo2zeta*a_dxxs1
a_dxxdxx0 = PBx*a_dxxpx0 - PCx*a_dxxpx1 + 2.0d0*ai_a_pxpx0 - 2.0d0*ai_a_pxpx1 + bi_a_dxxs0 - bi_a_dxxs1
a_dxxdyy0 = PBy*a_dxxpy0 - PCy*a_dxxpy1 + bi_a_dxxs0 - bi_a_dxxs1
a_dxxdzz0 = PBz*a_dxxpz0 - PCz*a_dxxpz1 + bi_a_dxxs0 - bi_a_dxxs1
a_dxxdxy0 = PBy*a_dxxpx0 - PCy*a_dxxpx1
a_dxxdxz0 = PBz*a_dxxpx0 - PCz*a_dxxpx1
a_dxxdyz0 = PBz*a_dxxpy0 - PCz*a_dxxpy1
bi_a_dyys0 = oo2zeta*a_dyys0
bi_a_dyys1 = oo2zeta*a_dyys1
a_dyydxx0 = PBx*a_dyypx0 - PCx*a_dyypx1 + bi_a_dyys0 - bi_a_dyys1
ai_a_pypy0 = oo2zeta*a_pypy0
ai_a_pypy1 = oo2zeta*a_pypy1
a_dyydyy0 = PBy*a_dyypy0 - PCy*a_dyypy1 + 2.0d0*ai_a_pypy0 - 2.0d0*ai_a_pypy1 + bi_a_dyys0 - bi_a_dyys1
a_dyydzz0 = PBz*a_dyypz0 - PCz*a_dyypz1 + bi_a_dyys0 - bi_a_dyys1
ai_a_pypx0 = oo2zeta*a_pypx0
ai_a_pypx1 = oo2zeta*a_pypx1
a_dyydxy0 = PBy*a_dyypx0 - PCy*a_dyypx1 + 2.0d0*ai_a_pypx0 - 2.0d0*ai_a_pypx1
a_dyydxz0 = PBz*a_dyypx0 - PCz*a_dyypx1
a_dyydyz0 = PBz*a_dyypy0 - PCz*a_dyypy1
bi_a_dzzs0 = oo2zeta*a_dzzs0
bi_a_dzzs1 = oo2zeta*a_dzzs1
a_dzzdxx0 = PBx*a_dzzpx0 - PCx*a_dzzpx1 + bi_a_dzzs0 - bi_a_dzzs1
a_dzzdyy0 = PBy*a_dzzpy0 - PCy*a_dzzpy1 + bi_a_dzzs0 - bi_a_dzzs1
ai_a_pzpz0 = oo2zeta*a_pzpz0
ai_a_pzpz1 = oo2zeta*a_pzpz1
a_dzzdzz0 = PBz*a_dzzpz0 - PCz*a_dzzpz1 + 2.0d0*ai_a_pzpz0 - 2.0d0*ai_a_pzpz1 + bi_a_dzzs0 - bi_a_dzzs1
a_dzzdxy0 = PBy*a_dzzpx0 - PCy*a_dzzpx1
ai_a_pzpx0 = oo2zeta*a_pzpx0
ai_a_pzpx1 = oo2zeta*a_pzpx1
a_dzzdxz0 = PBz*a_dzzpx0 - PCz*a_dzzpx1 + 2.0d0*ai_a_pzpx0 - 2.0d0*ai_a_pzpx1
ai_a_pzpy0 = oo2zeta*a_pzpy0
ai_a_pzpy1 = oo2zeta*a_pzpy1
a_dzzdyz0 = PBz*a_dzzpy0 - PCz*a_dzzpy1 + 2.0d0*ai_a_pzpy0 - 2.0d0*ai_a_pzpy1
bi_a_dxys0 = oo2zeta*a_dxys0
bi_a_dxys1 = oo2zeta*a_dxys1
a_dxydxx0 = PBx*a_dxypx0 - PCx*a_dxypx1 + ai_a_pypx0 - ai_a_pypx1 + bi_a_dxys0 - bi_a_dxys1
ai_a_pxpy0 = oo2zeta*a_pxpy0
ai_a_pxpy1 = oo2zeta*a_pxpy1
a_dxydyy0 = PBy*a_dxypy0 - PCy*a_dxypy1 + ai_a_pxpy0 - ai_a_pxpy1 + bi_a_dxys0 - bi_a_dxys1
a_dxydzz0 = PBz*a_dxypz0 - PCz*a_dxypz1 + bi_a_dxys0 - bi_a_dxys1
a_dxydxy0 = PBy*a_dxypx0 - PCy*a_dxypx1 + ai_a_pxpx0 - ai_a_pxpx1
a_dxydxz0 = PBz*a_dxypx0 - PCz*a_dxypx1
a_dxydyz0 = PBz*a_dxypy0 - PCz*a_dxypy1
bi_a_dxzs0 = oo2zeta*a_dxzs0
bi_a_dxzs1 = oo2zeta*a_dxzs1
a_dxzdxx0 = PBx*a_dxzpx0 - PCx*a_dxzpx1 + ai_a_pzpx0 - ai_a_pzpx1 + bi_a_dxzs0 - bi_a_dxzs1
a_dxzdyy0 = PBy*a_dxzpy0 - PCy*a_dxzpy1 + bi_a_dxzs0 - bi_a_dxzs1
ai_a_pxpz0 = oo2zeta*a_pxpz0
ai_a_pxpz1 = oo2zeta*a_pxpz1
a_dxzdzz0 = PBz*a_dxzpz0 - PCz*a_dxzpz1 + ai_a_pxpz0 - ai_a_pxpz1 + bi_a_dxzs0 - bi_a_dxzs1
a_dxzdxy0 = PBy*a_dxzpx0 - PCy*a_dxzpx1
a_dxzdxz0 = PBz*a_dxzpx0 - PCz*a_dxzpx1 + ai_a_pxpx0 - ai_a_pxpx1
a_dxzdyz0 = PBz*a_dxzpy0 - PCz*a_dxzpy1 + ai_a_pxpy0 - ai_a_pxpy1
bi_a_dyzs0 = oo2zeta*a_dyzs0
bi_a_dyzs1 = oo2zeta*a_dyzs1
a_dyzdxx0 = PBx*a_dyzpx0 - PCx*a_dyzpx1 + bi_a_dyzs0 - bi_a_dyzs1
a_dyzdyy0 = PBy*a_dyzpy0 - PCy*a_dyzpy1 + ai_a_pzpy0 - ai_a_pzpy1 + bi_a_dyzs0 - bi_a_dyzs1
ai_a_pypz0 = oo2zeta*a_pypz0
ai_a_pypz1 = oo2zeta*a_pypz1
a_dyzdzz0 = PBz*a_dyzpz0 - PCz*a_dyzpz1 + ai_a_pypz0 - ai_a_pypz1 + bi_a_dyzs0 - bi_a_dyzs1
a_dyzdxy0 = PBy*a_dyzpx0 - PCy*a_dyzpx1 + ai_a_pzpx0 - ai_a_pzpx1
a_dyzdxz0 = PBz*a_dyzpx0 - PCz*a_dyzpx1 + ai_a_pypx0 - ai_a_pypx1
a_dyzdyz0 = PBz*a_dyzpy0 - PCz*a_dyzpy1 + ai_a_pypy0 - ai_a_pypy1
a_dxxpx2 = PBx*a_dxxs2 - PCx*a_dxxs3 + 2.0d0*ai_a_pxs2 - 2.0d0*ai_a_pxs3
a_dxxpy2 = PBy*a_dxxs2 - PCy*a_dxxs3
a_dxxpz2 = PBz*a_dxxs2 - PCz*a_dxxs3
a_dyypx2 = PBx*a_dyys2 - PCx*a_dyys3
a_dyypy2 = PBy*a_dyys2 - PCy*a_dyys3 + 2.0d0*ai_a_pys2 - 2.0d0*ai_a_pys3
a_dyypz2 = PBz*a_dyys2 - PCz*a_dyys3
a_dzzpx2 = PBx*a_dzzs2 - PCx*a_dzzs3
a_dzzpy2 = PBy*a_dzzs2 - PCy*a_dzzs3
a_dzzpz2 = PBz*a_dzzs2 - PCz*a_dzzs3 + 2.0d0*ai_a_pzs2 - 2.0d0*ai_a_pzs3
a_dxypx2 = PBx*a_dxys2 - PCx*a_dxys3 + ai_a_pys2 - ai_a_pys3
a_dxypy2 = PBy*a_dxys2 - PCy*a_dxys3 + ai_a_pxs2 - ai_a_pxs3
a_dxypz2 = PBz*a_dxys2 - PCz*a_dxys3
a_dxzpx2 = PBx*a_dxzs2 - PCx*a_dxzs3 + ai_a_pzs2 - ai_a_pzs3
a_dxzpy2 = PBy*a_dxzs2 - PCy*a_dxzs3
a_dxzpz2 = PBz*a_dxzs2 - PCz*a_dxzs3 + ai_a_pxs2 - ai_a_pxs3
a_dyzpx2 = PBx*a_dyzs2 - PCx*a_dyzs3
a_dyzpy2 = PBy*a_dyzs2 - PCy*a_dyzs3 + ai_a_pzs2 - ai_a_pzs3
a_dyzpz2 = PBz*a_dyzs2 - PCz*a_dyzs3 + ai_a_pys2 - ai_a_pys3
a_pxpx2 = PBx*a_pxs2 - PCx*a_pxs3 + ai_a_ss2 - ai_a_ss3
a_pxpy2 = PBy*a_pxs2 - PCy*a_pxs3
a_pxpz2 = PBz*a_pxs2 - PCz*a_pxs3
a_pypx2 = PBx*a_pys2 - PCx*a_pys3
a_pypy2 = PBy*a_pys2 - PCy*a_pys3 + ai_a_ss2 - ai_a_ss3
a_pypz2 = PBz*a_pys2 - PCz*a_pys3
a_pzpx2 = PBx*a_pzs2 - PCx*a_pzs3
a_pzpy2 = PBy*a_pzs2 - PCy*a_pzs3
a_pzpz2 = PBz*a_pzs2 - PCz*a_pzs3 + ai_a_ss2 - ai_a_ss3
ai_a_pxpx2 = oo2zeta*a_pxpx2
bi_a_dxxs2 = oo2zeta*a_dxxs2
a_dxxdxx1 = PBx*a_dxxpx1 - PCx*a_dxxpx2 + 2.0d0*ai_a_pxpx1 - 2.0d0*ai_a_pxpx2 + bi_a_dxxs1 - bi_a_dxxs2
a_dxxdyy1 = PBy*a_dxxpy1 - PCy*a_dxxpy2 + bi_a_dxxs1 - bi_a_dxxs2
a_dxxdzz1 = PBz*a_dxxpz1 - PCz*a_dxxpz2 + bi_a_dxxs1 - bi_a_dxxs2
a_dxxdxy1 = PBy*a_dxxpx1 - PCy*a_dxxpx2
a_dxxdxz1 = PBz*a_dxxpx1 - PCz*a_dxxpx2
a_dxxdyz1 = PBz*a_dxxpy1 - PCz*a_dxxpy2
bi_a_dyys2 = oo2zeta*a_dyys2
a_dyydxx1 = PBx*a_dyypx1 - PCx*a_dyypx2 + bi_a_dyys1 - bi_a_dyys2
ai_a_pypy2 = oo2zeta*a_pypy2
a_dyydyy1 = PBy*a_dyypy1 - PCy*a_dyypy2 + 2.0d0*ai_a_pypy1 - 2.0d0*ai_a_pypy2 + bi_a_dyys1 - bi_a_dyys2
a_dyydzz1 = PBz*a_dyypz1 - PCz*a_dyypz2 + bi_a_dyys1 - bi_a_dyys2
ai_a_pypx2 = oo2zeta*a_pypx2
a_dyydxy1 = PBy*a_dyypx1 - PCy*a_dyypx2 + 2.0d0*ai_a_pypx1 - 2.0d0*ai_a_pypx2
a_dyydxz1 = PBz*a_dyypx1 - PCz*a_dyypx2
a_dyydyz1 = PBz*a_dyypy1 - PCz*a_dyypy2
bi_a_dzzs2 = oo2zeta*a_dzzs2
a_dzzdxx1 = PBx*a_dzzpx1 - PCx*a_dzzpx2 + bi_a_dzzs1 - bi_a_dzzs2
a_dzzdyy1 = PBy*a_dzzpy1 - PCy*a_dzzpy2 + bi_a_dzzs1 - bi_a_dzzs2
ai_a_pzpz2 = oo2zeta*a_pzpz2
a_dzzdzz1 = PBz*a_dzzpz1 - PCz*a_dzzpz2 + 2.0d0*ai_a_pzpz1 - 2.0d0*ai_a_pzpz2 + bi_a_dzzs1 - bi_a_dzzs2
a_dzzdxy1 = PBy*a_dzzpx1 - PCy*a_dzzpx2
ai_a_pzpx2 = oo2zeta*a_pzpx2
a_dzzdxz1 = PBz*a_dzzpx1 - PCz*a_dzzpx2 + 2.0d0*ai_a_pzpx1 - 2.0d0*ai_a_pzpx2
ai_a_pzpy2 = oo2zeta*a_pzpy2
a_dzzdyz1 = PBz*a_dzzpy1 - PCz*a_dzzpy2 + 2.0d0*ai_a_pzpy1 - 2.0d0*ai_a_pzpy2
bi_a_dxys2 = oo2zeta*a_dxys2
a_dxydxx1 = PBx*a_dxypx1 - PCx*a_dxypx2 + ai_a_pypx1 - ai_a_pypx2 + bi_a_dxys1 - bi_a_dxys2
ai_a_pxpy2 = oo2zeta*a_pxpy2
a_dxydyy1 = PBy*a_dxypy1 - PCy*a_dxypy2 + ai_a_pxpy1 - ai_a_pxpy2 + bi_a_dxys1 - bi_a_dxys2
a_dxydzz1 = PBz*a_dxypz1 - PCz*a_dxypz2 + bi_a_dxys1 - bi_a_dxys2
a_dxydxy1 = PBy*a_dxypx1 - PCy*a_dxypx2 + ai_a_pxpx1 - ai_a_pxpx2
a_dxydxz1 = PBz*a_dxypx1 - PCz*a_dxypx2
a_dxydyz1 = PBz*a_dxypy1 - PCz*a_dxypy2
bi_a_dxzs2 = oo2zeta*a_dxzs2
a_dxzdxx1 = PBx*a_dxzpx1 - PCx*a_dxzpx2 + ai_a_pzpx1 - ai_a_pzpx2 + bi_a_dxzs1 - bi_a_dxzs2
a_dxzdyy1 = PBy*a_dxzpy1 - PCy*a_dxzpy2 + bi_a_dxzs1 - bi_a_dxzs2
ai_a_pxpz2 = oo2zeta*a_pxpz2
a_dxzdzz1 = PBz*a_dxzpz1 - PCz*a_dxzpz2 + ai_a_pxpz1 - ai_a_pxpz2 + bi_a_dxzs1 - bi_a_dxzs2
a_dxzdxy1 = PBy*a_dxzpx1 - PCy*a_dxzpx2
a_dxzdxz1 = PBz*a_dxzpx1 - PCz*a_dxzpx2 + ai_a_pxpx1 - ai_a_pxpx2
a_dxzdyz1 = PBz*a_dxzpy1 - PCz*a_dxzpy2 + ai_a_pxpy1 - ai_a_pxpy2
bi_a_dyzs2 = oo2zeta*a_dyzs2
a_dyzdxx1 = PBx*a_dyzpx1 - PCx*a_dyzpx2 + bi_a_dyzs1 - bi_a_dyzs2
a_dyzdyy1 = PBy*a_dyzpy1 - PCy*a_dyzpy2 + ai_a_pzpy1 - ai_a_pzpy2 + bi_a_dyzs1 - bi_a_dyzs2
ai_a_pypz2 = oo2zeta*a_pypz2
a_dyzdzz1 = PBz*a_dyzpz1 - PCz*a_dyzpz2 + ai_a_pypz1 - ai_a_pypz2 + bi_a_dyzs1 - bi_a_dyzs2
a_dyzdxy1 = PBy*a_dyzpx1 - PCy*a_dyzpx2 + ai_a_pzpx1 - ai_a_pzpx2
a_dyzdxz1 = PBz*a_dyzpx1 - PCz*a_dyzpx2 + ai_a_pypx1 - ai_a_pypx2
a_dyzdyz1 = PBz*a_dyzpy1 - PCz*a_dyzpy2 + ai_a_pypy1 - ai_a_pypy2
a_spx2 = PBx*a_ss2 - PCx*a_ss3
a_spy2 = PBy*a_ss2 - PCy*a_ss3
a_spz2 = PBz*a_ss2 - PCz*a_ss3
ai_a_spx2 = oo2zeta*a_spx2
bi_a_pxs2 = oo2zeta*a_pxs2
a_pxdxx1 = PBx*a_pxpx1 - PCx*a_pxpx2 + ai_a_spx1 - ai_a_spx2 + bi_a_pxs1 - bi_a_pxs2
a_pxdyy1 = PBy*a_pxpy1 - PCy*a_pxpy2 + bi_a_pxs1 - bi_a_pxs2
a_pxdzz1 = PBz*a_pxpz1 - PCz*a_pxpz2 + bi_a_pxs1 - bi_a_pxs2
a_pxdxy1 = PBy*a_pxpx1 - PCy*a_pxpx2
a_pxdxz1 = PBz*a_pxpx1 - PCz*a_pxpx2
a_pxdyz1 = PBz*a_pxpy1 - PCz*a_pxpy2
bi_a_pys2 = oo2zeta*a_pys2
a_pydxx1 = PBx*a_pypx1 - PCx*a_pypx2 + bi_a_pys1 - bi_a_pys2
ai_a_spy2 = oo2zeta*a_spy2
a_pydyy1 = PBy*a_pypy1 - PCy*a_pypy2 + ai_a_spy1 - ai_a_spy2 + bi_a_pys1 - bi_a_pys2
a_pydzz1 = PBz*a_pypz1 - PCz*a_pypz2 + bi_a_pys1 - bi_a_pys2
a_pydxy1 = PBy*a_pypx1 - PCy*a_pypx2 + ai_a_spx1 - ai_a_spx2
a_pydxz1 = PBz*a_pypx1 - PCz*a_pypx2
a_pydyz1 = PBz*a_pypy1 - PCz*a_pypy2
bi_a_pzs2 = oo2zeta*a_pzs2
a_pzdxx1 = PBx*a_pzpx1 - PCx*a_pzpx2 + bi_a_pzs1 - bi_a_pzs2
a_pzdyy1 = PBy*a_pzpy1 - PCy*a_pzpy2 + bi_a_pzs1 - bi_a_pzs2
ai_a_spz2 = oo2zeta*a_spz2
a_pzdzz1 = PBz*a_pzpz1 - PCz*a_pzpz2 + ai_a_spz1 - ai_a_spz2 + bi_a_pzs1 - bi_a_pzs2
a_pzdxy1 = PBy*a_pzpx1 - PCy*a_pzpx2
a_pzdxz1 = PBz*a_pzpx1 - PCz*a_pzpx2 + ai_a_spx1 - ai_a_spx2
a_pzdyz1 = PBz*a_pzpy1 - PCz*a_pzpy2 + ai_a_spy1 - ai_a_spy2
ai_a_pxdxx0 = oo2zeta*a_pxdxx0
ai_a_pxdxx1 = oo2zeta*a_pxdxx1
bi_a_dxxpx0 = oo2zeta*a_dxxpx0
bi_a_dxxpx1 = oo2zeta*a_dxxpx1
a_dxxfxxx0 = PBx*a_dxxdxx0 - PCx*a_dxxdxx1 + 2.0d0*ai_a_pxdxx0 - 2.0d0*ai_a_pxdxx1 + 2.0d0*bi_a_dxxpx0 - 2.0d0*bi_a_dxxpx1
bi_a_dxxpy0 = oo2zeta*a_dxxpy0
bi_a_dxxpy1 = oo2zeta*a_dxxpy1
a_dxxfyyy0 = PBy*a_dxxdyy0 - PCy*a_dxxdyy1 + 2.0d0*bi_a_dxxpy0 - 2.0d0*bi_a_dxxpy1
bi_a_dxxpz0 = oo2zeta*a_dxxpz0
bi_a_dxxpz1 = oo2zeta*a_dxxpz1
a_dxxfzzz0 = PBz*a_dxxdzz0 - PCz*a_dxxdzz1 + 2.0d0*bi_a_dxxpz0 - 2.0d0*bi_a_dxxpz1
a_dxxfxxy0 = PBy*a_dxxdxx0 - PCy*a_dxxdxx1
a_dxxfxxz0 = PBz*a_dxxdxx0 - PCz*a_dxxdxx1
ai_a_pxdyy0 = oo2zeta*a_pxdyy0
ai_a_pxdyy1 = oo2zeta*a_pxdyy1
a_dxxfyyx0 = PBx*a_dxxdyy0 - PCx*a_dxxdyy1 + 2.0d0*ai_a_pxdyy0 - 2.0d0*ai_a_pxdyy1
a_dxxfyyz0 = PBz*a_dxxdyy0 - PCz*a_dxxdyy1
ai_a_pxdzz0 = oo2zeta*a_pxdzz0
ai_a_pxdzz1 = oo2zeta*a_pxdzz1
a_dxxfzzx0 = PBx*a_dxxdzz0 - PCx*a_dxxdzz1 + 2.0d0*ai_a_pxdzz0 - 2.0d0*ai_a_pxdzz1
a_dxxfzzy0 = PBy*a_dxxdzz0 - PCy*a_dxxdzz1
a_dxxfxyz0 = PBz*a_dxxdxy0 - PCz*a_dxxdxy1
bi_a_dyypx0 = oo2zeta*a_dyypx0
bi_a_dyypx1 = oo2zeta*a_dyypx1
a_dyyfxxx0 = PBx*a_dyydxx0 - PCx*a_dyydxx1 + 2.0d0*bi_a_dyypx0 - 2.0d0*bi_a_dyypx1
ai_a_pydyy0 = oo2zeta*a_pydyy0
ai_a_pydyy1 = oo2zeta*a_pydyy1
bi_a_dyypy0 = oo2zeta*a_dyypy0
bi_a_dyypy1 = oo2zeta*a_dyypy1
a_dyyfyyy0 = PBy*a_dyydyy0 - PCy*a_dyydyy1 + 2.0d0*ai_a_pydyy0 - 2.0d0*ai_a_pydyy1 + 2.0d0*bi_a_dyypy0 - 2.0d0*bi_a_dyypy1
bi_a_dyypz0 = oo2zeta*a_dyypz0
bi_a_dyypz1 = oo2zeta*a_dyypz1
a_dyyfzzz0 = PBz*a_dyydzz0 - PCz*a_dyydzz1 + 2.0d0*bi_a_dyypz0 - 2.0d0*bi_a_dyypz1
ai_a_pydxx0 = oo2zeta*a_pydxx0
ai_a_pydxx1 = oo2zeta*a_pydxx1
a_dyyfxxy0 = PBy*a_dyydxx0 - PCy*a_dyydxx1 + 2.0d0*ai_a_pydxx0 - 2.0d0*ai_a_pydxx1
a_dyyfxxz0 = PBz*a_dyydxx0 - PCz*a_dyydxx1
a_dyyfyyx0 = PBx*a_dyydyy0 - PCx*a_dyydyy1
a_dyyfyyz0 = PBz*a_dyydyy0 - PCz*a_dyydyy1
a_dyyfzzx0 = PBx*a_dyydzz0 - PCx*a_dyydzz1
ai_a_pydzz0 = oo2zeta*a_pydzz0
ai_a_pydzz1 = oo2zeta*a_pydzz1
a_dyyfzzy0 = PBy*a_dyydzz0 - PCy*a_dyydzz1 + 2.0d0*ai_a_pydzz0 - 2.0d0*ai_a_pydzz1
a_dyyfxyz0 = PBz*a_dyydxy0 - PCz*a_dyydxy1
bi_a_dzzpx0 = oo2zeta*a_dzzpx0
bi_a_dzzpx1 = oo2zeta*a_dzzpx1
a_dzzfxxx0 = PBx*a_dzzdxx0 - PCx*a_dzzdxx1 + 2.0d0*bi_a_dzzpx0 - 2.0d0*bi_a_dzzpx1
bi_a_dzzpy0 = oo2zeta*a_dzzpy0
bi_a_dzzpy1 = oo2zeta*a_dzzpy1
a_dzzfyyy0 = PBy*a_dzzdyy0 - PCy*a_dzzdyy1 + 2.0d0*bi_a_dzzpy0 - 2.0d0*bi_a_dzzpy1
ai_a_pzdzz0 = oo2zeta*a_pzdzz0
ai_a_pzdzz1 = oo2zeta*a_pzdzz1
bi_a_dzzpz0 = oo2zeta*a_dzzpz0
bi_a_dzzpz1 = oo2zeta*a_dzzpz1
a_dzzfzzz0 = PBz*a_dzzdzz0 - PCz*a_dzzdzz1 + 2.0d0*ai_a_pzdzz0 - 2.0d0*ai_a_pzdzz1 + 2.0d0*bi_a_dzzpz0 - 2.0d0*bi_a_dzzpz1
a_dzzfxxy0 = PBy*a_dzzdxx0 - PCy*a_dzzdxx1
ai_a_pzdxx0 = oo2zeta*a_pzdxx0
ai_a_pzdxx1 = oo2zeta*a_pzdxx1
a_dzzfxxz0 = PBz*a_dzzdxx0 - PCz*a_dzzdxx1 + 2.0d0*ai_a_pzdxx0 - 2.0d0*ai_a_pzdxx1
a_dzzfyyx0 = PBx*a_dzzdyy0 - PCx*a_dzzdyy1
ai_a_pzdyy0 = oo2zeta*a_pzdyy0
ai_a_pzdyy1 = oo2zeta*a_pzdyy1
a_dzzfyyz0 = PBz*a_dzzdyy0 - PCz*a_dzzdyy1 + 2.0d0*ai_a_pzdyy0 - 2.0d0*ai_a_pzdyy1
a_dzzfzzx0 = PBx*a_dzzdzz0 - PCx*a_dzzdzz1
a_dzzfzzy0 = PBy*a_dzzdzz0 - PCy*a_dzzdzz1
ai_a_pzdxy0 = oo2zeta*a_pzdxy0
ai_a_pzdxy1 = oo2zeta*a_pzdxy1
a_dzzfxyz0 = PBz*a_dzzdxy0 - PCz*a_dzzdxy1 + 2.0d0*ai_a_pzdxy0 - 2.0d0*ai_a_pzdxy1
bi_a_dxypx0 = oo2zeta*a_dxypx0
bi_a_dxypx1 = oo2zeta*a_dxypx1
a_dxyfxxx0 = PBx*a_dxydxx0 - PCx*a_dxydxx1 + ai_a_pydxx0 - ai_a_pydxx1 + 2.0d0*bi_a_dxypx0 - 2.0d0*bi_a_dxypx1
bi_a_dxypy0 = oo2zeta*a_dxypy0
bi_a_dxypy1 = oo2zeta*a_dxypy1
a_dxyfyyy0 = PBy*a_dxydyy0 - PCy*a_dxydyy1 + ai_a_pxdyy0 - ai_a_pxdyy1 + 2.0d0*bi_a_dxypy0 - 2.0d0*bi_a_dxypy1
bi_a_dxypz0 = oo2zeta*a_dxypz0
bi_a_dxypz1 = oo2zeta*a_dxypz1
a_dxyfzzz0 = PBz*a_dxydzz0 - PCz*a_dxydzz1 + 2.0d0*bi_a_dxypz0 - 2.0d0*bi_a_dxypz1
a_dxyfxxy0 = PBy*a_dxydxx0 - PCy*a_dxydxx1 + ai_a_pxdxx0 - ai_a_pxdxx1
a_dxyfxxz0 = PBz*a_dxydxx0 - PCz*a_dxydxx1
a_dxyfyyx0 = PBx*a_dxydyy0 - PCx*a_dxydyy1 + ai_a_pydyy0 - ai_a_pydyy1
a_dxyfyyz0 = PBz*a_dxydyy0 - PCz*a_dxydyy1
a_dxyfzzx0 = PBx*a_dxydzz0 - PCx*a_dxydzz1 + ai_a_pydzz0 - ai_a_pydzz1
a_dxyfzzy0 = PBy*a_dxydzz0 - PCy*a_dxydzz1 + ai_a_pxdzz0 - ai_a_pxdzz1
a_dxyfxyz0 = PBz*a_dxydxy0 - PCz*a_dxydxy1
bi_a_dxzpx0 = oo2zeta*a_dxzpx0
bi_a_dxzpx1 = oo2zeta*a_dxzpx1
a_dxzfxxx0 = PBx*a_dxzdxx0 - PCx*a_dxzdxx1 + ai_a_pzdxx0 - ai_a_pzdxx1 + 2.0d0*bi_a_dxzpx0 - 2.0d0*bi_a_dxzpx1
bi_a_dxzpy0 = oo2zeta*a_dxzpy0
bi_a_dxzpy1 = oo2zeta*a_dxzpy1
a_dxzfyyy0 = PBy*a_dxzdyy0 - PCy*a_dxzdyy1 + 2.0d0*bi_a_dxzpy0 - 2.0d0*bi_a_dxzpy1
bi_a_dxzpz0 = oo2zeta*a_dxzpz0
bi_a_dxzpz1 = oo2zeta*a_dxzpz1
a_dxzfzzz0 = PBz*a_dxzdzz0 - PCz*a_dxzdzz1 + ai_a_pxdzz0 - ai_a_pxdzz1 + 2.0d0*bi_a_dxzpz0 - 2.0d0*bi_a_dxzpz1
a_dxzfxxy0 = PBy*a_dxzdxx0 - PCy*a_dxzdxx1
a_dxzfxxz0 = PBz*a_dxzdxx0 - PCz*a_dxzdxx1 + ai_a_pxdxx0 - ai_a_pxdxx1
a_dxzfyyx0 = PBx*a_dxzdyy0 - PCx*a_dxzdyy1 + ai_a_pzdyy0 - ai_a_pzdyy1
a_dxzfyyz0 = PBz*a_dxzdyy0 - PCz*a_dxzdyy1 + ai_a_pxdyy0 - ai_a_pxdyy1
a_dxzfzzx0 = PBx*a_dxzdzz0 - PCx*a_dxzdzz1 + ai_a_pzdzz0 - ai_a_pzdzz1
a_dxzfzzy0 = PBy*a_dxzdzz0 - PCy*a_dxzdzz1
ai_a_pxdxy0 = oo2zeta*a_pxdxy0
ai_a_pxdxy1 = oo2zeta*a_pxdxy1
a_dxzfxyz0 = PBz*a_dxzdxy0 - PCz*a_dxzdxy1 + ai_a_pxdxy0 - ai_a_pxdxy1
bi_a_dyzpx0 = oo2zeta*a_dyzpx0
bi_a_dyzpx1 = oo2zeta*a_dyzpx1
a_dyzfxxx0 = PBx*a_dyzdxx0 - PCx*a_dyzdxx1 + 2.0d0*bi_a_dyzpx0 - 2.0d0*bi_a_dyzpx1
bi_a_dyzpy0 = oo2zeta*a_dyzpy0
bi_a_dyzpy1 = oo2zeta*a_dyzpy1
a_dyzfyyy0 = PBy*a_dyzdyy0 - PCy*a_dyzdyy1 + ai_a_pzdyy0 - ai_a_pzdyy1 + 2.0d0*bi_a_dyzpy0 - 2.0d0*bi_a_dyzpy1
bi_a_dyzpz0 = oo2zeta*a_dyzpz0
bi_a_dyzpz1 = oo2zeta*a_dyzpz1
a_dyzfzzz0 = PBz*a_dyzdzz0 - PCz*a_dyzdzz1 + ai_a_pydzz0 - ai_a_pydzz1 + 2.0d0*bi_a_dyzpz0 - 2.0d0*bi_a_dyzpz1
a_dyzfxxy0 = PBy*a_dyzdxx0 - PCy*a_dyzdxx1 + ai_a_pzdxx0 - ai_a_pzdxx1
a_dyzfxxz0 = PBz*a_dyzdxx0 - PCz*a_dyzdxx1 + ai_a_pydxx0 - ai_a_pydxx1
a_dyzfyyx0 = PBx*a_dyzdyy0 - PCx*a_dyzdyy1
a_dyzfyyz0 = PBz*a_dyzdyy0 - PCz*a_dyzdyy1 + ai_a_pydyy0 - ai_a_pydyy1
a_dyzfzzx0 = PBx*a_dyzdzz0 - PCx*a_dyzdzz1
a_dyzfzzy0 = PBy*a_dyzdzz0 - PCy*a_dyzdzz1 + ai_a_pzdzz0 - ai_a_pzdzz1
ai_a_pydxy0 = oo2zeta*a_pydxy0
ai_a_pydxy1 = oo2zeta*a_pydxy1
a_dyzfxyz0 = PBz*a_dyzdxy0 - PCz*a_dyzdxy1 + ai_a_pydxy0 - ai_a_pydxy1
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
a_dxxdxx_dAx = a_dxxdxx_dAx + alpha2*a_fxxxdxx0 - 2.0d0*a_pxdxx0
a_dxxdyy_dAx = a_dxxdyy_dAx + alpha2*a_fxxxdyy0 - 2.0d0*a_pxdyy0
a_dxxdzz_dAx = a_dxxdzz_dAx + alpha2*a_fxxxdzz0 - 2.0d0*a_pxdzz0
a_dxxdxy_dAx = a_dxxdxy_dAx + alpha2*a_fxxxdxy0 - 2.0d0*a_pxdxy0
a_dxxdxz_dAx = a_dxxdxz_dAx + alpha2*a_fxxxdxz0 - 2.0d0*a_pxdxz0
a_dxxdyz_dAx = a_dxxdyz_dAx + alpha2*a_fxxxdyz0 - 2.0d0*a_pxdyz0
a_dyydxx_dAx = a_dyydxx_dAx + alpha2*a_fyyxdxx0
a_dyydyy_dAx = a_dyydyy_dAx + alpha2*a_fyyxdyy0
a_dyydzz_dAx = a_dyydzz_dAx + alpha2*a_fyyxdzz0
a_dyydxy_dAx = a_dyydxy_dAx + alpha2*a_fyyxdxy0
a_dyydxz_dAx = a_dyydxz_dAx + alpha2*a_fyyxdxz0
a_dyydyz_dAx = a_dyydyz_dAx + alpha2*a_fyyxdyz0
a_dzzdxx_dAx = a_dzzdxx_dAx + alpha2*a_fzzxdxx0
a_dzzdyy_dAx = a_dzzdyy_dAx + alpha2*a_fzzxdyy0
a_dzzdzz_dAx = a_dzzdzz_dAx + alpha2*a_fzzxdzz0
a_dzzdxy_dAx = a_dzzdxy_dAx + alpha2*a_fzzxdxy0
a_dzzdxz_dAx = a_dzzdxz_dAx + alpha2*a_fzzxdxz0
a_dzzdyz_dAx = a_dzzdyz_dAx + alpha2*a_fzzxdyz0
a_dxydxx_dAx = a_dxydxx_dAx + alpha2*a_fxxydxx0 - a_pydxx0
a_dxydyy_dAx = a_dxydyy_dAx + alpha2*a_fxxydyy0 - a_pydyy0
a_dxydzz_dAx = a_dxydzz_dAx + alpha2*a_fxxydzz0 - a_pydzz0
a_dxydxy_dAx = a_dxydxy_dAx + alpha2*a_fxxydxy0 - a_pydxy0
a_dxydxz_dAx = a_dxydxz_dAx + alpha2*a_fxxydxz0 - a_pydxz0
a_dxydyz_dAx = a_dxydyz_dAx + alpha2*a_fxxydyz0 - a_pydyz0
a_dxzdxx_dAx = a_dxzdxx_dAx + alpha2*a_fxxzdxx0 - a_pzdxx0
a_dxzdyy_dAx = a_dxzdyy_dAx + alpha2*a_fxxzdyy0 - a_pzdyy0
a_dxzdzz_dAx = a_dxzdzz_dAx + alpha2*a_fxxzdzz0 - a_pzdzz0
a_dxzdxy_dAx = a_dxzdxy_dAx + alpha2*a_fxxzdxy0 - a_pzdxy0
a_dxzdxz_dAx = a_dxzdxz_dAx + alpha2*a_fxxzdxz0 - a_pzdxz0
a_dxzdyz_dAx = a_dxzdyz_dAx + alpha2*a_fxxzdyz0 - a_pzdyz0
a_dyzdxx_dAx = a_dyzdxx_dAx + alpha2*a_fxyzdxx0
a_dyzdyy_dAx = a_dyzdyy_dAx + alpha2*a_fxyzdyy0
a_dyzdzz_dAx = a_dyzdzz_dAx + alpha2*a_fxyzdzz0
a_dyzdxy_dAx = a_dyzdxy_dAx + alpha2*a_fxyzdxy0
a_dyzdxz_dAx = a_dyzdxz_dAx + alpha2*a_fxyzdxz0
a_dyzdyz_dAx = a_dyzdyz_dAx + alpha2*a_fxyzdyz0
a_dxxdxx_dAy = a_dxxdxx_dAy + alpha2*a_fxxydxx0
a_dxxdyy_dAy = a_dxxdyy_dAy + alpha2*a_fxxydyy0
a_dxxdzz_dAy = a_dxxdzz_dAy + alpha2*a_fxxydzz0
a_dxxdxy_dAy = a_dxxdxy_dAy + alpha2*a_fxxydxy0
a_dxxdxz_dAy = a_dxxdxz_dAy + alpha2*a_fxxydxz0
a_dxxdyz_dAy = a_dxxdyz_dAy + alpha2*a_fxxydyz0
a_dyydxx_dAy = a_dyydxx_dAy + alpha2*a_fyyydxx0 - 2.0d0*a_pydxx0
a_dyydyy_dAy = a_dyydyy_dAy + alpha2*a_fyyydyy0 - 2.0d0*a_pydyy0
a_dyydzz_dAy = a_dyydzz_dAy + alpha2*a_fyyydzz0 - 2.0d0*a_pydzz0
a_dyydxy_dAy = a_dyydxy_dAy + alpha2*a_fyyydxy0 - 2.0d0*a_pydxy0
a_dyydxz_dAy = a_dyydxz_dAy + alpha2*a_fyyydxz0 - 2.0d0*a_pydxz0
a_dyydyz_dAy = a_dyydyz_dAy + alpha2*a_fyyydyz0 - 2.0d0*a_pydyz0
a_dzzdxx_dAy = a_dzzdxx_dAy + alpha2*a_fzzydxx0
a_dzzdyy_dAy = a_dzzdyy_dAy + alpha2*a_fzzydyy0
a_dzzdzz_dAy = a_dzzdzz_dAy + alpha2*a_fzzydzz0
a_dzzdxy_dAy = a_dzzdxy_dAy + alpha2*a_fzzydxy0
a_dzzdxz_dAy = a_dzzdxz_dAy + alpha2*a_fzzydxz0
a_dzzdyz_dAy = a_dzzdyz_dAy + alpha2*a_fzzydyz0
a_dxydxx_dAy = a_dxydxx_dAy + alpha2*a_fyyxdxx0 - a_pxdxx0
a_dxydyy_dAy = a_dxydyy_dAy + alpha2*a_fyyxdyy0 - a_pxdyy0
a_dxydzz_dAy = a_dxydzz_dAy + alpha2*a_fyyxdzz0 - a_pxdzz0
a_dxydxy_dAy = a_dxydxy_dAy + alpha2*a_fyyxdxy0 - a_pxdxy0
a_dxydxz_dAy = a_dxydxz_dAy + alpha2*a_fyyxdxz0 - a_pxdxz0
a_dxydyz_dAy = a_dxydyz_dAy + alpha2*a_fyyxdyz0 - a_pxdyz0
a_dxzdxx_dAy = a_dxzdxx_dAy + alpha2*a_fxyzdxx0
a_dxzdyy_dAy = a_dxzdyy_dAy + alpha2*a_fxyzdyy0
a_dxzdzz_dAy = a_dxzdzz_dAy + alpha2*a_fxyzdzz0
a_dxzdxy_dAy = a_dxzdxy_dAy + alpha2*a_fxyzdxy0
a_dxzdxz_dAy = a_dxzdxz_dAy + alpha2*a_fxyzdxz0
a_dxzdyz_dAy = a_dxzdyz_dAy + alpha2*a_fxyzdyz0
a_dyzdxx_dAy = a_dyzdxx_dAy + alpha2*a_fyyzdxx0 - a_pzdxx0
a_dyzdyy_dAy = a_dyzdyy_dAy + alpha2*a_fyyzdyy0 - a_pzdyy0
a_dyzdzz_dAy = a_dyzdzz_dAy + alpha2*a_fyyzdzz0 - a_pzdzz0
a_dyzdxy_dAy = a_dyzdxy_dAy + alpha2*a_fyyzdxy0 - a_pzdxy0
a_dyzdxz_dAy = a_dyzdxz_dAy + alpha2*a_fyyzdxz0 - a_pzdxz0
a_dyzdyz_dAy = a_dyzdyz_dAy + alpha2*a_fyyzdyz0 - a_pzdyz0
a_dxxdxx_dAz = a_dxxdxx_dAz + alpha2*a_fxxzdxx0
a_dxxdyy_dAz = a_dxxdyy_dAz + alpha2*a_fxxzdyy0
a_dxxdzz_dAz = a_dxxdzz_dAz + alpha2*a_fxxzdzz0
a_dxxdxy_dAz = a_dxxdxy_dAz + alpha2*a_fxxzdxy0
a_dxxdxz_dAz = a_dxxdxz_dAz + alpha2*a_fxxzdxz0
a_dxxdyz_dAz = a_dxxdyz_dAz + alpha2*a_fxxzdyz0
a_dyydxx_dAz = a_dyydxx_dAz + alpha2*a_fyyzdxx0
a_dyydyy_dAz = a_dyydyy_dAz + alpha2*a_fyyzdyy0
a_dyydzz_dAz = a_dyydzz_dAz + alpha2*a_fyyzdzz0
a_dyydxy_dAz = a_dyydxy_dAz + alpha2*a_fyyzdxy0
a_dyydxz_dAz = a_dyydxz_dAz + alpha2*a_fyyzdxz0
a_dyydyz_dAz = a_dyydyz_dAz + alpha2*a_fyyzdyz0
a_dzzdxx_dAz = a_dzzdxx_dAz + alpha2*a_fzzzdxx0 - 2.0d0*a_pzdxx0
a_dzzdyy_dAz = a_dzzdyy_dAz + alpha2*a_fzzzdyy0 - 2.0d0*a_pzdyy0
a_dzzdzz_dAz = a_dzzdzz_dAz + alpha2*a_fzzzdzz0 - 2.0d0*a_pzdzz0
a_dzzdxy_dAz = a_dzzdxy_dAz + alpha2*a_fzzzdxy0 - 2.0d0*a_pzdxy0
a_dzzdxz_dAz = a_dzzdxz_dAz + alpha2*a_fzzzdxz0 - 2.0d0*a_pzdxz0
a_dzzdyz_dAz = a_dzzdyz_dAz + alpha2*a_fzzzdyz0 - 2.0d0*a_pzdyz0
a_dxydxx_dAz = a_dxydxx_dAz + alpha2*a_fxyzdxx0
a_dxydyy_dAz = a_dxydyy_dAz + alpha2*a_fxyzdyy0
a_dxydzz_dAz = a_dxydzz_dAz + alpha2*a_fxyzdzz0
a_dxydxy_dAz = a_dxydxy_dAz + alpha2*a_fxyzdxy0
a_dxydxz_dAz = a_dxydxz_dAz + alpha2*a_fxyzdxz0
a_dxydyz_dAz = a_dxydyz_dAz + alpha2*a_fxyzdyz0
a_dxzdxx_dAz = a_dxzdxx_dAz + alpha2*a_fzzxdxx0 - a_pxdxx0
a_dxzdyy_dAz = a_dxzdyy_dAz + alpha2*a_fzzxdyy0 - a_pxdyy0
a_dxzdzz_dAz = a_dxzdzz_dAz + alpha2*a_fzzxdzz0 - a_pxdzz0
a_dxzdxy_dAz = a_dxzdxy_dAz + alpha2*a_fzzxdxy0 - a_pxdxy0
a_dxzdxz_dAz = a_dxzdxz_dAz + alpha2*a_fzzxdxz0 - a_pxdxz0
a_dxzdyz_dAz = a_dxzdyz_dAz + alpha2*a_fzzxdyz0 - a_pxdyz0
a_dyzdxx_dAz = a_dyzdxx_dAz + alpha2*a_fzzydxx0 - a_pydxx0
a_dyzdyy_dAz = a_dyzdyy_dAz + alpha2*a_fzzydyy0 - a_pydyy0
a_dyzdzz_dAz = a_dyzdzz_dAz + alpha2*a_fzzydzz0 - a_pydzz0
a_dyzdxy_dAz = a_dyzdxy_dAz + alpha2*a_fzzydxy0 - a_pydxy0
a_dyzdxz_dAz = a_dyzdxz_dAz + alpha2*a_fzzydxz0 - a_pydxz0
a_dyzdyz_dAz = a_dyzdyz_dAz + alpha2*a_fzzydyz0 - a_pydyz0
beta2 = 2.0d0*beta
a_dxxdxx_dBx = a_dxxdxx_dBx + beta2*a_dxxfxxx0 - 2.0d0*a_dxxpx0
a_dxxdyy_dBx = a_dxxdyy_dBx + beta2*a_dxxfyyx0
a_dxxdzz_dBx = a_dxxdzz_dBx + beta2*a_dxxfzzx0
a_dxxdxy_dBx = a_dxxdxy_dBx + beta2*a_dxxfxxy0 - a_dxxpy0
a_dxxdxz_dBx = a_dxxdxz_dBx + beta2*a_dxxfxxz0 - a_dxxpz0
a_dxxdyz_dBx = a_dxxdyz_dBx + beta2*a_dxxfxyz0
a_dyydxx_dBx = a_dyydxx_dBx + beta2*a_dyyfxxx0 - 2.0d0*a_dyypx0
a_dyydyy_dBx = a_dyydyy_dBx + beta2*a_dyyfyyx0
a_dyydzz_dBx = a_dyydzz_dBx + beta2*a_dyyfzzx0
a_dyydxy_dBx = a_dyydxy_dBx + beta2*a_dyyfxxy0 - a_dyypy0
a_dyydxz_dBx = a_dyydxz_dBx + beta2*a_dyyfxxz0 - a_dyypz0
a_dyydyz_dBx = a_dyydyz_dBx + beta2*a_dyyfxyz0
a_dzzdxx_dBx = a_dzzdxx_dBx + beta2*a_dzzfxxx0 - 2.0d0*a_dzzpx0
a_dzzdyy_dBx = a_dzzdyy_dBx + beta2*a_dzzfyyx0
a_dzzdzz_dBx = a_dzzdzz_dBx + beta2*a_dzzfzzx0
a_dzzdxy_dBx = a_dzzdxy_dBx + beta2*a_dzzfxxy0 - a_dzzpy0
a_dzzdxz_dBx = a_dzzdxz_dBx + beta2*a_dzzfxxz0 - a_dzzpz0
a_dzzdyz_dBx = a_dzzdyz_dBx + beta2*a_dzzfxyz0
a_dxydxx_dBx = a_dxydxx_dBx + beta2*a_dxyfxxx0 - 2.0d0*a_dxypx0
a_dxydyy_dBx = a_dxydyy_dBx + beta2*a_dxyfyyx0
a_dxydzz_dBx = a_dxydzz_dBx + beta2*a_dxyfzzx0
a_dxydxy_dBx = a_dxydxy_dBx + beta2*a_dxyfxxy0 - a_dxypy0
a_dxydxz_dBx = a_dxydxz_dBx + beta2*a_dxyfxxz0 - a_dxypz0
a_dxydyz_dBx = a_dxydyz_dBx + beta2*a_dxyfxyz0
a_dxzdxx_dBx = a_dxzdxx_dBx + beta2*a_dxzfxxx0 - 2.0d0*a_dxzpx0
a_dxzdyy_dBx = a_dxzdyy_dBx + beta2*a_dxzfyyx0
a_dxzdzz_dBx = a_dxzdzz_dBx + beta2*a_dxzfzzx0
a_dxzdxy_dBx = a_dxzdxy_dBx + beta2*a_dxzfxxy0 - a_dxzpy0
a_dxzdxz_dBx = a_dxzdxz_dBx + beta2*a_dxzfxxz0 - a_dxzpz0
a_dxzdyz_dBx = a_dxzdyz_dBx + beta2*a_dxzfxyz0
a_dyzdxx_dBx = a_dyzdxx_dBx + beta2*a_dyzfxxx0 - 2.0d0*a_dyzpx0
a_dyzdyy_dBx = a_dyzdyy_dBx + beta2*a_dyzfyyx0
a_dyzdzz_dBx = a_dyzdzz_dBx + beta2*a_dyzfzzx0
a_dyzdxy_dBx = a_dyzdxy_dBx + beta2*a_dyzfxxy0 - a_dyzpy0
a_dyzdxz_dBx = a_dyzdxz_dBx + beta2*a_dyzfxxz0 - a_dyzpz0
a_dyzdyz_dBx = a_dyzdyz_dBx + beta2*a_dyzfxyz0
a_dxxdxx_dBy = a_dxxdxx_dBy + beta2*a_dxxfxxy0
a_dxxdyy_dBy = a_dxxdyy_dBy + beta2*a_dxxfyyy0 - 2.0d0*a_dxxpy0
a_dxxdzz_dBy = a_dxxdzz_dBy + beta2*a_dxxfzzy0
a_dxxdxy_dBy = a_dxxdxy_dBy + beta2*a_dxxfyyx0 - a_dxxpx0
a_dxxdxz_dBy = a_dxxdxz_dBy + beta2*a_dxxfxyz0
a_dxxdyz_dBy = a_dxxdyz_dBy + beta2*a_dxxfyyz0 - a_dxxpz0
a_dyydxx_dBy = a_dyydxx_dBy + beta2*a_dyyfxxy0
a_dyydyy_dBy = a_dyydyy_dBy + beta2*a_dyyfyyy0 - 2.0d0*a_dyypy0
a_dyydzz_dBy = a_dyydzz_dBy + beta2*a_dyyfzzy0
a_dyydxy_dBy = a_dyydxy_dBy + beta2*a_dyyfyyx0 - a_dyypx0
a_dyydxz_dBy = a_dyydxz_dBy + beta2*a_dyyfxyz0
a_dyydyz_dBy = a_dyydyz_dBy + beta2*a_dyyfyyz0 - a_dyypz0
a_dzzdxx_dBy = a_dzzdxx_dBy + beta2*a_dzzfxxy0
a_dzzdyy_dBy = a_dzzdyy_dBy + beta2*a_dzzfyyy0 - 2.0d0*a_dzzpy0
a_dzzdzz_dBy = a_dzzdzz_dBy + beta2*a_dzzfzzy0
a_dzzdxy_dBy = a_dzzdxy_dBy + beta2*a_dzzfyyx0 - a_dzzpx0
a_dzzdxz_dBy = a_dzzdxz_dBy + beta2*a_dzzfxyz0
a_dzzdyz_dBy = a_dzzdyz_dBy + beta2*a_dzzfyyz0 - a_dzzpz0
a_dxydxx_dBy = a_dxydxx_dBy + beta2*a_dxyfxxy0
a_dxydyy_dBy = a_dxydyy_dBy + beta2*a_dxyfyyy0 - 2.0d0*a_dxypy0
a_dxydzz_dBy = a_dxydzz_dBy + beta2*a_dxyfzzy0
a_dxydxy_dBy = a_dxydxy_dBy + beta2*a_dxyfyyx0 - a_dxypx0
a_dxydxz_dBy = a_dxydxz_dBy + beta2*a_dxyfxyz0
a_dxydyz_dBy = a_dxydyz_dBy + beta2*a_dxyfyyz0 - a_dxypz0
a_dxzdxx_dBy = a_dxzdxx_dBy + beta2*a_dxzfxxy0
a_dxzdyy_dBy = a_dxzdyy_dBy + beta2*a_dxzfyyy0 - 2.0d0*a_dxzpy0
a_dxzdzz_dBy = a_dxzdzz_dBy + beta2*a_dxzfzzy0
a_dxzdxy_dBy = a_dxzdxy_dBy + beta2*a_dxzfyyx0 - a_dxzpx0
a_dxzdxz_dBy = a_dxzdxz_dBy + beta2*a_dxzfxyz0
a_dxzdyz_dBy = a_dxzdyz_dBy + beta2*a_dxzfyyz0 - a_dxzpz0
a_dyzdxx_dBy = a_dyzdxx_dBy + beta2*a_dyzfxxy0
a_dyzdyy_dBy = a_dyzdyy_dBy + beta2*a_dyzfyyy0 - 2.0d0*a_dyzpy0
a_dyzdzz_dBy = a_dyzdzz_dBy + beta2*a_dyzfzzy0
a_dyzdxy_dBy = a_dyzdxy_dBy + beta2*a_dyzfyyx0 - a_dyzpx0
a_dyzdxz_dBy = a_dyzdxz_dBy + beta2*a_dyzfxyz0
a_dyzdyz_dBy = a_dyzdyz_dBy + beta2*a_dyzfyyz0 - a_dyzpz0
a_dxxdxx_dBz = a_dxxdxx_dBz + beta2*a_dxxfxxz0
a_dxxdyy_dBz = a_dxxdyy_dBz + beta2*a_dxxfyyz0
a_dxxdzz_dBz = a_dxxdzz_dBz + beta2*a_dxxfzzz0 - 2.0d0*a_dxxpz0
a_dxxdxy_dBz = a_dxxdxy_dBz + beta2*a_dxxfxyz0
a_dxxdxz_dBz = a_dxxdxz_dBz + beta2*a_dxxfzzx0 - a_dxxpx0
a_dxxdyz_dBz = a_dxxdyz_dBz + beta2*a_dxxfzzy0 - a_dxxpy0
a_dyydxx_dBz = a_dyydxx_dBz + beta2*a_dyyfxxz0
a_dyydyy_dBz = a_dyydyy_dBz + beta2*a_dyyfyyz0
a_dyydzz_dBz = a_dyydzz_dBz + beta2*a_dyyfzzz0 - 2.0d0*a_dyypz0
a_dyydxy_dBz = a_dyydxy_dBz + beta2*a_dyyfxyz0
a_dyydxz_dBz = a_dyydxz_dBz + beta2*a_dyyfzzx0 - a_dyypx0
a_dyydyz_dBz = a_dyydyz_dBz + beta2*a_dyyfzzy0 - a_dyypy0
a_dzzdxx_dBz = a_dzzdxx_dBz + beta2*a_dzzfxxz0
a_dzzdyy_dBz = a_dzzdyy_dBz + beta2*a_dzzfyyz0
a_dzzdzz_dBz = a_dzzdzz_dBz + beta2*a_dzzfzzz0 - 2.0d0*a_dzzpz0
a_dzzdxy_dBz = a_dzzdxy_dBz + beta2*a_dzzfxyz0
a_dzzdxz_dBz = a_dzzdxz_dBz + beta2*a_dzzfzzx0 - a_dzzpx0
a_dzzdyz_dBz = a_dzzdyz_dBz + beta2*a_dzzfzzy0 - a_dzzpy0
a_dxydxx_dBz = a_dxydxx_dBz + beta2*a_dxyfxxz0
a_dxydyy_dBz = a_dxydyy_dBz + beta2*a_dxyfyyz0
a_dxydzz_dBz = a_dxydzz_dBz + beta2*a_dxyfzzz0 - 2.0d0*a_dxypz0
a_dxydxy_dBz = a_dxydxy_dBz + beta2*a_dxyfxyz0
a_dxydxz_dBz = a_dxydxz_dBz + beta2*a_dxyfzzx0 - a_dxypx0
a_dxydyz_dBz = a_dxydyz_dBz + beta2*a_dxyfzzy0 - a_dxypy0
a_dxzdxx_dBz = a_dxzdxx_dBz + beta2*a_dxzfxxz0
a_dxzdyy_dBz = a_dxzdyy_dBz + beta2*a_dxzfyyz0
a_dxzdzz_dBz = a_dxzdzz_dBz + beta2*a_dxzfzzz0 - 2.0d0*a_dxzpz0
a_dxzdxy_dBz = a_dxzdxy_dBz + beta2*a_dxzfxyz0
a_dxzdxz_dBz = a_dxzdxz_dBz + beta2*a_dxzfzzx0 - a_dxzpx0
a_dxzdyz_dBz = a_dxzdyz_dBz + beta2*a_dxzfzzy0 - a_dxzpy0
a_dyzdxx_dBz = a_dyzdxx_dBz + beta2*a_dyzfxxz0
a_dyzdyy_dBz = a_dyzdyy_dBz + beta2*a_dyzfyyz0
a_dyzdzz_dBz = a_dyzdzz_dBz + beta2*a_dyzfzzz0 - 2.0d0*a_dyzpz0
a_dyzdxy_dBz = a_dyzdxy_dBz + beta2*a_dyzfxyz0
a_dyzdxz_dBz = a_dyzdxz_dBz + beta2*a_dyzfzzx0 - a_dyzpx0
a_dyzdyz_dBz = a_dyzdyz_dBz + beta2*a_dyzfzzy0 - a_dyzpy0
        end do

     end do
v(1,1,1) = a_dxxdxx_dAx
v(2,1,1) = a_dxxdyy_dAx
v(3,1,1) = a_dxxdzz_dAx
v(4,1,1) = a_dxxdxy_dAx*root3
v(5,1,1) = a_dxxdxz_dAx*root3
v(6,1,1) = a_dxxdyz_dAx*root3
v(7,1,1) = a_dyydxx_dAx
v(8,1,1) = a_dyydyy_dAx
v(9,1,1) = a_dyydzz_dAx
v(10,1,1) = a_dyydxy_dAx*root3
v(11,1,1) = a_dyydxz_dAx*root3
v(12,1,1) = a_dyydyz_dAx*root3
v(13,1,1) = a_dzzdxx_dAx
v(14,1,1) = a_dzzdyy_dAx
v(15,1,1) = a_dzzdzz_dAx
v(16,1,1) = a_dzzdxy_dAx*root3
v(17,1,1) = a_dzzdxz_dAx*root3
v(18,1,1) = a_dzzdyz_dAx*root3
v(19,1,1) = a_dxydxx_dAx*root3
v(20,1,1) = a_dxydyy_dAx*root3
v(21,1,1) = a_dxydzz_dAx*root3
v(22,1,1) = a_dxydxy_dAx*root3x2
v(23,1,1) = a_dxydxz_dAx*root3x2
v(24,1,1) = a_dxydyz_dAx*root3x2
v(25,1,1) = a_dxzdxx_dAx*root3
v(26,1,1) = a_dxzdyy_dAx*root3
v(27,1,1) = a_dxzdzz_dAx*root3
v(28,1,1) = a_dxzdxy_dAx*root3x2
v(29,1,1) = a_dxzdxz_dAx*root3x2
v(30,1,1) = a_dxzdyz_dAx*root3x2
v(31,1,1) = a_dyzdxx_dAx*root3
v(32,1,1) = a_dyzdyy_dAx*root3
v(33,1,1) = a_dyzdzz_dAx*root3
v(34,1,1) = a_dyzdxy_dAx*root3x2
v(35,1,1) = a_dyzdxz_dAx*root3x2
v(36,1,1) = a_dyzdyz_dAx*root3x2
v(1,2,1) = a_dxxdxx_dAy
v(2,2,1) = a_dxxdyy_dAy
v(3,2,1) = a_dxxdzz_dAy
v(4,2,1) = a_dxxdxy_dAy*root3
v(5,2,1) = a_dxxdxz_dAy*root3
v(6,2,1) = a_dxxdyz_dAy*root3
v(7,2,1) = a_dyydxx_dAy
v(8,2,1) = a_dyydyy_dAy
v(9,2,1) = a_dyydzz_dAy
v(10,2,1) = a_dyydxy_dAy*root3
v(11,2,1) = a_dyydxz_dAy*root3
v(12,2,1) = a_dyydyz_dAy*root3
v(13,2,1) = a_dzzdxx_dAy
v(14,2,1) = a_dzzdyy_dAy
v(15,2,1) = a_dzzdzz_dAy
v(16,2,1) = a_dzzdxy_dAy*root3
v(17,2,1) = a_dzzdxz_dAy*root3
v(18,2,1) = a_dzzdyz_dAy*root3
v(19,2,1) = a_dxydxx_dAy*root3
v(20,2,1) = a_dxydyy_dAy*root3
v(21,2,1) = a_dxydzz_dAy*root3
v(22,2,1) = a_dxydxy_dAy*root3x2
v(23,2,1) = a_dxydxz_dAy*root3x2
v(24,2,1) = a_dxydyz_dAy*root3x2
v(25,2,1) = a_dxzdxx_dAy*root3
v(26,2,1) = a_dxzdyy_dAy*root3
v(27,2,1) = a_dxzdzz_dAy*root3
v(28,2,1) = a_dxzdxy_dAy*root3x2
v(29,2,1) = a_dxzdxz_dAy*root3x2
v(30,2,1) = a_dxzdyz_dAy*root3x2
v(31,2,1) = a_dyzdxx_dAy*root3
v(32,2,1) = a_dyzdyy_dAy*root3
v(33,2,1) = a_dyzdzz_dAy*root3
v(34,2,1) = a_dyzdxy_dAy*root3x2
v(35,2,1) = a_dyzdxz_dAy*root3x2
v(36,2,1) = a_dyzdyz_dAy*root3x2
v(1,3,1) = a_dxxdxx_dAz
v(2,3,1) = a_dxxdyy_dAz
v(3,3,1) = a_dxxdzz_dAz
v(4,3,1) = a_dxxdxy_dAz*root3
v(5,3,1) = a_dxxdxz_dAz*root3
v(6,3,1) = a_dxxdyz_dAz*root3
v(7,3,1) = a_dyydxx_dAz
v(8,3,1) = a_dyydyy_dAz
v(9,3,1) = a_dyydzz_dAz
v(10,3,1) = a_dyydxy_dAz*root3
v(11,3,1) = a_dyydxz_dAz*root3
v(12,3,1) = a_dyydyz_dAz*root3
v(13,3,1) = a_dzzdxx_dAz
v(14,3,1) = a_dzzdyy_dAz
v(15,3,1) = a_dzzdzz_dAz
v(16,3,1) = a_dzzdxy_dAz*root3
v(17,3,1) = a_dzzdxz_dAz*root3
v(18,3,1) = a_dzzdyz_dAz*root3
v(19,3,1) = a_dxydxx_dAz*root3
v(20,3,1) = a_dxydyy_dAz*root3
v(21,3,1) = a_dxydzz_dAz*root3
v(22,3,1) = a_dxydxy_dAz*root3x2
v(23,3,1) = a_dxydxz_dAz*root3x2
v(24,3,1) = a_dxydyz_dAz*root3x2
v(25,3,1) = a_dxzdxx_dAz*root3
v(26,3,1) = a_dxzdyy_dAz*root3
v(27,3,1) = a_dxzdzz_dAz*root3
v(28,3,1) = a_dxzdxy_dAz*root3x2
v(29,3,1) = a_dxzdxz_dAz*root3x2
v(30,3,1) = a_dxzdyz_dAz*root3x2
v(31,3,1) = a_dyzdxx_dAz*root3
v(32,3,1) = a_dyzdyy_dAz*root3
v(33,3,1) = a_dyzdzz_dAz*root3
v(34,3,1) = a_dyzdxy_dAz*root3x2
v(35,3,1) = a_dyzdxz_dAz*root3x2
v(36,3,1) = a_dyzdyz_dAz*root3x2
v(1,1,2) = a_dxxdxx_dBx
v(2,1,2) = a_dxxdyy_dBx
v(3,1,2) = a_dxxdzz_dBx
v(4,1,2) = a_dxxdxy_dBx*root3
v(5,1,2) = a_dxxdxz_dBx*root3
v(6,1,2) = a_dxxdyz_dBx*root3
v(7,1,2) = a_dyydxx_dBx
v(8,1,2) = a_dyydyy_dBx
v(9,1,2) = a_dyydzz_dBx
v(10,1,2) = a_dyydxy_dBx*root3
v(11,1,2) = a_dyydxz_dBx*root3
v(12,1,2) = a_dyydyz_dBx*root3
v(13,1,2) = a_dzzdxx_dBx
v(14,1,2) = a_dzzdyy_dBx
v(15,1,2) = a_dzzdzz_dBx
v(16,1,2) = a_dzzdxy_dBx*root3
v(17,1,2) = a_dzzdxz_dBx*root3
v(18,1,2) = a_dzzdyz_dBx*root3
v(19,1,2) = a_dxydxx_dBx*root3
v(20,1,2) = a_dxydyy_dBx*root3
v(21,1,2) = a_dxydzz_dBx*root3
v(22,1,2) = a_dxydxy_dBx*root3x2
v(23,1,2) = a_dxydxz_dBx*root3x2
v(24,1,2) = a_dxydyz_dBx*root3x2
v(25,1,2) = a_dxzdxx_dBx*root3
v(26,1,2) = a_dxzdyy_dBx*root3
v(27,1,2) = a_dxzdzz_dBx*root3
v(28,1,2) = a_dxzdxy_dBx*root3x2
v(29,1,2) = a_dxzdxz_dBx*root3x2
v(30,1,2) = a_dxzdyz_dBx*root3x2
v(31,1,2) = a_dyzdxx_dBx*root3
v(32,1,2) = a_dyzdyy_dBx*root3
v(33,1,2) = a_dyzdzz_dBx*root3
v(34,1,2) = a_dyzdxy_dBx*root3x2
v(35,1,2) = a_dyzdxz_dBx*root3x2
v(36,1,2) = a_dyzdyz_dBx*root3x2
v(1,2,2) = a_dxxdxx_dBy
v(2,2,2) = a_dxxdyy_dBy
v(3,2,2) = a_dxxdzz_dBy
v(4,2,2) = a_dxxdxy_dBy*root3
v(5,2,2) = a_dxxdxz_dBy*root3
v(6,2,2) = a_dxxdyz_dBy*root3
v(7,2,2) = a_dyydxx_dBy
v(8,2,2) = a_dyydyy_dBy
v(9,2,2) = a_dyydzz_dBy
v(10,2,2) = a_dyydxy_dBy*root3
v(11,2,2) = a_dyydxz_dBy*root3
v(12,2,2) = a_dyydyz_dBy*root3
v(13,2,2) = a_dzzdxx_dBy
v(14,2,2) = a_dzzdyy_dBy
v(15,2,2) = a_dzzdzz_dBy
v(16,2,2) = a_dzzdxy_dBy*root3
v(17,2,2) = a_dzzdxz_dBy*root3
v(18,2,2) = a_dzzdyz_dBy*root3
v(19,2,2) = a_dxydxx_dBy*root3
v(20,2,2) = a_dxydyy_dBy*root3
v(21,2,2) = a_dxydzz_dBy*root3
v(22,2,2) = a_dxydxy_dBy*root3x2
v(23,2,2) = a_dxydxz_dBy*root3x2
v(24,2,2) = a_dxydyz_dBy*root3x2
v(25,2,2) = a_dxzdxx_dBy*root3
v(26,2,2) = a_dxzdyy_dBy*root3
v(27,2,2) = a_dxzdzz_dBy*root3
v(28,2,2) = a_dxzdxy_dBy*root3x2
v(29,2,2) = a_dxzdxz_dBy*root3x2
v(30,2,2) = a_dxzdyz_dBy*root3x2
v(31,2,2) = a_dyzdxx_dBy*root3
v(32,2,2) = a_dyzdyy_dBy*root3
v(33,2,2) = a_dyzdzz_dBy*root3
v(34,2,2) = a_dyzdxy_dBy*root3x2
v(35,2,2) = a_dyzdxz_dBy*root3x2
v(36,2,2) = a_dyzdyz_dBy*root3x2
v(1,3,2) = a_dxxdxx_dBz
v(2,3,2) = a_dxxdyy_dBz
v(3,3,2) = a_dxxdzz_dBz
v(4,3,2) = a_dxxdxy_dBz*root3
v(5,3,2) = a_dxxdxz_dBz*root3
v(6,3,2) = a_dxxdyz_dBz*root3
v(7,3,2) = a_dyydxx_dBz
v(8,3,2) = a_dyydyy_dBz
v(9,3,2) = a_dyydzz_dBz
v(10,3,2) = a_dyydxy_dBz*root3
v(11,3,2) = a_dyydxz_dBz*root3
v(12,3,2) = a_dyydyz_dBz*root3
v(13,3,2) = a_dzzdxx_dBz
v(14,3,2) = a_dzzdyy_dBz
v(15,3,2) = a_dzzdzz_dBz
v(16,3,2) = a_dzzdxy_dBz*root3
v(17,3,2) = a_dzzdxz_dBz*root3
v(18,3,2) = a_dzzdyz_dBz*root3
v(19,3,2) = a_dxydxx_dBz*root3
v(20,3,2) = a_dxydyy_dBz*root3
v(21,3,2) = a_dxydzz_dBz*root3
v(22,3,2) = a_dxydxy_dBz*root3x2
v(23,3,2) = a_dxydxz_dBz*root3x2
v(24,3,2) = a_dxydyz_dBz*root3x2
v(25,3,2) = a_dxzdxx_dBz*root3
v(26,3,2) = a_dxzdyy_dBz*root3
v(27,3,2) = a_dxzdzz_dBz*root3
v(28,3,2) = a_dxzdxy_dBz*root3x2
v(29,3,2) = a_dxzdxz_dBz*root3x2
v(30,3,2) = a_dxzdyz_dBz*root3x2
v(31,3,2) = a_dyzdxx_dBz*root3
v(32,3,2) = a_dyzdyy_dBz*root3
v(33,3,2) = a_dyzdzz_dBz*root3
v(34,3,2) = a_dyzdxy_dBz*root3x2
v(35,3,2) = a_dyzdxz_dBz*root3x2
v(36,3,2) = a_dyzdyz_dBz*root3x2

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do jx = jao, jao + 6-1
           do ix = jx, iao + 6-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = ( ix-iao ) * 6 + jx-jao +1
              do ixyz = 1, 3
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz,1)*ci
              end do
           end do
        end do
        do jx = jao, jao + 6-1
           do ix = jx, iao + 6-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = ( ix-iao ) * 6 + jx-jao +1
              do ixyz = 1, 3
                 g(ixyz,jatom) = g(ixyz,jatom) + dm(ix,jx)*v(key,ixyz,2)*ci
              end do
           end do
        end do
     else
        do jx = jao, jao + 6-1
           do ix = iao, iao + 6-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = ( ix-iao ) * 6 + jx-jao +1
              do ixyz = 1, 3
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz,1)*ci
              end do
           end do
        end do
        do jx = jao, jao + 6-1
           do ix = iao, iao + 6-1
              ci = 2.0d0
              if (ix == jx) ci = 1.0d0
              key = ( ix-iao ) * 6 + jx-jao +1
              do ixyz = 1, 3
                 g(ixyz,jatom) = g(ixyz,jatom) + dm(ix,jx)*v(key,ixyz,2)*ci
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dnai_dd_v2
